(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    531350,      14098]
NotebookOptionsPosition[    501155,      13105]
NotebookOutlinePosition[    501649,      13124]
CellTagsIndexPosition[    501606,      13121]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["LGS Bloch", "Title"],

Cell[CellGroupData[{

Cell["Reference", "Section"],

Cell[CellGroupData[{

Cell["Title", "Subsection"],

Cell["LGSBloch.m", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Authors", "Subsection"],

Cell["\<\
Simon Rochester (simon[at]rochesterscientific.com) and Ron \
Holzl\[ODoubleDot]hner (rholzloe[at]eso.org)\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Summary", "Subsection"],

Cell["\<\
Bloch equation simulation code to model the return flux from sodium laser \
guide stars (LGS)\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Copyright", "Subsection"],

Cell["\<\
\[Copyright] Copyright 2009--2016, Simon Rochester, Ron Holzl\[ODoubleDot]hner
This software is under the GNU Public License, see \
http://www.gnu.org/licenses/gpl.html

DISCLAIMER:
No responsibility will be taken for the use of any information found in this \
software whatsoever. It is not fit for any particular purpose of any kind. In \
particular, the information found in this software is not to be construed as \
an official publication or opinion of the institutions that the authors work \
for or have been working for.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Notebook Version", "Subsection"],

Cell["Version 27July2016", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " version"
}], "Subsection"],

Cell["7.0+", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Keywords", "Subsection"],

Cell["\<\
Sodium laser guide stars, Bloch equations, fluorescence, Larmor precession\
\>", "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Interface", "Section"],

Cell["This part declares publicly visible symbols.", "Text"],

Cell[CellGroupData[{

Cell["Begin package context", "Subsection"],

Cell[BoxData[
 RowBox[{"BeginPackage", "[", 
  RowBox[{"\"\<LGSBloch`\>\"", ",", 
   RowBox[{"{", "\"\<AtomicDensityMatrix`\>\"", "}"}]}], "]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"Get", "[", "\"\<AtomicDensityMatrix`\>\"", "]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"(*", 
  RowBox[{"BeginPackage", "[", 
   RowBox[{"\"\<AtomicDensityMatrix`LGSBloch`\>\"", ",", 
    RowBox[{"{", 
     RowBox[{
     "\"\<PhysicalConstants`\>\"", ",", "\"\<Units`\>\"", ",", 
      "\"\<AtomicDensityMatrix`Common`\>\"", ",", "\"\<VectorAnalysis`\>\"", 
      ",", "\"\<Splines`\>\"", ",", 
      "\"\<AtomicDensityMatrix`AtomicDensityMatrix`\>\"", ",", 
      "\"\<AtomicDensityMatrix`AtomicData`\>\"", ",", 
      "\"\<VectorAnalysis`\>\""}], "}"}]}], "]"}], "*)"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"Unprotect", "[", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"$Context", "<>", "\"\<*\>\""}], "]"}], "]"}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Usage messages", "Subsection"],

Cell[CellGroupData[{

Cell["Bloch", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SparseReplace", "::", "usage"}], "=", "\"\<SparseReplace[\!\(\*
StyleBox[\"expr\", \"TI\"]\),\!\(\*
StyleBox[\"rules\", \"TI\"]\)] generalizes ReplaceAll to apply to elements of \
a SparseArray.\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"SparseReplace[\\!\\(\\*\\nStyleBox[\\\"expr\\\", \
\\\"TI\\\"]\\),\\!\\(\\*\\nStyleBox[\\\"rules\\\", \\\"TI\\\"]\\)] \
generalizes ReplaceAll to apply to elements of a SparseArray.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SelfApply", "::", "usage"}], "=", "\"\<SelfApply[\!\(\*
StyleBox[\"rules\", \"TI\"]\)] repeatedly applies a list of rules to itself, \
transforming the right-hand sides of the rules until they no longer change.\>\
\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"SelfApply[\\!\\(\\*\\nStyleBox[\\\"rules\\\", \
\\\"TI\\\"]\\)] repeatedly applies a list of rules to itself, transforming \
the right-hand sides of the rules until they no longer change.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GenerateLGSSystem", "::", "usage"}], "=", 
  "\"\<GenerateLGSSystem[\!\(\*
StyleBox[\"atom\", \"TI\"]\), \!\(\*
StyleBox[\"line\", \"TI\"]\)] returns a list of data describing the specified \
atomic system suitable for use as the first parameter of a ReturnFlux call. \
Generally useful elements of the list include the first element, which is the \
dimension of the Bloch matrix (algebraic system) describing a particular \
region and velocity group, and the last element, which is a list of rules \
giving numerical values for various atomic parameters. It may be convenient \
to save the result of this function using Export[] and then Get[] it at the \
beginning of each session.\nGenerateLGSSystem[] is equivalent to \
GenerateLGSSystem[\\\"Na\\\",\\\"D2\\\"].\n\n\!\(\*
StyleBox[\"atom\", \"TI\"]\) and \!\(\*
StyleBox[\"line\", \"TI\"]\) are strings; possible values for \!\(\*
StyleBox[\"atom\", \"TI\"]\) are \\\"Na\\\", \\\"K39\\\", \\\"Rb87\\\", \
\\\"Rb85\\\", and \\\"Cs\\\", and possible values for \!\(\*
StyleBox[\"line\", \"TI\"]\) are \\\"D1\\\" and \\\"D2\\\". \n\nOptions: \
PrintDiagnostics, NeglectHyperfineCoherences, \
NeglectExcitedStateMixing.\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"GenerateLGSSystem[\\!\\(\\*\\nStyleBox[\\\"atom\\\", \\\"TI\
\\\"]\\), \\!\\(\\*\\nStyleBox[\\\"line\\\", \\\"TI\\\"]\\)] returns a list \
of data describing the specified atomic system suitable for use as the first \
parameter of a ReturnFlux call. Generally useful elements of the list include \
the first element, which is the dimension of the Bloch matrix (algebraic \
system) describing a particular region and velocity group, and the last \
element, which is a list of rules giving numerical values for various atomic \
parameters. It may be convenient to save the result of this function using \
Save or DumpSave and then Get it at the beginning of each \
session.\\nGenerateLGSSystem[] is equivalent to GenerateLGSSystem[\\\"Na\\\",\
\\\"D2\\\"].\\n\\n\\!\\(\\*\\nStyleBox[\\\"atom\\\", \\\"TI\\\"]\\) and \
\\!\\(\\*\\nStyleBox[\\\"line\\\", \\\"TI\\\"]\\) are strings; possible \
values for \\!\\(\\*\\nStyleBox[\\\"atom\\\", \\\"TI\\\"]\\) are \\\"Na\\\", \
\\\"K39\\\", \\\"Rb87\\\", \\\"Rb85\\\", and \\\"Cs\\\", and possible values \
for \\!\\(\\*\\nStyleBox[\\\"line\\\", \\\"TI\\\"]\\) are \\\"D1\\\" and \
\\\"D2\\\". \\n\\nOptions: PrintDiagnostics, NeglectHyperfineCoherences, \
NeglectExcitedStateMixing.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PrintDiagnostics", "::", "usage"}], "=", 
  "\"\<PrintDiagnostics is an option for GenerateLGSSystem that specifies \
whether to print the results of various intermediate steps during \
execution.\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"PrintDiagnostics is an option for GenerateLGSSystem that \
specifies whether to print the results of various intermediate steps during \
execution.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"NeglectHyperfineCoherences", "::", "usage"}], "=", 
  "\"\<NeglectHyperfineCoherences is an option for GenerateLGSSystem that \
specifies whether to set all ground- and excited- state hyperfine coherences \
to zero in order to reduce the size of the algebraic system.\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"NeglectHyperfineCoherences is an option for \
GenerateLGSSystem that specifies whether to set all ground- and excited- \
state hyperfine coherences to zero in order to reduce the size of the \
algebraic system.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"NeglectExcitedStateMixing", "::", "usage"}], "=", 
  "\"\<NeglectExcitedStateMixing is an option for GenerateLGSSystem that \
specifies whether to neglect magnetic-field-induced mixing of excited-state \
hyperfine sublevels.\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"NeglectExcitedStateMixing is an option for \
GenerateLGSSystem that specifies whether to neglect magnetic-field-induced \
mixing of excited-state hyperfine sublevels.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"UseBreitRabi", "::", "usage"}], "=", 
  "\"\<UseBreitRabi is an option for GenerateLGSSystem that specifies whether \
to use the BreitRabi formula to describe nonlinear Zeeman shifts in the \
ground state.\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"UseBreitRabi is an option for GenerateLGSSystem that \
specifies whether to use the BreitRabi formula to describe nonlinear Zeeman \
shifts in the ground state.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"NDither", "::", "usage"}], "=", 
  "\"\<ReturnFlux input parameter: Number of regions to be used for the \
dithering simulation.\>\"", " ", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "ndither"}], "*)"}], " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnFlux input parameter: Number of regions to be used \
for the dithering simulation.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DitherRate", "::", "usage"}], "=", 
  "\"\<ReturnFlux input parameter: Transit rate between dithering regions (\!\
\(\*SuperscriptBox[\(s\), \(-1\)]\)).\>\"", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "\[Gamma]dither"}], "*)"}], " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnFlux input parameter: Transit rate between dithering \
regions (\\!\\(\\*SuperscriptBox[\\(s\\), \\(-1\\)]\\)).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DitherDetuning", "::", "usage"}], "=", 
  "\"\<ReturnFlux input parameter: List specifying light detunings for each \
dithering region (rad/s).\>\"", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "\[CapitalDelta]dither"}], "*)"}], " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnFlux input parameter: List specifying light detunings \
for each dithering region (rad/s).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LightPhase", "::", "usage"}], "=", 
  "\"\<ReturnFlux input parameter: List specifying light phase for each \
dithering region.\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnFlux input parameter: List specifying light phase for \
each dithering region.\"\>"], "Output"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"ModeDetuning", "::", "usage"}], "=", "\"\<\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"LaserWidth", "::", "usage"}], "=", "\"\<\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"DitherRateFraction", "::", "usage"}], "=", "\"\<\>\""}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DitherCoherenceTransfer", "::", "usage"}], "=", 
  "\"\<ReturnFlux input parameter: Fraction of optical coherences to transfer \
when atoms travel between dithering regions.\>\"", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "\[Eta]dither"}], "*)"}], " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnFlux input parameter: Fraction of optical coherences \
to transfer when atoms travel between dithering regions.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"EllipticityA", "::", "usage"}], "=", 
  "\"\<ReturnFlux input parameter: Polarization ellipticity angle for the A \
beam.\>\"", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "\[Epsilon]a"}], "*)"}], " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnFlux input parameter: Polarization ellipticity angle \
for the A beam.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DetuningA", "::", "usage"}], "=", 
  "\"\<ReturnFlux input parameter: Light detuning for the A beam \
(rad/s).\>\"", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "\[CapitalDelta]a"}], "*)"}], " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnFlux input parameter: Light detuning for the A beam \
(rad/s).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LightIntensityA", "::", "usage"}], "=", 
  "\"\<ReturnFlux input parameter: Light intensity for the A beam \
(watt/\!\(\*SuperscriptBox[\(m\), \(2\)]\))\>\"", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "LIa"}], "*)"}], " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnFlux input parameter: Light intensity for the A beam \
(watt/\\!\\(\\*SuperscriptBox[\\(m\\), \\(2\\)]\\))\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"RabiFrequencyA", "::", "usage"}], "=", 
  "\"\<Rabi frequency for the A beam (rad/s).\>\"", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "\[CapitalOmega]Ra"}], "*)"}], " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"Rabi frequency for the A beam (rad/s).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"EllipticityB", "::", "usage"}], "=", 
  "\"\<ReturnFlux input parameter: Polarization ellipticity angle for the B \
beam.\>\"", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "\[Epsilon]b"}], "*)"}], " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnFlux input parameter: Polarization ellipticity angle \
for the B beam.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DetuningB", "::", "usage"}], "=", 
  "\"\<ReturnFlux input parameter: Light detuning for the B beam \
(rad/s).\>\"", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "\[CapitalDelta]b"}], "*)"}], " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnFlux input parameter: Light detuning for the B beam \
(rad/s).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LightIntensityB", "::", "usage"}], "=", 
  "\"\<ReturnFlux input parameter: Light intensity for the B beam \
(watt/\!\(\*SuperscriptBox[\(m\), \(2\)]\))\>\"", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "LIb"}], "*)"}], " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnFlux input parameter: Light intensity for the B beam \
(watt/\\!\\(\\*SuperscriptBox[\\(m\\), \\(2\\)]\\))\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"RabiFrequencyB", "::", "usage"}], "=", 
  "\"\<Rabi frequency for the B beam (rad/s).\>\"", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "\[CapitalOmega]Rb"}], "*)"}], " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"Rabi frequency for the B beam (rad/s).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"AtomicDensity", "::", "usage"}], "=", 
  "\"\<ReturnFlux input parameter: Atomic density \
(atoms/\!\(\*SuperscriptBox[\(m\), \(3\)]\)).\>\"", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "n0"}], "*)"}], " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnFlux input parameter: Atomic density \
(atoms/\\!\\(\\*SuperscriptBox[\\(m\\), \\(3\\)]\\)).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"BeamTransitRate", "::", "usage"}], "=", 
  "\"\<ReturnFlux input parameter: Rate of transit of atoms through the light \
beam (\!\(\*SuperscriptBox[\(s\), \(-1\)]\)).\>\"", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "\[Gamma]"}], "*)"}], " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnFlux input parameter: Rate of transit of atoms \
through the light beam (\\!\\(\\*SuperscriptBox[\\(s\\), \
\\(-1\\)]\\)).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SDampingRate", "::", "usage"}], "=", 
  "\"\<ReturnFlux input parameter: Overall S-damping rate \
(\!\(\*SuperscriptBox[\(s\), \(-1\)]\)).\>\"", " ", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "\[Gamma]s"}], "*)"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnFlux input parameter: Overall S-damping rate \
(\\!\\(\\*SuperscriptBox[\\(s\\), \\(-1\\)]\\)).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"VccRateFunction", "::", "usage"}], "=", 
  "\"\<ReturnFlux input parameter: Cumulative distribution function of rate \
of velocity-changing collisions. VccRateFunction[\[CapitalDelta]1,\
\[CapitalDelta]2] gives the rate of transfer of atoms with Doppler shift \
\[CapitalDelta]1 into velocity groups with Doppler shifts less than \
\[CapitalDelta]2 (\!\(\*SuperscriptBox[\(s\), \(-1\)]\)).\>\"", " ", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "\[Gamma]c"}], "*)"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnFlux input parameter: Cumulative distribution \
function of rate of velocity-changing collisions. VccRateFunction[\
\[CapitalDelta]1,\[CapitalDelta]2] gives the rate of transfer of atoms with \
Doppler shift \[CapitalDelta]1 into velocity groups with Doppler shifts less \
than \[CapitalDelta]2 (\\!\\(\\*SuperscriptBox[\\(s\\), \\(-1\\)]\\)).\"\>"], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"RMSVelocity", "::", "usage"}], "=", 
  "\"\<Atomic rms velocity (m/s).\>\"", " ", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "Vrms"}], "*)"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"Atomic rms velocity (m/s).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Temperature", "::", "usage"}], "=", 
  "\"\<ReturnFlux input parameter: System temperature in degrees Kelvin.\>\"",
   " ", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "T"}], "*)"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnFlux input parameter: System temperature in degrees \
Kelvin.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ResonanceFrequencies", "::", "usage"}], "=", 
  "\"\<List of possible optical resonance frequencies for the atomic system \
(rad/s).\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"List of possible optical resonance frequencies for the \
atomic system (rad/s).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DopplerWidth", "::", "usage"}], "=", 
  "\"\<Doppler width of the atomic thermal distribution (rad/s).\>\""}]], \
"Input",
 InitializationCell->True],

Cell[BoxData["\<\"Doppler width of the atomic thermal distribution (rad/s).\"\
\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LarmorFrequency", "::", "usage"}], "=", 
  "\"\<ReturnFlux input parameter: Larmor frequency of the highest angular \
momentum hyperfine ground state (rad/s).\>\"", " ", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "\[CapitalOmega]L"}], "*)"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnFlux input parameter: Larmor frequency of the highest \
angular momentum hyperfine ground state (rad/s).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MagneticZenith", "::", "usage"}], "=", 
  "\"\<ReturnFlux input parameter: Zenith angle of the magnetic field in the \
local coordinate system, where the laser points at +z (angle between B and \
z). MagneticZenith = 0 corresponds to the B-field pointing along the laser \
propagation direction. CAUTION: The geomagnetic field points from \
geographicSouth to North! This means e.g. that MagneticZenith cannot become 0 \
in the northern magnetic hemisphere unless the laser is pointing towards the \
ground\>\"", " ", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "\[Theta]Magnetic"}], "*)"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnFlux input parameter: Zenith angle of the magnetic \
field (angle between B and z).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MagneticAzimuth", "::", "usage"}], "=", 
  "\"\<ReturnFlux input parameter: Azimuthal angle of the magnetic field \
(angle between x and the projection of B on the x-y plane).\>\"", " ", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "\[Phi]Magnetic"}], "*)"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnFlux input parameter: Azimuthal angle of the magnetic \
field (angle between x and the projection of B on the x-y plane).\"\>"], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"RecoilShift", "::", "usage"}], "=", 
  "\"\<ReturnFlux input parameter: Doppler shift corresponding to the atomic \
recoil velocity (rad/s).\>\"", " ", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "vr"}], "*)"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnFlux input parameter: Doppler shift corresponding to \
the atomic recoil velocity (rad/s).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DitherRegionFraction", "::", "usage"}], "=", 
  "\"\<DitherRegionFraction[\!\(\*
StyleBox[\"i\", \"TI\"]\)] represents the fraction of atoms present in \
dithering region \!\(\*
StyleBox[\"i\", \"TI\"]\).\>\"", " ", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "BF"}], "*)"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"DitherRegionFraction[\\!\\(\\*\\nStyleBox[\\\"i\\\", \\\"TI\
\\\"]\\)] represents the fraction of atoms present in dithering region \\!\\(\
\\*\\nStyleBox[\\\"i\\\", \\\"TI\\\"]\\).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"VGDensity", "::", "usage"}], "=", "\"\<VGDensity[\!\(\*
StyleBox[\"i\", \"TI\"]\)] represents the fraction of atoms in the velocity \
group \!\(\*
StyleBox[\"i\", \"TI\"]\).\>\"", " ", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "G"}], "*)"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"VGDensity[\\!\\(\\*\\nStyleBox[\\\"i\\\", \\\"TI\\\"]\\)] \
represents the fraction of atoms in the velocity group \
\\!\\(\\*\\nStyleBox[\\\"i\\\", \\\"TI\\\"]\\).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DopplerShift", "::", "usage"}], "=", "\"\<DopplerShift[\!\(\*
StyleBox[\"i\", \"TI\"]\)] represents the Doppler shift for atoms in the \
velocity group \!\(\*
StyleBox[\"i\", \"TI\"]\).\>\"", " ", 
  RowBox[{"(*", 
   RowBox[{"was", " ", "S"}], "*)"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"DopplerShift[\\!\\(\\*\\nStyleBox[\\\"i\\\", \
\\\"TI\\\"]\\)] represents the Doppler shift for atoms in the velocity group \
\\!\\(\\*\\nStyleBox[\\\"i\\\", \\\"TI\\\"]\\).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"VelocityGroups1", "::", "usage"}], "=", 
  "\"\<VelocityGroups1 returns a list of velocity bins formed using one \
(small) bin size near a resonance frequency and another (larger) bin size \
away from any resonance frequencies.\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"VelocityGroups1 returns a list of velocity bins formed \
using one (small) bin size near a resonance frequency and another (larger) \
bin size away from any resonance frequencies.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"VelocityGroups2", "::", "usage"}], "=", 
  "\"\<VelocityGroups2 returns a list of velocity bins found by dividing the \
integral of a supplied weighting function evenly into a specified number of \
bins.\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"VelocityGroups2 returns a list of velocity bins found by \
dividing the integral of a supplied weighting function evenly into a \
specified number of bins.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ShowPlot", "::", "usage"}], "=", 
  "\"\<ShowPlot is an option for the VelocityGroups functions that specifies \
whether to generate a plot of the list of velocity groups returned by the \
function.\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ShowPlot is an option for the VelocityGroups functions that \
specifies whether to generate a plot of the list of velocity groups returned \
by the function.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ReturnFlux", "::", "usage"}], "=", "\"\<ReturnFlux[\!\(\*
StyleBox[\"system\", \"TI\"]\), \!\(\*
StyleBox[\"params\", \"TI\"]\)] returns the result of a calculation of \
observed fluorescence for the atomic and experimental system described by \
\!\(\*
StyleBox[\"system\", \"TI\"]\) (as given by the output of GenerateLGSSystem), \
using experimental paramters specified by \!\(\*
StyleBox[\"params\", \"TI\"]\). Additional output information can be \
requested using the FluxReport option. A list of velocity group bins is \
automatically created using the parameters specified with the option \
InitVGParams; if the option RefineResult is specified the calculation will be \
run multiple times, refining the velocity bins on successive runs.\n\
ReturnFlux[\!\(\*
StyleBox[\"system\", \"TI\"]\), \!\(\*
StyleBox[\"params\", \"TI\"]\), \!\(\*
StyleBox[\"bins\", \"TI\"]\)] does the first run of the calculation using the \
supplied list of velocity bins \!\(\*
StyleBox[\"bins\", \"TI\"]\).\n\nThe input \!\(\*
StyleBox[\"params\", \"TI\"]\) must contain the parameters NDither, \
DitherRate, DitherDetuning, LightPhase, DitherCoherenceTransfer, \
EllipticityA, DetuningA, LightIntensityA, EllipticityB, DetuningB, \
LightIntensityB, AtomicDensity, BeamTransitRate, SDampingRate, VccRate, \
Temperature, LarmorFrequency, MagneticZenith, MagneticAzimuth, RecoilShift.\n\
\nOptions: FluxReport, RefineResult, InitVGParams, ReturnAllSteps, \
PlotVelocityGroups\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnFlux[\\!\\(\\*\\nStyleBox[\\\"system\\\", \\\"TI\\\"]\
\\), \\!\\(\\*\\nStyleBox[\\\"params\\\", \\\"TI\\\"]\\)] returns the result \
of a calculation of observed fluorescence for the atomic and experimental \
system described by \\!\\(\\*\\nStyleBox[\\\"system\\\", \\\"TI\\\"]\\) (as \
given by the output of GenerateLGSSystem), using experimental paramters \
specified by \\!\\(\\*\\nStyleBox[\\\"params\\\", \\\"TI\\\"]\\). Additional \
output information can be requested using the FluxReport option. A list of \
velocity group bins is automatically created using the parameters specified \
with the option InitVGParams; if the option RefineResult is specified the \
calculation will be run multiple times, refining the velocity bins on \
successive runs.\\nReturnFlux[\\!\\(\\*\\nStyleBox[\\\"system\\\", \
\\\"TI\\\"]\\), \\!\\(\\*\\nStyleBox[\\\"params\\\", \\\"TI\\\"]\\), \
\\!\\(\\*\\nStyleBox[\\\"bins\\\", \\\"TI\\\"]\\)] does the first run of the \
calculation using the supplied list of velocity bins \
\\!\\(\\*\\nStyleBox[\\\"bins\\\", \\\"TI\\\"]\\).\\n\\nThe input \
\\!\\(\\*\\nStyleBox[\\\"params\\\", \\\"TI\\\"]\\) must contain the \
parameters NDither, DitherRate, DitherDetuning, LightPhase, \
DitherCoherenceTransfer, EllipticityA, DetuningA, LightIntensityA, \
EllipticityB, DetuningB, LightIntensityB, AtomicDensity, BeamTransitRate, \
SDampingRate, VccRate, Temperature, LarmorFrequency, MagneticZenith, \
MagneticAzimuth, RecoilShift.\\n\\nOptions: FluxReport, RefineResult, \
InitVGParams, ReturnAllSteps, PlotVelocityGroups\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FluxReport", "::", "usage"}], "=", 
  "\"\<FluxReport is an option for ReturnFlux that specifies addtional \
information to be returned as a list of replacement values.\nA list should be \
specified, contaning zero or more of the following elements:\n\
VelocitySpectrum: Returns a list giving the contribution to the flux from \
each velocity group.\nVelocityGroups: Returns the list of velocity bins used \
in the calculation.\nDMSolution: Returns the computed values of the density \
matrix elements for each velocity group and dithering region.\nBlochMatrix: \
Returns a list of the full matrix that was generated and solved, the RHS \
vector, and the vector that was dotted with the solution vector to find the \
total flux. \nRFParams: Return the parameter set of ReturnFlux[] (internal \
use)\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"FluxReport is an option for ReturnFlux that specifies \
addtional information to be returned as a list of replacement values.\\nA \
list should be specified, contaning zero or more of the following \
elements:\\nVelocitySpectrum: Returns a list giving the contribution to the \
flux from each velocity group.\\nVelocityGroups: Returns the list of velocity \
bins used in the calculation.\\nDMSolution: Returns the computed values of \
the density matrix elements for each velocity group and dithering \
region.\\nBlochMatrix: Returns a list of the full matrix that was generated \
and solved, the RHS vector, and the vector that was dotted with the solution \
vector to find the total flux. \\nRFParams: Return the parameter set of \
ReturnFlux[] (internal use)\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SkipLinearSolve", "::", "usage"}], " ", "=", "\n", 
  "\"\<SkipLinearSolve is an option for ReturnFlux that specifies if the \
linear Bloch equation system should be solved, or if the solution should be \
skipped, and only the Bloch matrices should be prepared for the FluxReport. \
For internal use.\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"SkipLinearSolve is an option for ReturnFlux that specifies \
if the linear Bloch equation system should be solved, or if the solution \
should be skipped, and only the Bloch matrices should be prepared for the \
FluxReport. For internal use.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"RefineResult", "::", "usage"}], "=", 
  "\"\<RefineResult is an option for ReturnFlux that specifies whether the \
calculation should be repeated using the preliminary results to refine the \
list of velocity bins. If a list {\!\(\*SubscriptBox[
StyleBox[\"i\", \"TI\"], \"1\"]\),\!\(\*SubscriptBox[
StyleBox[\"i\", \"TI\"], \"2\"]\)...\!\(\*
StyleBox[SubscriptBox[\"i\", \"n\"], \"TI\"]\)} of integers \!\(\*
StyleBox[SubscriptBox[\"i\", \"j\"], \"TI\"]\) is specified, the calculation \
is run \!\(\*
StyleBox[\"n\", \"TI\"]\) times after the initial run, each time using the \
velocity spectrum obtained from the previous run to create a weighting \
function used to generate a list of \!\(\*
StyleBox[SubscriptBox[\"i\", \"j\"], \"TI\"]\) velocity bins. The option \
ReturnAllSteps can be used to specify whether the results of all calculations \
should be returned, or just the last.\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"RefineResult is an option for ReturnFlux that specifies \
whether the calculation should be repeated using the preliminary results to \
refine the list of velocity bins. If a list {\\!\\(\\*SubscriptBox[\\n \
StyleBox[\\\"i\\\", \\\"TI\\\"], \\\"1\\\"]\\),\\!\\(\\*SubscriptBox[\\n \
StyleBox[\\\"i\\\", \\\"TI\\\"], \
\\\"2\\\"]\\)...\\!\\(\\*\\nStyleBox[SubscriptBox[\\\"i\\\", \\\"n\\\"], \
\\\"TI\\\"]\\)} of integers \\!\\(\\*\\nStyleBox[SubscriptBox[\\\"i\\\", \
\\\"j\\\"], \\\"TI\\\"]\\) is specified, the calculation is run \
\\!\\(\\*\\nStyleBox[\\\"n\\\", \\\"TI\\\"]\\) times after the initial run, \
each time using the velocity spectrum obtained from the previous run to \
create a weighting function used to generate a list of \
\\!\\(\\*\\nStyleBox[SubscriptBox[\\\"i\\\", \\\"j\\\"], \\\"TI\\\"]\\) \
velocity bins. The option ReturnAllSteps can be used to specify whether the \
results of all calculations should be returned, or just the last.\"\>"], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"InitVGParams", "::", "usage"}], "=", 
  "\"\<InitVGParams is an option for ReturnFlux that specifies the parameters \
used to generate the velocity group bins for the first run of the \
calculation. Velocities are specified in terms of their corresponding Doppler \
shift. The option is specified in the form {{\!\(\*
StyleBox[SubscriptBox[\"width\", \"res\"], \"TI\"]\),\!\(\*
StyleBox[SubscriptBox[\"n\", \"res\"], \"TI\"]\)},{\!\(\*
StyleBox[SubscriptBox[\"width\", \"nonres\"], \"TI\"]\),\!\(\*
StyleBox[SubscriptBox[\"range\", \"nonres\"], \"TI\"]\)}}, where \!\(\*
StyleBox[SubscriptBox[\"width\", \"res\"], \"TI\"]\) is the width of bins to \
be created near resonance frequencies (in units of the natural width), \!\(\*
StyleBox[SubscriptBox[\"n\", \"res\"], \"TI\"]\) specifies that 2\!\(\*
StyleBox[SubscriptBox[\"n\", \"res\"], \"TI\"]\)+1 bins should be created \
near each resonance frequency, \!\(\*
StyleBox[SubscriptBox[\"width\", \"nonres\"], \"TI\"]\) is the width of bins \
not near resonance frequencies (in units of the Doppler width), and \!\(\*
StyleBox[SubscriptBox[\"range\", \"nonres\"], \"TI\"]\) (units of the Doppler \
width) gives the half-range over which velocity bins should be \
generated.\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"InitVGParams is an option for ReturnFlux that specifies the \
parameters used to generate the velocity group bins for the first run of the \
calculation. Velocities are specified in terms of their corresponding Doppler \
shift. The option is specified in the form \
{{\\!\\(\\*\\nStyleBox[SubscriptBox[\\\"width\\\", \\\"res\\\"], \
\\\"TI\\\"]\\),\\!\\(\\*\\nStyleBox[SubscriptBox[\\\"n\\\", \\\"res\\\"], \
\\\"TI\\\"]\\)},{\\!\\(\\*\\nStyleBox[SubscriptBox[\\\"width\\\", \
\\\"nonres\\\"], \\\"TI\\\"]\\),\\!\\(\\*\\nStyleBox[SubscriptBox[\\\"range\\\
\", \\\"nonres\\\"], \\\"TI\\\"]\\)}}, where \
\\!\\(\\*\\nStyleBox[SubscriptBox[\\\"width\\\", \\\"res\\\"], \\\"TI\\\"]\\) \
is the width of bins to be created near resonance frequencies (in units of \
the natural width), \\!\\(\\*\\nStyleBox[SubscriptBox[\\\"n\\\", \
\\\"res\\\"], \\\"TI\\\"]\\) specifies that \
2\\!\\(\\*\\nStyleBox[SubscriptBox[\\\"n\\\", \\\"res\\\"], \\\"TI\\\"]\\)+1 \
bins should be created near each resonance frequency, \
\\!\\(\\*\\nStyleBox[SubscriptBox[\\\"width\\\", \\\"nonres\\\"], \\\"TI\\\"]\
\\) is the width of bins not near resonance frequencies (in units of the \
Doppler width), and \\!\\(\\*\\nStyleBox[SubscriptBox[\\\"range\\\", \
\\\"nonres\\\"], \\\"TI\\\"]\\) (units of the Doppler width) gives the \
half-range over which velocity bins should be generated.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ReturnAllSteps", "::", "usage"}], "=", 
  "\"\<ReturnAllSteps is an option for ReturnFlux that specifies whether the \
results of all iterations (specified using RefineResult) should be returned, \
or just the last.\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ReturnAllSteps is an option for ReturnFlux that specifies \
whether the results of all iterations (specified using RefineResult) should \
be returned, or just the last.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PlotVelocityGroups", "::", "usage"}], "=", 
  "\"\<PlotVelocityGroups is an option for ReturnFlux that specifies whether \
diagnostic plots showing the generated velocity bins should be shown during \
execution.\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"PlotVelocityGroups is an option for ReturnFlux that \
specifies whether diagnostic plots showing the generated velocity bins should \
be shown during execution.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"VelocitySpectrum", "::", "usage"}], "=", 
  "\"\<VelocitySpectrum is a value that can be specified by the FluxReport \
option of ReturnFlux to request a list giving the contribution to the flux \
from each velocity group.\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"VelocitySpectrum is a value that can be specified by the \
FluxReport option of ReturnFlux to request a list giving the contribution to \
the flux from each velocity group.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"VelocityGroups", "::", "usage"}], "=", 
  "\"\<VelocityGroups is a value that can be specified by the FluxReport \
option of ReturnFlux to request the list of velocity bins used in the \
calculation.\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"VelocityGroups is a value that can be specified by the \
FluxReport option of ReturnFlux to request the list of velocity bins used in \
the calculation.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DMSolution", "::", "usage"}], "=", 
  "\"\<DMSolution is a value that can be specified by the FluxReport option \
of ReturnFlux to request the computed values of the density matrix elements \
for each velocity group and dithering region.\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"DMSolution is a value that can be specified by the \
FluxReport option of ReturnFlux to request the computed values of the density \
matrix elements for each velocity group and dithering region.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"BlochMatrix", "::", "usage"}], "=", 
  "\"\<BlochMatrix is a value that can be specified by the FluxReport option \
of ReturnFlux to request a list of the full matrix that was generated and \
solved, the RHS vector, and the vector that was dotted with the solution \
vector to find the total flux.\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"BlochMatrix is a value that can be specified by the \
FluxReport option of ReturnFlux to request a list of the full matrix that was \
generated and solved, the RHS vector, and the vector that was dotted with the \
solution vector to find the total flux.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"RFParams", "::", "usage"}], "=", "\[IndentingNewLine]", 
  "\"\<RFParams is the parameter set of ReturnFlux[]. For internal use in \
StepPulseFlux[].\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"RFParams is the parameter set of ReturnFlux[]. For internal \
use in StepPulseFlux[].\"\>"], "Output"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"phiv", "::", "usage"}], "=", 
  "\"\<[w, err] = phiv( t, A, u, v, tol, m )\n phiv computes an approximation \
of w = exp(t*A)*v + t*phi(t*A)*u\n for a general matrix A using Krylov \
subspace projection techniques.\n Here, phi(z) = (exp(z)-1)/z and w is the \
solution of the \n nonhomogeneous linear ODE problem w' = Aw + u, w(0) = v.\n \
It does not compute the matrix functions in isolation but instead,\n it \
computes directly the action of these functions on the \n operand vectors. \
This way of doing so allows for addressing large\n sparse problems. The \
matrix under consideration interacts only\n via matrix-vector products \
(matrix-free method).\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"VerbosityLevel", "::", "usage"}], "=", "\"\<\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"InitVG", "::", "usage"}], "=", "\"\<\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"CDFGaussian", "::", "usage"}], "=", "\"\<\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"PulsedSetup", "::", "usage"}], "=", "\"\<\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"FlipDoppler", "::", "usage"}], "=", "\"\<\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"VccRate", "::", "usage"}], " ", "=", " ", "\"\<\>\""}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["LGS Constants and Default Parameters", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"BGParanal", "::", "usage"}], "=", 
  "\"\<BGParanal is a constant giving the geomagnetic field in Paranal at \
altitude 90km in 2009 [G] from [MSIS90].\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"BGParanal is a constant giving the geomagnetic field in \
Paranal at altitude 90km in 2009 [G] from [MSIS90].\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"BGSOR", "::", "usage"}], "=", 
  "\"\<BGSOR is a constant giving the geomagnetic field at SOR at altutude \
90km in 2009 [G] from [MSIS90].\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"BGSOR is a constant giving the geomagnetic field at SOR at \
altutude 90km in 2009 [G] from [MSIS90].\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"BGWendelstein", "::", "usage"}], "=", 
  "\"\<BGWendelstein is a constant giving the geomagnetic field over \
Wendelstein at altutude 90km in 2009 [G] from [MSIS90].\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"BGWendelstein is a constant giving the geomagnetic field \
over Wendelstein at altutude 90km in 2009 [G] from [MSIS90].\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"BGMaunaKea", "::", "usage"}], "=", 
  "\"\<BGMaunaKea is a constant giving the geomagnetic field over Mauna Kea \
at altutude 90km in 2009 [G] from [MSIS90].\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"BGMaunaKea is a constant giving the geomagnetic field over \
Mauna Kea at altutude 90km in 2009 [G] from [MSIS90].\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"\[CapitalOmega]NaRecoil", "::", "usage"}], "=", 
  "\"\<\[CapitalOmega]NaRecoil is a constant giving the recoil frequency of \
sodium atoms at 589nm [Hz].\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"\[CapitalOmega]NaRecoil is a constant giving the recoil \
frequency of sodium atoms at 589nm [Hz].\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"fOffsetD2a", "::", "usage"}], "=", 
  "\"\<fOffsetD2a is a constant giving the offset of the Na D2a line from the \
D2 line center [Hz]. Do not alter this parameter when varying the laser \
central frequency, but use \[CapitalDelta]fLaser instead.\>\"", 
  " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"fOffsetD2a is a constant giving the offset of the Na D2a \
line from the D2 line center [Hz]. Do not alter this parameter when varying \
the laser central frequency, but use \[CapitalDelta]fLaser instead.\"\>"], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"fOffsetD2ab", "::", "usage"}], "=", 
  "\"\<fOffsetD2ab is a constant giving the optimal frequency offset between \
Na D2a and D2b in sodium [Hz]. Normally, this constant should not be altered. \
Use \[CapitalDelta]fLaserDabOff to vary to repumping frequency offset.\>\"", 
  " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"fOffsetD2ab is a constant giving the optimal frequency \
offset between Na D2a and D2b in sodium [Hz]. Normally, this constant should \
not be altered. Use \[CapitalDelta]fLaserDabOff to vary to repumping \
frequency offset.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"IMeso", "::", "usage"}], "=", 
  "\"\<IMeso represents mesospheric laser intensity \!\(\*FormBox[
RowBox[{
RowBox[{\"[\", 
RowBox[{
StyleBox[\"W\",
FontSlant->\"Plain\"], \"/\", SuperscriptBox[
StyleBox[\"m\",
FontSlant->\"Plain\"], \"2\"]}], \"]\"}], \".\"}],
TraditionalForm]\)\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"IMeso represents mesospheric laser intensity \
\\!\\(\\*FormBox[\\n RowBox[{\\n  RowBox[{\\\"[\\\", \\n   RowBox[{\\n    \
StyleBox[\\\"W\\\",\\nFontSlant->\\\"Plain\\\"], \\\"/\\\", \
SuperscriptBox[\\n     StyleBox[\\\"m\\\",\\nFontSlant->\\\"Plain\\\"], \\\"2\
\\\"]}], \\\"]\\\"}], \\\".\\\"}],\\n TraditionalForm]\\)\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"RepumpFraction", "::", "usage"}], "=", 
  "\"\<RepumpFraction represents the repumping intensity fraction (fraction \
of total laser power in D2b line).\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"RepumpFraction represents the repumping intensity fraction \
(fraction of total laser power in D2b line).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LightPolarization", "::", "usage"}], "=", 
  "\"\<LightPolarization represents the normalized light ellipticity angle (\
\[Epsilon]/(\[Pi]/4)).\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"LightPolarization represents the normalized light \
ellipticity angle (\[Epsilon]/(\[Pi]/4)).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LightWavelength", "::", "usage"}], "=", 
  "\"\<LightWavelength represents the laser center wavelength [m].\>\"", 
  " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"LightWavelength represents the laser center wavelength [m].\
\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Phase", "::", "usage"}], "=", 
  "\"\<Phase represents overall light phase [rad].\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"Phase represents overall light phase [rad].\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FWHMbw", "::", "usage"}], "=", 
  "\"\<FWHMbw represents the Laser FWHM bandwidth, assuming a Lorentzian \
lineshape [Hz].\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"FWHMbw represents the Laser FWHM bandwidth, assuming a \
Lorentzian lineshape [Hz].\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"\[CapitalDelta]fLaser", "::", "usage"}], "=", 
  "\"\<\[CapitalDelta]fLaser represents the laser line detuning from optimum \
(both D2a and D2b) [Hz]. Use this parameter to vary the laser line frequency \
offset. Positive values shift to the blue (higher frequency).\>\"", 
  " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"\[CapitalDelta]fLaser represents the laser line detuning \
from optimum (both D2a and D2b) [Hz]. Use this parameter to vary the laser \
line frequency offset. Positive values shift to the blue (higher \
frequency).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"\[CapitalDelta]fLaserDabOff", "::", "usage"}], "=", 
  "\"\<\[CapitalDelta]fLaserDabOff represents the offset of D2a--D2b \
frequency spacing from fOffsetD2ab [Hz]. Positive values shift to the blue \
(higher frequency).\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"\[CapitalDelta]fLaserDabOff represents the offset of \
D2a--D2b frequency spacing from fOffsetD2ab [Hz]. Positive values shift to \
the blue (higher frequency).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"BG", "::", "usage"}], "=", 
  "\"\<BG represents the mesospheric geomagnetic field strength \!\(\*FormBox[
RowBox[{\"[\", 
StyleBox[\"G\",
FontSlant->\"Plain\"], \"]\"}],
TraditionalForm]\).\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"BG represents the mesospheric geomagnetic field strength \
\\!\\(\\*FormBox[\\n RowBox[{\\\"[\\\", \\n  \
StyleBox[\\\"G\\\",\\nFontSlant->\\\"Plain\\\"], \\\"]\\\"}],\\n \
TraditionalForm]\\).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Ta", "::", "usage"}], "=", 
  "\"\<Ta represents the one-way atmospheric light transmission at \[Lambda] \
at zenith.\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"Ta represents the one-way atmospheric light transmission at \
\[Lambda] at zenith.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SDampingTime", "::", "usage"}], "=", 
  "\"\<SDampingTime represents the mean time between S-damping collisions \
[s].\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"SDampingTime represents the mean time between S-damping \
collisions [s].\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"VccTime", "::", "usage"}], "=", 
  "\"\<VccTime represents the mean time between velocity-changing collisions \
[s].\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"VccTime represents the mean time between velocity-changing \
collisions [s].\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"HNa", "::", "usage"}], "=", 
  "\"\<HNa represents the altitude of sodium centroid above sea level \
[m].\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"HNa represents the altitude of sodium centroid above sea \
level [m].\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"vrms\[Gamma]", "::", "usage"}], "=", 
  "\"\<vrms\[Gamma] represents the rms velocity of diffusion effects [m/s], \
\[Gamma] = vrms\[Gamma]/(DLT/\[Mu]LT).\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"vrms\[Gamma] represents the rms velocity of diffusion \
effects [m/s], \[Gamma] = vrms\[Gamma]/(DLT/\[Mu]LT).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PLaunch", "::", "usage"}], "=", 
  "\"\<PLaunch represents the launched power (output of launch telescope, not \
power on sky) [W].\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"PLaunch represents the launched power (output of launch \
telescope, not power on sky) [W].\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Zenith", "::", "usage"}], "=", 
  "\"\<Zenith represents the Zenith angle of the laser beam [rad].\>\"", 
  " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"Zenith represents the Zenith angle of the laser beam [rad].\
\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Htele", "::", "usage"}], "=", 
  "\"\<Htele represents the altitude of launch telescope above sea level [m].\
\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"Htele represents the altitude of launch telescope above sea \
level [m].\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DLT", "::", "usage"}], "=", 
  "\"\<DLT represents the launch telescope aperture [m].\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"DLT represents the launch telescope aperture [m].\"\>"], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Mu]LT", "::", "usage"}], "=", 
  "\"\<\[Mu]LT represents the ratio of DLT to \
\!\(TraditionalForm\`1/\*SuperscriptBox[\(e\), \(2\)]\) lauch beam intensity \
diameter (not radius!), see Yura, Eq.(2.7).\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"\[Mu]LT represents the ratio of DLT to \
\\!\\(TraditionalForm\\`1/\\*SuperscriptBox[\\(e\\), \\(2\\)]\\) lauch beam \
intensity diameter (not radius!), see Yura, Eq.(2.7).\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MsqrLT", "::", "usage"}], "=", 
  "\"\<MsqrLT represents the \!\(TraditionalForm\`\*SuperscriptBox[\(M\), \(2\
\)]\\\ beam\\\ quality\\\ factor\\\ out\\\ of\\\ the\\\ launch\\\ \
telescope\).\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"MsqrLT represents the \
\\!\\(TraditionalForm\\`\\*SuperscriptBox[\\(M\\), \\(2\\)]\\\\ beam\\\\ \
quality\\\\ factor\\\\ out\\\\ of\\\\ the\\\\ launch\\\\ telescope\\).\"\>"], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"IMesoMax", "::", "usage"}], "=", 
  "\"\<IMesoMax is an option for BeamRepository and related functions that \
specfies the maximum intensity in mesosphere [\!\(\*
StyleBox[\"W\",
FontSlant->\"Plain\"]\)/\!\(\*SuperscriptBox[
StyleBox[\"m\",
FontSlant->\"Plain\"], \"2\"]\)]. If set to zero it is determined \
automatically.\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"IMesoMax is an option for BeamRepository and related \
functions that specfies the maximum intensity in mesosphere \
[\\!\\(\\*\\nStyleBox[\\\"W\\\",\\nFontSlant->\\\"Plain\\\"]\\)/\\!\\(\\*\
SuperscriptBox[\\n StyleBox[\\\"m\\\",\\nFontSlant->\\\"Plain\\\"], \
\\\"2\\\"]\\)]. If set to zero it is determined automatically.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"IMesoInitial", "::", "usage"}], "=", 
  "\"\<IMesoInitial is an option for BeamRepository and related functions \
that specifies the starting mesospheric laser intensity \!\(\*FormBox[
RowBox[{
RowBox[{
StyleBox[\"[\",
FontSlant->\"Plain\"], 
RowBox[{
StyleBox[\"W\",
FontSlant->\"Plain\"], \"/\", SuperscriptBox[
StyleBox[\"m\",
FontSlant->\"Plain\"], \"2\"]}], \"]\"}], \".\"}],
TraditionalForm]\)\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"IMesoInitial is an option for BeamRepository and related \
functions that specifies the starting mesospheric laser intensity \
\\!\\(\\*FormBox[\\n RowBox[{\\n  RowBox[{\\n   \
StyleBox[\\\"[\\\",\\nFontSlant->\\\"Plain\\\"], \\n   RowBox[{\\n    \
StyleBox[\\\"W\\\",\\nFontSlant->\\\"Plain\\\"], \\\"/\\\", \
SuperscriptBox[\\n     StyleBox[\\\"m\\\",\\nFontSlant->\\\"Plain\\\"], \\\"2\
\\\"]}], \\\"]\\\"}], \\\".\\\"}],\\n TraditionalForm]\\)\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MaxStepRatio", "::", "usage"}], "=", 
  "\"\<MaxStepRatio is an option for BeamRepository and related functions \
that controls the intensity step size.\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"MaxStepRatio is an option for BeamRepository and related \
functions that controls the intensity step size.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"IMesoFactor", "::", "usage"}], "=", 
  "\"\<IMesoFactor is an option for BeamRepository and related functions that \
specifies the factor by which to multiply peak intensity to compute maximum \
repository intensity.\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"IMesoFactor is an option for BeamRepository and related \
functions that specifies the factor by which to multiply peak intensity to \
compute maximum repository intensity.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"NSamples", "::", "usage"}], "=", 
  "\"\<NSamples is an option for BeamRepository and related functions that \
specifies the number of samples in the repository.\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"NSamples is an option for BeamRepository and related \
functions that specifies the number of samples in the repository.\"\>"], \
"Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LaserSpotMesoFWHM", "::", "usage"}], "=", 
  "\"\<LaserSpotMesoFWHM is an option for sce and interBeamMeso that equals \
the FWHM laser spot size in the mesosphere. This option is especially useful \
when computing sce for pulsed lasers, where the peak irradiance cannot be \
computed from the average power.\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"LaserSpotMesoFWHM is an option for sce and interBeamMeso \
that equals the FWHM laser spot size in the mesosphere. This option is \
especially useful when computing sce for pulsed lasers, where the peak \
irradiance cannot be computed from the average power.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"$DefaultLGSParameters", "::", "usage"}], "=", 
  "\"\<$DefaultLGSParameters is a global variable that is the default for the \
DefaultParameters option for PsiMeso and related functions.\>\"", 
  " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"$DefaultLGSParameters is a global variable that is the \
default for the DefaultParameters option for PsiMeso and related functions.\"\
\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"NaLGSDefaults", "::", "usage"}], "=", 
  "\"\<NaLGSDefaults is a constant giving a list of default parameters for a \
Na D2 laser guide star.\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"NaLGSDefaults is a constant giving a list of default \
parameters for a Na D2 laser guide star.\"\>"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["PsiMeso", "Subsubsection"],

Cell[BoxData[
 RowBox[{
  RowBox[{"PsiMeso", "::", "usage"}], "=", "\"\<\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"DefaultParameters", "::", "usage"}], "=", "\"\<\>\"", 
  " "}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["BeamRepository", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"BeamRepositorySamples", "::", "usage"}], "=", 
  "\"\<BeamRepositorySamples computes the beam repository samples.\>\"", 
  " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"BeamRepositorySamples computes the beam repository samples.\
\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"BeamRepository", "::", "usage"}], "=", 
  "\"\<BeamRepository computes a repository of return flux values. \>\"", 
  " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"BeamRepository computes repository of return flux values. \
\"\>"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["InterBeamMeso", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"InterBeamMeso", "::", "usage"}], "=", 
  "\"\<InterBeamMeso integrates beam return, and computes flux and beam \
diameter based on the 2nd moment.\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"InterBeamMeso integrates beam return, and computes flux and \
beam diameter based on the 2nd moment.\"\>"], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Sce", "Subsubsection"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Sce", "::", "usage"}], "=", 
  "\"\<Compute beam figure of merit \
\!\(TraditionalForm\`\*SubscriptBox[\(s\), \(ce\)]\). Note that the following \
options may be changed from their values when BeamRepository[] was called, \
unless the peak intensity exceeds that of the interpolating function:\n\t\
PLaunch: Launched power [W]\n\tZenith: Zenith angle [rad]   (note that \
MagneticZenith and MagneticAzimuth cannot be changed!)\n\tHNa: Altitude of \
sodium centroid above sea level [m]\n\tHtele: Altitude of launch telescope \
above sea level [m]\n\tDLT: Launch telescope aperture [m]\n\t\[Mu]LT: Ratio \
of DLT to \!\(TraditionalForm\`1/\*SuperscriptBox[\(e\), \(2\)]\) lauch beam \
intensity diameter (not radius!), see Yura, Eq.(2.7)\n\tMsqrLT:  Effective \!\
\(TraditionalForm\`\*SuperscriptBox[\(M\), \(2\)]\) beam quality factor out \
of the launch telescope, also taking into account the effect of speckles\n\t\
LaserSpotMesoFWHM: Mesospheric FWHM spot size [m]. If this option is set to a \
nonzero value, DLT and \[Mu]LT are not used to compute the mesospheric \
irradiance.\n\nOutput values:\n\tsceOut: Merit parameter \
\!\(TraditionalForm\`\*SubscriptBox[\(s\), \(ce\)]\)\!\(\*
StyleBox[\" \",
FontSlant->\"Italic\"]\)[ph/s/W/(atom/m^2)]\n\tGroundInt: Integrated flux on \
the ground [ph/s/atom]\n\tFluxMesoInt: Integrated flux in the mesosphere \
[ph*m^2/s/sr/atom]\n\tFWHMSpeckle: Instantaneous laser spot size in the \
mesosphere (central speckle) [m]\n\tFWHM2mom: Return light spot size based on \
second moment [m]\n\tIMesoMax: Peak mesospheric laser light intensity [W/m^2]\
\>\"", " "}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["AbsCrossSection", "Subsubsection"],

Cell[BoxData[
 RowBox[{
  RowBox[{"AbsCrossSection", "::", "usage"}], "=", 
  "\"\<Absorption cross section and cycle time\n\nInput values: As for \
PsiMeso[]\n\nOutput values:\nsig: absorption cross section [m^2]\nps: Result \
of PhiSec (computed for 200 velocity classes)  [ph/s/sr/(W/m^2)]\nW: number \
of spontaneous excitations per time [ph/s/atom], inverse of cycle time \
(neglecting stimulated emission)\n\nExample:\nAbsCrossSection[{IMeso->50.0,\
\[CapitalDelta]fLaser->0.0*\!\(\*SuperscriptBox[\(10\), \
\(9\)]\),RepumpFraction->0.12,BG->0.0,LightPolarization->1}]yields \
{1.1424135229284697`*^-15,402.5300545224092`,169414.22073090656`}, hence a \
cross-section of 1.14 times the low-irradiance cross section (1.0 x 10^(-11) \
cm^2), psi=402 ph/s/sr/(W/m^2), and the cycle time is \
1/169414.22073090656`*10^6 us = 5.90 us, hence very close to the Larmor time \
in Paranal (5.8us).\>\"", " "}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["StepPulseFlux", "Subsubsection"],

Cell[BoxData[
 RowBox[{
  RowBox[{"StepPulseFlux", "::", "usage"}], "=", 
  "\"\<Solve the time-dependent problem for periodic irradiation pattern and \
integrate flux return over the last period. Uses phiv[] to\nsolve the \
inhomogeneous problem (d/dt)\[Rho] = A\[Rho]+\!\(\*
StyleBox[\"b\",
FontSlant->\"Italic\"]\) for piecewise constant coefficients.\nInput:\n\t\
syst: Bloch equation system\n\tparams: Physical simulation parameters for \
ReturnFlux[]\n\tnperiods: maximum number of periods to simulate\n\tcomplexE: \
Vector, subdividing the pulse into n subspans, scaling the \n\t\tirradiance \
in each subspan so that I[[j]] == IMeso |complexE[[j,2]]|^2, \n\t\tand \
Arg[complexE[[j,2]]] is the optical phase at step j. The entries \
complexE[[j,1]] contain the corresponding \ttime steps relative to the \
beginning of the simulated period. Hence, complexE[[-1,1]] equals the \
duration of the simulated period. The time steps do not have to be constant. \
Note that all time steps, including complexE[[1,1]], must be larger than 0 \
and strictly increasing.\n\tchirpRate: Chirp rate (positive values yield blue \
shifting of laser line over time) [Hz/s]\n\tfname: File name to save the \
results structure \\\"res\\\" in, using Export[fname,res]\n\topts: option set \
for PsiMeso[]\nOutput:\nGlobal`Res={psiAvg, psiAvgTotal, psiSpanAll, \
psiAvgHist, Iavg, popgr8All, popex16All, {PhiVErrors -> errAll, PsiIavg -> \
psiIavg1, GroundPopulation -> popgr, ExcitedPopulation -> popex, \
FinalGroundPops -> groundPops, FinalExcitedPops -> excitedPops, ExcitedPops \
-> excitedPopsAll, LastDM -> {w, vg}, RFParamsOut -> {rfparams1, params1, \
opts}}}\n\n\tpsiAvg: Specific return flux averaged over the last period \
[ph/s/sr/atom/(W/m^2)]\n\tpsiAvgTotal: psiAvg summed over all periods \
[ph/s/sr/atom/(W/m^2)]\n\tpsiSpanAll: vector of pairs {tn, return flux(tn)}, \
return flux [ph/s/sr/atom/(W/m^2)]\n\tpsiAvgHist: vector of psiAvg values for \
each period\n\tIavg: Irradiance, averaged over a simulation period [W/m^2]\n\t\
popgr8All: Array of dimension Nx8, where N equals the length of psiSpanAll \
and corresponds to the time steps given by psiSpanAll[[All,1]]. The values of \
popgr8All yield the 8 ground state populations of the velocity class that is \
closest to +5 MHz, where in many systems a peak of the polarization is \
observed (\\\"busy velocity class\\\"), normalized by the total \
ground+excited state population in this velocity class. For darkness in \
equilibrium we thus have popgr8All[[All,All]]==1/8. The states are ordered \
{(F=2,m=2), (F=2,m=1)..., (F=2,m=-2), (F=1,m=1), (F=1,m=0), (F=2,m=-1)} (with \
LightPolarization->+1, the (F=2,m=2) is pumped most strongly)\n\tpopex16All: \
same as popgr8All, but for the 16 excited states. Sequence starts with \
(F=3,m=3).\n\tPhiVErrors: vector of numerical errors returned by phiv[], \
corresponding to psiSpanAll\n\tpsiIavg: value of psi for average irradiance\n\
\tpopgr: ground state population\n\tpopex: Excited state population\n\tw: \
final solution vector (b)\nThe routine computes the average return flux \
psiAvg defined as\n\n   psiAvg = 1/(trep * Iavg)  int_{t0}^{t0+trep} Psi(t) \
dt,\n\nwhere Psi(t) is the instantaneous non-normalized return flux in \
ph/s/sr/atom, t0 is the begin of the last period, and Iavg = Ipulse*duty, \
where Ipulse is the pulse irradiance.\n\nThe iteration starts from thermal \
equilibrium (InitialState -> LightOff) or from the steady-state solution at \
Ipulse (InitialState -> LightOn) and stops once the difference of psiAvg \
between two subsequent iterations drops below some threshold. In the verbose \
simulation output, \\\"peak cw psi\\\" is the steady-state result \
psi(I_peak), and \\\"flux from avg irradiance\\\" is similarly psi(I_avg).\n\
To achieve convergence, allow for a total simulation time of several times \
16ns. At low irradiance, you may have to simulate several collision time \
scales.\>\"", " "}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"KrylovDimension", "::", "usage"}], "=", 
  "\"\<KrylovDimension is an option for StepPulseFlux that specifies the \
dimension of the Krylov subspace to be used in the matrix exponential \
calculation.\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"KrylovDimension is an option for StepPulseFlux that \
specifies the dimension of the Krylov subspace to be used in the matrix \
exponential calculation.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Tolerance", "::", "usage"}], "=", 
  "\"\<Tolerance is an option for StepPulseFlux that specifies the tolerance \
to use in phiv[].\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"Tolerance is an option for StepPulseFlux that specifies the \
tolerance to use in phiv[].\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"TerminationThresh", "::", "usage"}], "=", 
  "\"\<TerminationThresh is an option for StepPulseFlux that specifies that \
the iteration shall be terminated once the absolute difference between two \
subsequent values of the averaged psi falls below TerminationThresh\>\""}]], \
"Input",
 InitializationCell->True],

Cell[BoxData["\<\"TerminationThresh is an option for StepPulseFlux that \
specifies that the iteration shall be terminated once the absolute difference \
between two subsequent values of the averaged psi falls below \
TerminationThresh\"\>"], "Output"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"InitialState", "::", "usage"}], "=", 
  "\"\<InitialState is an option for StepPulseFlux that specifies the state \
of the system at the beginning of the time-dependent calculation.\n\n\
InitialState->LightOn specifies the steady state with cw light.\n\
InitialState->LightOff specifies the steady state with no light.\n\
InitialState->{\!\(\*
StyleBox[\"sol\", \"TI\"]\),\!\(\*
StyleBox[\"vg\", \"TI\"]\)} specifies a state given by density-matrix \
solution vector \!\(\*
StyleBox[\"sol\", \"TI\"]\) corresponding to velocity group binning \!\(\*
StyleBox[\"vg\", \"TI\"]\).\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\"\<InitialState is an option for StepPulseFlux that specifies \
the state of the system at the beginning of the time-dependent \
calculation.\\n\\nInitialState->LightOn specifies the steady state with cw \
light.\\nInitialState->LightOff specifies the steady state with no \
light.\\nInitialState->{\\!\\(\\*\\nStyleBox[\\\"sol\\\", \
\\\"TI\\\"]\\),\\!\\(\\*\\nStyleBox[\\\"vg\\\", \\\"TI\\\"]\\)} specifies a \
state given by density-matrix solution vector \
\\!\\(\\*\\nStyleBox[\\\"sol\\\", \\\"TI\\\"]\\) corresponding to velocity \
group binning \\!\\(\\*\\nStyleBox[\\\"vg\\\", \\\"TI\\\"]\\).\>\""], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LightOn", "::", "usage"}], "=", 
  "\"\<LightOn is a possible value for the InitialState option of \
StepPulseFlux that specifies that the initial state is the steady state with \
cw light.\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"LightOn is a possible value for the InitialState option of \
StepPulseFlux that specifies that the initial state is the steady state with \
cw light.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LightOff", "::", "usage"}], "=", 
  "\"\<LightOff is a possible value for the InitialState option of \
StepPulseFlux that specifies that the initial state is the steady state with \
no light.\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"LightOff is a possible value for the InitialState option of \
StepPulseFlux that specifies that the initial state is the steady state with \
no light.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"VGMethod", "::", "usage"}], "=", 
  "\"\<VGMethod is an option for StepPulseFlux that specifies the method used \
to account for the velocity group binning over multiple steps.\n\n\
VGMethod->FixVG specifies that the velocity group binning found for the first \
calculation (using the nominal laser parameters) should be reused for all \
subsequent steps.\nVGMethod->ConvertDM specifies that a new binning should be \
found for each step, and the density matrix solution vector found from the \
previous step should be rebinned to match the new velocity group \
binning.\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"VGMethod is an option for StepPulseFlux that specifies the \
method used to account for the velocity group binning over multiple steps.\\n\
\\nVGMethod->FixVG specifies that the velocity group binning found for the \
first calculation (using the nominal laser parameters) should be reused for \
all subsequent steps.\\nVGMethod->ConvertDM specifies that a new binning \
should be found for each step, and the density matrix solution vector found \
from the previous step should be rebinned to match the new velocity group \
binning.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FixVG", "::", "usage"}], "=", 
  "\"\<FixVG is a possible value for the VGMethod option for StepPulseFlux \
that specifies that the velocity group binning found for the first \
calculation (using the nominal laser parameters) should be reused for all \
subsequent steps.\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"FixVG is a possible value for the VGMethod option for \
StepPulseFlux that specifies that the velocity group binning found for the \
first calculation (using the nominal laser parameters) should be reused for \
all subsequent steps.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ConvertDM", "::", "usage"}], "=", 
  "\"\<ConvertDM is a possible value for the VGMethod option for \
StepPulseFlux that specifies that a new binning should be found for each \
step, and the density matrix solution vector found from the previous step \
should be rebinned to match the new velocity group binning.\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"ConvertDM is a possible value for the VGMethod option for \
StepPulseFlux that specifies that a new binning should be found for each \
step, and the density matrix solution vector found from the previous step \
should be rebinned to match the new velocity group binning.\"\>"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"VGfreezePeriod", "::", "usage"}], "=", "\[IndentingNewLine]", 
  "\"\<VGfreezePeriod is the period after starting at which the rebinning \
stops and the velocity bins remain constant. In particular when simulating \
pulse shapes with numerous samples, this option can significantly improve the \
computational speed.\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"VGfreezePeriod is the period after starting at which the \
rebinning stops and the velocity bins remain constant. In particular when \
simulating pulse shapes with numerous samples, this option can significantly \
improve the computational speed.\"\>"], "Output"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"PhiVErrors", "::", "usage"}], "=", "\"\<\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"PsiIavg", "::", "usage"}], "=", "\"\<\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"GroundPopulation", "::", "usage"}], "=", "\"\<\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"ExcitedPopulation", "::", "usage"}], "=", "\"\<\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"FinalGroundPops", "::", "usage"}], "=", "\"\<\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"FinalExcitedPops", "::", "usage"}], "=", "\"\<\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"ExcitedPops", "::", "usage"}], "=", "\"\<\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"LastDM", "::", "usage"}], "=", "\"\<\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"RFParamsOut", "::", "usage"}], "=", "\"\<\>\""}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Internal functions", "Subsubsection"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"setStates", "::", "usage"}], "=", "\"\<\>\""}], "\n", 
 RowBox[{
  RowBox[{"makeAtomicSystem", "::", "usage"}], "=", "\"\<\>\""}], "\n", 
 RowBox[{
  RowBox[{"makeExperimentalSystem", "::", "usage"}], "=", "\"\<\>\""}], "\n", 
 RowBox[{
  RowBox[{"makeAtomicData", "::", "usage"}], "=", "\"\<\>\""}], "\n", 
 RowBox[{
  RowBox[{"makeHamiltonian", "::", "usage"}], "=", "\"\<\>\""}], "\n", 
 RowBox[{
  RowBox[{"makeLightHamiltonian", "::", "usage"}], "=", 
  "\"\<\>\""}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"doRWA", "::", "usage"}], "=", "\"\<\>\""}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"makeRelaxationMatrix", "::", "usage"}], "=", 
  "\"\<\>\""}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"makeSDampingMatrix", "::", "usage"}], "=", 
  "\"\<\>\""}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"makeRepopulationMatrix", "::", "usage"}], "=", 
  "\"\<\>\""}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"deleteHyperfineCoherences", "::", "usage"}], "=", 
  "\"\<\>\""}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"makeVariableList", "::", "usage"}], "=", 
  "\"\<\>\""}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"makeBlochEquations", "::", "usage"}], "=", 
  "\"\<\>\""}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"makeBlochMatrix", "::", "usage"}], "=", 
  "\"\<\>\""}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"makeFluorescenceOperator", "::", "usage"}], "=", 
  "\"\<\>\""}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"findFluorescence", "::", "usage"}], "=", 
  "\"\<\>\""}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"makeFluorescenceVector", "::", "usage"}], "=", 
  "\"\<\>\""}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"makeBlochBlocks", "::", "usage"}], "=", 
  "\"\<\>\""}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"separateBlochBlocks", "::", "usage"}], "=", "\"\<\>\""}]}], "Input",
 InitializationCell->True]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Implementation", "Section"],

Cell[CellGroupData[{

Cell["Begin private context", "Subsection"],

Cell[BoxData[
 RowBox[{"Begin", "[", "\"\<`Private`\>\"", "]"}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Setup", "Subsection"],

Cell["Load the package, set options, and define utility functions", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"DensityMatrix", ",", 
    RowBox[{"TimeDependence", "->", "False"}], ",", 
    RowBox[{"ComplexExpandVariables", "->", "Subscript"}]}], "]"}], 
  ";"}]], "Input",
 InitializationCell->True],

Cell["\<\
Calculates the unperturbed resonance frequencies for an atomic system\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ResonanceFrequencies", "[", "AtomicSystem_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "ha", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"ha", "=", 
      RowBox[{"Hamiltonian", "[", 
       RowBox[{"AtomicSystem", ",", 
        RowBox[{"ElectricField", "->", 
         RowBox[{"{", 
          RowBox[{"E0", ",", "0", ",", "0"}], "}"}]}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Simplify", "[", 
      RowBox[{
       RowBox[{"Union", "@", 
        RowBox[{"Flatten", "@", 
         RowBox[{"MapIndexed", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"(", 
                RowBox[{"Less", "@@", "#2"}], ")"}], "&&", 
               RowBox[{"(", 
                RowBox[{"#1", "=!=", "0"}], ")"}]}], ",", 
              RowBox[{
               RowBox[{"ha", "[", 
                RowBox[{"[", 
                 RowBox[{
                  RowBox[{"#2", "[", 
                   RowBox[{"[", "2", "]"}], "]"}], ",", 
                  RowBox[{"#2", "[", 
                   RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "]"}], "-", 
               RowBox[{"ha", "[", 
                RowBox[{"[", 
                 RowBox[{
                  RowBox[{"#2", "[", 
                   RowBox[{"[", "1", "]"}], "]"}], ",", 
                  RowBox[{"#2", "[", 
                   RowBox[{"[", "1", "]"}], "]"}]}], "]"}], "]"}]}], ",", 
              RowBox[{"{", "}"}]}], "]"}], "&"}], ",", "ha", ",", 
           RowBox[{"{", "2", "}"}]}], "]"}]}]}], "-", 
       RowBox[{"Energy", "[", "2", "]"}]}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["Applies replacement rules to a sparse matrix", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SparseReplace", "[", 
   RowBox[{"expr_", ",", 
    RowBox[{"reps", ":", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"_Rule", "|", "_RuleDelayed", "|", "_Dispatch", "|", 
        RowBox[{"{", 
         RowBox[{"___Rule", "|", "___RuleDelayed"}], "}"}]}], ")"}], 
      ".."}]}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "AR", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Fold", "[", 
      RowBox[{"ReplaceAll", ",", 
       RowBox[{"expr", "/.", 
        RowBox[{"x_SparseArray", ":>", 
         RowBox[{"AR", "[", 
          RowBox[{
           RowBox[{"ArrayRules", "[", "x", "]"}], ",", 
           RowBox[{"Dimensions", "[", "x", "]"}]}], "]"}]}]}], ",", 
       RowBox[{"{", "reps", "}"}]}], "]"}], "/.", 
     RowBox[{"AR", "->", "SparseArray"}]}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell["Applies a set of rules to itself", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SelfApply", "[", "rules_", "]"}], ":=", 
  RowBox[{"rules", "//.", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x_", "->", "y_"}], ")"}], ":>", 
    RowBox[{"(", 
     RowBox[{"x", "->", 
      RowBox[{"(", 
       RowBox[{"y", "/.", "rules"}], ")"}]}], ")"}]}]}]}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Set up C compiler", "Subsection"],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{"$VersionNumber", ">=", "8."}], ",", 
    RowBox[{"Needs", "[", "\"\<CCompilerDriver`\>\"", "]"}]}], "]"}], 
  ";"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"ccompiler", "=", 
   RowBox[{
    RowBox[{"$VersionNumber", ">=", "8."}], "&&", 
    RowBox[{
     RowBox[{"CCompilers", "[", "]"}], "!=", 
     RowBox[{"{", "}"}]}]}]}], ";"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"compileropts", "=", 
  RowBox[{"If", "[", 
   RowBox[{"ccompiler", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"RuntimeOptions", "->", "\"\<Speed\>\""}], ",", 
      RowBox[{"Parallelization", "->", "True"}], ",", " ", 
      RowBox[{"CompilationTarget", "->", "\"\<C\>\""}]}], " ", "}"}], ",", 
    RowBox[{"{", "}"}]}], "]"}]}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["VelocityGroups1", "Subsection"],

Cell["\<\
Here we define the function VelocityGroups, which finds an appropriate set of \
atomic velocity bins for performing the calculation. It creates a set of \
narrow bins near the resonance frequencies, and wider bins to cover the rest \
of the Doppler distribution.\
\>", "Text"],

Cell[TextData[{
 "It is called with\n\tVelocityGroups[DopplerWidth, ResonanceFrequencies, \
{SmallBinSize, n1}, {LargeBinSize, n2}]\nwhere\n\tDopplerWidth is the Doppler \
width\n\tResonanceFrequencies are the optical resonance frequencies relative \
to the light frequency (i.e. transition frequencies in the rotating frame)\n\t\
SmallBinSize is the velocity group bin size for bins near the resonance \
frequencies\n\tn1 specifies that (",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"2", " ", "n1"}], "+", "1"}], ")"}], TraditionalForm]]],
 " bins are created near each resonance frequency\n\tLargeBinSize is the \
maximum size for a bin not near a resonance frequency\n\tn2 is the multiple \
of the Doppler width that determines the range over which velocity groups \
should be created\nIf ShowPlot",
 "->",
 "True is specified, a plot is produced that shows the limits of the velocity \
group bins (blue lines), and the weighted centers and number of atoms in each \
bin (black lines)."
}], "Text"],

Cell["\<\
We can probably do better by using an adaptive procedure that adjusts the bin \
sizes in response to the calculated signal due to each velocity group.\
\>", "Text"],

Cell[BoxData[
 RowBox[{"Off", "[", 
  RowBox[{"Solve", "::", "\"\<ifun\>\""}], "]"}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"IntG", "[", 
   RowBox[{"\[CapitalGamma]D_", ",", "a_", ",", "b_"}], "]"}], "=", 
  RowBox[{"Integrate", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Exp", "[", 
      RowBox[{"-", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"\[CapitalDelta]", "/", "\[CapitalGamma]D"}], ")"}], "2"]}], 
      "]"}], "/", 
     RowBox[{"(", 
      RowBox[{
       SqrtBox["\[Pi]"], " ", "\[CapitalGamma]D"}], ")"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"\[CapitalDelta]", ",", "a", ",", "b"}], "}"}], ",", 
    RowBox[{"Assumptions", "->", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\[CapitalGamma]D", ">", "0"}], ",", 
       RowBox[{"b", ">", "a", ">", "0"}]}], "}"}]}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"Erf", "[", 
      FractionBox["a", "\[CapitalGamma]D"], "]"}]}], "+", 
    RowBox[{"Erf", "[", 
     FractionBox["b", "\[CapitalGamma]D"], "]"}]}], ")"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"CDFGaussian", "[", 
   RowBox[{"\[CapitalGamma]D_", ",", "b_"}], "]"}], "=", 
  RowBox[{"Integrate", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Exp", "[", 
      RowBox[{"-", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"\[CapitalDelta]", "/", "\[CapitalGamma]D"}], ")"}], "2"]}], 
      "]"}], "/", 
     RowBox[{"(", 
      RowBox[{
       SqrtBox["\[Pi]"], " ", "\[CapitalGamma]D"}], ")"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"\[CapitalDelta]", ",", 
      RowBox[{"-", "Infinity"}], ",", "b"}], "}"}], ",", 
    RowBox[{"Assumptions", "->", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\[CapitalGamma]D", ">", "0"}], ",", 
       RowBox[{"b", "\[Element]", "Reals"}]}], "}"}]}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  RowBox[{"(", 
   RowBox[{"1", "+", 
    RowBox[{"Erf", "[", 
     FractionBox["b", "\[CapitalGamma]D"], "]"}]}], ")"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Gaussian", "[", 
   RowBox[{"\[CapitalGamma]_", ",", "\[CapitalDelta]_"}], "]"}], "=", 
  RowBox[{
   RowBox[{"Exp", "[", 
    RowBox[{"-", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"\[CapitalDelta]", "/", "\[CapitalGamma]"}], ")"}], "2"]}], 
    "]"}], "/", 
   RowBox[{"(", 
    RowBox[{
     SqrtBox["\[Pi]"], " ", "\[CapitalGamma]"}], ")"}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 FractionBox[
  SuperscriptBox["\[ExponentialE]", 
   RowBox[{"-", 
    FractionBox[
     SuperscriptBox["\[CapitalDelta]", "2"], 
     SuperscriptBox["\[CapitalGamma]", "2"]]}]], 
  RowBox[{
   SqrtBox["\[Pi]"], " ", "\[CapitalGamma]"}]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"InverseIntG", "[", 
   RowBox[{"\[CapitalGamma]D_", ",", "a_", ",", "n_"}], "]"}], "=", 
  RowBox[{"b", "/.", 
   RowBox[{
    RowBox[{"Solve", "[", 
     RowBox[{
      RowBox[{"n", "==", 
       RowBox[{"IntG", "[", 
        RowBox[{"\[CapitalGamma]D", ",", "a", ",", "b"}], "]"}]}], ",", "b"}],
      "]"}], "[", 
    RowBox[{"[", "1", "]"}], "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"\[CapitalGamma]D", " ", 
  RowBox[{"InverseErf", "[", 
   RowBox[{
    RowBox[{"2", " ", "n"}], "+", 
    RowBox[{"Erf", "[", 
     FractionBox["a", "\[CapitalGamma]D"], "]"}]}], "]"}]}]], "Output"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"BinCenter", "[", 
   RowBox[{"\[CapitalGamma]D_", ",", 
    RowBox[{"{", 
     RowBox[{"a_", ",", "b_"}], "}"}]}], "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InverseIntG", "[", 
       RowBox[{"\[CapitalGamma]D", ",", "a", ",", 
        RowBox[{"#", "/", "2"}]}], "]"}], ",", "#"}], "}"}], "&"}], "@", 
   RowBox[{"IntG", "[", 
    RowBox[{"\[CapitalGamma]D", ",", "a", ",", "b"}], "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "VelocityGroups1", "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"ShowPlot", "->", "True"}], "}"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"FitRange", "[", 
   RowBox[{"rmin_", ",", "rmax_", ",", "bin_"}], "]"}], ":=", 
  RowBox[{"Range", "[", 
   RowBox[{"rmin", ",", "rmax", ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"rmax", "-", "rmin"}], ")"}], "/", 
     RowBox[{"Max", "[", 
      RowBox[{
       RowBox[{"Round", "[", 
        RowBox[{"Abs", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"rmax", "-", "rmin"}], ")"}], "/", "bin"}], "]"}], "]"}], 
       ",", "1"}], "]"}]}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"VelocityGroups1", "[", 
   RowBox[{"GD_", ",", "res_", ",", 
    RowBox[{"{", 
     RowBox[{"binsize1_", ",", "n1_"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"binsize2_", ",", "n2_"}], "}"}], ",", 
    RowBox[{"OptionsPattern", "[", "]"}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "res1", ",", "bins0", ",", "bins1", ",", "bins2", ",", "bins", ",", 
      "groups"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"res1", "=", 
      RowBox[{"Sort", "@", 
       RowBox[{"Union", "@", 
        RowBox[{"Flatten", "@", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "n2"}], " ", "GD"}], ",", "res", ",", 
           RowBox[{"n2", " ", "GD"}]}], "}"}]}]}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"res1", "=", 
      RowBox[{"Mean", "/@", 
       RowBox[{"Split", "[", 
        RowBox[{"res1", ",", 
         RowBox[{
          RowBox[{
           RowBox[{"#2", "-", "#1"}], "<", 
           RowBox[{"n1", " ", "binsize1"}]}], "&"}]}], "]"}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"bins0", "=", 
      RowBox[{"Select", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Sort", "@", 
         RowBox[{"Flatten", "@", 
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{"{", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"Range", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"res1", "[", 
                  RowBox[{"[", "i", "]"}], "]"}], "-", 
                 RowBox[{".5", " ", "binsize1"}]}], ",", 
                RowBox[{"Max", "[", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"res1", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], "-", 
                   RowBox[{
                    RowBox[{"(", 
                    RowBox[{"n1", "+", ".5"}], ")"}], "binsize1"}]}], ",", 
                  RowBox[{"Mean", "[", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"res1", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"res1", "[", 
                    RowBox[{"[", 
                    RowBox[{"i", "-", "1"}], "]"}], "]"}]}], "}"}], "]"}], 
                  ",", 
                  RowBox[{
                   RowBox[{"-", "n2"}], " ", "GD"}]}], "]"}], ",", 
                RowBox[{"-", "binsize1"}]}], "]"}], ",", 
              RowBox[{"Range", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"res1", "[", 
                  RowBox[{"[", "i", "]"}], "]"}], "+", 
                 RowBox[{".5", " ", "binsize1"}]}], ",", 
                RowBox[{"Min", "[", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"res1", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], "+", 
                   RowBox[{
                    RowBox[{"(", 
                    RowBox[{"n1", "+", ".5"}], ")"}], "binsize1"}]}], ",", 
                  RowBox[{"Mean", "[", 
                   RowBox[{"{", 
                    RowBox[{
                    RowBox[{"res1", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"res1", "[", 
                    RowBox[{"[", 
                    RowBox[{"i", "+", "1"}], "]"}], "]"}]}], "}"}], "]"}], 
                  ",", 
                  RowBox[{"n2", " ", "GD"}]}], "]"}], ",", "binsize1"}], 
               "]"}]}], "\[IndentingNewLine]", "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"i", ",", "2", ",", 
              RowBox[{
               RowBox[{"Length", "[", "res1", "]"}], "-", "1"}]}], "}"}]}], 
           "]"}]}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"-", "n2"}], " ", "GD"}], "<=", "#", "<=", 
          RowBox[{"n2", " ", "GD"}]}], "&"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"bins1", "=", 
      RowBox[{"Union", "@", 
       RowBox[{"Flatten", "@", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "n2"}], " ", "GD"}], ",", "bins0", ",", 
          RowBox[{"n2", " ", "GD"}]}], "}"}]}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"bins1", "=", 
      RowBox[{"Mean", "/@", 
       RowBox[{"Split", "[", 
        RowBox[{"bins1", ",", 
         RowBox[{
          RowBox[{
           RowBox[{"#2", "-", "#1"}], "<", "binsize1"}], "&"}]}], "]"}]}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"bins2", "=", 
      RowBox[{"Union", "@", 
       RowBox[{"Flatten", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{"Most", "@", 
             RowBox[{"FitRange", "[", 
              RowBox[{
               RowBox[{"bins1", "[", 
                RowBox[{"[", "i", "]"}], "]"}], ",", 
               RowBox[{"bins1", "[", 
                RowBox[{"[", 
                 RowBox[{"i", "+", "1"}], "]"}], "]"}], ",", "binsize2"}], 
              "]"}]}], ",", 
            RowBox[{"{", 
             RowBox[{"i", ",", 
              RowBox[{
               RowBox[{"Length", "[", "bins1", "]"}], "-", "1"}]}], "}"}]}], 
           "]"}], ",", 
          RowBox[{"Last", "@", "bins1"}]}], "}"}], "]"}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"bins", "=", 
      RowBox[{"Transpose", "@", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Prepend", "[", 
          RowBox[{"bins2", ",", 
           RowBox[{"-", "Infinity"}]}], "]"}], ",", 
         RowBox[{"Append", "[", 
          RowBox[{"bins2", ",", "Infinity"}], "]"}]}], "}"}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"groups", "=", 
      RowBox[{
       RowBox[{
        RowBox[{"BinCenter", "[", 
         RowBox[{"GD", ",", "#"}], "]"}], "&"}], "/@", "bins"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"OptionValue", "[", "ShowPlot", "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Print", "@", 
        RowBox[{"Show", "[", 
         RowBox[{
          RowBox[{"Plot", "[", 
           RowBox[{
            RowBox[{"GD", " ", 
             RowBox[{"Gaussian", "[", 
              RowBox[{"GD", ",", "\[CapitalDelta]"}], "]"}]}], ",", 
            RowBox[{"{", 
             RowBox[{"\[CapitalDelta]", ",", 
              RowBox[{
               RowBox[{"-", "3"}], "GD"}], ",", 
              RowBox[{"3", "GD"}]}], "}"}], ",", 
            RowBox[{"PlotRange", "->", "All"}]}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"ListPlot", "[", 
           RowBox[{"groups", ",", 
            RowBox[{"Filling", "->", "0"}], ",", 
            RowBox[{"FillingStyle", "->", 
             RowBox[{"Directive", "[", "Black", "]"}]}], ",", 
            RowBox[{"Axes", "->", "False"}], ",", 
            RowBox[{"PlotRange", "->", "All"}]}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"ListPlot", "[", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"{", 
               RowBox[{"#", ",", 
                RowBox[{"GD", " ", 
                 RowBox[{"Gaussian", "[", 
                  RowBox[{"GD", ",", "#"}], "]"}]}]}], "}"}], "&"}], "/@", 
             RowBox[{"Flatten", "[", "bins", "]"}]}], ",", 
            RowBox[{"Filling", "->", "0"}], ",", 
            RowBox[{"PlotStyle", "->", "None"}]}], "]"}]}], 
         "\[IndentingNewLine]", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"MapThread", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Append", "[", 
         RowBox[{"#1", ",", "#2"}], "]"}], "&"}], ",", 
       RowBox[{"{", 
        RowBox[{"groups", ",", "bins"}], "}"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"(*", 
  RowBox[{"VelocityGroups", "[", 
   RowBox[{
    RowBox[{"250.", " ", 
     SuperscriptBox["10", "6"]}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "9"}], " ", 
       SuperscriptBox["10", "7"]}], ",", 
      RowBox[{".5", " ", 
       SuperscriptBox["10", "7"]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1.7", " ", 
       SuperscriptBox["10", "7"]}], ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"200", " ", 
       SuperscriptBox["10", "6"]}], ",", "2"}], "}"}]}], "]"}], 
  "*)"}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["VelocityGroups2", "Subsection"],

Cell["\<\
A second method for choosing velocity groups, using an estimate for the \
spectrum to make a weighting function.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Lorentzian", "[", 
   RowBox[{"a_", ",", "x_"}], "]"}], ":=", 
  RowBox[{
   FractionBox["a", 
    RowBox[{"2", "\[Pi]"}]], 
   FractionBox["1", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"a", "/", "2"}], ")"}], "2"], "+", 
     RowBox[{"x", "^", "2"}]}]]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"Gaussian", "[", 
   RowBox[{"a_", ",", "x_"}], "]"}], ":=", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"a", " ", 
     SqrtBox["\[Pi]"]}]], 
   RowBox[{"Exp", "[", 
    RowBox[{"-", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"x", "/", "a"}], ")"}], "2"]}], "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"NNIntegrate", "[", 
   RowBox[{"f_", ",", 
    RowBox[{"{", 
     RowBox[{"x_", ",", "x0_", ",", "x1_"}], "}"}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "int", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"int", "=", 
      RowBox[{
       RowBox[{"y", "[", "x", "]"}], "/.", 
       RowBox[{
        RowBox[{"NDSolve", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"y", "'"}], "[", "x", "]"}], "==", "f"}], ",", 
            RowBox[{
             RowBox[{"y", "[", "x0", "]"}], "==", "0"}]}], "}"}], ",", 
          RowBox[{"y", "[", "x", "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"x", ",", "x0", ",", "x1"}], "}"}], ",", 
          RowBox[{"MaxStepFraction", "->", 
           RowBox[{"1", "/", "50000"}]}], ",", 
          RowBox[{"MaxSteps", "->", 
           SuperscriptBox["10", "6"]}]}], "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Piecewise", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"int", "/", 
           RowBox[{"(", 
            RowBox[{"int", "/.", 
             RowBox[{"x", "->", "x1"}]}], ")"}]}], ",", 
          RowBox[{"x0", "<=", "x", "<=", "x1"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", 
          RowBox[{"x", ">", "x1"}]}], "}"}]}], "}"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"IntG", "[", 
   RowBox[{"\[CapitalGamma]D_", ",", "a_", ",", "b_"}], "]"}], "=", 
  RowBox[{"Integrate", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Exp", "[", 
      RowBox[{"-", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"\[CapitalDelta]", "/", "\[CapitalGamma]D"}], ")"}], "2"]}], 
      "]"}], "/", 
     RowBox[{"(", 
      RowBox[{
       SqrtBox["\[Pi]"], " ", "\[CapitalGamma]D"}], ")"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"\[CapitalDelta]", ",", "a", ",", "b"}], "}"}], ",", 
    RowBox[{"Assumptions", "->", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\[CapitalGamma]D", ">", "0"}], ",", 
       RowBox[{"b", ">", "a", ">", "0"}]}], "}"}]}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"Erf", "[", 
      FractionBox["a", "\[CapitalGamma]D"], "]"}]}], "+", 
    RowBox[{"Erf", "[", 
     FractionBox["b", "\[CapitalGamma]D"], "]"}]}], ")"}]}]], "Output"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"WeightFunction", "[", 
   RowBox[{"sig_", ",", 
    RowBox[{"{", 
     RowBox[{"a_", ",", "b_", ",", "c_"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x_", ",", "x0_", ",", "x1_"}], "}"}]}], "]"}], ":=", 
  RowBox[{
   RowBox[{"a", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x", " ", "-", "x0"}], ")"}], "/", 
     RowBox[{"(", 
      RowBox[{"x1", "-", "x0"}], ")"}]}]}], "+", 
   RowBox[{"b", " ", 
    RowBox[{"NNIntegrate", "[", 
     RowBox[{"sig", ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "x0", ",", "x1"}], "}"}]}], "]"}]}], "+", 
   RowBox[{"c", " ", 
    RowBox[{"NNIntegrate", "[", 
     RowBox[{
      RowBox[{"Abs", "[", 
       RowBox[{"D", "[", 
        RowBox[{"sig", ",", 
         RowBox[{"{", 
          RowBox[{"x", ",", "2"}], "}"}]}], "]"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "x0", ",", "x1"}], "}"}]}], "]"}]}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "VelocityGroups2", "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"ShowPlot", "->", "True"}], "}"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"VelocityGroups2", "[", 
   RowBox[{"wint_", ",", 
    RowBox[{"{", 
     RowBox[{"x_", ",", "first_", ",", "last_"}], "}"}], ",", "n_", ",", 
    "dw_", ",", "minstep_", ",", "freqs_", ",", 
    RowBox[{"OptionsPattern", "[", "]"}]}], "]"}], ":=", 
  RowBox[{"Module", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "bins", ",", "groups", ",", "points", ",", "edges", ",", "pops", ",", 
      "wint1"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"wint1", "[", "x1_", "]"}], "=", 
      RowBox[{"(", 
       RowBox[{"wint", "/.", 
        RowBox[{"x", "->", "x1"}]}], ")"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"points", "=", 
      RowBox[{"NestWhileList", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"Min", "[", 
          RowBox[{"last", ",", 
           RowBox[{"Max", "[", 
            RowBox[{
             RowBox[{"#", "+", "minstep"}], ",", 
             RowBox[{"(", 
              RowBox[{"x1", "/.", 
               RowBox[{"FindRoot", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"wint1", "[", "x1", "]"}], "-", 
                  RowBox[{"wint1", "[", "#", "]"}], "-", 
                  RowBox[{
                   RowBox[{"wint1", "[", "last", "]"}], "/", "n"}]}], ",", 
                 RowBox[{"{", 
                  RowBox[{"x1", ",", "#", ",", "last"}], "}"}]}], "]"}]}], 
              ")"}]}], "]"}]}], "]"}], "&"}], ",", "\[IndentingNewLine]", 
        "first", ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"#", "<", 
          RowBox[{"last", "-", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"last", "-", "first"}], ")"}], 
            SuperscriptBox["10", 
             RowBox[{"-", "6"}]]}]}]}], "&"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"edges", "=", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"-", "Infinity"}], "}"}], ",", 
        RowBox[{"Mean", "/@", 
         RowBox[{"Transpose", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Most", "@", "points"}], ",", 
            RowBox[{"Rest", "@", "points"}]}], "}"}], "]"}]}], ",", 
        RowBox[{"{", "Infinity", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"bins", "=", 
      RowBox[{"Transpose", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Most", "@", "edges"}], ",", 
         RowBox[{"Rest", "@", "edges"}]}], "}"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"pops", "=", " ", 
      RowBox[{
       RowBox[{
        RowBox[{"IntG", "[", 
         RowBox[{"dw", ",", 
          RowBox[{"#", "[", 
           RowBox[{"[", "1", "]"}], "]"}], ",", 
          RowBox[{"#", "[", 
           RowBox[{"[", "2", "]"}], "]"}]}], "]"}], "&"}], "/@", "bins"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"groups", "=", 
      RowBox[{"Transpose", "[", 
       RowBox[{"{", 
        RowBox[{"points", ",", "pops", ",", "bins"}], "}"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"OptionValue", "[", "ShowPlot", "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Print", "@", 
        RowBox[{"Show", "[", 
         RowBox[{
          RowBox[{"Plot", "[", 
           RowBox[{
            RowBox[{"dw", " ", 
             RowBox[{"Gaussian", "[", 
              RowBox[{"dw", ",", "\[CapitalDelta]"}], "]"}]}], ",", 
            RowBox[{"{", 
             RowBox[{"\[CapitalDelta]", ",", "first", ",", "last"}], "}"}], 
            ",", 
            RowBox[{"PlotRange", "->", "All"}], ",", 
            RowBox[{"Frame", "->", "True"}], ",", 
            RowBox[{"Axes", "->", "False"}]}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"ListPlot", "[", 
           RowBox[{
            RowBox[{"groups", "[", 
             RowBox[{"[", 
              RowBox[{"All", ",", 
               RowBox[{"1", ";;", "2"}]}], "]"}], "]"}], ",", 
            RowBox[{"Axes", "->", "False"}], ",", 
            RowBox[{"PlotRange", "->", "All"}]}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"ListLinePlot", "[", 
           RowBox[{
            RowBox[{"Join", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"groups", "[", 
                  RowBox[{"[", 
                   RowBox[{"1", ",", "3", ",", "1"}], "]"}], "]"}], ",", 
                 "0"}], "}"}], "}"}], ",", 
              RowBox[{"Flatten", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"#", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "1"}], "]"}], "]"}], ",", 
                    RowBox[{"#", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], "}"}], ",", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"#", "[", 
                    RowBox[{"[", 
                    RowBox[{"3", ",", "2"}], "]"}], "]"}], ",", 
                    RowBox[{"#", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}], "}"}]}], "}"}], "&"}], "/@",
                  "groups"}], ",", "1"}], "]"}], ",", 
              RowBox[{"{", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"groups", "[", 
                  RowBox[{"[", 
                   RowBox[{
                    RowBox[{"-", "1"}], ",", "3", ",", "2"}], "]"}], "]"}], 
                 ",", "0"}], "}"}], "}"}]}], "]"}], ",", 
            RowBox[{"Filling", "->", "0"}]}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"ListPlot", "[", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"{", 
               RowBox[{"#", ",", 
                RowBox[{"dw", " ", 
                 RowBox[{"Gaussian", "[", 
                  RowBox[{"dw", ",", "#"}], "]"}]}]}], "}"}], "&"}], "/@", 
             RowBox[{"Flatten", "[", 
              RowBox[{"groups", "[", 
               RowBox[{"[", 
                RowBox[{"All", ",", 
                 RowBox[{"-", "1"}]}], "]"}], "]"}], "]"}]}], ",", 
            RowBox[{"Filling", "->", "0"}], ",", 
            RowBox[{"PlotStyle", "->", "None"}]}], "]"}]}], 
         "\[IndentingNewLine]", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     "groups"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"Bend", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"p0", ":", 
      RowBox[{"{", 
       RowBox[{"x0_", ",", "y0_"}], "}"}]}], ",", 
     RowBox[{"p1", ":", 
      RowBox[{"{", 
       RowBox[{"x1_", ",", "y1_"}], "}"}]}], ",", 
     RowBox[{"p2", ":", 
      RowBox[{"{", 
       RowBox[{"x2_", ",", "y2_"}], "}"}]}]}], "}"}], "]"}], ":=", 
  FractionBox[
   RowBox[{
    RowBox[{"Slope", "[", 
     RowBox[{"{", 
      RowBox[{"p1", ",", "p2"}], "}"}], "]"}], "-", 
    RowBox[{"Slope", "[", 
     RowBox[{"{", 
      RowBox[{"p0", ",", "p1"}], "}"}], "]"}]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x2", "-", "x0"}], ")"}], "/", "2"}]]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"BendList", "[", "list_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"bl", ",", "max"}], "}"}], ",", 
    RowBox[{
     RowBox[{"bl", "=", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"list", "[", 
            RowBox[{"[", 
             RowBox[{"1", ",", "1"}], "]"}], "]"}], ",", "0"}], "}"}], "}"}], 
        ",", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"list", "[", 
             RowBox[{"[", 
              RowBox[{
               RowBox[{"i", "+", "1"}], ",", "1"}], "]"}], "]"}], ",", 
            RowBox[{"Bend", "[", 
             RowBox[{"list", "[", 
              RowBox[{"[", 
               RowBox[{"i", ";;", 
                RowBox[{"i", "+", "2"}]}], "]"}], "]"}], "]"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", 
            RowBox[{
             RowBox[{"Length", "[", "list", "]"}], "-", "2"}]}], "}"}]}], 
         "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"list", "[", 
            RowBox[{"[", 
             RowBox[{
              RowBox[{"-", "1"}], ",", "1"}], "]"}], "]"}], ",", "0"}], "}"}],
          "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"max", "=", 
      RowBox[{"Max", "[", 
       RowBox[{"Abs", "@", 
        RowBox[{"bl", "[", 
         RowBox[{"[", 
          RowBox[{"All", ",", "2"}], "]"}], "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", 
          RowBox[{"1", "/", "max"}]}], "}"}], "#"}], "&"}], "/@", "bl"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"NWeightFunction", "[", 
   RowBox[{"spec_", ",", "bend_", ",", 
    RowBox[{"{", 
     RowBox[{"a_", ",", "b_", ",", "c_"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x_", ",", "x0_", ",", "x1_"}], "}"}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"specint", "=", 
       RowBox[{"Integrate", "[", 
        RowBox[{
         RowBox[{"spec", "[", "x", "]"}], ",", "x"}], "]"}]}], ",", 
      RowBox[{"absbend", "=", 
       RowBox[{"FunctionInterpolation", "[", 
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"bend", "[", "x", "]"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"x", ",", 
           RowBox[{"bend", "[", 
            RowBox[{"[", 
             RowBox[{"1", ",", "1", ",", "1"}], "]"}], "]"}], ",", 
           RowBox[{"bend", "[", 
            RowBox[{"[", 
             RowBox[{"1", ",", "1", ",", "2"}], "]"}], "]"}]}], "}"}], ",", 
         RowBox[{"InterpolationOrder", "->", "1"}]}], "]"}]}], ",", 
      "bendint"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"bendint", "=", 
      RowBox[{"Integrate", "[", 
       RowBox[{
        RowBox[{"absbend", "[", "x", "]"}], ",", "x"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"a", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"x", " ", "-", "x0"}], ")"}], "/", 
        RowBox[{"(", 
         RowBox[{"x1", "-", "x0"}], ")"}]}]}], "+", 
      RowBox[{"b", " ", 
       RowBox[{"Piecewise", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"0", ",", 
            RowBox[{"x", "<", "x0"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"specint", "/", 
             RowBox[{"(", 
              RowBox[{"specint", "/.", 
               RowBox[{"x", "->", "x1"}]}], ")"}]}], ",", 
            RowBox[{"x0", "<=", "x", "<=", "x1"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", 
            RowBox[{"x", ">", "x1"}]}], "}"}]}], "}"}], "]"}]}], "+", 
      RowBox[{"c", " ", 
       RowBox[{"Piecewise", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"0", ",", 
            RowBox[{"x", "<", "x0"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"bendint", "/", 
             RowBox[{"(", 
              RowBox[{"bendint", "/.", 
               RowBox[{"x", "->", "x1"}]}], ")"}]}], ",", 
            RowBox[{"x0", "<=", "x", "<=", "x1"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", 
            RowBox[{"x", ">", "x1"}]}], "}"}]}], "}"}], "]"}]}]}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"FakeSignalGroups", "[", "]"}], ":=", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{
     RowBox[{"sig", "[", "x_", "]"}], "=", 
     RowBox[{
      RowBox[{
       RowBox[{"NaturalWidth", "[", "2", "]"}], "DopplerWidth", " ", 
       RowBox[{"Total", "[", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"Lorentzian", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"NaturalWidth", "[", "2", "]"}], "+", 
              RowBox[{
               SuperscriptBox["\[CapitalOmega]R", "2"], "/", 
               RowBox[{"NaturalWidth", "[", "2", "]"}]}]}], ",", 
             RowBox[{"x", "-", 
              RowBox[{"(", 
               RowBox[{"#", "-", "\[CapitalDelta]"}], ")"}]}]}], "]"}], "&"}],
           "/@", 
          RowBox[{"Drop", "[", 
           RowBox[{"freqs", ",", "2"}], "]"}]}], ")"}], "]"}], 
       RowBox[{"Gaussian", "[", 
        RowBox[{"DopplerWidth", ",", "x"}], "]"}]}], "/.", "params"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"wf", "=", 
     RowBox[{"WeightFunction", "[", 
      RowBox[{
       RowBox[{"sig", "[", "x", "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", 
         RowBox[{
          RowBox[{"-", "3"}], "dw"}], ",", 
         RowBox[{"3", "dw"}]}], "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"vg", "=", 
     RowBox[{"VelocityGroups2", "[", 
      RowBox[{"wf", ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", 
         RowBox[{
          RowBox[{"-", "3"}], "dw"}], ",", 
         RowBox[{"3", "dw"}]}], "}"}], ",", 
       RowBox[{"nog", "[", 
        RowBox[{"[", "i", "]"}], "]"}], ",", "dw", ",", 
       RowBox[{"ShowPlot", "->", "False"}]}], "]"}]}], ";"}], 
   ")"}]}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solve functions", "Subsection"],

Cell[TextData[{
 "We first find an appropriate set of velocity groups using the \
VelocityGroups function: \n\tVelocityGroups[DopplerWidth, \
ResonanceFrequencies, {SmallBinSize, n1}, {LargeBinSize, n2}]\nwhere\n\t\
ResonanceFrequencies are the optical resonance frequencies relative to the \
light frequency\n\tSmallBinSize is the velocity group bin size for bins near \
the resonance frequencies\n\tn1 specifies that (",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"2", " ", "n1"}], "+", "1"}], ")"}], TraditionalForm]]],
 " bins are created near each resonance frequency\n\tLargeBinSize is the \
minimum size for a bin not near a resonance frequency\n\tn2 is the multiple \
of the doppler width that determines the range over which velocity groups \
should be created\nIf ShowPlot",
 "->",
 "True is specified, a plot is produced that shows the limits of the velocity \
group bins (blue lines), and the weighted centers and number of atoms in each \
bin (black lines)."
}], "Text"],

Cell["\<\
The parameters used in VelocityGroups here are sufficient to obtain \
reasonable results, but more velocity groups are needed for complete accuracy.\
\>", "Text"],

Cell["\<\
We then substitute the specified numerical parameters into the submatrix \
describing one velocity group, and create a large matrix using copies of the \
submatrices. This matrix is solved using LinearSolve, and the solution is \
dotted with the vector describing fluorescence. The Krylov method, an \
iterative method, is used in LinearSolve. A preconditioner consisting of the \
diagonal blocks of the Bloch matrix is used.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "ReturnFlux", "]"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"RefineResult", "->", 
      RowBox[{"{", "}"}]}], ",", 
     RowBox[{"InitVGParams", "->", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", "2"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "3"}], "}"}]}], "}"}]}], ",", 
     RowBox[{"FluxReport", "->", 
      RowBox[{"{", "}"}]}], ",", 
     RowBox[{"ReturnAllSteps", "->", "False"}], ",", 
     RowBox[{"PlotVelocityGroups", "->", "False"}], ",", 
     RowBox[{"InitVG", "->", "Automatic"}], ",", 
     RowBox[{"SkipLinearSolve", "->", "False"}]}], "}"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"massageParams", "[", 
   RowBox[{"par_", ",", "atdata_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"nb", "=", 
      RowBox[{"NDither", "/.", "par"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Dispatch", "@", 
     RowBox[{"SelfApply", "@", 
      RowBox[{"Flatten", "@", 
       RowBox[{"{", 
        RowBox[{"atdata", ",", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"MapThread", "[", 
            RowBox[{"Rule", ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"DitherRegionFraction", "[", "i", "]"}], ",", 
                 RowBox[{"DetuningA", "[", "i", "]"}], ",", 
                 RowBox[{"LightIntensityA", "[", "i", "]"}], ",", 
                 RowBox[{"DetuningB", "[", "i", "]"}], ",", 
                 RowBox[{"LightIntensityB", "[", "i", "]"}]}], "}"}], ",", 
               RowBox[{
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"1", "/", "nb"}], ",", 
                   RowBox[{"DetuningA", "+", 
                    RowBox[{
                    RowBox[{"Sign", "[", 
                    RowBox[{"i", "-", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{"nb", "+", "1"}], ")"}], "/", "2"}]}], "]"}], 
                    RowBox[{
                    RowBox[{"DitherDetuning", "[", 
                    RowBox[{"Abs", "[", 
                    RowBox[{"i", "-", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{"nb", "+", "1"}], ")"}], "/", "2"}]}], "]"}], 
                    "]"}], "/", "2"}]}]}], ",", "LightIntensityA", ",", 
                   RowBox[{"DetuningB", "+", 
                    RowBox[{
                    RowBox[{"Sign", "[", 
                    RowBox[{"i", "-", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{"nb", "+", "1"}], ")"}], "/", "2"}]}], "]"}], 
                    RowBox[{
                    RowBox[{"DitherDetuning", "[", 
                    RowBox[{"Abs", "[", 
                    RowBox[{"i", "-", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{"nb", "+", "1"}], ")"}], "/", "2"}]}], "]"}], 
                    "]"}], "/", "2"}]}]}], ",", "LightIntensityB"}], "}"}], "/.", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"DitherDetuning", "[", 
                    RowBox[{"n_", "/;", 
                    RowBox[{"n", "\[NotElement]", "Integers"}]}], "]"}], "->", 
                   RowBox[{"DitherDetuning", "[", 
                    RowBox[{"n", "+", 
                    RowBox[{"1", "/", "2"}]}], "]"}]}], "}"}]}], "/.", 
                "par"}]}], "}"}]}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "nb"}], "}"}]}], "]"}], ",", 
         RowBox[{"DeleteCases", "[", 
          RowBox[{"par", ",", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"DetuningA", "->", "_"}], ")"}], "|", 
            RowBox[{"(", 
             RowBox[{"DetuningB", "->", "_"}], ")"}], "|", 
            RowBox[{"(", 
             RowBox[{"LightIntensityA", "->", "_"}], ")"}], "|", 
            RowBox[{"(", 
             RowBox[{"LightIntensityB", "->", "_"}], ")"}]}]}], "]"}]}], 
        "}"}]}]}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"ReturnFlux", "[", 
   RowBox[{"syst_", ",", "par_", ",", 
    RowBox[{"OptionsPattern", "[", "]"}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "nb", ",", "params", ",", "dw", ",", "nog", ",", "allfreqs", ",", "vg", 
      ",", "atdata", ",", "freqs", ",", "speclist", ",", "nvars", ",", 
      "resultTable", ",", "spec", ",", "bendfunc", ",", "wf", ",", "solve", 
      ",", "rep"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"nb", "=", 
      RowBox[{"NDither", "/.", "par"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"atdata", "=", 
      RowBox[{"Last", "[", "syst", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"nvars", "=", 
      RowBox[{"First", "[", "syst", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"params", "=", 
      RowBox[{"massageParams", "[", 
       RowBox[{"par", ",", "atdata"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"dw", "=", 
      RowBox[{"DopplerWidth", "/.", "params"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"nog", "=", 
      RowBox[{
       RowBox[{"OptionValue", "[", "RefineResult", "]"}], "/.", 
       RowBox[{
        RowBox[{"False", "|", "None"}], "->", 
        RowBox[{"{", "}"}]}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"rep", "=", 
      RowBox[{
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Head", "[", "#", "]"}], "===", "List"}], ",", "#", ",", 
          RowBox[{"{", "#", "}"}]}], "]"}], "&"}], "[", 
       RowBox[{
        RowBox[{"OptionValue", "[", "FluxReport", "]"}], "/.", 
        RowBox[{
         RowBox[{"False", "|", "None"}], "->", 
         RowBox[{"{", "}"}]}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"freqs", "=", 
      RowBox[{"ResonanceFrequencies", "/.", "atdata"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"allfreqs", "=", 
      RowBox[{"Union", "@", 
       RowBox[{"Flatten", "@", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"freqs", "-", 
                RowBox[{"DetuningA", "[", "i", "]"}]}], "/.", "params"}], 
              ")"}]}], ",", 
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"freqs", "-", 
                RowBox[{"DetuningB", "[", "i", "]"}]}], "/.", "params"}], 
              ")"}]}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "nb"}], "}"}]}], "]"}]}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"vg", "=", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"OptionValue", "[", "InitVG", "]"}], "===", "Automatic"}], 
        ",", 
        RowBox[{"VelocityGroups1", "@@", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"DopplerWidth", ",", "allfreqs", ",", 
             RowBox[{"Sequence", "@@", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"OptionValue", "[", "InitVGParams", "]"}], 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"NaturalWidth", "[", "2", "]"}], ",", "1"}], 
                   "}"}], ",", 
                  RowBox[{"{", 
                   RowBox[{"DopplerWidth", ",", "1"}], "}"}]}], "}"}]}], 
               ")"}]}], ",", 
             RowBox[{"ShowPlot", "->", 
              RowBox[{"OptionValue", "[", "PlotVelocityGroups", "]"}]}]}], 
            "}"}], "/.", "params"}], ")"}]}], ",", 
        RowBox[{"OptionValue", "[", "InitVG", "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"solve", "=", 
      RowBox[{"BuildAndSolve", "[", 
       RowBox[{"syst", ",", "params", ",", "vg", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"FluxReport", "->", 
           RowBox[{"Append", "[", 
            RowBox[{
             RowBox[{"OptionValue", "[", "FluxReport", "]"}], ",", 
             "VelocitySpectrum"}], "]"}]}], ",", 
          RowBox[{"SkipLinearSolve", "->", 
           RowBox[{"OptionValue", "[", "SkipLinearSolve", "]"}]}]}], "}"}]}], 
       "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"speclist", "=", 
      RowBox[{"VelocitySpectrum", "/.", 
       RowBox[{"solve", "[", 
        RowBox[{"[", 
         RowBox[{"-", "1"}], "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"MemberQ", "[", 
         RowBox[{
          RowBox[{"OptionValue", "[", "FluxReport", "]"}], ",", 
          "VelocitySpectrum"}], "]"}]}], ",", 
       RowBox[{"solve", "=", 
        RowBox[{"DeleteCases", "[", 
         RowBox[{"solve", ",", 
          RowBox[{"VelocitySpectrum", "->", "_"}], ",", 
          RowBox[{"{", "2", "}"}]}], "]"}]}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"resultTable", "=", 
      RowBox[{"{", "solve", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Table", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"spec", "=", 
         RowBox[{"Interpolation", "[", 
          RowBox[{"speclist", ",", 
           RowBox[{"InterpolationOrder", "->", "1"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"bendfunc", "=", 
         RowBox[{"Interpolation", "[", 
          RowBox[{
           RowBox[{"BendList", "[", "speclist", "]"}], ",", 
           RowBox[{"InterpolationOrder", "->", "1"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"wf", "=", 
         RowBox[{"NWeightFunction", "[", 
          RowBox[{"spec", ",", "bendfunc", ",", 
           RowBox[{"{", 
            RowBox[{"5", ",", "5", ",", "5"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"x", ",", 
             RowBox[{
              RowBox[{"-", "3"}], "dw"}], ",", 
             RowBox[{"3", "dw"}]}], "}"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"vg", "=", 
         RowBox[{"VelocityGroups2", "[", 
          RowBox[{"wf", ",", 
           RowBox[{"{", 
            RowBox[{"x", ",", 
             RowBox[{
              RowBox[{"-", "3"}], "dw"}], ",", 
             RowBox[{"3", "dw"}]}], "}"}], ",", 
           RowBox[{"nog", "[", 
            RowBox[{"[", "i", "]"}], "]"}], ",", "dw", ",", 
           RowBox[{
            RowBox[{"5", " ", "RecoilShift"}], "/.", "params"}], ",", 
           "allfreqs", ",", 
           RowBox[{"ShowPlot", "->", 
            RowBox[{"OptionValue", "[", "PlotVelocityGroups", "]"}]}]}], 
          "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"solve", "=", 
         RowBox[{"BuildAndSolve", "[", 
          RowBox[{"syst", ",", "params", ",", "vg", ",", 
           RowBox[{"FluxReport", "->", 
            RowBox[{"Append", "[", 
             RowBox[{
              RowBox[{"OptionValue", "[", "FluxReport", "]"}], ",", 
              "VelocitySpectrum"}], "]"}]}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"speclist", "=", 
         RowBox[{"VelocitySpectrum", "/.", 
          RowBox[{"solve", "[", 
           RowBox[{"[", 
            RowBox[{"-", "1"}], "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"!", 
           RowBox[{"MemberQ", "[", 
            RowBox[{
             RowBox[{"OptionValue", "[", "FluxReport", "]"}], ",", 
             "VelocitySpectrum"}], "]"}]}], ",", 
          RowBox[{"solve", "=", 
           RowBox[{"DeleteCases", "[", 
            RowBox[{"solve", ",", 
             RowBox[{"VelocitySpectrum", "->", "_"}], ",", 
             RowBox[{"{", "2", "}"}]}], "]"}]}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"AppendTo", "[", 
         RowBox[{"resultTable", ",", "solve"}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{"Length", "[", "nog", "]"}]}], "}"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", 
         RowBox[{"OptionValue", "[", "FluxReport", "]"}], "]"}], "===", "0"}],
        ",", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"OptionValue", "[", "ReturnAllSteps", "]"}], ",", 
         RowBox[{"resultTable", "[", 
          RowBox[{"[", 
           RowBox[{"All", ",", "1"}], "]"}], "]"}], ",", 
         RowBox[{"resultTable", "[", 
          RowBox[{"[", 
           RowBox[{
            RowBox[{"-", "1"}], ",", "1"}], "]"}], "]"}]}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"OptionValue", "[", "ReturnAllSteps", "]"}], ",", 
         "resultTable", ",", 
         RowBox[{"resultTable", "[", 
          RowBox[{"[", 
           RowBox[{"-", "1"}], "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]",
       "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"(*", 
  RowBox[{
   RowBox[{"ReturnFlux", "[", 
    RowBox[{"syst_", ",", "par_", ",", "vg_", ",", 
     RowBox[{"OptionsPattern", "[", "]"}]}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"params", ",", "atdata", ",", "solve"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"params", "=", 
       RowBox[{"massageParams", "[", 
        RowBox[{"par", ",", 
         RowBox[{"Last", "[", "syst", "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"solve", "=", 
       RowBox[{"BuildAndSolve", "[", 
        RowBox[{"syst", ",", "params", ",", "vg", ",", 
         RowBox[{"FluxReport", "->", 
          RowBox[{"Append", "[", 
           RowBox[{
            RowBox[{"OptionValue", "[", "FluxReport", "]"}], ",", 
            "VelocitySpectrum"}], "]"}]}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"!", 
         RowBox[{"MemberQ", "[", 
          RowBox[{
           RowBox[{"OptionValue", "[", "FluxReport", "]"}], ",", 
           "VelocitySpectrum"}], "]"}]}], ",", 
        RowBox[{"solve", "=", 
         RowBox[{"DeleteCases", "[", 
          RowBox[{"solve", ",", 
           RowBox[{"VelocitySpectrum", "->", "_"}], ",", 
           RowBox[{"{", "2", "}"}]}], "]"}]}]}], "]"}], ";", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"MemberQ", "[", 
         RowBox[{
          RowBox[{"OptionValue", "[", "FluxReport", "]"}], ",", 
          "VelocityGroups"}], "]"}], ",", 
        RowBox[{"solve", "=", 
         RowBox[{"Insert", "[", 
          RowBox[{"solve", ",", 
           RowBox[{"VelocityGroups", "->", "vg"}], ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "1"}], ",", 
             RowBox[{"-", "1"}]}], "}"}]}], "]"}]}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", 
          RowBox[{"OptionValue", "[", "FluxReport", "]"}], "]"}], "===", 
         "0"}], ",", 
        RowBox[{"solve", "[", 
         RowBox[{"[", "1", "]"}], "]"}], ",", "solve"}], "]"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], "*)"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "BuildAndSolve", "]"}], "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"FluxReport", "->", 
      RowBox[{"{", "}"}]}], ",", 
     RowBox[{"SkipLinearSolve", "->", "False"}]}], "}"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 StyleBox[
  RowBox[{
   RowBox[{"BuildAndSolve", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "nvars_", ",", "A110_", ",", "A11S_", ",", "A11G_", ",", "A11R_", ",", 
       "A12G_", ",", "A21R_", ",", "A12B_", ",", "b1G_", ",", "fluorvec1_", 
       ",", "atdata_"}], "}"}], ",", "params_", ",", "vg_", ",", 
     RowBox[{"OptionsPattern", "[", "]"}]}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "nd", ",", "nb", ",", "fluorvecn", ",", "A110list", ",", "A110nList", 
       ",", "A11Sn", ",", "A11Gn", ",", "A11Rn", ",", "A12Gn", ",", "A21Rn", 
       ",", "A12Bn", ",", "bGn", ",", "A21RList", ",", "A12BList", ",", 
       "blocks", ",", "fullAmat", ",", "diagprecmat", ",", "prec", ",", 
       "fullbvec", ",", "fullfluorvec", ",", "sol", ",", "fl", ",", "vccfunc",
        ",", "vccrate1", ",", "vccrate2", ",", "vcr1", ",", "vcr2"}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{"Number", " ", "of", " ", "dithering", " ", "regions"}], "*)"}],
      "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"nb", "=", 
       RowBox[{"NDither", "/.", "params"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{"Number", " ", "of", " ", "velocity", " ", "groups"}], "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"nd", "=", 
       RowBox[{"Length", "[", "vg", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
       "Function", " ", "describing", " ", "transfer", " ", "rate", " ", 
        "between", " ", "velocity", " ", "groups", " ", "due", " ", "to", " ",
         "collisions"}], "*)"}], "\[IndentingNewLine]", 
      RowBox[{"vccfunc", "=", 
       RowBox[{"VccRateFunction", "/.", "params"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
       "Substitute", " ", "numerical", " ", "values", " ", "into", " ", 
        "input", " ", "matrices"}], "*)"}], "\[IndentingNewLine]", 
      RowBox[{"A110list", "=", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"nb", "==", "1"}], ",", 
         RowBox[{"{", "A110", "}"}], ",", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"SparseReplace", "[", 
            RowBox[{"A110", ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{
                RowBox[{"RabiFrequencyA", "[", "1", "]"}], "->", 
                RowBox[{"RabiFrequencyA", "[", "i", "]"}]}], ",", 
               RowBox[{
                RowBox[{"RabiFrequencyB", "[", "1", "]"}], "->", 
                RowBox[{"RabiFrequencyB", "[", "i", "]"}]}], ",", 
               RowBox[{
                RowBox[{"DitherRegionFraction", "[", "1", "]"}], "->", 
                RowBox[{"DitherRegionFraction", "[", "i", "]"}]}], ",", 
               RowBox[{
                RowBox[{"DetuningA", "[", "1", "]"}], "->", 
                RowBox[{"DetuningA", "[", "i", "]"}]}], ",", 
               RowBox[{
                RowBox[{"DetuningB", "[", "1", "]"}], "->", 
                RowBox[{"DetuningB", "[", "i", "]"}]}], ",", 
               RowBox[{
                RowBox[{"LightPhase", "[", "1", "]"}], "->", 
                RowBox[{"LightPhase", "[", "i", "]"}]}]}], "}"}]}], "]"}], 
           ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "nb"}], "}"}]}], "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
        "fluorvecn", ",", "A110nList", ",", "A11Sn", ",", "A11Gn", ",", 
         "A11Rn", ",", "A12Gn", ",", "A21Rn", ",", "A12Bn", ",", "bGn"}], 
        "}"}], "=", 
       RowBox[{"N", "[", 
        RowBox[{"SparseReplace", "[", 
         RowBox[{
          RowBox[{"SparseReplace", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
             "fluorvec1", ",", "A110list", ",", "A11S", ",", "A11G", ",", 
              "A11R", ",", "A12G", ",", "A21R", ",", "A12B", ",", "b1G"}], 
             "}"}], ",", "params"}], "]"}], ",", "params"}], "]"}], "]"}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
       "Check", " ", "if", " ", "matrices", " ", "are", " ", "numerical"}], 
       "*)"}], "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"!", 
         RowBox[{"And", "@@", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"ArrayQ", "[", 
              RowBox[{"#", ",", "_", ",", "NumericQ"}], "]"}], "&"}], "/@", 
            RowBox[{"{", 
             RowBox[{
             "fluorvecn", ",", "A110nList", ",", "A11Sn", ",", "A11Gn", ",", 
              "A11Rn", ",", "A12Gn", ",", "A21Rn", ",", "A12Bn", ",", "bGn"}],
              "}"}]}], ")"}]}]}], ",", 
        RowBox[{
         RowBox[{"Print", "[", "\"\<Nonnumerical matrix.\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{"A12List", "=", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"A12Gn", " ", 
            RowBox[{"vg", "[", 
             RowBox[{"[", 
              RowBox[{"i", ",", "2"}], "]"}], "]"}]}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "nd"}], "}"}]}], "]"}]}], ";"}], "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"A21RList", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"A21Rn", " ", "/", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"vg", "[", 
             RowBox[{"[", 
              RowBox[{"i", ",", "1"}], "]"}], "]"}], "-", 
            RowBox[{"vg", "[", 
             RowBox[{"[", 
              RowBox[{
               RowBox[{"i", "-", "1"}], ",", "1"}], "]"}], "]"}]}], ")"}]}], 
         ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "2", ",", "nd"}], "}"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"A12BList", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"A12Bn", "#"}], "&"}], "/@", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{"DitherRegionFraction", "[", "i", "]"}], " ", ",", 
            RowBox[{"{", 
             RowBox[{"i", ",", "nb"}], "}"}]}], "]"}], "/.", "params"}], 
         ")"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"!", 
         RowBox[{"And", "@@", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"ArrayQ", "[", 
              RowBox[{"#", ",", "_", ",", "NumericQ"}], "]"}], "&"}], "/@", 
            RowBox[{"{", 
             RowBox[{"A21RList", ",", "A12BList"}], "}"}]}], ")"}]}]}], ",", 
        RowBox[{
         RowBox[{
         "Print", "[", 
          "\"\<BuildAndSolve[]: Nonnumerical matrix (1), check the option \
list.\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{"For", " ", "hard", " ", "collisions", " ", 
         RowBox[{"(", 
          RowBox[{"VCCRateFunction", " ", "->", " ", "Automatic"}], ")"}]}], 
        ",", " ", 
        RowBox[{
        "the", " ", "formulas", " ", "for", " ", "transfer", " ", "rates", 
         " ", "can", " ", "be", " ", 
         RowBox[{"simplified", ".", " ", "We"}], " ", "use", " ", "this", " ",
          "here", " ", "to", " ", "optimize", " ", "the", " ", "hard", " ", 
         "collision", " ", 
         RowBox[{"case", "."}]}]}], "*)"}], "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"vccfunc", "===", "Automatic"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"vcr1", "=", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"A11Gn", 
               RowBox[{"(", 
                RowBox[{"1", "-", "#"}], ")"}]}], "&"}], "/@", 
             RowBox[{"vg", "[", 
              RowBox[{"[", 
               RowBox[{"All", ",", "2"}], "]"}], "]"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"VccTime", "/.", "params"}], ")"}]}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"vcr2", "=", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"A12Gn", " ", "#"}], "&"}], "/@", 
             RowBox[{"vg", "[", 
              RowBox[{"[", 
               RowBox[{"All", ",", "2"}], "]"}], "]"}]}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"VccTime", "/.", "params"}], ")"}]}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"!", 
            RowBox[{"And", "@@", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{
                RowBox[{"ArrayQ", "[", 
                 RowBox[{"#", ",", "_", ",", "NumericQ"}], "]"}], "&"}], "/@", 
               RowBox[{"{", 
                RowBox[{"vcr1", ",", "vcr2"}], "}"}]}], ")"}]}]}], ",", 
           RowBox[{
            RowBox[{
            "Print", "[", 
             "\"\<BuildAndSolve[]: Nonnumerical matrix (2), check the option \
list.\>\"", "]"}], ";", 
            RowBox[{"Abort", "[", "]"}]}]}], "]"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"vccrate1", "[", "i_", "]"}], ":=", 
          RowBox[{"vcr1", "[", 
           RowBox[{"[", "i", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"vccrate2", "[", 
           RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
          RowBox[{"vcr2", "[", 
           RowBox[{"[", "i", "]"}], "]"}]}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"vccrate1", "[", "i_", "]"}], ":=", 
          RowBox[{"A11Gn", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"vccfunc", "[", 
              RowBox[{
               RowBox[{"vg", "[", 
                RowBox[{"[", 
                 RowBox[{"i", ",", "1"}], "]"}], "]"}], ",", "Infinity"}], 
              "]"}], "-", 
             RowBox[{"vccfunc", "[", 
              RowBox[{
               RowBox[{"vg", "[", 
                RowBox[{"[", 
                 RowBox[{"i", ",", "1"}], "]"}], "]"}], ",", 
               RowBox[{"vg", "[", 
                RowBox[{"[", 
                 RowBox[{"i", ",", "3", ",", "2"}], "]"}], "]"}]}], "]"}], 
             "+", 
             RowBox[{"vccfunc", "[", 
              RowBox[{
               RowBox[{"vg", "[", 
                RowBox[{"[", 
                 RowBox[{"i", ",", "1"}], "]"}], "]"}], ",", 
               RowBox[{"vg", "[", 
                RowBox[{"[", 
                 RowBox[{"i", ",", "3", ",", "1"}], "]"}], "]"}]}], "]"}]}], 
            ")"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"vccrate2", "[", 
           RowBox[{"i_", ",", "j_"}], "]"}], ":=", 
          RowBox[{"A12Gn", 
           RowBox[{"(", 
            RowBox[{
             FractionBox["1", 
              RowBox[{"vg", "[", 
               RowBox[{"[", 
                RowBox[{"j", ",", "2"}], "]"}], "]"}]], 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"vccfunc", "[", 
                RowBox[{
                 RowBox[{"vg", "[", 
                  RowBox[{"[", 
                   RowBox[{"i", ",", "1"}], "]"}], "]"}], ",", 
                 RowBox[{"vg", "[", 
                  RowBox[{"[", 
                   RowBox[{"j", ",", "3", ",", "2"}], "]"}], "]"}]}], "]"}], 
               "-", 
               RowBox[{"vccfunc", "[", 
                RowBox[{
                 RowBox[{"vg", "[", 
                  RowBox[{"[", 
                   RowBox[{"i", ",", "1"}], "]"}], "]"}], ",", 
                 RowBox[{"vg", "[", 
                  RowBox[{"[", 
                   RowBox[{"j", ",", "3", ",", "1"}], "]"}], "]"}]}], "]"}]}],
               ")"}], 
             RowBox[{"vg", "[", 
              RowBox[{"[", 
               RowBox[{"i", ",", "2"}], "]"}], "]"}]}], ")"}]}]}]}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"blocks", "=", 
       RowBox[{"Table", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Which", "[", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"k", "==", "l"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"A110nList", "[", 
             RowBox[{"[", "k", "]"}], "]"}], "+", 
            RowBox[{
             RowBox[{"vg", "[", 
              RowBox[{"[", 
               RowBox[{"i", ",", "1"}], "]"}], "]"}], " ", "A11Sn"}], "+", 
            RowBox[{"vccrate1", "[", "i", "]"}], "+", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"i", "<", "nd"}], ",", 
              RowBox[{"A11Rn", "/", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"vg", "[", 
                  RowBox[{"[", 
                   RowBox[{
                    RowBox[{"i", "+", "1"}], ",", "1"}], "]"}], "]"}], "-", 
                 RowBox[{"vg", "[", 
                  RowBox[{"[", 
                   RowBox[{"i", ",", "1"}], "]"}], "]"}]}], ")"}]}], ",", 
              "0"}], "]"}]}], ",", "\[IndentingNewLine]", 
           RowBox[{"k", "!=", "l"}], ",", "\[IndentingNewLine]", 
           RowBox[{"A12BList", "[", 
            RowBox[{"[", "k", "]"}], "]"}]}], "\[IndentingNewLine]", "]"}], 
         ",", "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"i", ",", "nd"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"k", ",", "nb"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"l", ",", "nb"}], "}"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"fullAmat", "=", 
       RowBox[{"ArrayFlatten", "@", 
        RowBox[{"ArrayFlatten", "@", 
         RowBox[{"Table", "[", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"i", "==", "j"}], ",", "\[IndentingNewLine]", 
             RowBox[{"blocks", "[", 
              RowBox[{"[", "i", "]"}], "]"}], ",", "\[IndentingNewLine]", 
             RowBox[{"Table", "[", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"If", "[", "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"k", "==", "l"}], ",", "\[IndentingNewLine]", 
                 RowBox[{
                  RowBox[{"vccrate2", "[", 
                   RowBox[{"i", ",", "j"}], "]"}], "+", 
                  RowBox[{"If", "[", 
                   RowBox[{
                    RowBox[{"j", "==", 
                    RowBox[{"i", "-", "1"}]}], ",", 
                    RowBox[{"A21RList", "[", 
                    RowBox[{"[", "j", "]"}], "]"}], ",", "0"}], "]"}]}], ",", 
                 "\[IndentingNewLine]", "0."}], "\[IndentingNewLine]", "]"}], 
               ",", "\[IndentingNewLine]", 
               RowBox[{"{", 
                RowBox[{"k", ",", "nb"}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"l", ",", "nb"}], "}"}]}], "]"}]}], "]"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{"i", ",", "nd"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"j", ",", "nd"}], "}"}]}], "]"}]}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"diagprecmat", "=", 
       RowBox[{"ArrayFlatten", "@", 
        RowBox[{"ArrayFlatten", "@", 
         RowBox[{"Table", "[", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"i", "==", "j"}], ",", "\[IndentingNewLine]", 
             RowBox[{"blocks", "[", 
              RowBox[{"[", "i", "]"}], "]"}], ",", "\[IndentingNewLine]", 
             "0."}], "]"}], ",", "\[IndentingNewLine]", 
           RowBox[{"{", 
            RowBox[{"i", ",", "nd"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"j", ",", "nd"}], "}"}]}], "]"}]}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"Change", " ", 
        RowBox[{"R", ".", "H", ".", " ", "02"}], "Sep09"}], " ", "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"OptionValue", "[", "SkipLinearSolve", "]"}], "==", 
         "False"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"prec", "[", "x_", "]"}], "=", 
          RowBox[{
           RowBox[{"LinearSolve", "[", "diagprecmat", "]"}], "[", "x", 
           "]"}]}], ";"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"fullbvec", "=", 
       RowBox[{"Flatten", "@", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"vg", "[", 
              RowBox[{"[", 
               RowBox[{"i", ",", "2"}], "]"}], "]"}], "bGn"}], ",", 
            RowBox[{"{", "nb", "}"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "nd"}], "}"}]}], "]"}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"fullfluorvec", "=", 
       RowBox[{"Flatten", "@", 
        RowBox[{"Table", "[", 
         RowBox[{"fluorvecn", ",", 
          RowBox[{"{", 
           RowBox[{"nd", " ", "nb"}], "}"}]}], "]"}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"!", 
         RowBox[{"And", "@@", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"ArrayQ", "[", 
              RowBox[{"#", ",", "_", ",", "NumericQ"}], "]"}], "&"}], "/@", 
            RowBox[{"{", 
             RowBox[{
             "fullAmat", ",", "diagprecmat", ",", "fullbvec", ",", 
              "fullfluorvec"}], "}"}]}], ")"}]}]}], ",", 
        RowBox[{
         RowBox[{
         "Print", "[", 
          "\"\<BuildAndSolve[]: Nonnumerical matrix (3), check the option \
list.\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"Change", " ", 
        RowBox[{"R", ".", "H", ".", " ", "02"}], "Sep09"}], " ", "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"sol", "=", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"OptionValue", "[", "SkipLinearSolve", "]"}], "==", 
          "False"}], ",", "\[IndentingNewLine]", 
         RowBox[{"LinearSolve", "[", 
          RowBox[{"fullAmat", ",", "fullbvec", ",", 
           RowBox[{"Method", "->", 
            RowBox[{"{", 
             RowBox[{"Krylov", ",", 
              RowBox[{"Preconditioner", "->", "prec"}]}], "}"}]}]}], "]"}], 
         ",", "0"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"fl", "=", 
       RowBox[{"Re", "[", 
        RowBox[{"fullfluorvec", ".", "sol"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"fl", ",", 
        RowBox[{"Join", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"MemberQ", "[", 
             RowBox[{
              RowBox[{"OptionValue", "[", "FluxReport", "]"}], ",", 
              "VelocitySpectrum"}], "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"VelocitySpectrum", "->", 
              RowBox[{"MapThread", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{"#3", ",", 
                   RowBox[{"#1", "/", 
                    RowBox[{"(", 
                    RowBox[{"-", 
                    RowBox[{"Subtract", "@@", "#2"}]}], ")"}]}]}], "}"}], 
                 "&"}], ",", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"Total", "/@", 
                   RowBox[{"Partition", "[", 
                    RowBox[{
                    RowBox[{"Total", "/@", 
                    RowBox[{"Partition", "[", 
                    RowBox[{
                    RowBox[{"fullfluorvec", " ", "sol"}], ",", "nvars"}], 
                    "]"}]}], ",", "nb"}], "]"}]}], ",", 
                  RowBox[{"vg", "[", 
                   RowBox[{"[", 
                    RowBox[{"All", ",", "3"}], "]"}], "]"}], ",", 
                  RowBox[{"vg", "[", 
                   RowBox[{"[", 
                    RowBox[{"All", ",", "1"}], "]"}], "]"}]}], "}"}]}], 
               "]"}]}], "}"}], ",", 
            RowBox[{"{", "}"}]}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"MemberQ", "[", 
             RowBox[{
              RowBox[{"OptionValue", "[", "FluxReport", "]"}], ",", 
              "VelocityGroups"}], "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"VelocityGroups", "->", "vg"}], "}"}], ",", 
            RowBox[{"{", "}"}]}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"MemberQ", "[", 
             RowBox[{
              RowBox[{"OptionValue", "[", "FluxReport", "]"}], ",", 
              "DMSolution"}], "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"DMSolution", "->", "sol"}], "}"}], ",", 
            RowBox[{"{", "}"}]}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"MemberQ", "[", 
             RowBox[{
              RowBox[{"OptionValue", "[", "FluxReport", "]"}], ",", 
              "BlochMatrix"}], "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"BlochMatrix", "->", 
              RowBox[{"{", 
               RowBox[{"fullAmat", ",", "fullbvec", ",", "fullfluorvec"}], 
               "}"}]}], "}"}], ",", 
            RowBox[{"{", "}"}]}], "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{"Change", " ", 
            RowBox[{"R", ".", "H", ".", " ", "02"}], "Sep09"}], " ", "*)"}], 
          "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"MemberQ", "[", 
             RowBox[{
              RowBox[{"OptionValue", "[", "FluxReport", "]"}], ",", 
              "RFParams"}], "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"RFParams", "->", "params"}], "}"}], ",", 
            RowBox[{"{", "}"}]}], "]"}]}], "\[IndentingNewLine]", "]"}]}], 
       "\[IndentingNewLine]", "}"}]}]}], "\[IndentingNewLine]", "]"}]}],
  Background->None]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"BuildLightDependentPart", "[", 
   RowBox[{"nvars_", ",", "A110_", ",", "atdata_", ",", "par_", ",", "vg_"}], 
   "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "nd", ",", "nb", ",", "A110list", ",", "A110nList", ",", "blocks", ",", 
      "params", ",", "params0", ",", "params1"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"params", "=", 
      RowBox[{"Join", "[", 
       RowBox[{"par", ",", 
        RowBox[{"DefaultParameters", "/.", 
         RowBox[{"Options", "[", "PsiMeso", "]"}]}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"params0", "=", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"RepumpFraction", "/.", "params"}], ")"}], ">", "0"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"Join", "[", 
         RowBox[{
          RowBox[{"PhiOptDefault", "/.", "params"}], ",", "params"}], "]"}], 
        ",", "\[IndentingNewLine]", 
        RowBox[{"Join", "[", 
         RowBox[{
          RowBox[{"PhiOptDefault2", "/.", "params"}], ",", "params"}], "]"}], 
        ",", 
        RowBox[{
         RowBox[{
         "Print", "[", 
          "\"\<BuildLightDependentPart[]: Cannot decide if RepumpFraction is \
nonzero\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"params1", "=", 
      RowBox[{"massageParams", "[", 
       RowBox[{"params0", ",", "atdata"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{"Number", " ", "of", " ", "dithering", " ", "regions"}], "*)"}],
      "\[IndentingNewLine]", 
     RowBox[{"nb", "=", 
      RowBox[{"NDither", "/.", "params1"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{"Number", " ", "of", " ", "velocity", " ", "groups"}], "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"nd", "=", 
      RowBox[{"Length", "[", "vg", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
      "Substitute", " ", "numerical", " ", "values", " ", "into", " ", 
       "input", " ", "matrices"}], "*)"}], "\[IndentingNewLine]", 
     RowBox[{"A110list", "=", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"nb", "==", "1"}], ",", 
        RowBox[{"{", "A110", "}"}], ",", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"SparseReplace", "[", 
           RowBox[{"A110", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{
               RowBox[{"RabiFrequencyA", "[", "1", "]"}], "->", 
               RowBox[{"RabiFrequencyA", "[", "i", "]"}]}], ",", 
              RowBox[{
               RowBox[{"RabiFrequencyB", "[", "1", "]"}], "->", 
               RowBox[{"RabiFrequencyB", "[", "i", "]"}]}], ",", 
              RowBox[{
               RowBox[{"DetuningA", "[", "1", "]"}], "->", 
               RowBox[{"DetuningA", "[", "i", "]"}]}], ",", 
              RowBox[{
               RowBox[{"DetuningB", "[", "1", "]"}], "->", 
               RowBox[{"DetuningB", "[", "i", "]"}]}], ",", 
              RowBox[{
               RowBox[{"LightPhase", "[", "1", "]"}], "->", 
               RowBox[{"LightPhase", "[", "i", "]"}]}]}], "}"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "nb"}], "}"}]}], "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"{", "A110nList", "}"}], "=", 
      RowBox[{"N", "[", 
       RowBox[{"SparseReplace", "[", 
        RowBox[{
         RowBox[{"SparseReplace", "[", 
          RowBox[{
           RowBox[{"{", "A110list", "}"}], ",", "params1"}], "]"}], ",", 
         "params1"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
      "Check", " ", "if", " ", "matrices", " ", "are", " ", "numerical"}], 
      "*)"}], "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"And", "@@", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"ArrayQ", "[", 
             RowBox[{"#", ",", "_", ",", "NumericQ"}], "]"}], "&"}], "/@", 
           RowBox[{"{", "A110nList", "}"}]}], ")"}]}]}], ",", 
       RowBox[{
        RowBox[{"Print", "[", "\"\<Nonnumerical matrix.\>\"", "]"}], ";", 
        RowBox[{"Abort", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"blocks", "=", 
      RowBox[{"Table", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"If", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"k", "==", "l"}], ",", "\[IndentingNewLine]", 
          RowBox[{"A110nList", "[", 
           RowBox[{"[", "k", "]"}], "]"}], ",", "\[IndentingNewLine]", "0."}],
          "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"i", ",", "nd"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"k", ",", "nb"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"l", ",", "nb"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ArrayFlatten", "@", 
      RowBox[{"ArrayFlatten", "@", 
       RowBox[{"Table", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"i", "==", "j"}], ",", "\[IndentingNewLine]", 
           RowBox[{"blocks", "[", 
            RowBox[{"[", "i", "]"}], "]"}], ",", "\[IndentingNewLine]", 
           "0."}], "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{"i", ",", "nd"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"j", ",", "nd"}], "}"}]}], "]"}]}]}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Bloch equations setup", "Subsection"],

Cell["\<\
Designation for the ground and excited states used in the AtomicData database.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"setStates", "[", 
   RowBox[{"atom_", ",", "line_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"gstate", ",", "estate"}], "}"}], "=", 
   RowBox[{"Switch", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"atom", ",", "line"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\"\<Na\>\"", "|", "\"\<Na23\>\""}], ",", "\"\<D1\>\""}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\"\<Na\>\"", ",", "23", ",", 
         RowBox[{"{", 
          RowBox[{"\"\<Ne\>\"", ",", 
           RowBox[{"{", 
            RowBox[{"3", ",", "\"\<s\>\""}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "\"\<S\>\"", ",", 
           FractionBox["1", "2"]}], "}"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\"\<Na\>\"", ",", "23", ",", 
         RowBox[{"{", 
          RowBox[{"\"\<Ne\>\"", ",", 
           RowBox[{"{", 
            RowBox[{"3", ",", "\"\<p\>\""}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "\"\<P\>\"", ",", 
           FractionBox["1", "2"]}], "}"}]}], "}"}]}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\"\<Na\>\"", "|", "\"\<Na23\>\""}], ",", "\"\<D2\>\""}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\"\<Na\>\"", ",", "23", ",", 
         RowBox[{"{", 
          RowBox[{"\"\<Ne\>\"", ",", 
           RowBox[{"{", 
            RowBox[{"3", ",", "\"\<s\>\""}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "\"\<S\>\"", ",", 
           FractionBox["1", "2"]}], "}"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\"\<Na\>\"", ",", "23", ",", 
         RowBox[{"{", 
          RowBox[{"\"\<Ne\>\"", ",", 
           RowBox[{"{", 
            RowBox[{"3", ",", "\"\<p\>\""}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "\"\<P\>\"", ",", 
           FractionBox["3", "2"]}], "}"}]}], "}"}]}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\"\<K\>\"", "|", "\"\<K39\>\""}], ",", "\"\<D1\>\""}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\"\<K\>\"", ",", " ", "39", ",", " ", 
         RowBox[{"{", 
          RowBox[{"\"\<Ar\>\"", ",", " ", 
           RowBox[{"{", 
            RowBox[{"4", ",", " ", "\"\<s\>\""}], "}"}]}], "}"}], ",", " ", 
         RowBox[{"{", 
          RowBox[{"2", ",", " ", "\"\<S\>\"", ",", " ", 
           RowBox[{"1", "/", "2"}]}], "}"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\"\<K\>\"", ",", " ", "39", ",", " ", 
         RowBox[{"{", 
          RowBox[{"\"\<Ar\>\"", ",", " ", 
           RowBox[{"{", 
            RowBox[{"4", ",", " ", "\"\<p\>\""}], "}"}]}], "}"}], ",", " ", 
         RowBox[{"{", 
          RowBox[{"2", ",", " ", "\"\<P\>\"", ",", " ", 
           RowBox[{"1", "/", "2"}]}], "}"}]}], "}"}]}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\"\<K\>\"", "|", "\"\<K39\>\""}], ",", "\"\<D2\>\""}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\"\<K\>\"", ",", " ", "39", ",", " ", 
         RowBox[{"{", 
          RowBox[{"\"\<Ar\>\"", ",", " ", 
           RowBox[{"{", 
            RowBox[{"4", ",", " ", "\"\<s\>\""}], "}"}]}], "}"}], ",", " ", 
         RowBox[{"{", 
          RowBox[{"2", ",", " ", "\"\<S\>\"", ",", " ", 
           RowBox[{"1", "/", "2"}]}], "}"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\"\<K\>\"", ",", " ", "39", ",", " ", 
         RowBox[{"{", 
          RowBox[{"\"\<Ar\>\"", ",", " ", 
           RowBox[{"{", 
            RowBox[{"4", ",", " ", "\"\<p\>\""}], "}"}]}], "}"}], ",", " ", 
         RowBox[{"{", 
          RowBox[{"2", ",", " ", "\"\<P\>\"", ",", " ", 
           RowBox[{"3", "/", "2"}]}], "}"}]}], "}"}]}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<Rb87\>\"", ",", "\"\<D1\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\"\<Rb\>\"", ",", "87", ",", 
         RowBox[{"{", 
          RowBox[{"\"\<Kr\>\"", ",", 
           RowBox[{"{", 
            RowBox[{"5", ",", "\"\<s\>\""}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "\"\<S\>\"", ",", 
           FractionBox["1", "2"]}], "}"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\"\<Rb\>\"", ",", "87", ",", 
         RowBox[{"{", 
          RowBox[{"\"\<Kr\>\"", ",", 
           RowBox[{"{", 
            RowBox[{"5", ",", "\"\<p\>\""}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "\"\<P\>\"", ",", 
           FractionBox["1", "2"]}], "}"}]}], "}"}]}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<Rb87\>\"", ",", "\"\<D2\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\"\<Rb\>\"", ",", "87", ",", 
         RowBox[{"{", 
          RowBox[{"\"\<Kr\>\"", ",", 
           RowBox[{"{", 
            RowBox[{"5", ",", "\"\<s\>\""}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "\"\<S\>\"", ",", 
           FractionBox["1", "2"]}], "}"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\"\<Rb\>\"", ",", "87", ",", 
         RowBox[{"{", 
          RowBox[{"\"\<Kr\>\"", ",", 
           RowBox[{"{", 
            RowBox[{"5", ",", "\"\<p\>\""}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "\"\<P\>\"", ",", 
           FractionBox["3", "2"]}], "}"}]}], "}"}]}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<Rb85\>\"", ",", "\"\<D1\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\"\<Rb\>\"", ",", "85", ",", 
         RowBox[{"{", 
          RowBox[{"\"\<Kr\>\"", ",", 
           RowBox[{"{", 
            RowBox[{"5", ",", "\"\<s\>\""}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "\"\<S\>\"", ",", 
           FractionBox["1", "2"]}], "}"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\"\<Rb\>\"", ",", "85", ",", 
         RowBox[{"{", 
          RowBox[{"\"\<Kr\>\"", ",", 
           RowBox[{"{", 
            RowBox[{"5", ",", "\"\<p\>\""}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "\"\<P\>\"", ",", 
           FractionBox["1", "2"]}], "}"}]}], "}"}]}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{"\"\<Rb85\>\"", ",", "\"\<D2\>\""}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\"\<Rb\>\"", ",", "85", ",", 
         RowBox[{"{", 
          RowBox[{"\"\<Kr\>\"", ",", 
           RowBox[{"{", 
            RowBox[{"5", ",", "\"\<s\>\""}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "\"\<S\>\"", ",", 
           FractionBox["1", "2"]}], "}"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\"\<Rb\>\"", ",", "85", ",", 
         RowBox[{"{", 
          RowBox[{"\"\<Kr\>\"", ",", 
           RowBox[{"{", 
            RowBox[{"5", ",", "\"\<p\>\""}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "\"\<P\>\"", ",", 
           FractionBox["3", "2"]}], "}"}]}], "}"}]}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\"\<Cs\>\"", "|", "\"\<Cs133\>\""}], ",", "\"\<D1\>\""}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\"\<Cs\>\"", ",", "133", ",", 
         RowBox[{"{", 
          RowBox[{"\"\<Xe\>\"", ",", 
           RowBox[{"{", 
            RowBox[{"6", ",", "\"\<s\>\""}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "\"\<S\>\"", ",", 
           FractionBox["1", "2"]}], "}"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\"\<Cs\>\"", ",", "133", ",", 
         RowBox[{"{", 
          RowBox[{"\"\<Xe\>\"", ",", 
           RowBox[{"{", 
            RowBox[{"6", ",", "\"\<p\>\""}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "\"\<P\>\"", ",", 
           FractionBox["1", "2"]}], "}"}]}], "}"}]}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\"\<Cs\>\"", "|", "\"\<Cs133\>\""}], ",", "\"\<D2\>\""}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"\"\<Cs\>\"", ",", "133", ",", 
         RowBox[{"{", 
          RowBox[{"\"\<Xe\>\"", ",", 
           RowBox[{"{", 
            RowBox[{"6", ",", "\"\<s\>\""}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "\"\<S\>\"", ",", 
           FractionBox["1", "2"]}], "}"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\"\<Cs\>\"", ",", "133", ",", 
         RowBox[{"{", 
          RowBox[{"\"\<Xe\>\"", ",", 
           RowBox[{"{", 
            RowBox[{"6", ",", "\"\<p\>\""}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"2", ",", "\"\<P\>\"", ",", 
           FractionBox["3", "2"]}], "}"}]}], "}"}]}], "}"}], ",", 
     "\[IndentingNewLine]", "_", ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Print", "[", "\"\<Invalid atom or transition.\>\"", "]"}], ";", 
      RowBox[{"Abort", "[", "]"}], ";"}]}], "\[IndentingNewLine]", 
    "]"}]}]}]], "Input",
 InitializationCell->True],

Cell["Generate a list of the Zeeman sublevels of the system.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"makeAtomicSystem", "[", "]"}], ":=", 
  RowBox[{"atsys", "=", 
   RowBox[{"Sublevels", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"AtomicState", "[", 
       RowBox[{"1", ",", 
        RowBox[{"Join", "[", 
         RowBox[{
          RowBox[{"AtomicData", "[", 
           RowBox[{"gstate", ",", 
            RowBox[{"{", 
             RowBox[{
             "J", ",", "L", ",", "S", ",", "Energy", ",", "NaturalWidth", ",",
               "NuclearSpin"}], "}"}]}], "]"}], ",", 
          RowBox[{"{", "}"}]}], "]"}]}], "]"}], ",", 
      RowBox[{"AtomicState", "[", 
       RowBox[{"2", ",", 
        RowBox[{"Join", "[", 
         RowBox[{
          RowBox[{"AtomicData", "[", 
           RowBox[{"estate", ",", 
            RowBox[{"{", 
             RowBox[{"J", ",", "L", ",", "S", ",", "NuclearSpin"}], "}"}]}], 
           "]"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"BranchingRatio", "[", "1", "]"}], "->", "1"}], "}"}]}], 
         "]"}]}], "]"}]}], "}"}], "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Make a list of numerical parameters describing the system. All frequencies, \
widths, and rates are in omega units. \nRMSVelocity is rms atomic velocity in \
meter/sec\nLI is light intensity in ",
 Cell[BoxData[
  FormBox[
   RowBox[{"watt", "/", 
    SuperscriptBox["meter", "2"]}], TraditionalForm]]],
 "\nT is temperature in Kelvin\n",
 "LandeGF",
 " is g-factor of highest-angular-momentum ground state (first state in the \
atsys list)."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"makeAtomicData", "[", "]"}], ":=", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"atdata", "=", 
     RowBox[{
      RowBox[{"Flatten", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"AtomicData", "[", 
            RowBox[{"gstate", ",", 
             RowBox[{"{", "HyperfineA", "}"}], ",", 
             RowBox[{"StateLabel", "->", "1"}]}], "]"}], ",", 
           RowBox[{"AtomicData", "[", 
            RowBox[{"estate", ",", 
             RowBox[{"{", 
              RowBox[{
              "HyperfineA", ",", "HyperfineB", ",", "NaturalWidth", ",", 
               "Energy"}], "}"}], ",", 
             RowBox[{"StateLabel", "->", "2"}]}], "]"}]}], "}"}], ",", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"DopplerWidth", "->", 
            RowBox[{"2", "\[Pi]", " ", 
             RowBox[{"Convert", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"Wavenumber", " ", 
                 SqrtBox[
                  FractionBox[
                   RowBox[{
                   "2", " ", "BoltzmannConstant", " ", "Temperature", " ", 
                    "Kelvin"}], 
                   RowBox[{"AtomicWeight", " ", "AtomicMassUnit"}]]]}], "/.", 
                RowBox[{"AtomicData", "[", "estate", "]"}]}], ",", 
               RowBox[{"1", "/", "Second"}]}], "]"}]}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{"RMSVelocity", "->", 
            RowBox[{"Convert", "[", 
             RowBox[{
              RowBox[{
               SqrtBox[
                FractionBox[
                 RowBox[{
                 "BoltzmannConstant", " ", "Temperature", " ", "Kelvin"}], 
                 RowBox[{"AtomicWeight", " ", "AtomicMassUnit"}]]], "/.", 
               RowBox[{"AtomicData", "[", "estate", "]"}]}], ",", 
              RowBox[{"Meter", "/", "Second"}]}], "]"}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"RabiFrequencyA", "[", "b_", "]"}], "->", " ", 
            SuperscriptBox[
             RowBox[{"Convert", "[", 
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"ExpandDipoleRME", "[", 
                    RowBox[{"atsys", ",", 
                    RowBox[{"ReducedME", "[", 
                    RowBox[{"1", ",", 
                    RowBox[{"{", 
                    RowBox[{"Dipole", ",", "1"}], "}"}], ",", "2"}], "]"}]}], 
                    "]"}], "2"], 
                  SuperscriptBox["E0", "2"], 
                  FractionBox[
                   SuperscriptBox["SpeedOfLight", "3"], 
                   "PlanckConstantReduced"]}], "/.", 
                 RowBox[{
                  SuperscriptBox["E0", "2"], "->", 
                  FractionBox[
                   RowBox[{"8", "\[Pi]", " ", 
                    RowBox[{"LightIntensityA", "[", "b", "]"}]}], 
                   "SpeedOfLight"]}]}], " ", "/.", 
                RowBox[{"AtomicData", "[", 
                 RowBox[{"estate", ",", 
                  RowBox[{"StateLabel", "->", "2"}]}], "]"}]}], ",", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{"1", "/", "Second"}], ")"}], "2"], " ", 
                FractionBox[
                 SuperscriptBox["Meter", "2"], "Watt"]}]}], "]"}], 
             RowBox[{"1", "/", "2"}]]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"RabiFrequencyB", "[", "b_", "]"}], "->", " ", 
            SuperscriptBox[
             RowBox[{"Convert", "[", 
              RowBox[{
               RowBox[{
                RowBox[{
                 RowBox[{
                  SuperscriptBox[
                   RowBox[{"ExpandDipoleRME", "[", 
                    RowBox[{"atsys", ",", 
                    RowBox[{"ReducedME", "[", 
                    RowBox[{"1", ",", 
                    RowBox[{"{", 
                    RowBox[{"Dipole", ",", "1"}], "}"}], ",", "2"}], "]"}]}], 
                    "]"}], "2"], 
                  SuperscriptBox["E0", "2"], 
                  FractionBox[
                   SuperscriptBox["SpeedOfLight", "3"], 
                   "PlanckConstantReduced"]}], "/.", 
                 RowBox[{
                  SuperscriptBox["E0", "2"], "->", 
                  FractionBox[
                   RowBox[{"8", "\[Pi]", " ", 
                    RowBox[{"LightIntensityB", "[", "b", "]"}]}], 
                   "SpeedOfLight"]}]}], " ", "/.", 
                RowBox[{"AtomicData", "[", 
                 RowBox[{"estate", ",", 
                  RowBox[{"StateLabel", "->", "2"}]}], "]"}]}], ",", 
               RowBox[{
                SuperscriptBox[
                 RowBox[{"(", 
                  RowBox[{"1", "/", "Second"}], ")"}], "2"], " ", 
                FractionBox[
                 SuperscriptBox["Meter", "2"], "Watt"]}]}], "]"}], 
             RowBox[{"1", "/", "2"}]]}]}], "}"}], ",", "\[IndentingNewLine]", 
         RowBox[{"BohrMagneton", "->", 
          RowBox[{"Convert", "[", 
           RowBox[{
            FractionBox["ElectronCharge", 
             RowBox[{"2", " ", "ElectronMass"}]], ",", 
            RowBox[{
             RowBox[{"Mega", "/", "Second"}], "/", "Gauss"}]}], "]"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"LandeGF", "->", 
          RowBox[{"LandeGF", "[", 
           RowBox[{"atsys", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "]"}]}]}], "}"}], "]"}], "/.", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Mega", "->", 
         SuperscriptBox["10", "6"]}], ",", 
        RowBox[{"Second", "->", "1"}], ",", 
        RowBox[{"Meter", "->", "1"}], ",", 
        RowBox[{"Second", "->", "1"}], ",", 
        RowBox[{"Watt", "->", "1"}]}], "}"}]}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"AppendTo", "[", 
     RowBox[{"atdata", ",", 
      RowBox[{"ResonanceFrequencies", "->", 
       RowBox[{"Sort", "[", 
        RowBox[{
         RowBox[{"ResonanceFrequencies", "[", "atsys", "]"}], "/.", 
         "atdata"}], "]"}]}]}], "]"}]}], ")"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Define the environment that the atoms are subject to. A Region is a \
designator for a group of atoms all under the same conditions. In this case, \
the Regions are ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " velocity groups of the Doppler distribution, with Doppler shift \
DopplerShift[",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 "] and relative population ",
 "VGDensity[",
 StyleBox["i",
  FontSlant->"Italic"],
 "] . We could set up any number of velocity groups here, but that would \
involve a lot of redundant computation, since the equations for each group \
are similar. Instead, we will set up two velocity groups here, and use the \
resulting equations as building blocks to put together the evolution \
equations for the entire system."
}], "Text"],

Cell["\<\
\[Bullet] TransitRate[VoidRegion] is the rate at which atoms enter and leave \
the light beam.
\[Bullet] TransitRate[r1] is the rate at which atoms are transferred from the \
particular velocity group r to another velocity group r1. This depends on the \
overall collision rate VccRate, and the fraction of atoms VGDensity[r1] in \
the velocity group r1.
\[Bullet] Density is the atomic density for atoms in a given velocity group. \
AtomicDensity is the overall atomic density, and VGDensity[r] is the fraction \
in that group.
\[Bullet] MagneticField is the magnetic field vector in the Cartesian basis. \
It is specified in terms of the Larmor frequency \[CapitalOmega]L for the F=2 \
ground state, and its spherical angles \[Theta], \[Phi].
\[Bullet] Optical parameters are the parameters for the laser beam: \
{frequency, electric field, {polarization angle, ellipticity}}. 
\tThe propagation  and reference polarization directions can be specified \
through the options PolarizationVector and PropagationVector. 
\tIf they are not, the default values PolarizationVector -> {1, 0, 0}, \
PropagationVector -> {0, 0, 1} are used.
\tAn ellipticity angle value of zero corresponds to linear polarization, \
\[PlusMinus]\[Pi]/4 corresponds to circular polarization. A symbolic value \
can be used here and a numerical value substituted when a calculation is \
performed.\t\
\>", "Text"],

Cell["\<\
WHAT HAPPENS TO OPTICAL COHERENCES DURING VELOCITY-CHANGING COLLISIONS OR \
WHEN TRAVELING BETWEEN DITHERING REGIONS?\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"makeExperimentalSystem", "[", 
   RowBox[{"pulsedSetup_", ",", "flipDoppler_"}], "]"}], ":=", 
  RowBox[{"expsys", "=", 
   RowBox[{"Flatten", "@", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"Region", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"r", ",", "b"}], "}"}], ",", 
        RowBox[{"{", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"TransitRate", "[", "VoidRegion", "]"}], "->", 
           "BeamTransitRate"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"TransitRate", "[", 
            RowBox[{"{", 
             RowBox[{"rother_", ",", "bother_"}], "}"}], "]"}], "->", 
           RowBox[{"Which", "[", 
            RowBox[{
             RowBox[{"bother", "==", "b"}], ",", 
             RowBox[{"Evaluate", "[", 
              RowBox[{"VccRate", "[", 
               RowBox[{"r", ",", "rother"}], "]"}], "]"}], ",", 
             RowBox[{"Evaluate", "[", 
              RowBox[{"rother", "==", "r"}], "]"}], ",", 
             RowBox[{
              RowBox[{"DitherRegionFraction", "[", "bother", "]"}], 
              "DitherRate"}], ",", "True", ",", "0"}], "]"}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"Density", "->", 
           RowBox[{"AtomicDensity", " ", 
            RowBox[{"VGDensity", "[", "r", "]"}], " ", 
            RowBox[{"DitherRegionFraction", "[", "b", "]"}]}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"OpticalParameters", "->", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{
              RowBox[{"Energy", "[", "2", "]"}], "+", 
              RowBox[{"\[CapitalDelta]", "[", "b", "]"}], "-", 
              RowBox[{
               RowBox[{"If", "[", 
                RowBox[{"flipDoppler", ",", 
                 RowBox[{"-", "1"}], ",", "1"}], "]"}], 
               RowBox[{"DopplerShift", "[", "r", "]"}]}], "+", 
              RowBox[{"If", "[", 
               RowBox[{"pulsedSetup", ",", "ModeDetuning", ",", "0"}], 
               "]"}]}], ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{
                RowBox[{"\[CapitalOmega]R", "[", "b", "]"}], "/", 
                RowBox[{"ReducedME", "[", 
                 RowBox[{"1", ",", 
                  RowBox[{"{", 
                   RowBox[{"Dipole", ",", "1"}], "}"}], ",", "2"}], "]"}]}], 
               ",", 
               RowBox[{"LightPhase", "[", "b", "]"}]}], "}"}], ",", 
             RowBox[{"{", 
              RowBox[{"0", ",", "\[Epsilon]"}], "}"}]}], "}"}]}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"MagneticField", "->", 
           RowBox[{"LarmorFrequency", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{
                RowBox[{"Cos", "[", "MagneticAzimuth", "]"}], " ", 
                RowBox[{"Sin", "[", "MagneticZenith", "]"}]}], ",", 
               RowBox[{
                RowBox[{"Sin", "[", "MagneticAzimuth", "]"}], " ", 
                RowBox[{"Sin", "[", "MagneticZenith", "]"}]}], ",", 
               RowBox[{"Cos", "[", "MagneticZenith", "]"}]}], "}"}], "/", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"LandeGF", "/.", "atdata"}], ")"}], " ", 
               "BohrMagneton"}], ")"}]}]}]}]}], "\[IndentingNewLine]", 
         "}"}]}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"b", ",", "2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"r", ",", "2"}], "}"}]}], "]"}]}]}]}]], "Input",
 InitializationCell->True],

Cell["\<\
The Breit-Rabi Hamiltonian. 
DOES NOT USE EXPSYS, SO WILL NEED TO BE FIXED IF MAGNETIC FIELDS IN EXPSYS \
ARE CHANGED.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"BreitRabi", "[", 
    RowBox[{"s_AtomicState", ",", "B_"}], "]"}], "/;", 
   RowBox[{
    RowBox[{
     RowBox[{"J", "[", "s", "]"}], "===", 
     RowBox[{"1", "/", "2"}]}], "&&", 
    RowBox[{
     RowBox[{"NuclearSpin", "[", "s", "]"}], ">", "0"}]}]}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         FractionBox["\[CapitalDelta]", 
          RowBox[{"2", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"2", "N"}], "+", "1"}], ")"}]}]]}], "+", 
        RowBox[{"j", 
         FractionBox["\[CapitalDelta]", "2"], 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"1", "+", 
            FractionBox[
             RowBox[{"4", "m", " ", "\[Xi]"}], 
             RowBox[{
              RowBox[{"2", "N"}], "+", "1"}]], "+", 
            SuperscriptBox["\[Xi]", "2"]}], ")"}], 
          RowBox[{"1", "/", "2"}]]}]}], "/.", 
       RowBox[{"\[Xi]", "->", 
        RowBox[{
         RowBox[{"LandeGJ", "[", "s", "]"}], " ", "BohrMagneton", " ", 
         RowBox[{"B", "/", "\[CapitalDelta]"}]}]}]}], "/.", 
      RowBox[{"\[CapitalDelta]", "->", 
       RowBox[{"2", " ", 
        RowBox[{"HyperfineA", "[", "s", "]"}]}]}]}], "/.", 
     RowBox[{"N", "->", 
      RowBox[{"NuclearSpin", "[", "s", "]"}]}]}], "/.", 
    RowBox[{"m", "->", 
     RowBox[{"M", "[", "s", "]"}]}]}], "/.", 
   RowBox[{"j", "->", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"F", "[", "s", "]"}], "==", 
       RowBox[{
        RowBox[{"NuclearSpin", "[", "s", "]"}], "+", 
        RowBox[{"1", "/", "2"}]}]}], ",", "1", ",", 
      RowBox[{"-", "1"}]}], "]"}]}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"BRHamiltonian", "[", "]"}], ":=", 
  RowBox[{"WignerRotate", "[", 
   RowBox[{"atsys", ",", 
    RowBox[{"DiagonalMatrix", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"GroundStateQ", "[", "#", "]"}], ",", 
         RowBox[{"BreitRabi", "[", 
          RowBox[{"#", ",", 
           RowBox[{"LarmorFrequency", "/", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"LandeGF", "/.", "atdata"}], ")"}], " ", 
              "BohrMagneton"}], ")"}]}]}], "]"}], ",", "0"}], "]"}], "&"}], "/@",
       "atsys"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"MagneticAzimuth", ",", "MagneticZenith", ",", "0"}], "}"}], ",", 
    RowBox[{"StandardOperator", "->", "True"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
Generate the Hamiltonians for the atoms in the two velocity groups, subject \
to the magnetic field.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"makeHamiltonian", "[", 
    RowBox[{"neglectExcitedStateMixing_", ",", "useBreitRabi_"}], "]"}], ":=", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"H", "=", 
      RowBox[{"Hamiltonian", "[", 
       RowBox[{"atsys", ",", "expsys", ",", 
        RowBox[{"Interaction", "->", 
         RowBox[{"{", 
          RowBox[{"Internal", ",", "Magnetic"}], "}"}]}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"BRH", "=", 
      RowBox[{"If", "[", 
       RowBox[{"useBreitRabi", ",", 
        RowBox[{"BRHamiltonian", "[", "]"}], ",", "0"}], "]"}]}], ";", 
     RowBox[{"H1", "=", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"BRH", "+", 
          RowBox[{"#", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"DensityMatrix", "[", 
                 RowBox[{"atsys", ",", 
                  RowBox[{"ComplexExpandVariables", "->", "False"}]}], "]"}], 
                "/.", 
                RowBox[{"If", "[", 
                 RowBox[{"useBreitRabi", ",", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"DMElementPattern", "[", 
                    RowBox[{"ComplexExpandVariables", "->", "False"}], "]"}], 
                    "/;", 
                    RowBox[{"StateLabel1", "==", "StateLabel2", "==", "1"}]}],
                    "->", "0"}], ",", 
                  RowBox[{"{", "}"}]}], "]"}]}], "/.", 
               RowBox[{
                RowBox[{
                 RowBox[{"DMElementPattern", "[", 
                  RowBox[{"ComplexExpandVariables", "->", "False"}], "]"}], "/;", 
                 RowBox[{
                  RowBox[{"StateLabel1", "==", "StateLabel2", "==", "1"}], "&&", 
                  RowBox[{"F1", "!=", "F2"}]}]}], "->", "0"}]}], "/.", 
              RowBox[{"If", "[", 
               RowBox[{"neglectExcitedStateMixing", ",", 
                RowBox[{
                 RowBox[{
                  RowBox[{"DMElementPattern", "[", 
                   RowBox[{"ComplexExpandVariables", "->", "False"}], "]"}], "/;", 
                  RowBox[{
                   RowBox[{"StateLabel1", "==", "StateLabel2", "==", "2"}], "&&", 
                   RowBox[{"F1", "!=", "F2"}]}]}], "->", "0"}], ",", 
                RowBox[{"{", "}"}]}], "]"}]}], "/.", 
             RowBox[{
              RowBox[{"DMElementPattern", "[", 
               RowBox[{"ComplexExpandVariables", "->", "False"}], "]"}], "->",
               "1"}]}], ")"}]}]}], ")"}], "&"}], "/@", "H"}]}]}], ")"}]}], 
  ";"}]], "Input",
 InitializationCell->True],

Cell["Generate the Hamiltonians for the light.", "Text"],

Cell["\<\
Split up the light field Hamiltonian into two fields, one of which only \
interacts with the D2a (ground F=I+1/2) transition, and the other only \
interacts with the D2b (ground F=I-1/2) transition, in order to model the \
repump light.
THIS APPROXIMATION IS NOT VALID IF THE LIGHT FREQUENCIES STRAY TOO FAR FROM \
RESONANCE. \
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"makeLightHamiltonian", "[", "]"}], ":=", 
  RowBox[{"(", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"MatrixForm", "/@", 
     RowBox[{"(", 
      RowBox[{"Hlight", "=", 
       RowBox[{"Hamiltonian", "[", 
        RowBox[{"atsys", ",", "expsys", ",", 
         RowBox[{"Interaction", "->", 
          RowBox[{"{", "E1", "}"}]}]}], "]"}]}], ")"}]}], ";", 
    RowBox[{"MatrixForm", "/@", 
     RowBox[{"(", 
      RowBox[{"Hlighta", "=", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"DirectSelfProduct", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"StateLabel", "[", "#", "]"}], "==", "1"}], "&&", 
                 RowBox[{
                  RowBox[{"F", "[", "#", "]"}], "==", 
                  RowBox[{
                   RowBox[{"NuclearSpin", "[", "#", "]"}], "-", 
                   RowBox[{"J", "[", "#", "]"}]}]}]}], ",", "0", ",", "1"}], 
               "]"}], "&"}], "/@", "atsys"}], "]"}], "#"}], "&"}], "/@", 
         "Hlight"}], "/.", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Epsilon]", "->", "EllipticityA"}], ",", 
          RowBox[{"\[CapitalDelta]", "->", "DetuningA"}], ",", 
          RowBox[{"\[CapitalOmega]R", "->", "RabiFrequencyA"}]}], "}"}]}]}], 
      ")"}]}], ";", 
    RowBox[{"MatrixForm", "/@", 
     RowBox[{"(", 
      RowBox[{"Hlightb", "=", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"DirectSelfProduct", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"StateLabel", "[", "#", "]"}], "==", "1"}], "&&", 
                 RowBox[{
                  RowBox[{"F", "[", "#", "]"}], "==", 
                  RowBox[{
                   RowBox[{"NuclearSpin", "[", "#", "]"}], "+", 
                   RowBox[{"J", "[", "#", "]"}]}]}]}], ",", "0", ",", "1"}], 
               "]"}], "&"}], "/@", "atsys"}], "]"}], "#"}], "&"}], "/@", 
         "Hlight"}], "/.", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"\[Epsilon]", "->", "EllipticityB"}], ",", 
          RowBox[{"\[CapitalDelta]", "->", "DetuningB"}], ",", 
          RowBox[{"\[CapitalOmega]R", "->", "RabiFrequencyB"}]}], "}"}]}]}], 
      ")"}]}], ";"}], "\[IndentingNewLine]", ")"}]}]], "Input",
 InitializationCell->True],

Cell["Perform the rotating wave approximation on the Hamiltonians.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"doRWA", "[", "flipDoppler_", "]"}], ":=", 
   RowBox[{"(", 
    RowBox[{"Hrwa", "=", 
     RowBox[{"Table", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"vgn", ",", "beamn"}], "}"}], "=", 
         RowBox[{"RegionLabel", "[", 
          RowBox[{"expsys", "[", 
           RowBox[{"[", "i", "]"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"sign", "=", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"OrderedQ", "[", 
            RowBox[{"{", 
             RowBox[{"\"\<09.12.08\>\"", ",", "$ADMVersion"}], "}"}], "]"}], 
           ",", 
           RowBox[{"-", "1"}], ",", "1"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"tm", "=", 
         RowBox[{"MatrixExp", "[", 
          RowBox[{"sign", " ", "\[ImaginaryI]", " ", "t", " ", 
           RowBox[{"DiagonalMatrix", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"Which", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"StateLabel", "[", "#", "]"}], "==", "1"}], "&&", 
                 RowBox[{
                  RowBox[{"F", "[", "#", "]"}], "==", 
                  RowBox[{
                   RowBox[{"NuclearSpin", "[", "#", "]"}], "+", 
                   RowBox[{"J", "[", "#", "]"}]}]}]}], ",", 
                RowBox[{"-", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"DetuningA", "[", "beamn", "]"}], "+", 
                   RowBox[{"Energy", "[", "2", "]"}], "-", 
                   RowBox[{
                    RowBox[{"If", "[", 
                    RowBox[{"flipDoppler", ",", 
                    RowBox[{"-", "1"}], ",", "1"}], "]"}], 
                    RowBox[{"DopplerShift", "[", "vgn", "]"}]}]}], ")"}]}], 
                ",", 
                RowBox[{
                 RowBox[{
                  RowBox[{"StateLabel", "[", "#", "]"}], "==", "1"}], "&&", 
                 RowBox[{
                  RowBox[{"F", "[", "#", "]"}], "==", 
                  RowBox[{
                   RowBox[{"NuclearSpin", "[", "#", "]"}], "-", 
                   RowBox[{"J", "[", "#", "]"}]}]}]}], ",", 
                RowBox[{"-", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"DetuningB", "[", "beamn", "]"}], "+", 
                   RowBox[{"Energy", "[", "2", "]"}], "-", 
                   RowBox[{
                    RowBox[{"If", "[", 
                    RowBox[{"flipDoppler", ",", 
                    RowBox[{"-", "1"}], ",", "1"}], "]"}], 
                    RowBox[{"DopplerShift", "[", "vgn", "]"}]}]}], ")"}]}], 
                ",", "True", ",", "0"}], "]"}], "&"}], "/@", "atsys"}], 
            "]"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"RotatingWaveApproximation", "[", 
         RowBox[{"atsys", ",", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"H1", "+", "Hlighta", "+", "Hlightb"}], ")"}], "[", 
           RowBox[{"[", "i", "]"}], "]"}], ",", 
          RowBox[{"Energy", "[", "2", "]"}], ",", 
          RowBox[{"TransformMatrix", "->", "tm"}]}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "4"}], "}"}]}], "]"}]}], ")"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell["\<\
Generate the matrices describing relaxation due to spontaneous decay, and \
effective relaxation due to atom transit of the light beam and velocity \
changing collisions, which cause atoms to leave one velocity group and be \
transferred to another.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"makeRelaxationMatrix", "[", "]"}], ":=", 
  RowBox[{"MatrixForm", "/@", 
   RowBox[{"(", 
    RowBox[{"rel", "=", 
     RowBox[{"Relaxation", "[", 
      RowBox[{"atsys", ",", "expsys"}], "]"}]}], ")"}]}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Generate the matrix describing S-damping of the ground state: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox["J", "2"], "\[Rho]"}], "-", 
    RowBox[{"J", "\[CenterDot]", 
     RowBox[{"(", 
      RowBox[{"\[Rho]", " ", "J"}], ")"}]}]}], TraditionalForm]]],
 ". \nWignerEckart[atsys, {J, 1}] generates the matrix representation of the \
spherical components of the J operator for the specified atomic system."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"makeSDampingMatrix", "[", "]"}], ":=", 
  RowBox[{"(", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"gs", "=", 
     RowBox[{"WignerEckart", "[", 
      RowBox[{"atsys", ",", 
       RowBox[{"{", 
        RowBox[{"GroundState", ",", "0", ",", "0"}], "}"}]}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"jcart", "=", 
     RowBox[{"ToCartesian", "@", 
      RowBox[{"WignerEckart", "[", 
       RowBox[{"atsys", ",", 
        RowBox[{"{", 
         RowBox[{"J", ",", "1"}], "}"}]}], "]"}]}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"dm", "=", 
     RowBox[{"DensityMatrix", "[", 
      RowBox[{"atsys", ",", 
       RowBox[{"DMLabel", "->", "temp"}]}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"jsqr", "=", 
     RowBox[{"Simplify", "[", 
      RowBox[{"jcart", "\[CenterDot]", "jcart"}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"sd", "=", 
     RowBox[{
      RowBox[{"SDampingRate", " ", 
       RowBox[{"gs", ".", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"jsqr", ".", "dm"}], "-", 
          RowBox[{"jcart", "\[CenterDot]", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"(", 
               RowBox[{"dm", ".", "#"}], ")"}], "&"}], "/@", "jcart"}], 
            ")"}]}]}], ")"}], ".", "gs"}]}], "//", "Expand"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"sdamp", "=", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{"sd", "/.", 
         RowBox[{"temp", "->", 
          RowBox[{"RegionLabel", "[", "#", "]"}]}]}], ")"}], "&"}], "/@", 
      "expsys"}]}]}], "\[IndentingNewLine]", ")"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
Generate the matrices describing the repopulation of the ground state due to \
spontaneous decay, and the appearance of atoms in each velocity group due to \
atoms entering the light beam and changing their velocity from another \
velocity group.\
\>", "Text"],

Cell["\<\
SHOULD DECIDE WHAT HAPPENS TO EXCITED-STATE ATOMS AND OPTICAL COHERENCES \
DURING VELOCITY-CHANGING COLLISIONS.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"makeRepopulationMatrix", "[", "]"}], ":=", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"rep", "=", 
     RowBox[{
      RowBox[{"ExpandAll", "[", 
       RowBox[{"Repopulation", "[", 
        RowBox[{"atsys", ",", "expsys"}], "]"}], "]"}], "/.", 
      RowBox[{
       RowBox[{
        RowBox[{"x", ":", 
         RowBox[{"(", 
          RowBox[{"DitherRate", " ", 
           RowBox[{"DMElementPattern", "[", "]"}]}], ")"}]}], "/;", 
        RowBox[{"StateLabel1", "!=", "StateLabel2"}]}], "->", 
       RowBox[{"DitherCoherenceTransfer", " ", "x"}]}]}]}], ";", 
    RowBox[{"recoil", "=", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"rep", "/.", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"BeamTransitRate", "->", "0"}], ",", 
           RowBox[{
            RowBox[{"VccRate", "[", "__", "]"}], "->", "0"}], ",", 
           RowBox[{"DitherRate", "->", "0"}]}], "}"}]}], "/.", 
        RowBox[{
         SubscriptBox["\[Rho]", 
          RowBox[{"re_", ",", 
           RowBox[{"{", 
            RowBox[{"1", ",", "b_"}], "}"}], ",", "s1_", ",", "s2_"}]], "->", 
         "0"}]}], "/.", 
       RowBox[{
        SubscriptBox["\[Rho]", 
         RowBox[{"re_", ",", 
          RowBox[{"{", 
           RowBox[{"v_", ",", "b_"}], "}"}], ",", "s1_", ",", "s2_"}]], "->", 
        SubscriptBox["\[Rho]", 
         RowBox[{"re", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"v", "-", "1"}], ",", "b"}], "}"}], ",", "s1", ",", 
          "s2"}]]}]}], "/.", 
      RowBox[{
       RowBox[{"NaturalWidth", "[", "2", "]"}], "->", 
       RowBox[{
        FractionBox["RecoilShift", "dv"], 
        RowBox[{"NaturalWidth", "[", "2", "]"}]}]}]}]}], ";", 
    RowBox[{"rep1", "=", 
     RowBox[{
      RowBox[{"MapThread", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"#1", "/.", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"RegionLabel", "[", "#2", "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], "==", "1"}], ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"NaturalWidth", "[", "2", "]"}], "->", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"1", "-", 
                 FractionBox["RecoilShift", "dv"]}], ")"}], 
               RowBox[{"NaturalWidth", "[", "2", "]"}]}]}], "}"}], ",", 
            RowBox[{"{", "}"}]}], "]"}]}], "&"}], ",", 
        RowBox[{"{", 
         RowBox[{"rep", ",", "expsys"}], "}"}]}], "]"}], "-", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"LaserWidth", " ", 
         RowBox[{
          RowBox[{"DensityMatrix", "[", 
           RowBox[{"atsys", ",", "expsys"}], "]"}], "/", "2"}]}], "/.", 
        RowBox[{
         RowBox[{
          RowBox[{"DMElementPattern", "[", "]"}], "/;", 
          RowBox[{"StateLabel1", "==", "StateLabel2"}]}], "->", "0"}]}], 
       ")"}]}]}]}], "\[IndentingNewLine]", ")"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
Use the Hamiltonian, relaxation, and repopulation matrices in the Liouville \
equation to produce the Bloch equations.\
\>", "Text"],

Cell["\<\
Here we set ground-state and excited-state hyperfine coherences to zero. \
SHOULD QUANTIFY WHEN THIS APPROXIMATION IS VALID.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"deleteHyperfineCoherences", "[", "]"}], ":=", 
  RowBox[{"delpos", "=", 
   RowBox[{"Position", "[", 
    RowBox[{
     RowBox[{"DMVariables", "[", 
      RowBox[{"atsys", ",", "expsys"}], "]"}], ",", 
     RowBox[{
      RowBox[{"DMElementPattern", "[", "]"}], "/;", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"StateLabel1", "==", "StateLabel2"}], "&&", 
         RowBox[{"F1", "!=", "F2"}]}], ")"}], "||", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"StateLabel1", "!=", "StateLabel2"}], "&&", 
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"F1", "-", "F2"}], "]"}], ">", "1"}]}], ")"}]}]}], ",", 
     "1"}], "]"}]}]}]], "Input",
 InitializationCell->True],

Cell["\<\
A list of the variables describing the density matrix elements.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"makeVariableList", "[", "]"}], ":=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"delvars", "=", 
     RowBox[{"Extract", "[", 
      RowBox[{
       RowBox[{"DMVariables", "[", 
        RowBox[{"atsys", ",", "expsys"}], "]"}], ",", "delpos"}], "]"}]}], 
    ",", 
    RowBox[{"vars", "=", 
     RowBox[{"Delete", "[", 
      RowBox[{
       RowBox[{"DMVariables", "[", 
        RowBox[{"atsys", ",", "expsys"}], "]"}], ",", "delpos"}], "]"}]}]}], 
   "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"makeBlochEquations", "[", "]"}], ":=", 
  RowBox[{"BlochEqs", "=", 
   RowBox[{
    RowBox[{"Delete", "[", 
     RowBox[{
      RowBox[{"LiouvilleEquation", "[", 
       RowBox[{"atsys", ",", "expsys", ",", "Hrwa", ",", "rel", ",", 
        RowBox[{"rep1", "+", "recoil", "-", "sdamp"}]}], "]"}], ",", 
      "delpos"}], "]"}], "/.", 
    RowBox[{"Dispatch", "[", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"#", "->", "0"}], ")"}], "&"}], "/@", "delvars"}], ")"}], 
     "]"}]}]}]}]], "Input",
 InitializationCell->True],

Cell["\<\
Produce the equivalent matrix describing the Bloch equations. I refer to this \
as the \"Bloch matrix\". This can also be thought of as the matrix for the \
evolution operator in the \"Liouville\" vector representation of the density \
matrix.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"makeBlochMatrix", "[", "]"}], ":=", 
  RowBox[{"BlochMat", "=", 
   RowBox[{"Chop", "@", 
    RowBox[{"Reverse", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "1"}], ",", "1"}], "}"}], 
      RowBox[{"CoefficientArrays", "[", 
       RowBox[{
        RowBox[{"BlochEqs", "[", 
         RowBox[{"[", 
          RowBox[{"All", ",", "2"}], "]"}], "]"}], ",", "vars"}], "]"}]}], 
     "]"}]}]}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Produce the operator describing fluorescence into the solid angle at ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Theta]", "=", "0"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Phi]", "=", "0"}], TraditionalForm]]],
 ". The Fluorescence operator is in units of power/sr, so we divide by photon \
energy to get units of photons/s/sr."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"makeFluorescenceOperator", "[", "]"}], ":=", 
  RowBox[{"flop", "=", 
   RowBox[{
    RowBox[{"Simplify", "@", 
     RowBox[{"FluorescenceOperator", "[", 
      RowBox[{"atsys", ",", 
       RowBox[{"Energy", "[", "2", "]"}], ",", "\[Pi]", ",", "0"}], "]"}]}], 
    "/", 
    RowBox[{"Energy", "[", "2", "]"}]}]}]}]], "Input",
 InitializationCell->True],

Cell["\<\
Find the expectation value of the fluorescence operator to get an expression \
for the fluorescence in terms of the density matrix elements.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"findFluorescence", "[", "]"}], ":=", 
  RowBox[{"fluor", "=", 
   RowBox[{"Total", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Tr", "[", 
       RowBox[{"flop", ".", "#"}], "]"}], "&"}], "/@", 
     RowBox[{"DensityMatrix", "[", 
      RowBox[{"atsys", ",", "expsys"}], "]"}]}], "]"}]}]}]], "Input",
 InitializationCell->True],

Cell["\<\
Convert the expression for fluorescence into a vector suitable for dotting \
with the density matrix in vector form (aka in \"Liouville space\").\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"makeFluorescenceVector", "[", "]"}], ":=", 
  RowBox[{"fluorvec", "=", 
   RowBox[{"Normal", "@", 
    RowBox[{
     RowBox[{"CoefficientArrays", "[", 
      RowBox[{"fluor", ",", "vars"}], "]"}], "[", 
     RowBox[{"[", "2", "]"}], "]"}]}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"makeTotalFluorescenceOperator", "[", "]"}], ":=", 
  RowBox[{"totflop", "=", 
   RowBox[{
    RowBox[{"Simplify", "@", 
     RowBox[{"FluorescenceOperator", "[", 
      RowBox[{"atsys", ",", 
       RowBox[{"Energy", "[", "2", "]"}]}], "]"}]}], "/", 
    RowBox[{"Energy", "[", "2", "]"}]}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"findTotalFluorescence", "[", "]"}], ":=", 
  RowBox[{"totfluor", "=", 
   RowBox[{"Total", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Tr", "[", 
       RowBox[{"totflop", ".", "#"}], "]"}], "&"}], "/@", 
     RowBox[{"DensityMatrix", "[", 
      RowBox[{"atsys", ",", "expsys"}], "]"}]}], "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"makeTotalFluorescenceVector", "[", "]"}], ":=", 
  RowBox[{"totfluorvec", "=", 
   RowBox[{"Normal", "@", 
    RowBox[{
     RowBox[{"CoefficientArrays", "[", 
      RowBox[{"totfluor", ",", "vars"}], "]"}], "[", 
     RowBox[{"[", "2", "]"}], "]"}]}]}]}]], "Input",
 InitializationCell->True],

Cell["\<\
We now chop up the Bloch matrix into submatrices for each velocity group and \
subbeam (or, for the off diagonal blocks, the interaction between two \
velocity groups or beams.) We will then put these blocks together to create a \
matrix describing as many velocity groups and beams as required.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"makeBlochBlocks", "[", "]"}], ":=", 
  RowBox[{"(", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"nvars", "=", 
     RowBox[{
      RowBox[{"Length", "[", "vars", "]"}], "/", 
      RowBox[{"Length", "[", "expsys", "]"}]}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"vars1", "  ", "=", " ", 
     RowBox[{"Take", "[", 
      RowBox[{"vars", ",", "nvars"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"A11", "=", 
     RowBox[{"SparseReplace", "[", 
      RowBox[{
       RowBox[{"BlochMat", "[", 
        RowBox[{"[", 
         RowBox[{"1", ",", 
          RowBox[{";;", "nvars"}], ",", 
          RowBox[{";;", "nvars"}]}], "]"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"VGDensity", "[", "2", "]"}], "->", 
          RowBox[{"(", 
           RowBox[{"1", "-", 
            RowBox[{"VGDensity", "[", "1", "]"}]}], ")"}]}], ",", 
         RowBox[{
          RowBox[{"DitherRegionFraction", "[", "2", "]"}], "->", 
          RowBox[{"1", "-", 
           RowBox[{"DitherRegionFraction", "[", "1", "]"}]}]}]}], "}"}]}], 
      "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"A12vg", "=", 
     RowBox[{"BlochMat", "[", 
      RowBox[{"[", 
       RowBox[{"1", ",", 
        RowBox[{";;", "nvars"}], ",", 
        RowBox[{
         RowBox[{"nvars", "+", "1"}], ";;", 
         RowBox[{"2", "nvars"}]}]}], "]"}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"A21vg", "=", 
     RowBox[{"BlochMat", "[", 
      RowBox[{"[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"nvars", "+", "1"}], ";;", 
         RowBox[{"2", "nvars"}]}], ",", 
        RowBox[{";;", "nvars"}]}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"A12beam", "=", 
     RowBox[{"BlochMat", "[", 
      RowBox[{"[", 
       RowBox[{"1", ",", 
        RowBox[{";;", "nvars"}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"2", "nvars"}], "+", "1"}], ";;", 
         RowBox[{"3", "nvars"}]}]}], "]"}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"A21beam", "=", 
     RowBox[{"BlochMat", "[", 
      RowBox[{"[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"2", "nvars"}], "+", "1"}], ";;", 
         RowBox[{"3", "nvars"}]}], ",", 
        RowBox[{";;", "nvars"}]}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"b1", "=", 
     RowBox[{
      RowBox[{"BlochMat", "[", 
       RowBox[{"[", 
        RowBox[{"2", ",", 
         RowBox[{"1", ";;", "nvars"}]}], "]"}], "]"}], "//", "Normal"}]}], 
    ";", "\[IndentingNewLine]", 
    RowBox[{"fluorvec1", "=", 
     RowBox[{
      RowBox[{"fluorvec", "[", 
       RowBox[{"[", 
        RowBox[{";;", "nvars"}], "]"}], "]"}], "//", "Normal"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"totfluorvec1", "=", 
     RowBox[{
      RowBox[{"totfluorvec", "[", 
       RowBox[{"[", 
        RowBox[{";;", "nvars"}], "]"}], "]"}], "//", "Normal"}]}], ";"}], 
   "\[IndentingNewLine]", ")"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "The only difference between the blocks for each velocity group is the \
values of the Doppler shift DopplerShift[",
 StyleBox["i",
  FontSlant->"Italic"],
 "] and the relative density ",
 "VGDensity[",
 StyleBox["i",
  FontSlant->"Italic"],
 "]. We create separate matrices describing the dependence on each of these \
parameters, allowing more efficient manipulation of the submatrices."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"separateBlochBlocks", "[", "]"}], ":=", 
  RowBox[{"(", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"A110", "=", 
     RowBox[{"SparseReplace", "[", 
      RowBox[{"A11", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"DopplerShift", "[", "1", "]"}], "->", "0"}], ",", 
         RowBox[{
          RowBox[{"VccRate", "[", 
           RowBox[{"1", ",", "2"}], "]"}], "->", "0"}], ",", 
         RowBox[{"RecoilShift", "->", "0"}]}], "}"}]}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"A11S", "=", 
     RowBox[{
      RowBox[{"SparseArray", "@@", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Expand", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"ArrayRules", "[", 
             RowBox[{"A11", "-", 
              RowBox[{"SparseReplace", "[", 
               RowBox[{"A11", ",", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"DopplerShift", "[", "1", "]"}], "->", "0"}], 
                 "}"}]}], "]"}]}], "]"}], ",", 
            RowBox[{"Dimensions", "[", "A11", "]"}]}], "}"}], "]"}], "/.", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"x_Integer", ",", "y_Integer"}], "}"}], "->", "0"}], 
           ")"}], "->", 
          RowBox[{"Sequence", "[", "]"}]}]}], ")"}]}], "/", 
      RowBox[{"DopplerShift", "[", "1", "]"}]}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"A11G", "=", 
     RowBox[{
      RowBox[{"SparseArray", "@@", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Expand", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"ArrayRules", "[", 
             RowBox[{"A11", "-", 
              RowBox[{"SparseReplace", "[", 
               RowBox[{"A11", ",", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"VccRate", "[", 
                   RowBox[{"1", ",", "2"}], "]"}], "->", "0"}], "}"}]}], 
               "]"}]}], "]"}], ",", 
            RowBox[{"Dimensions", "[", "A11", "]"}]}], "}"}], "]"}], "/.", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"x_Integer", ",", "y_Integer"}], "}"}], "->", "0"}], 
           ")"}], "->", 
          RowBox[{"Sequence", "[", "]"}]}]}], ")"}]}], "/", 
      RowBox[{"VccRate", "[", 
       RowBox[{"1", ",", "2"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
    RowBox[{"A11R", "=", 
     RowBox[{
      RowBox[{"SparseArray", "@@", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Expand", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"ArrayRules", "[", 
             RowBox[{"A11", "-", 
              RowBox[{"SparseReplace", "[", 
               RowBox[{"A11", ",", 
                RowBox[{"{", 
                 RowBox[{"RecoilShift", "->", "0"}], "}"}]}], "]"}]}], "]"}], 
            ",", 
            RowBox[{"Dimensions", "[", "A11", "]"}]}], "}"}], "]"}], "/.", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"x_Integer", ",", "y_Integer"}], "}"}], "->", "0"}], 
           ")"}], "->", 
          RowBox[{"Sequence", "[", "]"}]}]}], ")"}]}], "dv"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"A12G", "=", 
     RowBox[{"A12vg", "/", 
      RowBox[{"(", 
       FractionBox[
        RowBox[{
         RowBox[{"VccRate", "[", 
          RowBox[{"1", ",", "2"}], "]"}], " ", 
         RowBox[{"VGDensity", "[", "1", "]"}]}], 
        RowBox[{"VGDensity", "[", "2", "]"}]], ")"}]}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"A21G", "=", 
     RowBox[{
      RowBox[{"SparseReplace", "[", 
       RowBox[{"A21vg", ",", 
        RowBox[{"RecoilShift", "->", "0"}]}], "]"}], "/", 
      RowBox[{"(", 
       FractionBox[
        RowBox[{
         RowBox[{"VccRate", "[", 
          RowBox[{"2", ",", "1"}], "]"}], " ", 
         RowBox[{"VGDensity", "[", "2", "]"}]}], 
        RowBox[{"VGDensity", "[", "1", "]"}]], ")"}]}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"A21R", "=", 
     RowBox[{
      RowBox[{"SparseReplace", "[", 
       RowBox[{"A21vg", ",", 
        RowBox[{
         RowBox[{"VccRate", "[", 
          RowBox[{"2", ",", "1"}], "]"}], "->", "0"}]}], "]"}], "dv"}]}], ";",
     "\[IndentingNewLine]", 
    RowBox[{"A12B", "=", 
     RowBox[{"A12beam", "/", 
      RowBox[{"DitherRegionFraction", "[", "1", "]"}]}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"A21B", "=", 
     RowBox[{"A21beam", "/", 
      RowBox[{"DitherRegionFraction", "[", "2", "]"}]}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"b1G", "=", 
     RowBox[{"b1", "/", 
      RowBox[{"VGDensity", "[", "1", "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   ")"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"Options", "[", "GenerateLGSSystem", "]"}], "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"NeglectHyperfineCoherences", "->", "True"}], ",", 
    RowBox[{"PrintDiagnostics", "->", "False"}], ",", 
    RowBox[{"NeglectExcitedStateMixing", "->", "True"}], ",", 
    RowBox[{"PulsedSetup", "->", "False"}], ",", 
    RowBox[{"FlipDoppler", "->", "False"}]}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"NeglectHyperfineCoherences", "->", "True"}], ",", 
   RowBox[{"PrintDiagnostics", "->", "False"}], ",", 
   RowBox[{"NeglectExcitedStateMixing", "->", "True"}]}], "}"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"GenerateLGSSystem", "[", 
   RowBox[{
    RowBox[{"atom_String:", "\"\<Na\>\""}], ",", 
    RowBox[{"line_String:", "\"\<D2\>\""}], ",", 
    RowBox[{"OptionsPattern", "[", "]"}]}], "]"}], ":=", 
  RowBox[{"(", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"diag", "=", 
     RowBox[{"OptionValue", "[", "PrintDiagnostics", "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"setStates", "[", 
     RowBox[{"atom", ",", "line"}], "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{"makeAtomicSystem", "[", "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{"diag", ",", 
      RowBox[{"Print", "[", "atsys", "]"}]}], "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"makeAtomicData", "[", "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{"diag", ",", 
      RowBox[{"Print", "[", "atdata", "]"}]}], "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"makeExperimentalSystem", "[", 
     RowBox[{
      RowBox[{"OptionValue", "[", "PulsedSetup", "]"}], ",", 
      RowBox[{"OptionValue", "[", "FlipDoppler", "]"}]}], "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"makeHamiltonian", "[", 
     RowBox[{
      RowBox[{"OptionValue", "[", "NeglectExcitedStateMixing", "]"}], ",", 
      "False"}], "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{"diag", ",", 
      RowBox[{"Print", "[", 
       RowBox[{"MatrixForm", "/@", "H1"}], "]"}]}], "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"makeLightHamiltonian", "[", "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{"doRWA", "[", 
     RowBox[{"OptionValue", "[", "FlipDoppler", "]"}], "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{"diag", ",", 
      RowBox[{"Print", "[", 
       RowBox[{"MatrixForm", "/@", "Hrwa"}], "]"}]}], "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"makeRelaxationMatrix", "[", "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{"makeSDampingMatrix", "[", "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{"makeRepopulationMatrix", "[", "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"OptionValue", "[", "NeglectHyperfineCoherences", "]"}], ",", 
      RowBox[{"deleteHyperfineCoherences", "[", "]"}], ",", 
      RowBox[{"delpos", "=", 
       RowBox[{"{", "}"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{"makeVariableList", "[", "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{"makeBlochEquations", "[", "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{"makeBlochMatrix", "[", "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{"diag", ",", 
      RowBox[{"Print", "@", 
       RowBox[{"MatrixPlot", "[", 
        RowBox[{
         RowBox[{"BlochMat", "[", 
          RowBox[{"[", "1", "]"}], "]"}], ",", 
         RowBox[{"Mesh", "->", 
          RowBox[{"{", 
           RowBox[{"3", ",", "3"}], "}"}]}], ",", 
         RowBox[{"MaxPlotPoints", "->", "2400"}]}], "]"}]}]}], "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"makeFluorescenceOperator", "[", "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"findFluorescence", "[", "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{"makeFluorescenceVector", "[", "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{"makeTotalFluorescenceOperator", "[", "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"findTotalFluorescence", "[", "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{"makeTotalFluorescenceVector", "[", "]"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{"makeBlochBlocks", "[", "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{"separateBlochBlocks", "[", "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{
     "nvars", ",", "A110", ",", "A11S", ",", "A11G", ",", "A11R", ",", "A12G",
       ",", "A21R", ",", "A12B", ",", "b1G", ",", "fluorvec1", ",", 
      RowBox[{"Sequence", "@@", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"OptionValue", "[", "PulsedSetup", "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"totfluorvec1", ",", "vars"}], "}"}], ",", 
         RowBox[{"{", "}"}]}], "]"}]}], ",", "atdata"}], "}"}]}], 
   "\[IndentingNewLine]", ")"}]}]], "Input",
 InitializationCell->True],

Cell[" ", "Text",
 Editable->False,
 Selectable->False,
 CellFrame->{{0, 0}, {0, 2}},
 ShowCellBracket->False,
 CellMargins->{{0, 0}, {1, 1}},
 CellElementSpacings->{"CellMinHeight"->1},
 CellFrameMargins->0,
 CellFrameColor->RGBColor[0, 0, 1],
 CellSize->{Inherited, 4}]
}, Open  ]],

Cell[CellGroupData[{

Cell["phiv", "Subsection"],

Cell["\<\
 [w, err] = phiv( t, A, u, v, tol, m )
 phiv computes an approximation of w = exp(t*A)*v + t*phi(t*A)*u
 for a general matrix A using Krylov subspace projection techniques.
 Here, phi(z) = (exp(z)-1)/z and w is the solution of the 
 nonhomogeneous linear ODE problem w' = Aw + u, w(0) = v.
 It does not compute the matrix functions in isolation but instead,
 it computes directly the action of these functions on the 
 operand vectors. This way of doing so allows for addressing large
 sparse problems. The matrix under consideration interacts only
 via matrix-vector products (matrix-free method).
 
 w = phiv( t, A, u, v ) 
 computes w = exp(t*A)*v + t*phi(t*A)*u using a default tol = 1.0e-7 
 and m = 30.
 
 [w, err] = phiv( t, A, u, v )
 renders an estimate of the error on the approximation.
 
 [w, err] = phiv( t, A, u, v, tol )
 overrides default tolerance.
 
 [w, err] = phiv( t, A, u, v, tol, m )
 overrides default tolerance and dimension of the Krylov subspace.
 
 Example 1:
 ----------
   n = 100;
   A = rand(n);
   u = zeros(n,1);
   v = eye(n,1);
   w = phiv(1,A,u,v);
 
 
 Example 2:
 ----------
   % generate a random sparse matrix
   n = 100;
   A = rand(n);
   for j = 1:n
       for i = 1:n
           if rand < 0.5, A(i,j) = 0; end;
       end;
   end;
   u = rand(n,1);
   v = eye(n,1);
   A = sparse(A); % invaluable for a large and sparse matrix.
 
   tic
   [w,err] = phiv(1,A,u,v);
   toc
 
   disp('w(1:10) ='); disp(w(1:10));
   disp('err =');     disp(err);
 
   tic
   x = A\\u;
   w_matlab = expm(full(A))*(v+x)-x;
   toc
 
   disp('w_matlab(1:10) ='); disp(w_matlab(1:10));
   gap = norm(w-w_matlab)/norm(w_matlab);
   disp('||w-w_matlab|| / ||w_matlab|| ='); disp(gap);
 
 In the above example, n could have been set to a larger value,
 but the computation of w_matlab will be too long (feel free to
 discard this computation).
 
 See also EXPV, MEXPV, EXPOKIT.
 
 Roger B. Sidje (rbs@maths.uq.edu.au)
 EXPOKIT: Software Package for Computing Matrix Exponentials. 
 ACM - Transactions On Mathematical Software, 24(1):130-156, 1998

Translated to Mathematica by Ronald Holzloehner, 20 March 2009
Function compared on 23Mar09 with MATLAB routine for 5x5 random complex A, u \
and v: results agree.

Further mucked with by S. Rochester, 12 August 2009\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "phiv", "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"VerbosityLevel", "->", "2"}], "}"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"phiv", "[", 
    RowBox[{"t_", ",", "A_", ",", "u_", ",", "v_", ",", 
     RowBox[{"obsmat_:", 
      RowBox[{"{", "}"}]}], ",", 
     RowBox[{"tspanMax_:", "Infinity"}], ",", 
     RowBox[{"tol_:", 
      RowBox[{"10.0", "^", 
       RowBox[{"(", 
        RowBox[{"-", "7"}], ")"}]}]}], ",", 
     RowBox[{"mIn_:", "-", "1"}], ",", 
     RowBox[{"indVgMaxEx_:", "-", "1"}], ",", 
     RowBox[{"OptionsPattern", "[", "]"}]}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "n", ",", "anorm", ",", "mxrej", ",", "btol", ",", "gamma", ",", 
       "delta", ",", "mb", ",", "tout", ",", "istep", ",", "tnew", ",", 
       "tnow", ",", "serror", ",", "rndoff", ",", "k1", ",", "xm", ",", 
       "beta", ",", "fact", ",", "s", ",", "sgn", ",", "w", ",", "tstep", ",",
        "Vt", ",", "H", ",", "p", ",", "i", ",", "j", ",", "avnorm", ",", 
       "ireject", ",", "mx", ",", "errloc", ",", "p1", ",", "p2", ",", "err", 
       ",", "m", ",", "h", ",", "obs", ",", "F", ",", "off1", ",", "time", 
       ",", "verb", ",", "popgr8", ",", "popex16", ",", "grStates", ",", 
       "exStates", ",", "wgr", ",", "wex", ",", "tot1", ",", "popgr8Weighted",
        ",", "wgrAll", ",", "wexAllTotal", ",", "popgr8WeightedLoc"}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
       RowBox[{"function", " ", 
        RowBox[{"(", 
         RowBox[{"w", ",", "err", ",", "hump"}], ")"}]}], "=", 
       RowBox[{"expv", " ", 
        RowBox[{"(", 
         RowBox[{"t", ",", "A", ",", "v", ",", "tol", ",", "m"}], ")"}]}]}], 
      "*)"}], "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"n", "=", 
       RowBox[{
        RowBox[{"Dimensions", "[", "A", "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"{", "n", "}"}], "!=", 
         RowBox[{"Dimensions", "[", "u", "]"}]}], ",", 
        RowBox[{
         RowBox[{"Print", "[", 
          RowBox[{
          "\"\<phiv[]: A must be conformable with v, Dimensions[A][[1]]: \
\>\"", ",", " ", 
           RowBox[{"{", "n", "}"}], ",", " ", "\"\<, Dimensions[u]: \>\"", 
           ",", " ", 
           RowBox[{"Dimensions", "[", "u", "]"}]}], "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}], ",", 
        RowBox[{
         RowBox[{"{", "}"}], ";"}], ",", 
        RowBox[{
         RowBox[{"Print", "[", "\"\<u\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"{", "n", "}"}], "!=", 
         RowBox[{"Dimensions", "[", "v", "]"}]}], ",", 
        RowBox[{
         RowBox[{"Print", "[", 
          RowBox[{
          "\"\<phiv[]: A must be conformable with v, Dimensions[A][[1]]: \
\>\"", ",", " ", 
           RowBox[{"{", "n", "}"}], ",", " ", "\"\<, Dimensions[v]: \>\"", 
           ",", " ", 
           RowBox[{"Dimensions", "[", "v", "]"}]}], "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}], ",", 
        RowBox[{
         RowBox[{"{", "}"}], ";"}], ",", 
        RowBox[{
         RowBox[{"Print", "[", "\"\<v\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"MatrixQ", "[", 
          RowBox[{"A", ",", "NumericQ"}], "]"}], "==", "False"}], " ", ",", 
        RowBox[{
         RowBox[{"Print", "[", "\"\<phiv[]: A is nonnumeric\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}], ",", 
        RowBox[{
         RowBox[{"{", "}"}], ";"}], ",", 
        RowBox[{
         RowBox[{"Print", "[", "\"\<A\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"MatrixQ", "[", 
          RowBox[{
           RowBox[{"{", "u", "}"}], ",", "NumericQ"}], "]"}], "==", "False"}],
         " ", ",", 
        RowBox[{
         RowBox[{"Print", "[", "\"\<phiv[]: u is nonnumeric\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}], ",", 
        RowBox[{
         RowBox[{"{", "}"}], ";"}], ",", 
        RowBox[{
         RowBox[{"Print", "[", "\"\<A\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"MatrixQ", "[", 
          RowBox[{
           RowBox[{"{", "v", "}"}], ",", "NumericQ"}], "]"}], "==", "False"}],
         " ", ",", 
        RowBox[{
         RowBox[{"Print", "[", "\"\<phiv[]: v is nonnumeric\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}], ",", 
        RowBox[{
         RowBox[{"{", "}"}], ";"}], ",", 
        RowBox[{
         RowBox[{"Print", "[", "\"\<A\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"MatrixQ", "[", 
          RowBox[{
           RowBox[{"{", "obsmat", "}"}], ",", "NumericQ"}], "]"}], "==", 
         "False"}], " ", ",", 
        RowBox[{
         RowBox[{
         "Print", "[", "\"\<phiv[]: resultmat is nonnumeric\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}], ",", 
        RowBox[{
         RowBox[{"{", "}"}], ";"}], ",", 
        RowBox[{
         RowBox[{"Print", "[", "\"\<A\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"verb", "=", 
       RowBox[{"OptionValue", "[", "VerbosityLevel", "]"}]}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "Positions", " ", "of", " ", "the", " ", "ground", " ", "state", " ", 
        "populations"}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"grStates", "=", 
       RowBox[{"Flatten", "[", 
        RowBox[{"Position", "[", 
         RowBox[{"u", ",", 
          RowBox[{"x_", "/;", 
           RowBox[{"x", "!=", "0"}]}]}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "Positions", " ", "of", " ", "the", " ", "excited", " ", "state", " ", 
        "populations"}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"exStates", "=", 
       RowBox[{"Flatten", "[", 
        RowBox[{"Position", "[", 
         RowBox[{"obsmat", ",", 
          RowBox[{"x_", "/;", 
           RowBox[{"x", ">", "0"}]}]}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"popgr8", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"popex16", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"popgr8Weighted", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{
         RowBox[{"[", 
          RowBox[{"n", ",", "n"}], "]"}], "=", 
         RowBox[{"size", " ", 
          RowBox[{"(", "A", ")"}]}]}], ";"}], "*)"}], 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{
         RowBox[{"if", " ", "nargin"}], "==", "3"}], ",", 
        RowBox[{
         RowBox[{"tol", "=", 
          RowBox[{"1.0", "*", 
           RowBox[{"10", "^", 
            RowBox[{"(", 
             RowBox[{"-", "7"}], ")"}]}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"m", "=", 
          RowBox[{"min", " ", 
           RowBox[{"(", 
            RowBox[{"n", ",", "30"}], ")"}]}]}], ";", "\[IndentingNewLine]", 
         "end", ";", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"if", " ", "nargin"}], "==", "4"}]}], ",", 
        RowBox[{
         RowBox[{"m", "=", 
          RowBox[{"min", " ", 
           RowBox[{"(", 
            RowBox[{"n", ",", "30"}], ")"}]}]}], ";", "\[IndentingNewLine]", 
         "end", ";"}]}], "*)"}], "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"mIn", "==", 
         RowBox[{"-", "1"}]}], ",", 
        RowBox[{
         RowBox[{"m", "=", 
          RowBox[{"Min", "[", 
           RowBox[{"n", ",", "30"}], "]"}]}], ";"}], ",", 
        RowBox[{
         RowBox[{"m", "=", "mIn"}], ";"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"anorm", "=", 
       RowBox[{"Norm", "[", 
        RowBox[{"A", ",", "Infinity"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"mxrej", "=", "10"}], ";", 
      RowBox[{"btol", "=", 
       RowBox[{"10.0", "^", 
        RowBox[{"(", 
         RowBox[{"-", "7"}], ")"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"gamma", "=", "0.9"}], ";", 
      RowBox[{"delta", "=", "1.2"}], ";", "\[IndentingNewLine]", 
      RowBox[{"mb", "=", "m"}], ";", 
      RowBox[{"tout", "=", 
       RowBox[{"N", "[", 
        RowBox[{"Abs", "[", "t", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"istep", "=", "0"}], ";", 
      RowBox[{"tnew", "=", "0"}], ";", "\[IndentingNewLine]", 
      RowBox[{"tnow", "=", "0"}], ";", 
      RowBox[{"serror", "=", "0"}], ";", "\[IndentingNewLine]", 
      RowBox[{"rndoff", "=", 
       RowBox[{"anorm", "*", "$MachineEpsilon"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"sgn", "=", 
       RowBox[{"N", "[", 
        RowBox[{"Sign", "[", "t", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"k1", "=", "3"}], ";", "\[IndentingNewLine]", 
      RowBox[{"xm", "=", 
       RowBox[{"1.0", "/", "m"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"obs", "=", 
       RowBox[{"{", "}"}]}], ";", " ", 
      RowBox[{"(*", " ", 
       RowBox[{
       "Used", " ", "to", " ", "accumulate", " ", "return", " ", "flux", " ", 
        "samples"}], " ", "*)"}], "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"w", "=", "v"}], ";", "\[IndentingNewLine]", 
      RowBox[{"While", "[", 
       RowBox[{
        RowBox[{"tnow", "<", "tout"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"time", "=", 
          RowBox[{"Timing", "[", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Vt", "=", 
             RowBox[{"ConstantArray", "[", 
              RowBox[{"0.0", ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"m", "+", "1"}], ",", "n"}], "}"}]}], "]"}]}], ";", 
            RowBox[{"(*", 
             RowBox[{
              RowBox[{"V", "=", 
               RowBox[{"zeros", " ", 
                RowBox[{"(", 
                 RowBox[{"n", ",", 
                  RowBox[{"m", "+", "1"}]}], ")"}]}]}], ";"}], "*)"}], 
            "\[IndentingNewLine]", 
            RowBox[{"H", "=", 
             RowBox[{"ConstantArray", "[", 
              RowBox[{"0.0", ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"m", "+", "3"}], ",", 
                 RowBox[{"m", "+", "3"}]}], "}"}]}], "]"}]}], ";", 
            RowBox[{"(*", 
             RowBox[{
              RowBox[{"H", "=", 
               RowBox[{"zeros", " ", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"m", "+", "3"}], ",", 
                  RowBox[{"m", "+", "3"}]}], ")"}]}]}], ";"}], "*)"}], 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"Vt", "[", 
              RowBox[{"[", "1", "]"}], "]"}], "=", 
             RowBox[{
              RowBox[{"A", " ", ".", " ", "w"}], "+", "u"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"beta", "=", 
             RowBox[{"Norm", "[", 
              RowBox[{
               RowBox[{"Vt", "[", 
                RowBox[{"[", "1", "]"}], "]"}], ",", "2"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"Vt", "[", 
              RowBox[{"[", "1", "]"}], "]"}], "*=", 
             RowBox[{"(", 
              RowBox[{"1.0", "/", "beta"}], ")"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"istep", "==", "0"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"fact", "=", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{"m", "+", "1.0"}], ")"}], "/", "E"}], ")"}], "^", 
                   RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}]}], ")"}], "*", 
                 RowBox[{"Sqrt", "[", 
                  RowBox[{"2.0", "*", "Pi", "*", 
                   RowBox[{"(", 
                    RowBox[{"m", "+", "1"}], ")"}]}], "]"}]}]}], ";", 
               "\[IndentingNewLine]", 
               RowBox[{"tnew", "=", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{"1.0", "/", "anorm"}], ")"}], "*", 
                 RowBox[{
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"(", 
                    RowBox[{"fact", "*", "tol"}], ")"}], "/", 
                    RowBox[{"(", 
                    RowBox[{"4", "*", "beta", "*", "anorm"}], ")"}]}], ")"}], 
                  "^", "xm"}]}]}], ";", "\[IndentingNewLine]", 
               RowBox[{"s", "=", 
                RowBox[{"10.0", "^", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"Floor", "[", 
                    RowBox[{"Log10", "[", "tnew", "]"}], "]"}], "-", "1"}], 
                  ")"}]}]}], ";", "\[IndentingNewLine]", 
               RowBox[{"tnew", "=", 
                RowBox[{
                 RowBox[{"Ceiling", "[", 
                  RowBox[{"tnew", "/", "s"}], "]"}], "*", "s"}]}], ";"}]}], 
             "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
            RowBox[{"istep", "++"}], ";", "\[IndentingNewLine]", 
            RowBox[{"tstep", "=", 
             RowBox[{"Min", "[", 
              RowBox[{
               RowBox[{"tout", "-", "tnow"}], ",", "tnew"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"tstep", "=", 
             RowBox[{"Min", "[", 
              RowBox[{"tstep", ",", "tspanMax"}], "]"}]}], ";", 
            "\[IndentingNewLine]", "\[IndentingNewLine]", 
            RowBox[{"(*", " ", 
             RowBox[{
              RowBox[{
               RowBox[{"V", "[", 
                RowBox[{"[", 
                 RowBox[{"All", ",", "1"}], "]"}], "]"}], "=", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"1.0", "/", "beta"}], ")"}], "*", "w"}]}], ";", 
              "    ", 
              RowBox[{
               RowBox[{"V", " ", 
                RowBox[{"(", 
                 RowBox[{":", 
                  RowBox[{",", "1"}]}], ")"}]}], "=", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"1", "/", "beta"}], ")"}], "*", "w"}]}], ";"}], 
             "*)"}], "\[IndentingNewLine]", 
            RowBox[{"For", "[", 
             RowBox[{
              RowBox[{"j", "=", "1"}], ",", 
              RowBox[{"j", "<=", "m"}], ",", 
              RowBox[{"j", "++"}], ",", 
              RowBox[{"(*", 
               RowBox[{
                RowBox[{"for", " ", "j"}], "=", 
                RowBox[{"1", ":", "m"}]}], "*)"}], "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"p", "=", 
                RowBox[{"A", ".", 
                 RowBox[{"Vt", "[", 
                  RowBox[{"[", "j", "]"}], "]"}]}]}], ";", 
               RowBox[{"(*", 
                RowBox[{
                 RowBox[{"p", "=", 
                  RowBox[{"A", "*", "V", " ", 
                   RowBox[{"(", 
                    RowBox[{":", 
                    RowBox[{",", "j"}]}], ")"}]}]}], ";"}], "*)"}], 
               "\[IndentingNewLine]", 
               RowBox[{"For", "[", 
                RowBox[{
                 RowBox[{"i", "=", "1"}], ",", 
                 RowBox[{"i", "<=", "j"}], ",", 
                 RowBox[{"i", "++"}], ",", 
                 RowBox[{"(*", 
                  RowBox[{
                   RowBox[{"for", " ", "i"}], "=", 
                   RowBox[{"1", ":", "j"}]}], "*)"}], "\[IndentingNewLine]", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"H", "[", 
                    RowBox[{"[", 
                    RowBox[{"i", ",", "j"}], "]"}], "]"}], "=", 
                   RowBox[{
                    RowBox[{"Conjugate", "[", 
                    RowBox[{"Vt", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], "]"}], ".", "p"}]}], ";", 
                  RowBox[{"(*", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"H", " ", 
                    RowBox[{"(", 
                    RowBox[{"i", ",", "j"}], ")"}]}], "=", 
                    RowBox[{"V", " ", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{":", 
                    RowBox[{",", "i"}]}], ")"}], "'"}], "*", "p"}]}], ";"}], 
                   "*)"}], "\[IndentingNewLine]", 
                  RowBox[{"p", "-=", 
                   RowBox[{
                    RowBox[{"H", "[", 
                    RowBox[{"[", 
                    RowBox[{"i", ",", "j"}], "]"}], "]"}], " ", "*", " ", 
                    RowBox[{"Vt", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}]}], ";"}]}], 
                RowBox[{"(*", 
                 RowBox[{
                  RowBox[{"p", "=", 
                   RowBox[{"p", "-", 
                    RowBox[{"H", " ", 
                    RowBox[{"(", 
                    RowBox[{"i", ",", "j"}], ")"}], "*", "V", " ", 
                    RowBox[{"(", 
                    RowBox[{":", 
                    RowBox[{",", "i"}]}], ")"}]}]}]}], ";"}], "*)"}], 
                "\[IndentingNewLine]", "]"}], ";", 
               RowBox[{"(*", 
                RowBox[{"for", " ", "i"}], "*)"}], "\[IndentingNewLine]", 
               RowBox[{"s", "=", 
                RowBox[{"Norm", "[", 
                 RowBox[{"p", ",", "2"}], "]"}]}], ";", 
               RowBox[{"(*", 
                RowBox[{
                 RowBox[{"s", "=", 
                  RowBox[{"norm", " ", 
                   RowBox[{"(", "p", ")"}]}]}], ";"}], "*)"}], 
               "\[IndentingNewLine]", 
               RowBox[{"If", "[", 
                RowBox[{
                 RowBox[{"s", "<", "btol"}], ",", 
                 RowBox[{"(*", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"if", " ", "s"}], "<", "btol"}], ","}], "*)"}], 
                 "\[IndentingNewLine]", 
                 RowBox[{
                  RowBox[{"k1", "=", "0"}], ";", "\[IndentingNewLine]", 
                  RowBox[{"mb", "=", "j"}], ";", "\[IndentingNewLine]", 
                  RowBox[{"tstep", "=", 
                   RowBox[{"tout", "-", "tnow"}]}], ";", 
                  "\[IndentingNewLine]", 
                  RowBox[{"tstep", "=", 
                   RowBox[{"Min", "[", 
                    RowBox[{"tstep", ",", "tspanMax"}], "]"}]}], ";", 
                  "\[IndentingNewLine]", 
                  RowBox[{"Break", "[", "]"}]}]}], 
                RowBox[{"(*", 
                 RowBox[{"break", ";"}], "*)"}], "\[IndentingNewLine]", "]"}],
                ";", 
               RowBox[{"(*", " ", "if", " ", "*)"}], "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"H", "[", 
                 RowBox[{"[", 
                  RowBox[{
                   RowBox[{"j", "+", "1"}], ",", "j"}], "]"}], "]"}], "=", 
                "s"}], ";", 
               RowBox[{"(*", 
                RowBox[{
                 RowBox[{
                  RowBox[{"H", " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"j", "+", "1"}], ",", "j"}], ")"}]}], "=", "s"}], 
                 ";"}], "*)"}], "\[IndentingNewLine]", 
               RowBox[{"(*", 
                RowBox[{
                 RowBox[{
                  RowBox[{"V", " ", 
                   RowBox[{"(", 
                    RowBox[{":", 
                    RowBox[{",", 
                    RowBox[{"j", "+", "1"}]}]}], ")"}]}], "=", 
                  RowBox[{
                   RowBox[{"(", 
                    RowBox[{"1", "/", "s"}], ")"}], "*", "p"}]}], ";"}], 
                "*)"}], "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"Vt", "[", 
                 RowBox[{"[", 
                  RowBox[{"j", "+", "1"}], "]"}], "]"}], "=", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{"1.0", "/", "s"}], ")"}], "*", "p"}]}], ";"}]}], 
             "\[IndentingNewLine]", "]"}], ";", 
            RowBox[{"(*", 
             RowBox[{
              RowBox[{"for", " ", "j"}], ";"}], "*)"}], "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"H", "[", 
              RowBox[{"[", 
               RowBox[{"1", ",", 
                RowBox[{"mb", "+", "1"}]}], "]"}], "]"}], "=", "1"}], ";", 
            "\[IndentingNewLine]", "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"k1", "!=", "0"}], ",", 
              RowBox[{"(*", 
               RowBox[{
                RowBox[{"if", " ", 
                 RowBox[{"k1", "~", 
                  RowBox[{"=", "0"}]}]}], ","}], "*)"}], 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{
                RowBox[{"H", "[", 
                 RowBox[{"[", 
                  RowBox[{
                   RowBox[{"m", "+", "1"}], ",", 
                   RowBox[{"m", "+", "2"}]}], "]"}], "]"}], "=", "1.0"}], ";",
                "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"H", "[", 
                 RowBox[{"[", 
                  RowBox[{
                   RowBox[{"m", "+", "2"}], ",", 
                   RowBox[{"m", "+", "3"}]}], "]"}], "]"}], "=", "1.0"}], ";",
                "\[IndentingNewLine]", 
               RowBox[{"h", "=", 
                RowBox[{"H", "[", 
                 RowBox[{"[", 
                  RowBox[{
                   RowBox[{"m", "+", "1"}], ",", "m"}], "]"}], "]"}]}], ";", 
               "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"H", "[", 
                 RowBox[{"[", 
                  RowBox[{
                   RowBox[{"m", "+", "1"}], ",", "m"}], "]"}], "]"}], "=", 
                "0.0"}], ";", "\[IndentingNewLine]", 
               RowBox[{"(*", 
                RowBox[{
                 RowBox[{"avnorm", "=", 
                  RowBox[{"norm", " ", 
                   RowBox[{"(", 
                    RowBox[{"A", "*", "V", " ", 
                    RowBox[{"(", 
                    RowBox[{":", 
                    RowBox[{",", 
                    RowBox[{"m", "+", "1"}]}]}], ")"}]}], ")"}]}]}], ";"}], 
                "*)"}], "\[IndentingNewLine]", 
               RowBox[{"avnorm", "=", 
                RowBox[{"Norm", "[", 
                 RowBox[{"A", ".", 
                  RowBox[{"Vt", "[", 
                   RowBox[{"[", 
                    RowBox[{"m", "+", "1"}], "]"}], "]"}]}], " ", "]"}]}], 
               ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
            RowBox[{"(*", "if", "*)"}], "\[IndentingNewLine]", 
            "\[IndentingNewLine]", 
            RowBox[{"ireject", "=", "0"}], ";", "\[IndentingNewLine]", 
            RowBox[{"While", "[", 
             RowBox[{
              RowBox[{"ireject", "<=", "mxrej"}], ",", 
              RowBox[{"(*", 
               RowBox[{
                RowBox[{
                 RowBox[{"while", " ", "ireject"}], "<=", "mxrej"}], ","}], 
               "*)"}], "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"mx", "=", 
                RowBox[{"mb", "+", 
                 RowBox[{"Max", "[", 
                  RowBox[{"1", ",", "k1"}], "]"}]}]}], ";", 
               "\[IndentingNewLine]", 
               RowBox[{"(*", " ", 
                RowBox[{
                 RowBox[{"Re", "[", "]"}], " ", "added", " ", "on", " ", "09",
                  "Sep11", " ", "since", " ", 
                 RowBox[{"MatrixExp", "[", "]"}], " ", "sometimes", " ", 
                 "returns", " ", "complex", " ", "numbers"}], " ", "*)"}], 
               "\[IndentingNewLine]", 
               RowBox[{"F", "=", 
                RowBox[{"Re", "[", 
                 RowBox[{"MatrixExp", "[", " ", 
                  RowBox[{"sgn", "*", "tstep", "*", " ", 
                   RowBox[{"H", "[", 
                    RowBox[{"[", 
                    RowBox[{
                    RowBox[{"1", ";;", "mx"}], ",", 
                    RowBox[{"1", ";;", "mx"}]}], "]"}], "]"}]}], " ", "]"}], 
                 "]"}]}], ";", 
               RowBox[{"(*", 
                RowBox[{
                 RowBox[{"F", "=", 
                  RowBox[{"expm", " ", 
                   RowBox[{"(", 
                    RowBox[{"sgn", "*", "t_step", "*", "H", " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"1", ":", "mx"}], ",", 
                    RowBox[{"1", ":", "mx"}]}], ")"}]}], ")"}]}]}], ";"}], 
                "*)"}], "\[IndentingNewLine]", 
               RowBox[{"If", "[", 
                RowBox[{
                 RowBox[{"k1", "==", "0"}], ",", 
                 RowBox[{"(*", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"if", " ", "k1"}], "==", "0"}], ","}], "*)"}], 
                 "\[IndentingNewLine]", 
                 RowBox[{
                  RowBox[{"errloc", "=", "btol"}], ";", "\[IndentingNewLine]", 
                  RowBox[{"Break", "[", "]"}], ";"}], ",", 
                 RowBox[{"(*", 
                  RowBox[{"break", ";"}], "*)"}], "\[IndentingNewLine]", 
                 RowBox[{"(*", " ", "Else", " ", "*)"}], 
                 "\[IndentingNewLine]", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"F", "[", 
                    RowBox[{"[", 
                    RowBox[{
                    RowBox[{"m", "+", "1"}], ",", 
                    RowBox[{"m", "+", "1"}]}], "]"}], "]"}], "=", 
                   RowBox[{"h", "*", 
                    RowBox[{"F", "[", 
                    RowBox[{"[", 
                    RowBox[{"m", ",", 
                    RowBox[{"m", "+", "2"}]}], "]"}], "]"}]}]}], ";", 
                  "\[IndentingNewLine]", 
                  RowBox[{
                   RowBox[{"F", "[", 
                    RowBox[{"[", 
                    RowBox[{
                    RowBox[{"m", "+", "2"}], ",", 
                    RowBox[{"m", "+", "1"}]}], "]"}], "]"}], "=", 
                   RowBox[{"h", "*", 
                    RowBox[{"F", "[", 
                    RowBox[{"[", 
                    RowBox[{"m", ",", 
                    RowBox[{"m", "+", "3"}]}], "]"}], "]"}]}]}], ";", 
                  "\[IndentingNewLine]", 
                  RowBox[{"p1", "=", 
                   RowBox[{"Abs", "[", 
                    RowBox[{"beta", "*", 
                    RowBox[{"F", "[", 
                    RowBox[{"[", 
                    RowBox[{
                    RowBox[{"m", "+", "1"}], ",", 
                    RowBox[{"m", "+", "1"}]}], "]"}], "]"}]}], "]"}]}], ";", 
                  RowBox[{"(*", 
                   RowBox[{
                    RowBox[{"abs", " ", 
                    RowBox[{"(", 
                    RowBox[{"beta", "*", "F", " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"m", "+", "1"}], ",", 
                    RowBox[{"m", "+", "1"}]}], ")"}]}], ")"}]}], ";"}], 
                   "*)"}], "\[IndentingNewLine]", 
                  RowBox[{"p2", "=", 
                   RowBox[{"Abs", "[", 
                    RowBox[{"beta", "*", 
                    RowBox[{"F", "[", 
                    RowBox[{"[", 
                    RowBox[{
                    RowBox[{"m", "+", "2"}], ",", 
                    RowBox[{"m", "+", "1"}]}], "]"}], "]"}], "*", "avnorm"}], 
                    "]"}]}], ";", 
                  RowBox[{"(*", 
                   RowBox[{
                    RowBox[{"abs", " ", 
                    RowBox[{"(", 
                    RowBox[{"beta", "*", "F", " ", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"m", "+", "2"}], ",", 
                    RowBox[{"m", "+", "1"}]}], ")"}], "*", "avnorm"}], 
                    ")"}]}], ";"}], "*)"}], "\[IndentingNewLine]", 
                  "\[IndentingNewLine]", 
                  RowBox[{"If", "[", 
                   RowBox[{
                    RowBox[{"p1", ">", 
                    RowBox[{"10.0", "*", "p2"}]}], ",", 
                    RowBox[{"(*", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"if", " ", "phi1"}], ">", 
                    RowBox[{"10", "*", "phi2"}]}], ","}], "*)"}], 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"errloc", "=", "p2"}], ";", "\[IndentingNewLine]", 
                    RowBox[{"xm", "=", 
                    RowBox[{"1.0", "/", "m"}]}], ";"}], ",", 
                    "\[IndentingNewLine]", 
                    RowBox[{"(*", " ", "Else", " ", "*)"}], 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{"p1", ">", "p2"}], ",", "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"errloc", "=", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{"p1", "*", "p2"}], ")"}], "/", 
                    RowBox[{"(", 
                    RowBox[{"p1", "-", "p2"}], ")"}]}]}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{"xm", "=", 
                    RowBox[{"1.0", "/", "m"}]}], ";"}], ",", 
                    "\[IndentingNewLine]", 
                    RowBox[{"(*", "Else", "*)"}], "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{"errloc", "=", "p1"}], ";", "\[IndentingNewLine]", 
                    RowBox[{"xm", "=", 
                    RowBox[{"1.0", "/", 
                    RowBox[{"(", 
                    RowBox[{"m", "-", "1"}], ")"}]}]}], ";"}]}], 
                    "\[IndentingNewLine]", "]"}], ";"}]}], 
                   "\[IndentingNewLine]", "]"}], ";"}]}], " ", 
                "\[IndentingNewLine]", "]"}], ";", 
               RowBox[{"(*", " ", 
                RowBox[{
                 RowBox[{"If", " ", "k"}], "==", "1"}], " ", "*)"}], 
               "\[IndentingNewLine]", 
               RowBox[{"If", "[", 
                RowBox[{
                 RowBox[{"errloc", "<=", 
                  RowBox[{"delta", "*", "tstep", "*", "tol"}]}], ",", 
                 RowBox[{"(*", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"if", " ", "err_loc"}], "<=", 
                    RowBox[{"delta", "*", "t_step", "*", "tol"}]}], ","}], 
                  "*)"}], "\[IndentingNewLine]", 
                 RowBox[{
                  RowBox[{"Break", "[", "]"}], ";"}], ",", 
                 "\[IndentingNewLine]", 
                 RowBox[{"(*", "Else", "*)"}], "\[IndentingNewLine]", 
                 RowBox[{
                  RowBox[{"tstep", "=", 
                   RowBox[{"gamma", "*", "tstep", "*", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{"tstep", "*", 
                    RowBox[{"tol", "/", "errloc"}]}], ")"}], "^", "xm"}]}]}], 
                  ";", "\[IndentingNewLine]", 
                  RowBox[{"tstep", "=", 
                   RowBox[{"Min", "[", 
                    RowBox[{"tstep", ",", "tspanMax"}], "]"}]}], ";", 
                  "\[IndentingNewLine]", 
                  RowBox[{"s", "=", 
                   RowBox[{"10.0", "^", 
                    RowBox[{"(", 
                    RowBox[{
                    RowBox[{"Floor", "[", 
                    RowBox[{"Log10", "[", "tstep", "]"}], "]"}], "-", "1"}], 
                    ")"}]}]}], ";", "\[IndentingNewLine]", 
                  RowBox[{"tstep", "=", 
                   RowBox[{
                    RowBox[{"Ceiling", "[", 
                    RowBox[{"tstep", "/", "s"}], "]"}], "*", "s"}]}], ";", 
                  "\[IndentingNewLine]", 
                  RowBox[{"If", "[", 
                   RowBox[{
                    RowBox[{"ireject", "==", "mxrej"}], ",", 
                    "\[IndentingNewLine]", 
                    RowBox[{
                    RowBox[{
                    "Print", "[", 
                    "\"\<phiv[]: The requested tolerance is too high!\>\"", 
                    "]"}], ";", "\[IndentingNewLine]", 
                    RowBox[{"Abort", "[", "]"}], ";"}]}], 
                   "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
                  RowBox[{"ireject", "++"}], ";"}]}], "\[IndentingNewLine]", 
                "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
            RowBox[{"(*", " ", 
             RowBox[{"While", "[", 
              RowBox[{
               RowBox[{"ireject", "<=", "mxrej"}], ",", "..."}], "]"}], " ", 
             "*)"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
            RowBox[{"mx", "=", 
             RowBox[{"mb", "+", 
              RowBox[{"Max", "[", 
               RowBox[{"0", ",", 
                RowBox[{"k1", "-", "2"}]}], "]"}]}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"w", "=", 
             RowBox[{
              RowBox[{
               RowBox[{"Transpose", "[", 
                RowBox[{"Vt", "[", 
                 RowBox[{"[", 
                  RowBox[{"1", ";;", "mx"}], "]"}], "]"}], "]"}], ".", 
               RowBox[{"(", 
                RowBox[{"beta", "*", 
                 RowBox[{"F", "[", 
                  RowBox[{"[", 
                   RowBox[{
                    RowBox[{"1", ";;", "mx"}], ",", 
                    RowBox[{"mb", "+", "1"}]}], "]"}], "]"}]}], ")"}]}], " ", 
              "+", " ", "w"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"(*", 
             RowBox[{
              RowBox[{"w", "=", 
               RowBox[{
                RowBox[{"V", " ", 
                 RowBox[{"(", 
                  RowBox[{":", 
                   RowBox[{",", 
                    RowBox[{"1", ":", "mx"}]}]}], ")"}], "*", 
                 RowBox[{"(", 
                  RowBox[{"beta", "*", "F", " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"1", ":", "mx"}], ",", 
                    RowBox[{"mb", "+", "1"}]}], ")"}]}], ")"}]}], " ", "+", 
                " ", "w"}]}], ";"}], "*)"}], "\[IndentingNewLine]", 
            "\[IndentingNewLine]", 
            RowBox[{"tnow", "+=", "tstep"}], ";", 
            RowBox[{"(*", 
             RowBox[{
              RowBox[{"t_now", "=", 
               RowBox[{"t_now", "+", "t_step"}]}], ";"}], "*)"}], 
            "\[IndentingNewLine]", 
            RowBox[{"tnew", "=", 
             RowBox[{"gamma", "*", "tstep", "*", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"tstep", "*", 
                 RowBox[{"tol", "/", "errloc"}]}], ")"}], "^", "xm"}]}]}], 
            ";", 
            RowBox[{"(*", 
             RowBox[{
              RowBox[{"t_new", "=", 
               RowBox[{"gamma", "*", "t_step", "*", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{"t_step", "*", 
                   RowBox[{"tol", "/", "err_loc"}]}], ")"}], "^", "xm"}]}]}], 
              ";"}], "*)"}], "\[IndentingNewLine]", 
            RowBox[{"s", "=", 
             RowBox[{"10.0", "^", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"Floor", "[", 
                 RowBox[{"Log10", "[", "tnew", "]"}], "]"}], "-", "1"}], 
               ")"}]}]}], ";", 
            RowBox[{"(*", 
             RowBox[{
              RowBox[{"s", "=", 
               RowBox[{"10", "^", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"floor", " ", 
                   RowBox[{"(", 
                    RowBox[{"log10", " ", 
                    RowBox[{"(", "t_new", ")"}]}], ")"}]}], "-", "1"}], 
                 ")"}]}]}], ";"}], "*)"}], "\[IndentingNewLine]", 
            RowBox[{"tnew", "=", 
             RowBox[{
              RowBox[{"Ceiling", "[", 
               RowBox[{"tnew", "/", "s"}], "]"}], "*", "s"}]}], ";", 
            RowBox[{"(*", 
             RowBox[{
              RowBox[{"t_new", "=", 
               RowBox[{"ceil", " ", 
                RowBox[{"(", 
                 RowBox[{"t_new", "/", "s"}], ")"}], "*", "s"}]}], ";"}], 
             "*)"}], "\[IndentingNewLine]", 
            RowBox[{"errloc", "=", 
             RowBox[{"Max", "[", 
              RowBox[{"errloc", ",", "rndoff"}], "]"}]}], ";", 
            RowBox[{"(*", 
             RowBox[{
              RowBox[{"err_loc", "=", 
               RowBox[{"max", " ", 
                RowBox[{"(", 
                 RowBox[{"err_loc", ",", "rndoff"}], ")"}]}]}], ";"}], "*)"}],
             "\[IndentingNewLine]", 
            RowBox[{"serror", "+=", "errloc"}], ";"}], "\[IndentingNewLine]", 
           "]"}]}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"istep", "<=", "4"}], "&&", 
             RowBox[{"verb", ">", "3"}]}], "||", 
            RowBox[{"verb", ">", "5"}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Print", "[", 
             RowBox[{
             "\"\<    Step \>\"", ",", "istep", ",", "\"\<: tstep = \>\"", 
              ",", " ", 
              RowBox[{
               RowBox[{"10.0", "^", "9"}], "*", "tstep"}], ",", " ", 
              "\"\< ns\>\"", ",", " ", "\"\<, tnow = \>\"", ",", " ", 
              RowBox[{
               RowBox[{"10.0", "^", "9"}], "*", "tnow"}], ",", " ", 
              "\"\< ns, (\>\"", ",", 
              RowBox[{"time", "[", 
               RowBox[{"[", "1", "]"}], "]"}], ",", "\"\< s)\>\""}], "]"}], 
            ";"}]}], "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{"Code", " ", "inserted", " ", "by", " ", "Ron", " ", 
           RowBox[{"Holzloehner", ":", " ", 
            RowBox[{
            "Compute", " ", "the", " ", "return", " ", "flux", " ", "at", " ",
              "tnow"}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"n", "==", 
            RowBox[{"Max", "[", 
             RowBox[{"Dimensions", "[", "obsmat", "]"}], "]"}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"AppendTo", "[", 
             RowBox[{"obs", ",", 
              RowBox[{"{", 
               RowBox[{"tnow", ",", 
                RowBox[{"Re", "[", 
                 RowBox[{"obsmat", ".", "w"}], "]"}]}], "}"}]}], "]"}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]",
          "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"indVgMaxEx", ">=", "1"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"off1", "=", 
             RowBox[{"8", "*", 
              RowBox[{"(", 
               RowBox[{"indVgMaxEx", "-", "1"}], ")"}]}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"wgr", "=", 
             RowBox[{
              RowBox[{"w", "[", 
               RowBox[{"[", "grStates", "]"}], "]"}], "[", 
              RowBox[{"[", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"1", "+", "off1"}], ")"}], ";;", 
                RowBox[{"(", 
                 RowBox[{"8", "+", "off1"}], ")"}]}], "]"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"off1", "=", 
             RowBox[{"16", "*", 
              RowBox[{"(", 
               RowBox[{"indVgMaxEx", "-", "1"}], ")"}]}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"wex", "=", 
             RowBox[{
              RowBox[{"w", "[", 
               RowBox[{"[", "exStates", "]"}], "]"}], "[", 
              RowBox[{"[", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"1", "+", "off1"}], ")"}], ";;", 
                RowBox[{"(", 
                 RowBox[{"16", "+", "off1"}], ")"}]}], "]"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"(*", " ", 
             RowBox[{
             "Normalize", " ", "by", " ", "number", " ", "of", " ", "atoms", 
              " ", "in", " ", "this", " ", "velocity", " ", "bin"}], " ", 
             "*)"}], "\[IndentingNewLine]", 
            RowBox[{"tot1", "=", 
             RowBox[{"N", "[", 
              RowBox[{
               RowBox[{"Total", "[", "wgr", "]"}], "+", 
               RowBox[{"Total", "[", "wex", "]"}]}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"popgr8", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"popgr8", ",", 
               RowBox[{"{", 
                RowBox[{"wgr", "/", "tot1"}], "}"}]}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"popex16", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"popex16", ",", 
               RowBox[{"{", 
                RowBox[{"wex", "/", "tot1"}], "}"}]}], "]"}]}], ";"}], 
           "\[IndentingNewLine]", ",", " ", 
           RowBox[{"(*", "else", "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"popgr8", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"popgr8", ",", 
               RowBox[{"{", 
                RowBox[{"ConstantArray", "[", 
                 RowBox[{
                  RowBox[{"-", "1.0"}], ",", "8"}], "]"}], "}"}]}], "]"}]}], 
            ";", "\[IndentingNewLine]", 
            RowBox[{"popex16", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"popex16", ",", 
               RowBox[{"{", 
                RowBox[{"ConstantArray", "[", 
                 RowBox[{
                  RowBox[{"-", "1.0"}], ",", "16"}], "]"}], "}"}]}], "]"}]}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{
           RowBox[{"Nx8", " ", "array"}], ",", " ", 
           RowBox[{
           "where", " ", "N", " ", "is", " ", "the", " ", "number", " ", "of",
             " ", "velocity", " ", "classes"}]}], "  ", "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"wgrAll", "=", 
          RowBox[{"Partition", "[", 
           RowBox[{
            RowBox[{"w", "[", 
             RowBox[{"[", "grStates", "]"}], "]"}], ",", "8"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{
          "Vector", " ", "of", " ", "length", " ", "N", " ", "with", " ", 
           "the", " ", "total", " ", "excited", " ", "populations", " ", 
           RowBox[{"(", 
            RowBox[{
            "total", " ", "of", " ", "all", " ", "16", " ", "states"}], ")"}],
            " ", "in", " ", "each", " ", "velocity", " ", "class"}], " ", 
          "*)"}], "\[IndentingNewLine]", 
         RowBox[{"wexAllTotal", "=", 
          RowBox[{"Total", "[", 
           RowBox[{
            RowBox[{"Partition", "[", 
             RowBox[{
              RowBox[{"w", "[", 
               RowBox[{"[", "exStates", "]"}], "]"}], ",", "16"}], "]"}], ",", 
            RowBox[{"{", "2", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"tot1", "=", 
          RowBox[{"Total", "[", "wexAllTotal", "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"tot1", ">", "0"}], ",", "\[IndentingNewLine]", 
           RowBox[{"(*", " ", 
            RowBox[{
            "Vector", " ", "of", " ", "length", " ", "8", " ", "containing", 
             " ", "the", " ", "ground", " ", "state", " ", "populations", " ",
              "weighted", " ", "by", " ", "the", " ", "excited", " ", "state",
              " ", "populations"}], " ", "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"popgr8WeightedLoc", "=", 
             RowBox[{
              RowBox[{"Total", "[", 
               RowBox[{
                RowBox[{"wgrAll", "*", "wexAllTotal"}], ",", 
                RowBox[{"{", "1", "}"}]}], "]"}], "/", "tot1"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"(*", " ", 
             RowBox[{"Normalize", " ", "to", " ", "total", " ", "1"}], " ", 
             "*)"}], "\[IndentingNewLine]", 
            RowBox[{"popgr8WeightedLoc", "/=", 
             RowBox[{"Total", "[", "popgr8WeightedLoc", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"popgr8Weighted", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"popgr8Weighted", ",", 
               RowBox[{"{", "popgr8WeightedLoc", "}"}]}], "]"}]}], ";"}], 
           "\[IndentingNewLine]", ",", " ", 
           RowBox[{"(*", " ", "ELSE", " ", "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"popgr8Weighted", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"popgr8Weighted", ",", 
               RowBox[{"{", 
                RowBox[{"ConstantArray", "[", 
                 RowBox[{
                  RowBox[{"-", "1.0"}], ",", "8"}], "]"}], "}"}]}], "]"}]}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";"}]}], 
       "\[IndentingNewLine]", "\[IndentingNewLine]", "]"}], ";", " ", 
      RowBox[{"(*", " ", 
       RowBox[{"end", " ", "of", " ", 
        RowBox[{"While", "[", "]"}]}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"err", "=", "serror"}], ";", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "w", ",", "err", ",", "obs", ",", "popgr8", ",", "popex16", ",", 
        "popgr8Weighted"}], "}"}]}]}], "\[IndentingNewLine]", "]"}]}], ";", 
  " ", 
  RowBox[{"(*", 
   RowBox[{"End", " ", "of", " ", 
    RowBox[{"Module", "[", "]"}], " ", 
    RowBox[{"phiv", "[", "]"}]}], " ", "*)"}]}]], "Input",
 InitializationCell->True],

Cell[" ", "Text",
 Editable->False,
 Selectable->False,
 CellFrame->{{0, 0}, {0, 2}},
 ShowCellBracket->False,
 CellMargins->{{0, 0}, {1, 1}},
 CellElementSpacings->{"CellMinHeight"->1},
 CellFrameMargins->0,
 CellFrameColor->RGBColor[0, 0, 1],
 CellSize->{Inherited, 4}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Modified", " ", "version", " ", "of", " ", 
   RowBox[{"phiv", "[", "]"}], " ", "with", " ", "a", " ", "compiled", " ", 
   "inner", " ", "loop"}], " ", "*)"}]], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Options", "[", "phivCompiled", "]"}], "=", 
    RowBox[{"{", 
     RowBox[{"VerbosityLevel", "->", "2"}], "}"}]}], ";"}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "The", " ", "sparse", " ", "matrix", " ", "A", " ", "is", " ", "passed", 
    " ", "to", " ", 
    RowBox[{"multiplyByA", "[", "]"}], " ", "via", " ", "the", " ", "global", 
    " ", "variable", " ", "Global`fullAmatG"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"phivCompiled", "[", 
    RowBox[{"t_", ",", "u_", ",", "v_", ",", 
     RowBox[{"obsmat_:", 
      RowBox[{"{", "}"}]}], ",", 
     RowBox[{"tspanMax_:", "1.0*^200"}], ",", 
     RowBox[{"tol_:", "1.0*^-7"}], ",", 
     RowBox[{"mIn_:", "-", "1"}], ",", 
     RowBox[{"indVgMaxEx_:", "-", "1"}], ",", 
     RowBox[{"OptionsPattern", "[", "]"}]}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "n", ",", "anorm", ",", "tout", ",", "istep", ",", "tnew", ",", "tnow", 
       ",", "serror", ",", "rndoff", ",", "sgn", ",", "w", ",", "err", ",", 
       "m", ",", "obs", ",", "off1", ",", "time", ",", "verb", ",", "popgr8", 
       ",", "popex16", ",", "grStates", ",", "exStates", ",", "wgr", ",", 
       "wex", ",", "tot1", ",", "popgr8Weighted", ",", "wgrAll", ",", 
       "wexAllTotal", ",", "popgr8WeightedLoc", ",", "tstep", ",", "xm", ",", 
       "k1", ",", "mb", ",", "out1", ",", "errloc", ",", "s"}], "}"}], ",", 
     "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{
       RowBox[{"function", " ", 
        RowBox[{"(", 
         RowBox[{"w", ",", "err", ",", "hump"}], ")"}]}], "=", 
       RowBox[{"expv", " ", 
        RowBox[{"(", 
         RowBox[{"t", ",", "A", ",", "v", ",", "tol", ",", "m"}], ")"}]}]}], 
      "*)"}], "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"n", "=", 
       RowBox[{
        RowBox[{"Dimensions", "[", "Global`fullAmatG", "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"{", "n", "}"}], "!=", 
         RowBox[{"Dimensions", "[", "u", "]"}]}], ",", 
        RowBox[{
         RowBox[{"Print", "[", 
          RowBox[{
          "\"\<phivCompiled[]: A must be conformable with v, \
Dimensions[A][[1]]: \>\"", ",", " ", 
           RowBox[{"{", "n", "}"}], ",", " ", "\"\<, Dimensions[u]: \>\"", 
           ",", " ", 
           RowBox[{"Dimensions", "[", "u", "]"}]}], "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}], ",", 
        RowBox[{
         RowBox[{"{", "}"}], ";"}], ",", 
        RowBox[{
         RowBox[{"Print", "[", "\"\<u\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"{", "n", "}"}], "!=", 
         RowBox[{"Dimensions", "[", "v", "]"}]}], ",", 
        RowBox[{
         RowBox[{"Print", "[", 
          RowBox[{
          "\"\<phivCompiled[]: A must be conformable with v, \
Dimensions[A][[1]]: \>\"", ",", " ", 
           RowBox[{"{", "n", "}"}], ",", " ", "\"\<, Dimensions[v]: \>\"", 
           ",", " ", 
           RowBox[{"Dimensions", "[", "v", "]"}]}], "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}], ",", 
        RowBox[{
         RowBox[{"{", "}"}], ";"}], ",", 
        RowBox[{
         RowBox[{"Print", "[", "\"\<v\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
        RowBox[{"Since", " ", "the", " ", "matrix", " ", "A", " ", 
         RowBox[{"doesn", "'"}], "t", " ", "have", " ", "to", " ", "be", " ", 
         "passed"}], ",", " ", 
        RowBox[{
        "check", " ", "here", " ", "if", " ", "the", " ", "2", "nd", " ", 
         "argument", " ", "u", " ", "is", " ", "a", " ", "matrix", " ", 
         "instead", " ", "of", " ", "a", " ", "vector"}]}], " ", "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"MatrixQ", "[", 
          RowBox[{"u", ",", "NumericQ"}], "]"}], "==", "True"}], " ", ",", 
        RowBox[{
         RowBox[{
         "Print", "[", 
          "\"\<phivCompiled[]: Second argument should be the vector u, not a \
matrix (different argument list than phiv[])!\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}], ",", 
        RowBox[{
         RowBox[{"{", "}"}], ";"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"MatrixQ", "[", 
          RowBox[{"Global`fullAmatG", ",", "NumericQ"}], "]"}], "==", 
         "False"}], " ", ",", 
        RowBox[{
         RowBox[{
         "Print", "[", "\"\<phivCompiled[]: A is nonnumeric\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}], ",", 
        RowBox[{
         RowBox[{"{", "}"}], ";"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"MatrixQ", "[", 
          RowBox[{
           RowBox[{"{", "u", "}"}], ",", "NumericQ"}], "]"}], "==", "False"}],
         " ", ",", 
        RowBox[{
         RowBox[{
         "Print", "[", "\"\<phivCompiled[]: u is nonnumeric\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}], ";"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"MatrixQ", "[", 
          RowBox[{
           RowBox[{"{", "v", "}"}], ",", "NumericQ"}], "]"}], "==", "False"}],
         " ", ",", 
        RowBox[{
         RowBox[{
         "Print", "[", "\"\<phivCompiled[]: v is nonnumeric\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}], ";"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"MatrixQ", "[", 
           RowBox[{
            RowBox[{"{", "obsmat", "}"}], ",", "NumericQ"}], "]"}], "==", 
          "False"}], " ", "&&", 
         RowBox[{"obsmat", "!=", 
          RowBox[{"{", "}"}]}]}], ",", 
        RowBox[{
         RowBox[{
         "Print", "[", "\"\<phivCompiled[]: resultmat is nonnumeric\>\"", 
          "]"}], ";", 
         RowBox[{"Abort", "[", "]"}], ";"}]}], "]"}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"verb", "=", 
       RowBox[{"OptionValue", "[", "VerbosityLevel", "]"}]}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "Positions", " ", "of", " ", "the", " ", "ground", " ", "state", " ", 
        "populations"}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"grStates", "=", 
       RowBox[{"Flatten", "[", 
        RowBox[{"Position", "[", 
         RowBox[{"u", ",", 
          RowBox[{"x_", "/;", 
           RowBox[{"x", "!=", "0"}]}]}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "Positions", " ", "of", " ", "the", " ", "excited", " ", "state", " ", 
        "populations"}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"exStates", "=", 
       RowBox[{"Flatten", "[", 
        RowBox[{"Position", "[", 
         RowBox[{"obsmat", ",", 
          RowBox[{"x_", "/;", 
           RowBox[{"x", ">", "0"}]}]}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"popgr8", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"popex16", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"popgr8Weighted", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{
         RowBox[{"[", 
          RowBox[{"n", ",", "n"}], "]"}], "=", 
         RowBox[{"size", " ", 
          RowBox[{"(", "A", ")"}]}]}], ";"}], "*)"}], 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{
         RowBox[{"if", " ", "nargin"}], "==", "3"}], ",", 
        RowBox[{
         RowBox[{"tol", "=", 
          RowBox[{"1.0", "*", 
           RowBox[{"10", "^", 
            RowBox[{"(", 
             RowBox[{"-", "7"}], ")"}]}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"m", "=", 
          RowBox[{"min", " ", 
           RowBox[{"(", 
            RowBox[{"n", ",", "30"}], ")"}]}]}], ";", "\[IndentingNewLine]", 
         "end", ";", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"if", " ", "nargin"}], "==", "4"}]}], ",", 
        RowBox[{
         RowBox[{"m", "=", 
          RowBox[{"min", " ", 
           RowBox[{"(", 
            RowBox[{"n", ",", "30"}], ")"}]}]}], ";", "\[IndentingNewLine]", 
         "end", ";"}]}], "*)"}], "\[IndentingNewLine]", 
      RowBox[{"m", "=", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"mIn", "==", 
          RowBox[{"-", "1"}]}], ",", 
         RowBox[{"Min", "[", 
          RowBox[{"n", ",", "30"}], "]"}], ",", "mIn"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"anorm", "=", 
       RowBox[{"Norm", "[", 
        RowBox[{"Global`fullAmatG", ",", "Infinity"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"tout", "=", 
       RowBox[{"N", "[", 
        RowBox[{"Abs", "[", "t", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"istep", "=", "0"}], ";", 
      RowBox[{"tnew", "=", "0"}], ";", "\[IndentingNewLine]", 
      RowBox[{"tnow", "=", "0"}], ";", 
      RowBox[{"serror", "=", "0"}], ";", "\[IndentingNewLine]", 
      RowBox[{"rndoff", "=", 
       RowBox[{"anorm", "*", "$MachineEpsilon"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"sgn", "=", 
       RowBox[{"N", "[", 
        RowBox[{"Sign", "[", "t", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"xm", "=", 
       RowBox[{"1.0", "/", "m"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"mb", "=", "m"}], ";", "\[IndentingNewLine]", 
      RowBox[{"k1", "=", "3"}], ";", "\[IndentingNewLine]", 
      RowBox[{"obs", "=", 
       RowBox[{"{", "}"}]}], ";", " ", 
      RowBox[{"(*", " ", 
       RowBox[{
       "Used", " ", "to", " ", "accumulate", " ", "return", " ", "flux", " ", 
        "samples"}], " ", "*)"}], "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"w", "=", "v"}], ";", "\[IndentingNewLine]", 
      RowBox[{"(*", "  ", 
       RowBox[{
        RowBox[{"===", "===", "===", "===", "===", " ", 
         RowBox[{"Main", " ", "Loop"}], " ", "===", "===", "===", "===", "===",
          "==="}], "="}], "  ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"While", "[", 
       RowBox[{
        RowBox[{"tnow", "<", "tout"}], ",", "\[IndentingNewLine]", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"time", "=", 
          RowBox[{"Timing", "[", "\[IndentingNewLine]", 
           RowBox[{"(*", " ", 
            RowBox[{
             RowBox[{"Due", " ", "to", " ", "restrictions", " ", "of", " ", 
              RowBox[{"Compile", "[", "]"}]}], ",", " ", 
             RowBox[{
             "the", " ", "output", " ", "is", " ", "a", " ", "single", " ", 
              "vector", " ", "with", " ", "the", " ", "following", " ", 
              RowBox[{"elements", ":", " ", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{
                 "w", ",", "istep", ",", "tstep", ",", "tnew", ",", "tnow", 
                  ",", "errloc", ",", "s", ",", "xm", ",", "mb"}], "}"}], 
                "\[IndentingNewLine]", "All", " ", "floating", " ", "point", 
                " ", "variables", " ", "are", " ", "real"}]}]}], ",", " ", 
             RowBox[{"not", " ", "complex"}]}], "  ", "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"out1", "=", 
             RowBox[{"phivLoopCompiled", "[", 
              RowBox[{
              "m", ",", "n", ",", "w", ",", "u", ",", "istep", ",", "tout", 
               ",", "tnow", ",", "tnew", ",", "tspanMax", ",", "sgn", ",", 
               "tol", ",", "rndoff", ",", "anorm", ",", "xm", ",", "mb", ",", 
               "k1"}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"w", "=", 
             RowBox[{"out1", "[", 
              RowBox[{"[", 
               RowBox[{"1", ";;", 
                RowBox[{"Length", "[", "v", "]"}]}], "]"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"istep", "=", 
             RowBox[{"Round", "[", 
              RowBox[{"out1", "[", 
               RowBox[{"[", 
                RowBox[{"-", "8"}], "]"}], "]"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"tstep", "=", 
             RowBox[{"out1", "[", 
              RowBox[{"[", 
               RowBox[{"-", "7"}], "]"}], "]"}]}], ";", 
            RowBox[{"tnew", "=", 
             RowBox[{"out1", "[", 
              RowBox[{"[", 
               RowBox[{"-", "6"}], "]"}], "]"}]}], ";", " ", 
            RowBox[{"tnow", "=", 
             RowBox[{"out1", "[", 
              RowBox[{"[", 
               RowBox[{"-", "5"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"errloc", "=", 
             RowBox[{"out1", "[", 
              RowBox[{"[", 
               RowBox[{"-", "4"}], "]"}], "]"}]}], ";", 
            RowBox[{"s", "=", 
             RowBox[{"out1", "[", 
              RowBox[{"[", 
               RowBox[{"-", "3"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"xm", "=", 
             RowBox[{"out1", "[", 
              RowBox[{"[", 
               RowBox[{"-", "2"}], "]"}], "]"}]}], ";", 
            RowBox[{"mb", "=", 
             RowBox[{"Round", "[", 
              RowBox[{"out1", "[", 
               RowBox[{"[", 
                RowBox[{"-", "1"}], "]"}], "]"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"serror", "+=", "errloc"}], ";"}], "\[IndentingNewLine]", 
           "]"}]}], ";", " ", 
         RowBox[{"(*", " ", 
          RowBox[{"End", " ", "of", " ", 
           RowBox[{"Timing", "[", "]"}]}], " ", "*)"}], "\[IndentingNewLine]",
          "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"istep", "<=", "4"}], "&&", 
             RowBox[{"verb", ">", "3"}]}], "||", 
            RowBox[{"verb", ">", "5"}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Print", "[", 
             RowBox[{
             "\"\<    Step \>\"", ",", "istep", ",", "\"\<: tstep = \>\"", 
              ",", " ", 
              RowBox[{
               RowBox[{"10.0", "^", "9"}], "*", "tstep"}], ",", " ", 
              "\"\< ns\>\"", ",", " ", "\"\<, tnow = \>\"", ",", " ", 
              RowBox[{
               RowBox[{"10.0", "^", "9"}], "*", "tnow"}], ",", " ", 
              "\"\< ns, (\>\"", ",", 
              RowBox[{"time", "[", 
               RowBox[{"[", "1", "]"}], "]"}], ",", "\"\< s)\>\""}], "]"}], 
            ";"}]}], "]"}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{"Code", " ", "inserted", " ", "by", " ", "Ron", " ", 
           RowBox[{"Holzloehner", ":", " ", 
            RowBox[{
            "Compute", " ", "the", " ", "return", " ", "flux", " ", "at", " ",
              "tnow"}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"n", "==", 
            RowBox[{"Max", "[", 
             RowBox[{"Dimensions", "[", "obsmat", "]"}], "]"}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"AppendTo", "[", 
             RowBox[{"obs", ",", 
              RowBox[{"{", 
               RowBox[{"tnow", ",", 
                RowBox[{"Re", "[", 
                 RowBox[{"obsmat", ".", "w"}], "]"}]}], "}"}]}], "]"}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]",
          "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"indVgMaxEx", ">=", "1"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"off1", "=", 
             RowBox[{"8", "*", 
              RowBox[{"(", 
               RowBox[{"indVgMaxEx", "-", "1"}], ")"}]}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"wgr", "=", 
             RowBox[{
              RowBox[{"w", "[", 
               RowBox[{"[", "grStates", "]"}], "]"}], "[", 
              RowBox[{"[", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"1", "+", "off1"}], ")"}], ";;", 
                RowBox[{"(", 
                 RowBox[{"8", "+", "off1"}], ")"}]}], "]"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"off1", "=", 
             RowBox[{"16", "*", 
              RowBox[{"(", 
               RowBox[{"indVgMaxEx", "-", "1"}], ")"}]}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"wex", "=", 
             RowBox[{
              RowBox[{"w", "[", 
               RowBox[{"[", "exStates", "]"}], "]"}], "[", 
              RowBox[{"[", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"1", "+", "off1"}], ")"}], ";;", 
                RowBox[{"(", 
                 RowBox[{"16", "+", "off1"}], ")"}]}], "]"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"(*", " ", 
             RowBox[{
             "Normalize", " ", "by", " ", "number", " ", "of", " ", "atoms", 
              " ", "in", " ", "this", " ", "velocity", " ", "bin"}], " ", 
             "*)"}], "\[IndentingNewLine]", 
            RowBox[{"tot1", "=", 
             RowBox[{"N", "[", 
              RowBox[{
               RowBox[{"Total", "[", "wgr", "]"}], "+", 
               RowBox[{"Total", "[", "wex", "]"}]}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"popgr8", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"popgr8", ",", 
               RowBox[{"{", 
                RowBox[{"wgr", "/", "tot1"}], "}"}]}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"popex16", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"popex16", ",", 
               RowBox[{"{", 
                RowBox[{"wex", "/", "tot1"}], "}"}]}], "]"}]}], ";"}], 
           "\[IndentingNewLine]", ",", " ", 
           RowBox[{"(*", "else", "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"popgr8", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"popgr8", ",", 
               RowBox[{"{", 
                RowBox[{"ConstantArray", "[", 
                 RowBox[{
                  RowBox[{"-", "1.0"}], ",", "8"}], "]"}], "}"}]}], "]"}]}], 
            ";", "\[IndentingNewLine]", 
            RowBox[{"popex16", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"popex16", ",", 
               RowBox[{"{", 
                RowBox[{"ConstantArray", "[", 
                 RowBox[{
                  RowBox[{"-", "1.0"}], ",", "16"}], "]"}], "}"}]}], "]"}]}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{
           RowBox[{"Nx8", " ", "array"}], ",", " ", 
           RowBox[{
           "where", " ", "N", " ", "is", " ", "the", " ", "number", " ", "of",
             " ", "velocity", " ", "classes"}]}], "  ", "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"wgrAll", "=", 
          RowBox[{"Partition", "[", 
           RowBox[{
            RowBox[{"w", "[", 
             RowBox[{"[", "grStates", "]"}], "]"}], ",", "8"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{
          "Vector", " ", "of", " ", "length", " ", "N", " ", "with", " ", 
           "the", " ", "total", " ", "excited", " ", "populations", " ", 
           RowBox[{"(", 
            RowBox[{
            "total", " ", "of", " ", "all", " ", "16", " ", "states"}], ")"}],
            " ", "in", " ", "each", " ", "velocity", " ", "class"}], " ", 
          "*)"}], "\[IndentingNewLine]", 
         RowBox[{"wexAllTotal", "=", 
          RowBox[{"Total", "[", 
           RowBox[{
            RowBox[{"Partition", "[", 
             RowBox[{
              RowBox[{"w", "[", 
               RowBox[{"[", "exStates", "]"}], "]"}], ",", "16"}], "]"}], ",", 
            RowBox[{"{", "2", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"tot1", "=", 
          RowBox[{"Total", "[", "wexAllTotal", "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"tot1", ">", "0"}], ",", "\[IndentingNewLine]", 
           RowBox[{"(*", " ", 
            RowBox[{
            "Vector", " ", "of", " ", "length", " ", "8", " ", "containing", 
             " ", "the", " ", "ground", " ", "state", " ", "populations", " ",
              "weighted", " ", "by", " ", "the", " ", "excited", " ", "state",
              " ", "populations"}], " ", "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"popgr8WeightedLoc", "=", 
             RowBox[{
              RowBox[{"Total", "[", 
               RowBox[{
                RowBox[{"wgrAll", "*", "wexAllTotal"}], ",", 
                RowBox[{"{", "1", "}"}]}], "]"}], "/", "tot1"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"(*", " ", 
             RowBox[{"Normalize", " ", "to", " ", "total", " ", "1"}], " ", 
             "*)"}], "\[IndentingNewLine]", 
            RowBox[{"popgr8WeightedLoc", "/=", 
             RowBox[{"Total", "[", "popgr8WeightedLoc", "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"popgr8Weighted", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"popgr8Weighted", ",", 
               RowBox[{"{", "popgr8WeightedLoc", "}"}]}], "]"}]}], ";"}], 
           "\[IndentingNewLine]", ",", " ", 
           RowBox[{"(*", " ", "ELSE", " ", "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"popgr8Weighted", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"popgr8Weighted", ",", 
               RowBox[{"{", 
                RowBox[{"ConstantArray", "[", 
                 RowBox[{
                  RowBox[{"-", "1.0"}], ",", "8"}], "]"}], "}"}]}], "]"}]}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";"}]}], 
       "\[IndentingNewLine]", "\[IndentingNewLine]", "]"}], ";", " ", 
      RowBox[{"(*", " ", 
       RowBox[{"end", " ", "of", " ", 
        RowBox[{"While", "[", "]"}]}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"err", "=", "serror"}], ";", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "w", ",", "err", ",", "obs", ",", "popgr8", ",", "popex16", ",", 
        "popgr8Weighted"}], "}"}]}]}], "\[IndentingNewLine]", "]"}]}], ";", 
  " ", 
  RowBox[{"(*", 
   RowBox[{"End", " ", "of", " ", 
    RowBox[{"Module", "[", "]"}]}], "*)"}]}]}], "Input",
 InitializationCell->True],

Cell[" ", "Text",
 Editable->False,
 Selectable->False,
 CellFrame->{{0, 0}, {0, 0.5}},
 ShowCellBracket->False,
 CellMargins->{{0, 0}, {1, 1}},
 CellElementSpacings->{"CellMinHeight"->1},
 CellFrameMargins->0,
 CellFrameColor->RGBColor[0, 0, 1],
 CellSize->{Inherited, 3}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Inner", " ", "loop", " ", "of", " ", 
    RowBox[{"Expokit", "'"}], "s", " ", "phiv", " ", "routine"}], ",", " ", 
   RowBox[{"sped", " ", "up", " ", "with", " ", 
    RowBox[{
     RowBox[{"Compile", "[", "]"}], ".", " ", "All"}], " ", "floating", " ", 
    "point", " ", "variables", " ", "are", " ", "of", " ", "type", " ", 
    "real"}], ",", " ", 
   RowBox[{"not", " ", 
    RowBox[{"complex", ".", "\[IndentingNewLine]", "Ron"}], " ", 
    "Holzloehner"}], ",", " ", 
   RowBox[{"12", "Sep2011"}]}], "\[IndentingNewLine]", " ", "*)"}]], "Input",
 CellChangeTimes->{{3.5244632226530867`*^9, 3.5244632361220956`*^9}, {
  3.524805254179756*^9, 3.524805301399413*^9}, {3.524805379150906*^9, 
  3.524805393744936*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"phivLoopCompiled", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"m", ",", "_Integer"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n", ",", "_Integer"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"wIn", ",", "_Real", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"u", ",", "_Real", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"istepIn", ",", "_Integer"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"tout", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"tnowIn", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"tnewIn", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"tspanMax", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"sgn", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"tol", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"rndoff", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"anorm", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"xmIn", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"mbIn", ",", "_Integer"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"k1In", ",", "_Integer"}], "}"}]}], "}"}], ",", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"mxrej", "=", "10"}], ",", 
         RowBox[{"btol", "=", "1.0*^-7"}], ",", 
         RowBox[{"gamma", "=", "0.9"}], ",", 
         RowBox[{"delta", "=", "1.2"}], ",", 
         RowBox[{"mb", "=", "mbIn"}], ",", 
         RowBox[{"k1", "=", "k1In"}], ",", 
         RowBox[{"xm", "=", "xmIn"}], ",", 
         RowBox[{"beta", "=", "0.0"}], ",", 
         RowBox[{"fact", "=", "0.0"}], ",", 
         RowBox[{"s", "=", "0.0"}], ",", 
         RowBox[{"tstep", "=", "0.0"}], ",", 
         RowBox[{"Vt", "=", 
          RowBox[{"ConstantArray", "[", 
           RowBox[{"0.0", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"m", "+", "1"}], ",", "n"}], "}"}]}], "]"}]}], ",", 
         RowBox[{"Vt1", "=", 
          RowBox[{
           RowBox[{"multiplyByA", "[", " ", "wIn", "]"}], "+", "u"}]}], ",", 
         RowBox[{"H", "=", 
          RowBox[{"ConstantArray", "[", 
           RowBox[{"0.0", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"m", "+", "3"}], ",", 
              RowBox[{"m", "+", "3"}]}], "}"}]}], "]"}]}], ",", 
         RowBox[{"p", "=", "u"}], ",", "i", ",", "j", ",", 
         RowBox[{"avnorm", "=", "0.0"}], ",", 
         RowBox[{"ireject", "=", "0"}], ",", 
         RowBox[{"mx", "=", "0"}], ",", 
         RowBox[{"errloc", "=", "0.0"}], ",", 
         RowBox[{"p1", "=", "0.0"}], ",", 
         RowBox[{"p2", "=", "0.0"}], ",", 
         RowBox[{"h", "=", "0.0"}], ",", 
         RowBox[{"F", "=", 
          RowBox[{"ConstantArray", "[", 
           RowBox[{"0.0", ",", 
            RowBox[{"{", 
             RowBox[{"2", ",", "2"}], "}"}]}], "]"}]}], ",", 
         RowBox[{"tnow", "=", "tnowIn"}], ",", 
         RowBox[{"tnew", "=", "tnewIn"}], ",", 
         RowBox[{"istep", "=", "istepIn"}], ",", 
         RowBox[{"w", "=", "wIn"}], ",", 
         RowBox[{"Hij", "=", "0.0"}], ",", 
         RowBox[{"Vti", "=", "u"}]}], "}"}], ",", "\[IndentingNewLine]", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Vt", "=", 
          RowBox[{"ConstantArray", "[", 
           RowBox[{"0.0", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"m", "+", "1"}], ",", "n"}], "}"}]}], "]"}]}], ";", 
         RowBox[{"(*", 
          RowBox[{
           RowBox[{"V", "=", 
            RowBox[{"zeros", " ", 
             RowBox[{"(", 
              RowBox[{"n", ",", 
               RowBox[{"m", "+", "1"}]}], ")"}]}]}], ";"}], "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"H", "=", 
          RowBox[{"ConstantArray", "[", 
           RowBox[{"0.0", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"m", "+", "3"}], ",", 
              RowBox[{"m", "+", "3"}]}], "}"}]}], "]"}]}], ";"}], 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{"H", "=", 
           RowBox[{"zeros", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"m", "+", "3"}], ",", 
              RowBox[{"m", "+", "3"}]}], ")"}]}]}], ";"}], "*)"}], 
        "\[IndentingNewLine]", "*)"}], "\[IndentingNewLine]", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{"A", " ", "has", " ", "dimensions", "  ", 
         RowBox[{"{", 
          RowBox[{"n", ",", 
           RowBox[{"Length", "[", "w", "]"}]}], "}"}]}], " ", "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
         RowBox[{
          RowBox[{"Vt1", " ", "=", " ", 
           RowBox[{
            RowBox[{"A", ".", "w"}], "+", "u"}]}], ";"}], "   ", "<", 
         RowBox[{
          RowBox[{"--", " ", "Now"}], " ", "in", " ", "parameter", " ", 
          "initialization"}]}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"beta", "=", 
         RowBox[{"Norm", "[", 
          RowBox[{"Vt1", ",", "2"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
          RowBox[{
           RowBox[{"V", "[", 
            RowBox[{"[", 
             RowBox[{"All", ",", "1"}], "]"}], "]"}], "=", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1.0", "/", "beta"}], ")"}], "*", "w"}]}], ";", "    ", 
          RowBox[{
           RowBox[{"V", " ", 
            RowBox[{"(", 
             RowBox[{":", 
              RowBox[{",", "1"}]}], ")"}]}], "=", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", "/", "beta"}], ")"}], "*", "w"}]}], ";"}], "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"Vt1", "*=", 
         RowBox[{"(", 
          RowBox[{"1.0", "/", "beta"}], ")"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Vt", "[", 
          RowBox[{"[", "1", "]"}], "]"}], "=", "Vt1"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"istep", "==", "0"}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"fact", "=", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{"m", "+", "1.0"}], ")"}], "/", "E"}], ")"}], "^", 
               RowBox[{"(", 
                RowBox[{"m", "+", "1"}], ")"}]}], ")"}], "*", 
             RowBox[{"Sqrt", "[", 
              RowBox[{"2.0", "*", "Pi", "*", 
               RowBox[{"(", 
                RowBox[{"m", "+", "1"}], ")"}]}], "]"}]}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"tnew", "=", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1.0", "/", "anorm"}], ")"}], "*", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"fact", "*", "tol"}], ")"}], "/", 
                RowBox[{"(", 
                 RowBox[{"4", "*", "beta", "*", "anorm"}], ")"}]}], ")"}], 
              "^", "xm"}]}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"s", "=", 
            RowBox[{"10.0", "^", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Floor", "[", 
                RowBox[{"Log10", "[", "tnew", "]"}], "]"}], "-", "1"}], 
              ")"}]}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"tnew", "=", 
            RowBox[{
             RowBox[{"Ceiling", "[", 
              RowBox[{"tnew", "/", "s"}], "]"}], "*", "s"}]}], ";"}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"istep", "++"}], ";", "\[IndentingNewLine]", 
        RowBox[{"tstep", "=", 
         RowBox[{"Min", "[", 
          RowBox[{
           RowBox[{"tout", "-", "tnow"}], ",", "tnew"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"tstep", "=", 
         RowBox[{"Min", "[", 
          RowBox[{"tstep", ",", "tspanMax"}], "]"}]}], ";", 
        "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"For", "[", 
         RowBox[{
          RowBox[{"j", "=", "1"}], ",", 
          RowBox[{"j", "<=", "m"}], ",", 
          RowBox[{"j", "++"}], ",", 
          RowBox[{"(*", 
           RowBox[{
            RowBox[{"for", " ", "j"}], "=", 
            RowBox[{"1", ":", "m"}]}], "*)"}], "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"p", "=", 
            RowBox[{"multiplyByA", "[", 
             RowBox[{"Vt", "[", 
              RowBox[{"[", "j", "]"}], "]"}], "]"}]}], ";", 
           RowBox[{"(*", 
            RowBox[{
             RowBox[{"p", "=", 
              RowBox[{"A", "*", "V", " ", 
               RowBox[{"(", 
                RowBox[{":", 
                 RowBox[{",", "j"}]}], ")"}]}]}], ";"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{"(*", " ", 
            RowBox[{
            "The", " ", "following", " ", "loop", " ", "eats", " ", "up", " ",
              "80", "%", " ", "or", " ", "so", " ", "of", " ", "the", " ", 
             "total", " ", "CPU", " ", "time"}], " ", "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{"For", "[", 
            RowBox[{
             RowBox[{"i", "=", "1"}], ",", 
             RowBox[{"i", "<=", "j"}], ",", 
             RowBox[{"i", "++"}], ",", 
             RowBox[{"(*", 
              RowBox[{
               RowBox[{"for", " ", "i"}], "=", 
               RowBox[{"1", ":", "j"}]}], "*)"}], "\[IndentingNewLine]", 
             RowBox[{"(*", " ", 
              RowBox[{"WARNING", ":", " ", 
               RowBox[{
               "The", " ", "following", " ", "line", " ", "should", " ", 
                "use", " ", "the", " ", 
                RowBox[{"Conjugate", "[", "]"}], " ", 
                RowBox[{"function", ".", " ", "We"}], " ", "omit", " ", "it", 
                " ", "to", " ", "speed", " ", "up", " ", "the", " ", "code", 
                " ", "for", " ", "type", " ", "real", " ", 
                RowBox[{
                 RowBox[{"only", "!!"}], "!"}]}]}], " ", "*)"}], 
             "\[IndentingNewLine]", 
             RowBox[{"(*", " ", 
              RowBox[{
               RowBox[{
                RowBox[{"H", "[", 
                 RowBox[{"[", 
                  RowBox[{"i", ",", "j"}], "]"}], "]"}], "=", 
                RowBox[{
                 RowBox[{"Conjugate", "[", 
                  RowBox[{"Vt", "[", 
                   RowBox[{"[", "i", "]"}], "]"}], "]"}], ".", "p"}]}], ";"}], 
              RowBox[{"(*", 
               RowBox[{
                RowBox[{
                 RowBox[{"H", " ", 
                  RowBox[{"(", 
                   RowBox[{"i", ",", "j"}], ")"}]}], "=", 
                 RowBox[{"V", " ", 
                  RowBox[{
                   RowBox[{"(", 
                    RowBox[{":", 
                    RowBox[{",", "i"}]}], ")"}], "'"}], "*", "p"}]}], ";"}], 
               "*)"}], "*)"}], "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"Vti", "=", 
               RowBox[{"Vt", "[", 
                RowBox[{"[", "i", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
              RowBox[{"(*", " ", 
               RowBox[{
                RowBox[{
                 RowBox[{"H", "[", 
                  RowBox[{"[", 
                   RowBox[{"i", ",", "j"}], "]"}], "]"}], "=", 
                 RowBox[{"Vti", ".", "p"}]}], ";"}], "*)"}], 
              "\[IndentingNewLine]", 
              RowBox[{"Hij", "=", 
               RowBox[{"Vti", ".", "p"}]}], ";", "\[IndentingNewLine]", 
              RowBox[{"p", "-=", 
               RowBox[{"Hij", " ", "*", " ", "Vti"}]}], ";", 
              RowBox[{"(*", 
               RowBox[{
                RowBox[{"p", "=", 
                 RowBox[{"p", "-", 
                  RowBox[{"H", " ", 
                   RowBox[{"(", 
                    RowBox[{"i", ",", "j"}], ")"}], "*", "V", " ", 
                   RowBox[{"(", 
                    RowBox[{":", 
                    RowBox[{",", "i"}]}], ")"}]}]}]}], ";"}], "*)"}], 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"H", "[", 
                RowBox[{"[", 
                 RowBox[{"i", ",", "j"}], "]"}], "]"}], "=", "Hij"}], ";"}]}],
             "\[IndentingNewLine]", "]"}], ";", 
           RowBox[{"(*", 
            RowBox[{"for", " ", "i"}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{"s", "=", 
            RowBox[{"Norm", "[", 
             RowBox[{"p", ",", "2"}], "]"}]}], ";", 
           RowBox[{"(*", 
            RowBox[{
             RowBox[{"s", "=", 
              RowBox[{"norm", " ", 
               RowBox[{"(", "p", ")"}]}]}], ";"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"s", "<", "btol"}], ",", 
             RowBox[{"(*", 
              RowBox[{
               RowBox[{
                RowBox[{"if", " ", "s"}], "<", "btol"}], ","}], "*)"}], 
             "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"k1", "=", "0"}], ";", "\[IndentingNewLine]", 
              RowBox[{"mb", "=", "j"}], ";", "\[IndentingNewLine]", 
              RowBox[{"tstep", "=", 
               RowBox[{"tout", "-", "tnow"}]}], ";", "\[IndentingNewLine]", 
              RowBox[{"tstep", "=", 
               RowBox[{"Min", "[", 
                RowBox[{"tstep", ",", "tspanMax"}], "]"}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"Break", "[", "]"}]}]}], 
            RowBox[{"(*", 
             RowBox[{"break", ";"}], "*)"}], "\[IndentingNewLine]", "]"}], 
           ";", 
           RowBox[{"(*", " ", "if", " ", "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"H", "[", 
             RowBox[{"[", 
              RowBox[{
               RowBox[{"j", "+", "1"}], ",", "j"}], "]"}], "]"}], "=", "s"}], 
           ";", 
           RowBox[{"(*", 
            RowBox[{
             RowBox[{
              RowBox[{"H", " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"j", "+", "1"}], ",", "j"}], ")"}]}], "=", "s"}], 
             ";"}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{"(*", 
            RowBox[{
             RowBox[{
              RowBox[{"V", " ", 
               RowBox[{"(", 
                RowBox[{":", 
                 RowBox[{",", 
                  RowBox[{"j", "+", "1"}]}]}], ")"}]}], "=", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"1", "/", "s"}], ")"}], "*", "p"}]}], ";"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Vt", "[", 
             RowBox[{"[", 
              RowBox[{"j", "+", "1"}], "]"}], "]"}], "=", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1.0", "/", "s"}], ")"}], "*", "p"}]}], ";"}]}], 
         "\[IndentingNewLine]", "]"}], ";", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{"for", " ", "j"}], ";"}], "*)"}], "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"H", "[", 
          RowBox[{"[", 
           RowBox[{"1", ",", 
            RowBox[{"mb", "+", "1"}]}], "]"}], "]"}], "=", "1"}], ";", 
        "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"k1", "!=", "0"}], ",", 
          RowBox[{"(*", 
           RowBox[{
            RowBox[{"if", " ", 
             RowBox[{"k1", "~", 
              RowBox[{"=", "0"}]}]}], ","}], "*)"}], "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{"H", "[", 
             RowBox[{"[", 
              RowBox[{
               RowBox[{"m", "+", "1"}], ",", 
               RowBox[{"m", "+", "2"}]}], "]"}], "]"}], "=", "1.0"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"H", "[", 
             RowBox[{"[", 
              RowBox[{
               RowBox[{"m", "+", "2"}], ",", 
               RowBox[{"m", "+", "3"}]}], "]"}], "]"}], "=", "1.0"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"h", "=", 
            RowBox[{"H", "[", 
             RowBox[{"[", 
              RowBox[{
               RowBox[{"m", "+", "1"}], ",", "m"}], "]"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"H", "[", 
             RowBox[{"[", 
              RowBox[{
               RowBox[{"m", "+", "1"}], ",", "m"}], "]"}], "]"}], "=", 
            "0.0"}], ";", "\[IndentingNewLine]", 
           RowBox[{"(*", 
            RowBox[{
             RowBox[{"avnorm", "=", 
              RowBox[{"norm", " ", 
               RowBox[{"(", 
                RowBox[{"A", "*", "V", " ", 
                 RowBox[{"(", 
                  RowBox[{":", 
                   RowBox[{",", 
                    RowBox[{"m", "+", "1"}]}]}], ")"}]}], ")"}]}]}], ";"}], 
            "*)"}], "\[IndentingNewLine]", 
           RowBox[{"avnorm", "=", 
            RowBox[{"Norm", "[", 
             RowBox[{"multiplyByA", "[", 
              RowBox[{"Vt", "[", 
               RowBox[{"[", 
                RowBox[{"m", "+", "1"}], "]"}], "]"}], "]"}], " ", "]"}]}], 
           ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
        RowBox[{"(*", "if", "*)"}], "\[IndentingNewLine]", 
        "\[IndentingNewLine]", 
        RowBox[{"ireject", "=", "0"}], ";", "\[IndentingNewLine]", 
        RowBox[{"While", "[", 
         RowBox[{
          RowBox[{"ireject", "<=", "mxrej"}], ",", 
          RowBox[{"(*", 
           RowBox[{
            RowBox[{
             RowBox[{"while", " ", "ireject"}], "<=", "mxrej"}], ","}], 
           "*)"}], "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"mx", "=", 
            RowBox[{"mb", "+", 
             RowBox[{"Max", "[", 
              RowBox[{"1", ",", "k1"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"(*", " ", 
            RowBox[{
             RowBox[{
              RowBox[{"If", " ", 
               RowBox[{"Range", "[", 
                RowBox[{"1", ",", "mx"}], "]"}], " ", "is", " ", "replaced", 
               " ", "by", " ", "1"}], ";;", "mx"}], ",", " ", 
             RowBox[{
             "we", " ", "get", " ", "a", " ", "compilation", " ", "error"}]}],
             " ", "*)"}], "\[IndentingNewLine]", 
           RowBox[{"(*", " ", 
            RowBox[{
             RowBox[{"Re", "[", "]"}], " ", "added", " ", "on", " ", "09", 
             "Sep11", " ", "since", " ", 
             RowBox[{"MatrixExp", "[", "]"}], " ", "sometimes", " ", 
             "returns", " ", "complex", " ", "numbers"}], " ", "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{"(*", " ", 
            RowBox[{"Runtime", " ", "of", " ", 
             RowBox[{"MatrixExp", "[", "]"}], " ", "was", " ", "checked", " ",
              "with", " ", 
             RowBox[{"Timing", "[", "]"}], " ", "and", " ", "was", " ", 
             "negligible"}], " ", "*)"}], "\[IndentingNewLine]", 
           RowBox[{"F", "=", 
            RowBox[{"Re", "[", 
             RowBox[{"MatrixExp", "[", " ", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"sgn", "*", "tstep"}], ")"}], "*", " ", 
               RowBox[{"H", "[", 
                RowBox[{"[", 
                 RowBox[{
                  RowBox[{"Range", "[", 
                   RowBox[{"1", ",", "mx"}], "]"}], ",", 
                  RowBox[{"1", ";;", "mx"}]}], "]"}], "]"}]}], " ", "]"}], 
             "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"(*", 
            RowBox[{
             RowBox[{"F", "=", 
              RowBox[{"expm", " ", 
               RowBox[{"(", 
                RowBox[{"sgn", "*", "t_step", "*", "H", " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"1", ":", "mx"}], ",", 
                   RowBox[{"1", ":", "mx"}]}], ")"}]}], ")"}]}]}], ";"}], 
            "*)"}], "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"k1", "==", "0"}], ",", 
             RowBox[{"(*", 
              RowBox[{
               RowBox[{
                RowBox[{"if", " ", "k1"}], "==", "0"}], ","}], "*)"}], 
             "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"errloc", "=", "btol"}], ";", "\[IndentingNewLine]", 
              RowBox[{"Break", "[", "]"}], ";"}], ",", 
             RowBox[{"(*", 
              RowBox[{"break", ";"}], "*)"}], "\[IndentingNewLine]", 
             RowBox[{"(*", " ", "Else", " ", "*)"}], "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{
               RowBox[{"F", "[", 
                RowBox[{"[", 
                 RowBox[{
                  RowBox[{"m", "+", "1"}], ",", 
                  RowBox[{"m", "+", "1"}]}], "]"}], "]"}], "=", 
               RowBox[{"h", "*", 
                RowBox[{"F", "[", 
                 RowBox[{"[", 
                  RowBox[{"m", ",", 
                   RowBox[{"m", "+", "2"}]}], "]"}], "]"}]}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"F", "[", 
                RowBox[{"[", 
                 RowBox[{
                  RowBox[{"m", "+", "2"}], ",", 
                  RowBox[{"m", "+", "1"}]}], "]"}], "]"}], "=", 
               RowBox[{"h", "*", 
                RowBox[{"F", "[", 
                 RowBox[{"[", 
                  RowBox[{"m", ",", 
                   RowBox[{"m", "+", "3"}]}], "]"}], "]"}]}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"p1", "=", 
               RowBox[{"Abs", "[", 
                RowBox[{"beta", "*", 
                 RowBox[{"F", "[", 
                  RowBox[{"[", 
                   RowBox[{
                    RowBox[{"m", "+", "1"}], ",", 
                    RowBox[{"m", "+", "1"}]}], "]"}], "]"}]}], "]"}]}], ";", 
              RowBox[{"(*", 
               RowBox[{
                RowBox[{"abs", " ", 
                 RowBox[{"(", 
                  RowBox[{"beta", "*", "F", " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"m", "+", "1"}], ",", 
                    RowBox[{"m", "+", "1"}]}], ")"}]}], ")"}]}], ";"}], 
               "*)"}], "\[IndentingNewLine]", 
              RowBox[{"p2", "=", 
               RowBox[{"Abs", "[", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{"beta", "*", "avnorm"}], ")"}], "*", 
                 RowBox[{"F", "[", 
                  RowBox[{"[", 
                   RowBox[{
                    RowBox[{"m", "+", "2"}], ",", 
                    RowBox[{"m", "+", "1"}]}], "]"}], "]"}]}], "]"}]}], ";", 
              RowBox[{"(*", 
               RowBox[{
                RowBox[{"abs", " ", 
                 RowBox[{"(", 
                  RowBox[{"beta", "*", "F", " ", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{"m", "+", "2"}], ",", 
                    RowBox[{"m", "+", "1"}]}], ")"}], "*", "avnorm"}], 
                  ")"}]}], ";"}], "*)"}], "\[IndentingNewLine]", 
              "\[IndentingNewLine]", 
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{"p1", ">", 
                 RowBox[{"10.0", "*", "p2"}]}], ",", 
                RowBox[{"(*", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"if", " ", "phi1"}], ">", 
                   RowBox[{"10", "*", "phi2"}]}], ","}], "*)"}], 
                "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"errloc", "=", "p2"}], ";", "\[IndentingNewLine]", 
                 RowBox[{"xm", "=", 
                  RowBox[{"1.0", "/", "m"}]}], ";"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"(*", " ", "Else", " ", "*)"}], "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"If", "[", 
                  RowBox[{
                   RowBox[{"p1", ">", "p2"}], ",", "\[IndentingNewLine]", 
                   RowBox[{
                    RowBox[{"errloc", "=", 
                    RowBox[{
                    RowBox[{"(", 
                    RowBox[{"p1", "*", "p2"}], ")"}], "/", 
                    RowBox[{"(", 
                    RowBox[{"p1", "-", "p2"}], ")"}]}]}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{"xm", "=", 
                    RowBox[{"1.0", "/", "m"}]}], ";"}], ",", 
                   "\[IndentingNewLine]", 
                   RowBox[{"(*", "Else", "*)"}], "\[IndentingNewLine]", 
                   RowBox[{
                    RowBox[{"errloc", "=", "p1"}], ";", "\[IndentingNewLine]", 
                    RowBox[{"xm", "=", 
                    RowBox[{"1.0", "/", 
                    RowBox[{"(", 
                    RowBox[{"m", "-", "1"}], ")"}]}]}], ";"}]}], 
                  "\[IndentingNewLine]", "]"}], ";"}]}], 
               "\[IndentingNewLine]", "]"}], ";"}]}], " ", 
            "\[IndentingNewLine]", "]"}], ";", 
           RowBox[{"(*", " ", 
            RowBox[{
             RowBox[{"If", " ", "k"}], "==", "1"}], " ", "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"errloc", "<=", 
              RowBox[{"delta", "*", "tstep", "*", "tol"}]}], ",", 
             RowBox[{"(*", 
              RowBox[{
               RowBox[{
                RowBox[{"if", " ", "err_loc"}], "<=", 
                RowBox[{"delta", "*", "t_step", "*", "tol"}]}], ","}], "*)"}],
              "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"Break", "[", "]"}], ";"}], ",", "\[IndentingNewLine]", 
             RowBox[{"(*", "Else", "*)"}], "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"tstep", "=", 
               RowBox[{"gamma", "*", "tstep", "*", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{"tstep", "*", 
                   RowBox[{"tol", "/", "errloc"}]}], ")"}], "^", "xm"}]}]}], 
              ";", "\[IndentingNewLine]", 
              RowBox[{"tstep", "=", 
               RowBox[{"Min", "[", 
                RowBox[{"tstep", ",", "tspanMax"}], "]"}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"s", "=", 
               RowBox[{"10.0", "^", 
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"Floor", "[", 
                   RowBox[{"Log10", "[", "tstep", "]"}], "]"}], "-", "1"}], 
                 ")"}]}]}], ";", "\[IndentingNewLine]", 
              RowBox[{"tstep", "=", 
               RowBox[{
                RowBox[{"Ceiling", "[", 
                 RowBox[{"tstep", "/", "s"}], "]"}], "*", "s"}]}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{"ireject", "==", "mxrej"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{
                 "Print", "[", 
                  "\"\<phivLoopCompiled[]: The requested tolerance is too \
high!\>\"", "]"}], ";", "\[IndentingNewLine]", 
                 RowBox[{"Abort", "[", "]"}], ";"}]}], "\[IndentingNewLine]", 
               "]"}], ";", "\[IndentingNewLine]", 
              RowBox[{"ireject", "++"}], ";"}]}], "\[IndentingNewLine]", 
            "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
        RowBox[{"(*", " ", 
         RowBox[{"While", "[", 
          RowBox[{
           RowBox[{"ireject", "<=", "mxrej"}], ",", "..."}], "]"}], " ", 
         "*)"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"mx", "=", 
         RowBox[{"mb", "+", 
          RowBox[{"Max", "[", 
           RowBox[{"0", ",", 
            RowBox[{"k1", "-", "2"}]}], "]"}]}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"w", "=", 
         RowBox[{
          RowBox[{
           RowBox[{"Transpose", "[", 
            RowBox[{"Vt", "[", 
             RowBox[{"[", 
              RowBox[{"1", ";;", "mx"}], "]"}], "]"}], "]"}], ".", 
           RowBox[{"(", 
            RowBox[{"beta", "*", 
             RowBox[{"F", "[", 
              RowBox[{"[", 
               RowBox[{
                RowBox[{"1", ";;", "mx"}], ",", 
                RowBox[{"mb", "+", "1"}]}], "]"}], "]"}]}], ")"}]}], " ", "+",
           " ", "w"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{"w", "=", 
           RowBox[{
            RowBox[{"V", " ", 
             RowBox[{"(", 
              RowBox[{":", 
               RowBox[{",", 
                RowBox[{"1", ":", "mx"}]}]}], ")"}], "*", 
             RowBox[{"(", 
              RowBox[{"beta", "*", "F", " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"1", ":", "mx"}], ",", 
                 RowBox[{"mb", "+", "1"}]}], ")"}]}], ")"}]}], " ", "+", " ", 
            "w"}]}], ";"}], "*)"}], "\[IndentingNewLine]", 
        "\[IndentingNewLine]", 
        RowBox[{"tnow", "+=", "tstep"}], ";", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{"t_now", "=", 
           RowBox[{"t_now", "+", "t_step"}]}], ";"}], "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"tnew", "=", 
         RowBox[{"gamma", "*", "tstep", "*", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"tstep", "*", 
             RowBox[{"tol", "/", "errloc"}]}], ")"}], "^", "xm"}]}]}], ";", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{"t_new", "=", 
           RowBox[{"gamma", "*", "t_step", "*", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"t_step", "*", 
               RowBox[{"tol", "/", "err_loc"}]}], ")"}], "^", "xm"}]}]}], 
          ";"}], "*)"}], "\[IndentingNewLine]", 
        RowBox[{"s", "=", 
         RowBox[{"10.0", "^", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Floor", "[", 
             RowBox[{"Log10", "[", "tnew", "]"}], "]"}], "-", "1"}], 
           ")"}]}]}], ";", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{"s", "=", 
           RowBox[{"10", "^", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"floor", " ", 
               RowBox[{"(", 
                RowBox[{"log10", " ", 
                 RowBox[{"(", "t_new", ")"}]}], ")"}]}], "-", "1"}], 
             ")"}]}]}], ";"}], "*)"}], "\[IndentingNewLine]", 
        RowBox[{"tnew", "=", 
         RowBox[{
          RowBox[{"Ceiling", "[", 
           RowBox[{"tnew", "/", "s"}], "]"}], "*", "s"}]}], ";", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{"t_new", "=", 
           RowBox[{"ceil", " ", 
            RowBox[{"(", 
             RowBox[{"t_new", "/", "s"}], ")"}], "*", "s"}]}], ";"}], "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"errloc", "=", 
         RowBox[{"Max", "[", 
          RowBox[{"errloc", ",", "rndoff"}], "]"}]}], ";", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{"err_loc", "=", 
           RowBox[{"max", " ", 
            RowBox[{"(", 
             RowBox[{"err_loc", ",", "rndoff"}], ")"}]}]}], ";"}], "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"(*", "  ", 
         RowBox[{
          RowBox[{
           RowBox[{
           "This", " ", "command", " ", "has", " ", "been", " ", "moved", " ",
             "up", " ", "to", " ", "the", " ", "calling", " ", "routine", 
            "\[IndentingNewLine]", "serror"}], "+=", "errloc"}], ";"}], 
         "*)"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
         "Combine", " ", "everything", " ", "in", " ", "one", " ", "vector", 
          " ", "since", " ", 
          RowBox[{"Compile", "[", "]"}], " ", "cannot", " ", "return", " ", 
          "irregular", " ", "arrays"}], " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{"Join", "[", 
         RowBox[{"w", ",", 
          RowBox[{"N", "[", 
           RowBox[{"{", 
            RowBox[{
            "istep", ",", "tstep", ",", "tnew", ",", "tnow", ",", "errloc", 
             ",", "s", ",", "xm", ",", "mb"}], "}"}], "]"}]}], "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], " ", 
     RowBox[{"(*", " ", 
      RowBox[{"End", " ", "of", " ", "Module"}], " ", "*)"}], 
     "\[IndentingNewLine]", ",", 
     RowBox[{"Evaluate", "[", 
      RowBox[{"Sequence", "@@", "compileropts"}], "]"}]}], 
    "\[IndentingNewLine]", "]"}]}], ";", 
  RowBox[{"(*", " ", 
   RowBox[{"End", " ", "of", " ", "Compile"}], " ", "*)"}]}]], "Input",
 InitializationCell->True],

Cell[" ", "Text",
 Editable->False,
 Selectable->False,
 CellFrame->{{0, 0}, {0, 0.5}},
 ShowCellBracket->False,
 CellMargins->{{0, 0}, {1, 1}},
 CellElementSpacings->{"CellMinHeight"->1},
 CellFrameMargins->0,
 CellFrameColor->RGBColor[0, 0, 1],
 CellSize->{Inherited, 3}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "Multiply", " ", "a", " ", "vector", " ", "v", " ", "by", " ", "the", " ", 
    "global", " ", "matrix", " ", "variable", " ", "Global`fullAmatG", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
      "which", " ", "may", " ", "be", " ", "a", " ", "sparse", " ", 
       "matrix"}], ",", " ", 
      RowBox[{"in", " ", "which", " ", "case", " ", 
       RowBox[{"Compile", "[", "]"}], " ", 
       RowBox[{"couldn", "'"}], "t", " ", "handle", " ", "the", " ", 
       "multiplication"}]}], ")"}]}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"multiplyByA", "[", "v_", "]"}], ":=", 
   RowBox[{"(", 
    RowBox[{"Global`fullAmatG", ".", "v"}], ")"}]}]}]], "Input",
 InitializationCell->True],

Cell[" ", "Text",
 Editable->False,
 Selectable->False,
 CellFrame->{{0, 0}, {0, 0.5}},
 ShowCellBracket->False,
 CellMargins->{{0, 0}, {1, 1}},
 CellElementSpacings->{"CellMinHeight"->1},
 CellFrameMargins->0,
 CellFrameColor->RGBColor[0, 0, 1],
 CellSize->{Inherited, 3}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Stripped", " ", "version", " ", "of", " ", 
   RowBox[{"phiv", "[", "]"}], " ", "to", " ", "work", " ", "with", " ", 
   "with", " ", 
   RowBox[{"Mathematica", "'"}], "s", " ", 
   RowBox[{"MatrixExp", "[", "]"}], " ", "function", " ", 
   RowBox[{"(", 
    RowBox[{"see", " ", 
     RowBox[{"StepPulseFlux", "[", "]"}]}], ")"}]}], " ", "*)"}]], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "phivMatrixExp", "]"}], "=", 
   RowBox[{"{", 
    RowBox[{"VerbosityLevel", "->", "2"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"phivMatrixExp", "[", 
    RowBox[{
    "t_", ",", "w_", ",", "grStates_", ",", "exStates_", ",", "obsmat_", ",", 
     RowBox[{"indVgMaxEx_:", "-", "1"}], ",", 
     RowBox[{"OptionsPattern", "[", "]"}]}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "verb", ",", "popgr8", ",", "popex16", ",", "wgr", ",", "wex", ",", 
       "tot1", ",", "popgr8Weighted", ",", "wgrAll", ",", "wexAllTotal", ",", 
       "err", ",", "off1", ",", "obs", ",", "popgr8WeightedLoc"}], "}"}], ",",
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"MatrixQ", "[", 
          RowBox[{
           RowBox[{"{", "w", "}"}], ",", "NumericQ"}], "]"}], "==", "False"}],
         " ", ",", 
        RowBox[{
         RowBox[{
         "Print", "[", "\"\<phivMatrixExp[]: w is nonnumeric\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}], ";"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"MatrixQ", "[", 
           RowBox[{
            RowBox[{"{", "obsmat", "}"}], ",", "NumericQ"}], "]"}], "==", 
          "False"}], " ", "&&", 
         RowBox[{"obsmat", "!=", 
          RowBox[{"{", "}"}]}]}], ",", 
        RowBox[{
         RowBox[{
         "Print", "[", "\"\<phivCompiled[]: resultmat is nonnumeric\>\"", 
          "]"}], ";", 
         RowBox[{"Abort", "[", "]"}], ";"}]}], "]"}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"verb", "=", 
       RowBox[{"OptionValue", "[", "VerbosityLevel", "]"}]}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"popgr8", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"popex16", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"popgr8Weighted", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"obs", "=", 
       RowBox[{"{", "}"}]}], ";", " ", 
      RowBox[{"(*", " ", 
       RowBox[{
       "Used", " ", "to", " ", "accumulate", " ", "return", " ", "flux", " ", 
        "samples"}], " ", "*)"}], "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"Code", " ", "inserted", " ", "by", " ", "Ron", " ", 
        RowBox[{"Holzloehner", ":", " ", 
         RowBox[{
         "Compute", " ", "the", " ", "return", " ", "flux", " ", "at", " ", 
          "tnow"}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"AppendTo", "[", 
       RowBox[{"obs", ",", 
        RowBox[{"{", 
         RowBox[{"t", ",", 
          RowBox[{"Re", "[", 
           RowBox[{"obsmat", ".", "w"}], "]"}]}], "}"}]}], "]"}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"indVgMaxEx", ">=", "1"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"off1", "=", 
          RowBox[{"8", "*", 
           RowBox[{"(", 
            RowBox[{"indVgMaxEx", "-", "1"}], ")"}]}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"wgr", "=", 
          RowBox[{
           RowBox[{"w", "[", 
            RowBox[{"[", "grStates", "]"}], "]"}], "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", "+", "off1"}], ")"}], ";;", 
             RowBox[{"(", 
              RowBox[{"8", "+", "off1"}], ")"}]}], "]"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"off1", "=", 
          RowBox[{"16", "*", 
           RowBox[{"(", 
            RowBox[{"indVgMaxEx", "-", "1"}], ")"}]}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"wex", "=", 
          RowBox[{
           RowBox[{"w", "[", 
            RowBox[{"[", "exStates", "]"}], "]"}], "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", "+", "off1"}], ")"}], ";;", 
             RowBox[{"(", 
              RowBox[{"16", "+", "off1"}], ")"}]}], "]"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{
          "Normalize", " ", "by", " ", "number", " ", "of", " ", "atoms", " ",
            "in", " ", "this", " ", "velocity", " ", "bin"}], " ", "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"tot1", "=", 
          RowBox[{"N", "[", 
           RowBox[{
            RowBox[{"Total", "[", "wgr", "]"}], "+", 
            RowBox[{"Total", "[", "wex", "]"}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"popgr8", "=", 
          RowBox[{"Join", "[", 
           RowBox[{"popgr8", ",", 
            RowBox[{"{", 
             RowBox[{"wgr", "/", "tot1"}], "}"}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"popex16", "=", 
          RowBox[{"Join", "[", 
           RowBox[{"popex16", ",", 
            RowBox[{"{", 
             RowBox[{"wex", "/", "tot1"}], "}"}]}], "]"}]}], ";"}], 
        "\[IndentingNewLine]", ",", " ", 
        RowBox[{"(*", "else", "*)"}], "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"popgr8", "=", 
          RowBox[{"Join", "[", 
           RowBox[{"popgr8", ",", 
            RowBox[{"{", 
             RowBox[{"ConstantArray", "[", 
              RowBox[{
               RowBox[{"-", "1.0"}], ",", "8"}], "]"}], "}"}]}], "]"}]}], ";",
          "\[IndentingNewLine]", 
         RowBox[{"popex16", "=", 
          RowBox[{"Join", "[", 
           RowBox[{"popex16", ",", 
            RowBox[{"{", 
             RowBox[{"ConstantArray", "[", 
              RowBox[{
               RowBox[{"-", "1.0"}], ",", "16"}], "]"}], "}"}]}], "]"}]}], 
         ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
        RowBox[{"Nx8", " ", "array"}], ",", " ", 
        RowBox[{
        "where", " ", "N", " ", "is", " ", "the", " ", "number", " ", "of", 
         " ", "velocity", " ", "classes"}]}], "  ", "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"wgrAll", "=", 
       RowBox[{"Partition", "[", 
        RowBox[{
         RowBox[{"w", "[", 
          RowBox[{"[", "grStates", "]"}], "]"}], ",", "8"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "Vector", " ", "of", " ", "length", " ", "N", " ", "with", " ", "the", 
        " ", "total", " ", "excited", " ", "populations", " ", 
        RowBox[{"(", 
         RowBox[{"total", " ", "of", " ", "all", " ", "16", " ", "states"}], 
         ")"}], " ", "in", " ", "each", " ", "velocity", " ", "class"}], " ", 
       "*)"}], "\[IndentingNewLine]", 
      RowBox[{"wexAllTotal", "=", 
       RowBox[{"Total", "[", 
        RowBox[{
         RowBox[{"Partition", "[", 
          RowBox[{
           RowBox[{"w", "[", 
            RowBox[{"[", "exStates", "]"}], "]"}], ",", "16"}], "]"}], ",", 
         RowBox[{"{", "2", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"tot1", "=", 
       RowBox[{"Total", "[", "wexAllTotal", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"tot1", ">", "0"}], ",", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
         "Vector", " ", "of", " ", "length", " ", "8", " ", "containing", " ",
           "the", " ", "ground", " ", "state", " ", "populations", " ", 
          "weighted", " ", "by", " ", "the", " ", "excited", " ", "state", 
          " ", "populations"}], " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"popgr8WeightedLoc", "=", 
          RowBox[{
           RowBox[{"Total", "[", 
            RowBox[{
             RowBox[{"wgrAll", "*", "wexAllTotal"}], ",", 
             RowBox[{"{", "1", "}"}]}], "]"}], "/", "tot1"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{"Normalize", " ", "to", " ", "total", " ", "1"}], " ", 
          "*)"}], "\[IndentingNewLine]", 
         RowBox[{"popgr8WeightedLoc", "/=", 
          RowBox[{"Total", "[", "popgr8WeightedLoc", "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"popgr8Weighted", "=", 
          RowBox[{"Join", "[", 
           RowBox[{"popgr8Weighted", ",", 
            RowBox[{"{", "popgr8WeightedLoc", "}"}]}], "]"}]}], ";"}], 
        "\[IndentingNewLine]", ",", " ", 
        RowBox[{"(*", " ", "ELSE", " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"popgr8Weighted", "=", 
          RowBox[{"Join", "[", 
           RowBox[{"popgr8Weighted", ",", 
            RowBox[{"{", 
             RowBox[{"ConstantArray", "[", 
              RowBox[{
               RowBox[{"-", "1.0"}], ",", "8"}], "]"}], "}"}]}], "]"}]}], 
         ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"err", "=", "0.0"}], ";", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "err", ",", "obs", ",", "popgr8", ",", "popex16", ",", 
        "popgr8Weighted"}], "}"}]}]}], "\[IndentingNewLine]", "]"}]}], ";", 
  " ", 
  RowBox[{"(*", 
   RowBox[{"End", " ", "of", " ", 
    RowBox[{"Module", "[", "]"}], " ", "phivMatrixExp"}], " ", 
   "*)"}]}]}], "Input",
 InitializationCell->True],

Cell[" ", "Text",
 Editable->False,
 Selectable->False,
 CellFrame->{{0, 0}, {0, 2}},
 ShowCellBracket->False,
 CellMargins->{{0, 0}, {1, 1}},
 CellElementSpacings->{"CellMinHeight"->1},
 CellFrameMargins->0,
 CellFrameColor->RGBColor[0, 0, 1],
 CellSize->{Inherited, 4}]
}, Open  ]],

Cell[CellGroupData[{

Cell["LGS Constants and Default Parameters", "Subsection"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"BGParanal", "=", "0.22750"}], ";"}], "  ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "geomagnetic", " ", "field", " ", "in", " ", "Paranal", " ", "at", " ", 
    "altitude", " ", "90", "km", " ", "in", " ", 
    RowBox[{"2009", " ", "[", "G", "]"}], " ", 
    RowBox[{"from", " ", "[", "MSIS90", "]"}]}], 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"BGSOR", "=", "0.48052"}], ";"}], "  ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "geomagnetic", " ", "field", " ", "at", " ", "SOR", " ", "at", " ", 
    "altutude", " ", "90", "km", " ", "in", " ", 
    RowBox[{"2009", " ", "[", "G", "]"}], " ", 
    RowBox[{"from", " ", "[", "MSIS90", "]"}]}], 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"BGWendelstein", "=", "0.46044"}], ";"}], "  ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "geomagnetic", " ", "field", " ", "over", " ", "Wendelstein", " ", "at", 
    " ", "altutude", " ", "90", "km", " ", "in", " ", 
    RowBox[{"2009", " ", "[", "G", "]"}], " ", 
    RowBox[{"from", " ", "[", "MSIS90", "]"}]}], 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"BGMaunaKea", "=", "0.35"}], ";"}], "  ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "geomagnetic", " ", "field", " ", "over", " ", "Mauna", " ", "Kea", " ", 
    "at", " ", "altutude", " ", "90", "km", " ", "in", " ", 
    RowBox[{"2009", " ", "[", "G", "]"}], " ", 
    RowBox[{"from", " ", "[", "MSIS90", "]"}]}], 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalOmega]NaRecoil", "=", 
    RowBox[{"2", "\[Pi]", " ", "50.", " ", 
     SuperscriptBox["10", "3"]}]}], ";"}], "  ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "recoil", " ", "frequency", " ", "of", " ", "sodium", " ", "atoms", " ", 
    "at", " ", "589", "nm"}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[Gamma]2FWHM", "=", 
       SqrtBox[
        RowBox[{
         RowBox[{"-", "1.0"}], "+", 
         SqrtBox["2"]}]]}], ";"}], "  ", "*", " ", "Conversion", " ", 
     "factor", " ", "from", " ", "Lorentzian"}], "-", 
    RowBox[{"square", " ", "\[Gamma]", " ", "to", " ", "FWHM"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"w2f", "=", 
    RowBox[{"Sqrt", "[", 
     RowBox[{"2.0", "*", 
      RowBox[{"Log", "[", "2", "]"}]}], "]"}]}], ";"}], "  ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "Conversion", " ", "factor", " ", "from", " ", "waist", " ", "radius", " ",
     "to", " ", "FWHM", " ", "of", " ", "a", " ", "Gaussian", " ", "beam"}], 
   " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"fOffsetD2a", "=", 
    RowBox[{
     RowBox[{"-", "0.6268"}], "*", 
     SuperscriptBox["10", "9"]}]}], ";"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "Offset", " ", "of", " ", "D2a", " ", "line", " ", "from", " ", "D2", " ", 
    "line", " ", "center"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"fOffsetD2ab", "=", 
    RowBox[{"1.710", "*", 
     SuperscriptBox["10", "9"]}]}], ";"}], 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
    "Optimal", " ", "frequency", " ", "offset", " ", "between", " ", "D2a", 
     " ", "and", " ", "D2b"}], ",", " ", 
    RowBox[{
    "modifed", " ", "on", " ", "17", "Nov11", " ", "with", " ", "the", " ", 
     "introduction", " ", "of", " ", "DopplerFlip"}]}], " ", 
   "*)"}]}]}], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Laser Parameters:\n\tIMeso: Mesospheric laser intensity ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{
     StyleBox["W",
      FontSlant->"Plain"], "/", 
     SuperscriptBox[
      StyleBox["m",
       FontSlant->"Plain"], "2"]}], "]"}], TraditionalForm]]],
 "\n\tRepumpFraction: Repumping intensity fraction\n\tLightPolarization: \
Normalized ellipticity angle (LightPolarization := \[Epsilon]/(\[Pi]/4))\n\t\
LightWavelength: Laser center wavelength [m]\n\tPhase: Overall light phase.\n\
\tFWHMbw: Laser FWHM bandwidth [Hz]\n\t\[CapitalDelta]fLaser: Laser line \
detuning from optimum (both D2a and D2b) [Hz]\n\t\[CapitalDelta]fLaserDabOff: \
Offset of D2a--D2b frequency spacing from fOffsetD2ab [Hz]"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"laserOptsDefault", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"IMeso", "->", "46.0"}], ",", 
     RowBox[{"RepumpFraction", "->", "0.12"}], ",", 
     RowBox[{"LightPolarization", "->", "1.0"}], ",", 
     RowBox[{"LightWavelength", "->", 
      RowBox[{"589.159", "\[Times]", 
       SuperscriptBox["10", 
        RowBox[{"-", "9"}]]}]}], ",", 
     RowBox[{"Phase", "->", "0.0"}], ",", 
     RowBox[{"FWHMbw", "->", "0.0"}], ",", 
     RowBox[{"\[CapitalDelta]fLaser", "->", "0.0"}], ",", 
     RowBox[{"\[CapitalDelta]fLaserDabOff", "->", "0.0"}], ",", 
     RowBox[{"LaserWidth", "->", "0"}]}], "}"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Atmospheric and Mesospheric Parameters:\n\tBG: Mesospheric geomagnetic \
field strength ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    StyleBox["G",
     FontSlant->"Plain"], "]"}], TraditionalForm]]],
 "\n\tTa: One-way transmission at \[Lambda] at zenith\n\tTemperature: \
Mesospheric temperature [K]\n\ttauCollMult: Multiplier for S- and v-damping \
collision times\n\tHNa: Altitude of sodium centroid above sea level [m]\n\t\
vrms\[Gamma]: rms velocity of diffusion effects [m/s], \[Gamma] = \
vrms\[Gamma] / (DLT/\[Mu]LT)"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"atmoOptsDefault", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"BG", "->", "BGParanal"}], ",", 
     RowBox[{"Ta", "->", "0.84"}], ",", 
     RowBox[{"Temperature", "->", "185.0"}], ",", 
     RowBox[{"SDampingTime", "->", 
      RowBox[{"245.0", "\[Times]", " ", 
       SuperscriptBox["10", 
        RowBox[{"-", "6"}]]}]}], ",", 
     RowBox[{"VccTime", "->", 
      RowBox[{"35.0", "\[Times]", " ", 
       SuperscriptBox["10", 
        RowBox[{"-", "6"}]]}]}], ",", 
     RowBox[{"HNa", "->", 
      RowBox[{"92.0", "\[Times]", 
       SuperscriptBox["10", "3"]}]}], ",", 
     RowBox[{"vrms\[Gamma]", "->", "38.0"}], ",", 
     RowBox[{"RecoilShift", "->", "\[CapitalOmega]NaRecoil"}], ",", 
     RowBox[{"VccRateFunction", "->", "Automatic"}]}], "}"}]}], 
  ";"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Launch Telescope System (LTS) Parameters:\n\tPLaunch: Launched power [W]\n\t\
",
 "Zenith",
 ": Zenith angle [rad]\n\tHtele: Altitude of launch telescope above sea level \
[m]\n\tDLT: Launch telescope aperture [m]\n\t\[Mu]LT: Ratio of DLT to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "/", 
    SuperscriptBox["e", "2"]}], TraditionalForm]]],
 " lauch beam intensity diameter (not radius!), see Yura, Eq.(2.7)\n\tMsqrLT: \
",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "2"], TraditionalForm]]],
 " beam quality factor out of the launch telescope\n\tMagneticZenith, \
MagneticAzimuth: elevation and azimuth angles of B vector in a system where \
the laser points along z "
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"LTSOptsDefault", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"PLaunch", "->", "20.0"}], ",", 
     RowBox[{"Zenith", "->", 
      RowBox[{"30.0", "*", 
       RowBox[{"\[Pi]", "/", "180.0"}]}]}], ",", 
     RowBox[{"Htele", "->", "2650.0"}], ",", 
     RowBox[{"DLT", "->", "0.4"}], ",", 
     RowBox[{"\[Mu]LT", "->", 
      RowBox[{"50.0", "/", "36.0"}]}], ",", " ", 
     RowBox[{"MsqrLT", "->", "1.1"}], ",", 
     RowBox[{"MagneticZenith", "->", 
      RowBox[{"\[Pi]", "/", "2.0"}]}], ",", 
     RowBox[{"MagneticAzimuth", "->", 
      RowBox[{"\[Pi]", "/", "2.0"}]}]}], "}"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Numerical Parameters:\n\tIMesoMax: Maximum intensity in mesosphere. If set \
to zero, the routine determines it automatically  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{
     StyleBox["W",
      FontSlant->"Plain"], "/", 
     SuperscriptBox[
      StyleBox["m",
       FontSlant->"Plain"], "2"]}], "]"}], TraditionalForm]]],
 "\n\tIMesoInitial: Starting mesospheric laser intensity  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"[", 
    RowBox[{
     StyleBox["W",
      FontSlant->"Plain"], "/", 
     SuperscriptBox[
      StyleBox["m",
       FontSlant->"Plain"], "2"]}], "]"}], TraditionalForm]]],
 "\n\tMaxStepRatio: controls intensity step size\n\tIMesoFactor: multiply \
peak intensity by this factor to compute maximum repository intensity\n\t\
NSamples: number of samples in repository"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"NumOptsDefault", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"IMesoMax", "->", "0.0"}], ",", 
     RowBox[{"IMesoInitial", "->", "0.35"}], ",", 
     RowBox[{"MaxStepRatio", "->", "20.0"}], ",", 
     RowBox[{"IMesoFactor", "->", "1.0"}], ",", 
     RowBox[{"NSamples", "->", "6"}]}], "}"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"(*", "  ", 
  RowBox[{"WARNING", ":", " ", 
   RowBox[{
   "NaLGSDefaults", " ", "and", " ", "PhiOptDefault", " ", "depend", " ", 
    "on", " ", "the", " ", "three", " ", "preceding", " ", "option", " ", 
    RowBox[{"sets", "!"}]}]}], " ", "*)"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"NaLGSDefaults", "=", 
  RowBox[{"Join", "[", 
   RowBox[{
   "laserOptsDefault", ",", "atmoOptsDefault", ",", "LTSOptsDefault"}], 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"IMeso", "->", "46.`"}], ",", 
   RowBox[{"RepumpFraction", "->", "0.12`"}], ",", 
   RowBox[{"LightPolarization", "->", "1.`"}], ",", 
   RowBox[{"LightWavelength", "->", "5.89159`*^-7"}], ",", 
   RowBox[{"Phase", "->", "0.`"}], ",", 
   RowBox[{"FWHMbw", "->", "0.`"}], ",", 
   RowBox[{"\[CapitalDelta]fLaser", "->", "0.`"}], ",", 
   RowBox[{"\[CapitalDelta]fLaserDabOff", "->", "0.`"}], ",", 
   RowBox[{"BG", "->", "0.2275`"}], ",", 
   RowBox[{"Ta", "->", "0.84`"}], ",", 
   RowBox[{"Temperature", "->", "185.`"}], ",", 
   RowBox[{"SDampingTime", "->", "0.000245`"}], ",", 
   RowBox[{"VccTime", "->", "0.000035`"}], ",", 
   RowBox[{"HNa", "->", "92000.`"}], ",", 
   RowBox[{"vrms\[Gamma]", "->", "38.`"}], ",", 
   RowBox[{"RecoilShift", "->", "314159.2653589793`"}], ",", 
   RowBox[{"VccRateFunction", "->", "Automatic"}], ",", 
   RowBox[{"PLaunch", "->", "20.`"}], ",", 
   RowBox[{"Zenith", "->", "0.5235987755982988`"}], ",", 
   RowBox[{"Htele", "->", "2650.`"}], ",", 
   RowBox[{"DLT", "->", "0.4`"}], ",", 
   RowBox[{"\[Mu]LT", "->", "1.3888888888888888`"}], ",", 
   RowBox[{"MsqrLT", "->", "1.1`"}], ",", 
   RowBox[{"MagneticZenith", "->", "1.5707963267948966`"}], ",", 
   RowBox[{"MagneticAzimuth", "->", "1.5707963267948966`"}]}], "}"}]], \
"Input"],

Cell[BoxData[
 RowBox[{"$DefaultLGSParameters", "=", "NaLGSDefaults"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"IMeso", "->", "46.`"}], ",", 
   RowBox[{"RepumpFraction", "->", "0.12`"}], ",", 
   RowBox[{"LightPolarization", "->", "1.`"}], ",", 
   RowBox[{"LightWavelength", "->", "5.89159`*^-7"}], ",", 
   RowBox[{"Phase", "->", "0.`"}], ",", 
   RowBox[{"FWHMbw", "->", "0.`"}], ",", 
   RowBox[{"\[CapitalDelta]fLaser", "->", "0.`"}], ",", 
   RowBox[{"\[CapitalDelta]fLaserDabOff", "->", "0.`"}], ",", 
   RowBox[{"BG", "->", "0.2275`"}], ",", 
   RowBox[{"Ta", "->", "0.84`"}], ",", 
   RowBox[{"Temperature", "->", "185.`"}], ",", 
   RowBox[{"SDampingTime", "->", "0.000245`"}], ",", 
   RowBox[{"VccTime", "->", "0.000035`"}], ",", 
   RowBox[{"HNa", "->", "92000.`"}], ",", 
   RowBox[{"vrms\[Gamma]", "->", "38.`"}], ",", 
   RowBox[{"RecoilShift", "->", "314159.2653589793`"}], ",", 
   RowBox[{"VccRateFunction", "->", "Automatic"}], ",", 
   RowBox[{"PLaunch", "->", "20.`"}], ",", 
   RowBox[{"Zenith", "->", "0.5235987755982988`"}], ",", 
   RowBox[{"Htele", "->", "2650.`"}], ",", 
   RowBox[{"DLT", "->", "0.4`"}], ",", 
   RowBox[{"\[Mu]LT", "->", "1.3888888888888888`"}], ",", 
   RowBox[{"MsqrLT", "->", "1.1`"}], ",", 
   RowBox[{"MagneticZenith", "->", "1.5707963267948966`"}], ",", 
   RowBox[{"MagneticAzimuth", "->", "1.5707963267948966`"}]}], "}"}]], \
"Input"],

Cell[BoxData[
 RowBox[{"PhiOptDefault", "/.", "$DefaultLGSParameters"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"NDither", "->", "1"}], ",", 
   RowBox[{"DitherRate", "->", "0.`"}], ",", 
   RowBox[{
    RowBox[{"DitherDetuning", "[", "1", "]"}], "->", "0"}], ",", 
   RowBox[{
    RowBox[{"LightPhase", "[", "1", "]"}], "->", "0.`"}], ",", 
   RowBox[{
    RowBox[{"LightPhase", "[", "2", "]"}], "->", "3.141592653589793`"}], ",", 
   RowBox[{"DitherCoherenceTransfer", "->", "0"}], ",", 
   RowBox[{"EllipticityA", "->", "0.7853981633974483`"}], ",", 
   RowBox[{"DetuningA", "->", 
    RowBox[{"-", "3.9383005505401645`*^9"}]}], ",", 
   RowBox[{"LightIntensityA", "->", "40.48`"}], ",", 
   RowBox[{"EllipticityB", "->", "0.7853981633974483`"}], ",", 
   RowBox[{"DetuningB", "->", "6.854955170132929`*^9"}], ",", 
   RowBox[{"LightIntensityB", "->", "5.52`"}], ",", 
   RowBox[{"AtomicDensity", "->", "1"}], ",", 
   RowBox[{"BeamTransitRate", "->", "131.94444444444443`"}], ",", 
   RowBox[{"SDampingRate", "->", "4081.6326530612246`"}], ",", 
   RowBox[{"LarmorFrequency", "->", 
    RowBox[{
    "0.2275`", " ", "BohrMagneton", " ", "Gauss", " ", "LandeGF"}]}]}], "}"}]],\
 "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"PhiOptDefault", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"NDither", "->", 
      RowBox[{"1", "+", 
       RowBox[{"Sign", "[", "FWHMbw", "]"}]}]}], ",", 
     RowBox[{"DitherRate", "->", 
      RowBox[{
       RowBox[{"Sign", "[", "FWHMbw", "]"}], "2", "*", "\[Pi]", " ", "*", 
       "FWHMbw"}]}], ",", 
     RowBox[{
      RowBox[{"DitherDetuning", "[", "1", "]"}], "->", "0"}], ",", 
     RowBox[{
      RowBox[{"LightPhase", "[", "1", "]"}], "->", "Phase"}], ",", 
     RowBox[{
      RowBox[{"LightPhase", "[", "2", "]"}], "->", 
      RowBox[{"Phase", "+", "\[Pi]"}]}], ",", 
     RowBox[{"DitherCoherenceTransfer", "->", "0"}], ",", 
     RowBox[{"EllipticityA", "->", 
      RowBox[{"LightPolarization", "\[Times]", 
       RowBox[{"\[Pi]", "/", "4"}]}]}], ",", 
     RowBox[{"DetuningA", "->", 
      RowBox[{"2", " ", "\[Pi]", " ", 
       RowBox[{"(", 
        RowBox[{"\[CapitalDelta]fLaser", "+", "fOffsetD2a"}], ")"}]}]}], ",", 
     RowBox[{"LightIntensityA", "->", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"1.0", "-", "RepumpFraction"}], ")"}], "IMeso"}]}], ",", 
     RowBox[{"EllipticityB", "->", 
      RowBox[{"LightPolarization", "\[Times]", 
       RowBox[{"\[Pi]", "/", "4"}]}]}], ",", 
     RowBox[{"DetuningB", "->", 
      RowBox[{"2", " ", "\[Pi]", " ", 
       RowBox[{"(", 
        RowBox[{
        "\[CapitalDelta]fLaser", "+", "fOffsetD2a", "+", "fOffsetD2ab", "+", 
         "\[CapitalDelta]fLaserDabOff"}], ")"}]}]}], ",", 
     RowBox[{"LightIntensityB", "->", 
      RowBox[{"RepumpFraction", " ", "IMeso"}]}], ",", 
     RowBox[{"AtomicDensity", "->", "1"}], ",", 
     RowBox[{"BeamTransitRate", "->", 
      RowBox[{"vrms\[Gamma]", "/", 
       RowBox[{"(", 
        RowBox[{"DLT", "/", "\[Mu]LT"}], ")"}]}]}], ",", 
     RowBox[{"SDampingRate", "->", 
      RowBox[{"1", "/", "SDampingTime"}]}], ",", 
     RowBox[{"LarmorFrequency", "->", 
      RowBox[{"LandeGF", " ", "BohrMagneton", " ", "BG", " ", "Gauss"}]}]}], 
    "}"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell["\<\
This one is for no backpumping. In this case, we insert the power also into \
the \"D2b line\"\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"PhiOptDefault2", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"NDither", "->", 
      RowBox[{"1", "+", 
       RowBox[{"Sign", "[", "FWHMbw", "]"}]}]}], ",", 
     RowBox[{"DitherRate", "->", 
      RowBox[{
       RowBox[{"Sign", "[", "FWHMbw", "]"}], "2", "*", "\[Pi]", " ", "*", 
       "FWHMbw"}]}], ",", 
     RowBox[{
      RowBox[{"DitherDetuning", "[", "1", "]"}], "->", "0"}], ",", 
     RowBox[{
      RowBox[{"LightPhase", "[", "1", "]"}], "->", "Phase"}], ",", 
     RowBox[{
      RowBox[{"LightPhase", "[", "2", "]"}], "->", 
      RowBox[{"Phase", "+", "\[Pi]"}]}], ",", 
     RowBox[{"DitherCoherenceTransfer", "->", "0"}], ",", 
     RowBox[{"EllipticityA", "->", 
      RowBox[{"LightPolarization", "\[Times]", 
       RowBox[{"\[Pi]", "/", "4"}]}]}], ",", 
     RowBox[{"DetuningA", "->", 
      RowBox[{"2", " ", "\[Pi]", " ", 
       RowBox[{"(", 
        RowBox[{"\[CapitalDelta]fLaser", "+", "fOffsetD2a"}], ")"}]}]}], ",", 
     RowBox[{"LightIntensityA", "->", "IMeso"}], ",", 
     RowBox[{"EllipticityB", "->", 
      RowBox[{"LightPolarization", "\[Times]", 
       RowBox[{"\[Pi]", "/", "4"}]}]}], ",", 
     RowBox[{"DetuningB", "->", 
      RowBox[{"2", " ", "\[Pi]", 
       RowBox[{"(", 
        RowBox[{"\[CapitalDelta]fLaser", "+", "fOffsetD2a"}], ")"}]}]}], ",", 
     RowBox[{"LightIntensityB", "->", "IMeso"}], ",", 
     RowBox[{"AtomicDensity", "->", "1"}], ",", 
     RowBox[{"BeamTransitRate", "->", 
      RowBox[{"vrms\[Gamma]", "/", 
       RowBox[{"(", 
        RowBox[{"DLT", "/", "\[Mu]LT"}], ")"}]}]}], ",", 
     RowBox[{"SDampingRate", "->", 
      RowBox[{"1", "/", "SDampingTime"}]}], ",", 
     RowBox[{"LarmorFrequency", "->", 
      RowBox[{"LandeGF", " ", "BohrMagneton", " ", "BG", " ", "Gauss"}]}]}], 
    "}"}]}], ";"}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["PsiMeso", "Subsection"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "PsiMeso", "]"}], "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"DefaultParameters", ":>", "$DefaultLGSParameters"}], "}"}], 
     ",", 
     RowBox[{"Options", "[", "ReturnFlux", "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"SetOptions", "[", 
  RowBox[{"PsiMeso", ",", 
   RowBox[{"RefineResult", "->", 
    RowBox[{"{", "100", "}"}]}]}], "]"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DefaultParameters", ":>", "$DefaultLGSParameters"}], ",", 
   RowBox[{"RefineResult", "->", 
    RowBox[{"{", "100", "}"}]}], ",", 
   RowBox[{"InitVGParams", "->", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "3"}], "}"}]}], "}"}]}], ",", 
   RowBox[{"FluxReport", "->", 
    RowBox[{"{", "}"}]}], ",", 
   RowBox[{"ReturnAllSteps", "->", "False"}], ",", 
   RowBox[{"PlotVelocityGroups", "->", "False"}], ",", 
   RowBox[{"InitVG", "->", "Automatic"}], ",", 
   RowBox[{"SkipLinearSolve", "->", "False"}]}], "}"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"PsiMeso", "[", 
   RowBox[{"syst_", ",", "params_", ",", 
    RowBox[{"opts", ":", 
     RowBox[{"OptionsPattern", "[", "]"}]}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"flux", ",", "imes", ",", "params1", ",", "rfopts"}], "}"}], ",",
     "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"params1", "=", 
      RowBox[{"Join", "[", 
       RowBox[{"params", ",", 
        RowBox[{"OptionValue", "[", "DefaultParameters", "]"}]}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"rfopts", "=", 
      RowBox[{"FilterRules", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", "opts", "}"}], ",", 
          RowBox[{"Options", "[", "PsiMeso", "]"}]}], "}"}], ",", 
        RowBox[{"Options", "[", "ReturnFlux", "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{"Print", "[", 
        RowBox[{"\"\<Params: \>\"", ",", "params1"}], "]"}], ";"}], " ", 
      "*)"}], "\[IndentingNewLine]", 
     RowBox[{"flux", "=", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{"imes", "=", 
           RowBox[{"IMeso", "/.", "params1"}]}], ")"}], ">", "0"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{"RepumpFraction", "/.", "params1"}], ")"}], ">", "0"}], 
          ",", "\[IndentingNewLine]", 
          RowBox[{"ReturnFlux", "[", 
           RowBox[{"syst", ",", 
            RowBox[{"Join", "[", 
             RowBox[{
              RowBox[{"PhiOptDefault", "/.", "params1"}], ",", "params1"}], 
             "]"}], ",", "rfopts"}], "]"}], ",", 
          RowBox[{"ReturnFlux", "[", 
           RowBox[{"syst", ",", 
            RowBox[{"Join", "[", 
             RowBox[{
              RowBox[{"PhiOptDefault2", "/.", "params1"}], ",", "params1"}], 
             "]"}], ",", "rfopts"}], "]"}], ",", 
          RowBox[{
           RowBox[{"Print", "[", 
            RowBox[{
            "\"\<PsiMeso[]: Cannot decide if RepumpFraction is nonzero, \
RepumpFraction: \>\"", ",", 
             RowBox[{"RepumpFraction", "/.", "params1"}]}], "]"}], ";", 
           RowBox[{"Abort", "[", "]"}]}]}], "]"}], ",", "0.0", ",", 
        RowBox[{
         RowBox[{"Print", "[", 
          RowBox[{
          "\"\<PsiMeso[]: Cannot decide if IMeso is nonzero, IMeso: \>\"", 
           ",", 
           RowBox[{"IMeso", "/.", "params1"}]}], "]"}], ";", 
         RowBox[{"Abort", "[", "]"}]}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"NumericQ", "[", "flux", "]"}], ",", 
       RowBox[{"flux", "/", "imes"}], ",", 
       RowBox[{"flux", "/", 
        RowBox[{"{", 
         RowBox[{"imes", ",", "1"}], "}"}]}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["BeamRepository", "Subsection"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ImaxWA", "[", 
    RowBox[{"P_", ",", "FWHM_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"A", ",", "w", ",", "Imax"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"w", "=", 
       RowBox[{"FWHM", "/", "w2f"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"A", "=", 
       RowBox[{"\[Pi]", " ", 
        RowBox[{"w", "^", "2"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Imax", "=", 
       RowBox[{"2.0", " ", 
        RowBox[{"P", "/", "A"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"Imax", ",", "w", ",", "A"}], "}"}]}]}], "\[IndentingNewLine]",
     "]"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Speckle FWHM in mesosphere, see Andrews \"Laser Beam Propagation in Radom \
Media\", Ch .16 Eq.(102):\n\tDap : aperture diameter [m]\n\t",
 Cell[BoxData[
  RowBox[{"wLaunch", ":", " ", 
   RowBox[{
    RowBox[{"1", "/", 
     SuperscriptBox["e", "2"]}], " ", "radius", " ", "of", " ", "launched", 
    " ", "beam"}]}]],
  CellChangeTimes->{{3.445869046422124*^9, 3.445869075375064*^9}, {
   3.445869551481392*^9, 3.445869664621292*^9}}],
 " [m]\n\t\[Lambda] : wavelength of light [m]\n\tL : line of sight distance\n\
\tMsqr: ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "2"], TraditionalForm]]],
 "beam quality factor"
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"dSpeckle", "[", 
   RowBox[{"Dap_", ",", "wLaunch_", ",", "\[Lambda]_", ",", "L_", ",", 
    RowBox[{"Msqr_:", "1.0"}]}], "]"}], ":=", 
  RowBox[{"4.0", "*", 
   RowBox[{
    RowBox[{"Sqrt", "[", 
     RowBox[{"Log", "[", "2", "]"}], "]"}], "/", "\[Pi]"}], "*", "\[Lambda]", 
   "*", "L", "*", 
   RowBox[{"Msqr", "/", "Dap"}], "*", 
   RowBox[{"Sqrt", "[", 
    RowBox[{"1", "+", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Dap", "/", "2"}], "/", "wLaunch"}], ")"}], "2"]}], 
    "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"MATLAB", "'"}], "s", " ", "linspace", 
    RowBox[{"(", ")"}], " ", "routine", " ", "for", " ", "convenience"}], " ",
    "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"linspace", "[", 
     RowBox[{"a_", ",", "b_", ",", 
      RowBox[{"n_:", "100"}]}], "]"}], ":=", 
    RowBox[{"Range", "[", 
     RowBox[{"a", ",", "b", ",", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"b", "-", "a"}], ")"}], "/", 
       RowBox[{"(", 
        RowBox[{"n", "-", "1"}], ")"}]}]}], "]"}]}], ";"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
BeamRepositorySamples[): Compute the beam repository samples. Not meant to be \
called directly.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Options", "[", "BeamRepositorySamples", "]"}], "=", 
  RowBox[{"Join", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"DefaultParameters", ":>", " ", "$DefaultLGSParameters"}], "}"}],
     ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"IMesoMax", "->", "0.0"}], ",", 
      RowBox[{"IMesoInitial", "->", "0.35"}], ",", 
      RowBox[{"MaxStepRatio", "->", "20"}], ",", 
      RowBox[{"IMesoFactor", "->", "1.0"}], ",", 
      RowBox[{"NSamples", "->", "6"}], ",", 
      RowBox[{"LaserSpotMesoFWHM", "->", "0"}]}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DefaultParameters", ":>", "$DefaultLGSParameters"}], ",", 
   RowBox[{"IMesoMax", "->", "0.`"}], ",", 
   RowBox[{"IMesoInitial", "->", "0.35`"}], ",", 
   RowBox[{"MaxStepRatio", "->", "20"}], ",", 
   RowBox[{"IMesoFactor", "->", "1.`"}], ",", 
   RowBox[{"NSamples", "->", "6"}], ",", 
   RowBox[{"LaserSpotMesoFWHM", "->", "0"}]}], "}"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"BeamRepositorySamples", "[", 
    RowBox[{"params_", ",", 
     RowBox[{"opts", ":", 
      RowBox[{"OptionsPattern", "[", "]"}]}]}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "airmass", ",", "stepsNorm", ",", "fn", ",", "IMesoMax1", ",", 
       "IMeso1Loc", ",", "FWHMMeso1", ",", "wLaunch", ",", "dist", ",", 
       "IMesoAll", ",", "params1"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"params1", "=", 
       RowBox[{"Join", "[", 
        RowBox[{"params", ",", 
         RowBox[{"OptionValue", "[", "DefaultParameters", "]"}]}], "]"}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"IMesoMax1", "=", 
       RowBox[{"OptionValue", "[", "IMesoMax", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"FWHMMeso1", "=", 
       RowBox[{"OptionValue", "[", "LaserSpotMesoFWHM", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"airmass", " ", "=", " ", 
       RowBox[{
        RowBox[{"Sec", "[", "Zenith", "]"}], "/.", "params1"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"IMesoMax1", "<=", "0.0"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"FWHMMeso1", "<=", "0.0"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"wLaunch", "=", 
             RowBox[{
              RowBox[{"0.5", 
               RowBox[{"DLT", "/", "\[Mu]LT"}]}], "/.", "params1"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"(*", " ", 
             RowBox[{
             "distance", " ", "to", " ", "guide", " ", "star", " ", "in", " ",
               "line", " ", "of", " ", "sight"}], " ", "*)"}], 
            "\[IndentingNewLine]", 
            RowBox[{"dist", "=", 
             RowBox[{
              RowBox[{
               RowBox[{"(", 
                RowBox[{"HNa", "-", "Htele"}], ")"}], "*", "airmass"}], "/.", 
              "params1"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"(*", " ", 
             RowBox[{
             "Instantaneous", " ", "FWHM", " ", "spot", " ", "size", " ", 
              "in", " ", "mesosphere"}], " ", "*)"}], "\[IndentingNewLine]", 
            RowBox[{"FWHMMeso1", "=", 
             RowBox[{"dSpeckle", "@@", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{
                 "DLT", ",", "wLaunch", ",", "LightWavelength", ",", "dist", 
                  ",", "MsqrLT"}], "}"}], "/.", "params1"}], ")"}]}]}], 
            ";"}]}], " ", "\[IndentingNewLine]", "]"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"IMesoMax1", "=", 
          RowBox[{
           RowBox[{"ImaxWA", "[", 
            RowBox[{
             RowBox[{
              RowBox[{
               SuperscriptBox["Ta", "airmass"], "*", "PLaunch"}], "/.", 
              "params1"}], ",", "FWHMMeso1"}], "]"}], "[", 
           RowBox[{"[", "1", "]"}], "]"}]}], ";"}]}], " ", 
       RowBox[{"(*", " ", 
        RowBox[{"max", " ", "intensity", " ", "in", " ", "mesosphere"}], " ", 
        "*)"}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"OptionValue", "[", "NSamples", "]"}], ">", "1"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"stepsNorm", " ", "=", 
          RowBox[{"linspace", "[", 
           RowBox[{"1", ",", 
            RowBox[{"OptionValue", "[", "MaxStepRatio", "]"}], ",", 
            RowBox[{
             RowBox[{"OptionValue", "[", "NSamples", "]"}], "-", "1"}]}], 
           "]"}]}], ";", " ", 
         RowBox[{"(*", " ", 
          RowBox[{
          "compute", " ", "vector", " ", "of", " ", "normalized", " ", 
           "steps"}], " ", "*)"}], "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{"make", " ", "sure", " ", "iMeso1", " ", 
           RowBox[{"won", "'"}], "t", " ", "become", " ", "too", " ", 
           "large"}], " ", "*)"}], "\[IndentingNewLine]", 
         RowBox[{"IMeso1Loc", "=", 
          RowBox[{"Min", "[", 
           RowBox[{
            RowBox[{"OptionValue", "[", "IMesoInitial", "]"}], ",", 
            RowBox[{
             RowBox[{"IMesoMax1", "/", "10.0"}], "/", 
             RowBox[{"OptionValue", "[", "NSamples", "]"}]}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"fn", "=", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"OptionValue", "[", "IMesoFactor", "]"}], "*", 
              "IMesoMax1"}], "-", "IMeso1Loc"}], ")"}], "/", 
           RowBox[{"Total", "[", "stepsNorm", "]"}]}]}], ";", "  ", 
         RowBox[{"(*", " ", 
          RowBox[{"Normalization", " ", "factor"}], " ", "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"IMesoAll", "=", 
          RowBox[{"IMeso1Loc", "+", 
           RowBox[{"fn", "*", 
            RowBox[{"Join", "[", 
             RowBox[{
              RowBox[{"{", "0", "}"}], ",", 
              RowBox[{"Accumulate", "[", "stepsNorm", "]"}]}], "]"}]}]}]}], 
         ";"}], "    ", 
        RowBox[{"(*", " ", 
         RowBox[{"Intensity", " ", "vector"}], " ", "*)"}], 
        "\[IndentingNewLine]", ",", 
        RowBox[{"(*", " ", "ELSE", " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"IMesoAll", "=", 
          RowBox[{"{", 
           RowBox[{"OptionValue", "[", "IMesoInitial", "]"}], "}"}]}], 
         ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"IMesoAll", ",", "IMesoMax1", ",", "FWHMMeso1"}], "}"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], " ", ";"}]], "Input",
 InitializationCell->True],

Cell["\<\
BeamRepository[]: Compute repository of return flux values. Must be called \
with number of velocity bins, i.e. BeamRepository[{IMeso->46...}, {100}]\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "BeamRepository", "]"}], "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"Options", "[", "BeamRepositorySamples", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"RefineResult", "->", 
        RowBox[{"{", "100", "}"}]}], ",", 
       RowBox[{"InitVGParams", "->", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"1", ",", "2"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "3"}], "}"}]}], "}"}]}], ",", 
       RowBox[{"InitVG", "->", "Automatic"}], ",", 
       RowBox[{"PlotVelocityGroups", "->", "False"}]}], "}"}]}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"BeamRepository", "[", 
    RowBox[{"syst_", ",", "params_", ",", 
     RowBox[{"opts", ":", 
      RowBox[{"OptionsPattern", "[", "]"}]}]}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "fluxAll", ",", "brs", ",", "brsopts", ",", "nfopts", ",", "IMesoMax1", 
       ",", "FWHMMeso1"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"SetSharedVariable", "[", 
       RowBox[{"brs", ",", "nfopts"}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"brsopts", "=", 
       RowBox[{"FilterRules", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", "opts", "}"}], ",", 
           RowBox[{"Options", "[", "BeamRepository", "]"}]}], "}"}], ",", 
         RowBox[{"Options", "[", "BeamRepositorySamples", "]"}]}], "]"}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"brs", ",", "IMesoMax1", ",", "FWHMMeso1"}], "}"}], "=", 
       RowBox[{"BeamRepositorySamples", "[", 
        RowBox[{"params", ",", "brsopts"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"nfopts", "=", 
       RowBox[{"FilterRules", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"FluxReport", "->", 
              RowBox[{"{", "}"}]}], ",", 
             RowBox[{"ReturnAllSteps", "->", "False"}]}], "}"}], ",", 
           RowBox[{"{", "opts", "}"}], ",", 
           RowBox[{"Options", "[", "BeamRepository", "]"}]}], "}"}], ",", 
         RowBox[{"Options", "[", "PsiMeso", "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"fluxAll", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"brs", "[", 
            RowBox[{"[", "k", "]"}], "]"}], ",", 
           RowBox[{
            RowBox[{"brs", "[", 
             RowBox[{"[", "k", "]"}], "]"}], 
            RowBox[{"PsiMeso", "[", 
             RowBox[{"syst", ",", 
              RowBox[{"Join", "[", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"IMeso", "->", 
                  RowBox[{"brs", "[", 
                   RowBox[{"[", "k", "]"}], "]"}]}], "}"}], ",", "params"}], 
               "]"}], ",", "nfopts"}], "]"}]}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"k", ",", "1", ",", 
           RowBox[{"Length", "[", "brs", "]"}]}], "}"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"fluxAll", "=", 
       RowBox[{"Join", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"{", 
           RowBox[{"0.0", ",", "0.0"}], "}"}], "}"}], ",", "fluxAll"}], 
        "]"}]}], ";", "  ", 
      RowBox[{"(*", " ", 
       RowBox[{"add", " ", "zero", " ", "intensity", " ", "point"}], " ", 
       "*)"}], "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Interpolation", "[", 
         RowBox[{"fluxAll", ",", 
          RowBox[{"InterpolationOrder", "->", "2"}], ",", 
          RowBox[{"Method", "->", "\"\<Spline\>\""}]}], "]"}], ",", "fluxAll",
         ",", "IMesoMax1", ",", "FWHMMeso1"}], "}"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["InterBeamMeso", "Subsection"],

Cell["\<\
Integrate beam return, compute flux and mesospheric spot size FWHM based on \
the 2nd moment. The function f[] yields the flux return for a given intensity.
To test, use e.g. f[x_] := x\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "InterBeamMeso", "]"}], "=", 
   RowBox[{"Options", "[", "BeamRepositorySamples", "]"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"InterBeamMeso", "[", 
     RowBox[{"params_", ",", "PsiVector_", ",", 
      RowBox[{"opts", ":", 
       RowBox[{"OptionsPattern", "[", "]"}]}]}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "w", ",", "IMesoMax1", ",", "r", ",", "FluxMesoInt", ",", "r2", ",", 
        "airmass", ",", "FWHM2mom", ",", "GaussIntens", ",", "wLaunch", ",", 
        "dist", ",", "FWHMMeso1", ",", "params1", ",", "Psi0", ",", "I0", ",",
         "c", ",", "fitFunction", ",", "fitRules", ",", "errTotRel", ",", 
        "useSplines"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"params1", "=", 
        RowBox[{"Join", "[", 
         RowBox[{"params", ",", 
          RowBox[{"OptionValue", "[", "DefaultParameters", "]"}]}], "]"}]}], 
       ";", "\[IndentingNewLine]", 
       RowBox[{"IMesoMax1", "=", 
        RowBox[{"OptionValue", "[", "IMesoMax", "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"airmass", " ", "=", " ", 
        RowBox[{
         RowBox[{"Sec", "[", "Zenith", "]"}], "/.", "params1"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"FWHMMeso1", "=", 
        RowBox[{"OptionValue", "[", "LaserSpotMesoFWHM", "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"IMesoMax1", "<=", "0"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"FWHMMeso1", "<=", "0"}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"wLaunch", "=", 
              RowBox[{
               RowBox[{"0.5", 
                RowBox[{"DLT", "/", "\[Mu]LT"}]}], "/.", "params1"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"(*", " ", 
              RowBox[{
              "distance", " ", "to", " ", "guide", " ", "star", " ", "in", 
               " ", "line", " ", "of", " ", "sight"}], " ", "*)"}], 
             "\[IndentingNewLine]", 
             RowBox[{"dist", "=", 
              RowBox[{
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"HNa", "-", "Htele"}], ")"}], "*", "airmass"}], "/.",
                "params1"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"(*", " ", 
              RowBox[{
              "Instantaneous", " ", "FWHM", " ", "spot", " ", "size", " ", 
               "in", " ", "mesosphere"}], " ", "*)"}], "\[IndentingNewLine]", 
             RowBox[{"FWHMMeso1", "=", 
              RowBox[{"dSpeckle", "@@", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                  "DLT", ",", "wLaunch", ",", "LightWavelength", ",", "dist", 
                   ",", "MsqrLT"}], "}"}], "/.", "params1"}], ")"}]}]}], 
             ";"}]}], " ", "\[IndentingNewLine]", "]"}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{
           "Intensity", " ", "and", " ", "instantaneous", " ", "FWHM", " ", 
            "spot", " ", "size", " ", "in", " ", "mesosphere"}], " ", "*)"}], 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"IMesoMax1", ",", "w"}], "}"}], "=", 
           RowBox[{
            RowBox[{"ImaxWA", "[", 
             RowBox[{
              RowBox[{
               RowBox[{
                SuperscriptBox["Ta", "airmass"], "*", "PLaunch"}], "/.", 
               "params1"}], ",", "FWHMMeso1"}], "]"}], "[", 
            RowBox[{"[", 
             RowBox[{"1", ";;", "2"}], "]"}], "]"}]}], ";"}], 
         "\[IndentingNewLine]", ",", 
         RowBox[{"(*", " ", "ELSE", " ", "*)"}], "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"w", "=", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"OptionValue", "[", "LaserSpotMesoFWHM", "]"}], "<=", 
              "0"}], ",", "\[IndentingNewLine]", 
             RowBox[{"Sqrt", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"2.0", "/", "\[Pi]"}], " ", 
                SuperscriptBox["Ta", "airmass"], 
                RowBox[{"PLaunch", "/", "IMesoMax1"}]}], "/.", "params1"}], 
              "]"}], ",", 
             RowBox[{"FWHMMeso1", "/", "w2f"}]}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"FWHMMeso1", "=", "0.0"}], ";"}]}], "\[IndentingNewLine]", 
        "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{"Check", " ", "that", " ", 
         RowBox[{"we", "'"}], "re", " ", "staying", " ", "within", " ", "the",
          " ", "range", " ", "of", " ", "the", " ", "simulated", " ", 
         "irradiance", " ", "values"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"InterBeamMeso", "::", "IMaxOverflow"}], "=", " ", 
        "\"\<Maximum intensity exceeds that of the supplied simulation \
data!\>\""}], ";", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"IMesoMax1", " ", ">", " ", 
          RowBox[{"PsiVector", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", "1"}], "]"}], "]"}]}], ",", 
         RowBox[{
          RowBox[{"Message", "[", 
           RowBox[{"InterBeamMeso", "::", "IMaxOverflow"}], "]"}], ";"}]}], 
        "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"InterBeamMeso", "::", "PsiVector0"}], "=", " ", 
        "\"\<PsiVector must start with the values {0,0}!\>\""}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"PsiVector", "[", 
           RowBox[{"[", "1", "]"}], "]"}], "!=", 
          RowBox[{"{", 
           RowBox[{"0", ",", "0"}], "}"}]}], ",", 
         RowBox[{
          RowBox[{"Message", "[", 
           RowBox[{"InterBeamMeso", "::", "PsiVector0"}], "]"}], ";", 
          RowBox[{"Abort", "[", "]"}]}], ",", 
         RowBox[{"{", "}"}]}], "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"verbosityLevel", ">", "3"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Print", "[", 
           RowBox[{
           "\"\<--> InterBeamMeso[]: IMesoMax1: \>\"", ",", "IMesoMax1", ",", 
            "\"\<, w: \>\"", ",", "w", ",", "\"\<, PLaunch: \>\"", ",", 
            RowBox[{"PLaunch", "/.", "params1"}]}], "]"}], ";"}]}], "]"}], 
       ";", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
        "Assume", " ", "Gaussian", " ", "irradiance", " ", "distribution", 
         " ", "in", " ", 
         RowBox[{"mesosphere", ".", " ", "The"}], " ", 
         RowBox[{"Re", "[", "]"}], " ", "is", " ", "to", " ", "avoid", " ", 
         RowBox[{"Less", "::", "nord"}], " ", "errors", " ", "in", " ", "the",
          " ", "numerical", " ", "integration", " ", "below"}], " ", "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"GaussIntens", "[", "r_", "]"}], ":=", 
        RowBox[{"IMesoMax1", "*", 
         RowBox[{"Exp", "[", 
          RowBox[{"Re", "[", 
           RowBox[{
            RowBox[{"-", "2.0"}], 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"r", "/", "w"}], ")"}], "^", "2"}]}], "]"}], "]"}]}]}], 
       ";", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
         RowBox[{
         "Use", " ", "splines", " ", "instead", " ", "of", " ", "a", " ", 
          "logarithmic", " ", "fit", " ", "function", " ", "if", " ", 
          "\[CapitalPsi]", 
          RowBox[{"(", "I", ")"}], " ", "shows", " ", "a", " ", "roll"}], "-",
          "off"}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"useSplines", "=", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"PsiVector", "[", 
            RowBox[{"[", 
             RowBox[{
              RowBox[{"-", "1"}], ",", "2"}], "]"}], "]"}], "<", 
           RowBox[{"Max", "[", 
            RowBox[{"PsiVector", "[", 
             RowBox[{"[", 
              RowBox[{"All", ",", "2"}], "]"}], "]"}], "]"}]}], ",", "True", 
          ",", "False"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"!", "useSplines"}], ",", "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{
           RowBox[{
            RowBox[{
            "Fit", " ", "function", " ", "template", " ", "\[CapitalPsi]", 
             RowBox[{"(", "I", ")"}]}], "=", 
            RowBox[{
             SubscriptBox["\[CapitalPsi]", "0"], " ", 
             RowBox[{"ln", "[", 
              RowBox[{"1", "+", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{"I", "/", 
                  SubscriptBox["I", "0"]}], ")"}], "c"]}], "]"}], 
             "\[IndentingNewLine]", 
             RowBox[{"Abs", "[", "]"}], " ", "is", " ", "to", " ", "protect", 
             " ", "from", " ", "imaginary", " ", "function", " ", 
             RowBox[{"values", ".", " ", "Using"}], " ", "a", " ", "fit", " ",
              "function", " ", "is", " ", "much", " ", "more", " ", 
             "reliable", " ", "than", " ", "employing", " ", 
             RowBox[{"Interpolation", "[", "]"}]}]}], ",", " ", 
           RowBox[{
            RowBox[{"even", " ", "with", " ", "InterpolationOrder"}], "->", 
            "2"}]}], " ", "*)"}], "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"fitFunction", "[", "x_", "]"}], ":=", 
           RowBox[{"Psi0", " ", 
            RowBox[{"Log", "[", 
             RowBox[{"1.0", "+", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"x", "/", 
                 RowBox[{"Abs", "[", "I0", "]"}]}], " ", ")"}], "^", 
               RowBox[{"Abs", "[", "c", "]"}]}]}], "]"}]}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{"Fit", " ", "model", " ", "function", " ", "to", " ", 
            RowBox[{"PsiVector", ".", " ", "Supply"}], " ", "initial", " ", 
            "parameter", " ", "values", " ", "so", " ", "that", " ", "the", 
            " ", "value", " ", "of", " ", "Psi", " ", "at", " ", "the", " ", 
            "last", " ", "simulation", " ", "point", " ", "equals", " ", 
            "Psi0", "*", "ln", 
            RowBox[{"(", "2", ")"}]}], " ", "*)"}], "\[IndentingNewLine]", 
          RowBox[{"fitRules", "=", 
           RowBox[{"FindFit", "[", 
            RowBox[{"PsiVector", ",", 
             RowBox[{"fitFunction", "[", "x", "]"}], ",", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"Psi0", ",", " ", 
                 RowBox[{
                  RowBox[{"PsiVector", "[", 
                   RowBox[{"[", 
                    RowBox[{
                    RowBox[{"-", "1"}], ",", "2"}], "]"}], "]"}], "/", 
                  RowBox[{"Log", "[", "2.0", "]"}]}]}], "}"}], ",", 
               RowBox[{"{", 
                RowBox[{"I0", ",", " ", 
                 RowBox[{"PsiVector", "[", 
                  RowBox[{"[", 
                   RowBox[{
                    RowBox[{"-", "1"}], ",", "1"}], "]"}], "]"}]}], "}"}], 
               ",", 
               RowBox[{"{", 
                RowBox[{"c", ",", "1.0"}], "}"}]}], "}"}], ",", "x", ",", 
             RowBox[{"AccuracyGoal", "->", "6"}], ",", 
             RowBox[{"MaxIterations", "->", "300"}]}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{"Total", " ", "relative", " ", "error", " ", 
            RowBox[{"(", 
             RowBox[{"1", "st", " ", "moment"}], ")"}]}], " ", "*)"}], 
          "\[IndentingNewLine]", 
          RowBox[{"errTotRel", "=", 
           RowBox[{
            RowBox[{"Total", "[", 
             RowBox[{"Abs", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"PsiVector", "[", 
                 RowBox[{"[", 
                  RowBox[{"All", ",", "2"}], "]"}], "]"}], "-", 
                RowBox[{"fitFunction", "[", 
                 RowBox[{"PsiVector", "[", 
                  RowBox[{"[", 
                   RowBox[{"All", ",", "1"}], "]"}], "]"}], "]"}]}], "/.", 
               "fitRules"}], "]"}], "]"}], "/", 
            RowBox[{"Total", "[", 
             RowBox[{"PsiVector", "[", 
              RowBox[{"[", 
               RowBox[{"All", ",", "2"}], "]"}], "]"}], "]"}]}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"(*", " ", 
           RowBox[{
            RowBox[{"Fit", " ", "can", " ", "fail"}], ",", " ", 
            RowBox[{
             RowBox[{
              RowBox[{"e", ".", "g", ".", " ", "in"}], " ", "case", " ", "of",
               " ", "roll"}], "-", 
             RowBox[{"over", " ", "of", " ", 
              RowBox[{
               RowBox[{"\[CapitalPsi]", "[", "I", "]"}], ".", " ", "Use"}], 
              " ", "spline", " ", "interpolation", " ", "in", " ", "this", 
              " ", 
              RowBox[{"case", "."}]}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"InterBeamMeso", "::", "BadFit"}], "=", 
           "\"\<Large error in return flux fit of `1`%! Trying spline \
interpolation.\>\""}], ";", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"errTotRel", ">", "0.02"}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"Message", "[", 
              RowBox[{
               RowBox[{"InterBeamMeso", "::", "BadFit"}], ",", 
               RowBox[{"N", "[", 
                RowBox[{
                 RowBox[{"100.0", "*", "errTotRel"}], ",", "4"}], "]"}]}], 
              "]"}], ";", "\[IndentingNewLine]", 
             RowBox[{"useSplines", "=", "True"}], ";"}]}], 
           "\[IndentingNewLine]", "]"}], ";"}], "\[IndentingNewLine]", ",", 
         " ", 
         RowBox[{"(*", " ", "ELSE", " ", "*)"}], "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"fitRules", "=", 
           RowBox[{"{", "}"}]}], ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
       RowBox[{"(*", 
        RowBox[{"If", "[", 
         RowBox[{"!", "useSplines"}], "]"}], "*)"}], "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{"useSplines", ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"fitFunction", "=", 
           RowBox[{"Interpolation", "[", 
            RowBox[{"PsiVector", ",", 
             RowBox[{"InterpolationOrder", "->", "2"}], ",", 
             RowBox[{"Method", "->", "\"\<Spline\>\""}]}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"errTotRel", "=", 
           RowBox[{
            RowBox[{"Total", "[", 
             RowBox[{"Abs", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"PsiVector", "[", 
                 RowBox[{"[", 
                  RowBox[{"All", ",", "2"}], "]"}], "]"}], "-", 
                RowBox[{"fitFunction", "[", 
                 RowBox[{"PsiVector", "[", 
                  RowBox[{"[", 
                   RowBox[{"All", ",", "1"}], "]"}], "]"}], "]"}]}], "/.", 
               "fitRules"}], "]"}], "]"}], "/", 
            RowBox[{"Total", "[", 
             RowBox[{"PsiVector", "[", 
              RowBox[{"[", 
               RowBox[{"All", ",", "2"}], "]"}], "]"}], "]"}]}]}], ";"}]}], 
        "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
        "Turn", " ", "off", " ", "this", " ", "error", " ", "message"}], " ", 
        "*)"}], "\[IndentingNewLine]", 
       RowBox[{"Off", "[", 
        RowBox[{"InverseFunction", "::", "ifun"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
         RowBox[{"Carry", " ", "out", " ", "numerical", " ", "integral"}], 
         ",", " ", 
         RowBox[{
         "exploiting", " ", "the", " ", "rotational", " ", "symmetry", " ", 
          "of", " ", "a", " ", 
          RowBox[{"Gaussian", ".", " ", "FluxMesoInt"}], " ", "was", " ", 
          "denoted", " ", "by", " ", 
          FormBox[
           RowBox[{
            SubscriptBox["F", "m"], " "}],
           TraditionalForm], "in", " ", "Holzloehner", " ", "et", " ", 
          RowBox[{"al", "."}]}], ",", 
         RowBox[{
          RowBox[{"A", "&"}], "A", " ", "510"}], ",", 
         RowBox[{"A20", " ", 
          RowBox[{"(", "2010", ")"}]}], ",", " ", 
         RowBox[{"Eq", ".", 
          RowBox[{"(", "7", ")"}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"FluxMesoInt", "=", 
        RowBox[{"NIntegrate", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"2.0", "\[Pi]", " ", "r", "  ", 
            RowBox[{"fitFunction", "[", 
             RowBox[{"GaussIntens", "[", "r", "]"}], "]"}]}], "/.", 
           "fitRules"}], ",", 
          RowBox[{"{", 
           RowBox[{"r", ",", "0.0", ",", 
            RowBox[{"3.0", "w"}]}], "}"}], ",", 
          RowBox[{"AccuracyGoal", "->", "5"}]}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{"2", "nd", " ", "moment", " ", "computation", " ", 
         RowBox[{"(", 
          RowBox[{"weight", " ", "by", " ", 
           RowBox[{"r", "^", "2"}]}], ")"}]}], " ", "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"r2", "=", 
        RowBox[{"NIntegrate", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"2.0", "\[Pi]", "  ", 
            RowBox[{"r", "^", "3"}], "  ", 
            RowBox[{"fitFunction", "[", 
             RowBox[{"GaussIntens", "[", "r", "]"}], "]"}]}], "/.", 
           "fitRules"}], ",", 
          RowBox[{"{", 
           RowBox[{"r", ",", "0.0", ",", 
            RowBox[{"3.0", "w"}]}], "}"}], ",", 
          RowBox[{"AccuracyGoal", "->", "5"}]}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"On", "[", 
        RowBox[{"InverseFunction", "::", "ifun"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{
         RowBox[{"2", "nd", " ", 
          RowBox[{"moment", ".", " ", "In"}], " ", "the", " ", "absence", " ",
           "of", " ", "any", " ", "saturation", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"hence", " ", 
             RowBox[{"a", "/", 
              RowBox[{"b", "--"}]}]}], ">", 
            RowBox[{"Infinity", " ", "and", " ", "c"}], "==", "1"}], ")"}]}], 
         ",", " ", 
         RowBox[{
         "this", " ", "equals", " ", "the", " ", "FWHM", " ", "beam", " ", 
          RowBox[{"diameter", "."}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{"FWHM2mom", "=", 
        RowBox[{"w2f", "*", 
         RowBox[{"Sqrt", "[", 
          RowBox[{"2.0", 
           RowBox[{"r2", "/", "FluxMesoInt"}]}], "]"}]}]}], ";", 
       "\[IndentingNewLine]", " ", 
       RowBox[{"{", 
        RowBox[{
        "FluxMesoInt", ",", "FWHMMeso1", ",", " ", "FWHM2mom", ",", 
         "IMesoMax1", ",", "fitFunction", ",", "fitRules", ",", "errTotRel"}],
         "}"}]}]}], "\[IndentingNewLine]", "]"}]}], " ", ";"}], 
  "\n"}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Sce", "Subsection"],

Cell[TextData[{
 "Compute beam figure of merit ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["s", "ce"], TraditionalForm]]],
 ". Note that the following options may be changed from their values\nwhen \
BeamRepository[] was called, unless the peak intensity exceeds that of the \
interpolating function:\n\tPLaunch: Launched power [W]\n\tZenith: Zenith \
angle [rad]   (note that MagneticZenith and MagneticAzimuth cannot be \
changed!)\n\tHtele: Altitude of launch telescope above sea level [m]\n\tDLT: \
Launch telescope aperture [m]\n\t\[Mu]LT: Ratio of DLT to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "/", 
    SuperscriptBox["e", "2"]}], TraditionalForm]]],
 " lauch beam intensity diameter (not radius!), see Yura, Eq.(2.7)\n\tMsqrLT: \
 Effective ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["M", "2"], TraditionalForm]]],
 " beam quality factor out of the launch telescope, also taking into account \
the effect of speckles\nInput values:\n\tparams: parameters to PsiMeso[]\n\t\
PsiVector: Vector of simulated returns of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["I", "1"], ",", 
       RowBox[{"\[CapitalPsi]", "(", 
        FormBox[
         SubscriptBox["I", "1"],
         TraditionalForm], ")"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["I", "2"], ",", 
       RowBox[{"\[CapitalPsi]", "(", 
        FormBox[
         SubscriptBox["I", "2"],
         TraditionalForm], ")"}]}], "}"}]}]}], TraditionalForm]]],
 ",...}. Note this is capital \[CapitalPsi], hence not divided by irradiance.\
\n\topts: Options to Sce[]\nOutput values:\n\tsceOut: Merit parameter ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["s", "ce"], TraditionalForm]]],
 StyleBox[" ",
  FontSlant->"Italic"],
 "[ph/s/W/(atom/m^2)]\n\tGroundInt: Integrated flux on the ground per ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "Na"], TraditionalForm]]],
 " [ph/s/atom]. Multiply by ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "Na"], TraditionalForm]]],
 " to obtain \[CapitalPhi] in ph/s/m^2.\n\tFluxMesoInt: Integrated flux in \
the mesosphere (called ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["F", "m"], TraditionalForm]]],
 " in Holzloehner et al., A&A 510, A20 (2010), Eq.(7)) [ph*m^2/s/sr/atom]\n\t\
FWHMSpeckle: Instantaneous laser spot size in the mesosphere (central \
speckle) [m]\n\tFWHM2mom: Return light spot size based on second moment [m]\n\
\tIMesoMax1: Peak mesospheric laser light intensity [W/m^2]\n\tfitFunction: \
Fit function to fit PsiVector, hence approximate \[CapitalPsi](",
 StyleBox["I",
  FontSlant->"Italic"],
 ")\n\tfitRules: Rules for fit function.\nExample:\n\tresSce = \
Sce[myPsiMesoParams,PsiVector,mySceOpts]\n\tPlot[resSce[[-2]][x] /. \
resSce[[-1",
 "]]",
 ", {x, 0, x1}]\nFunction compared on 19Mar09 with MATLAB routine \
Exciter_beam_interp(): agrees."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Options", "[", "Sce", "]"}], "=", 
  RowBox[{"Options", "[", "InterBeamMeso", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DefaultParameters", ":>", "$DefaultLGSParameters"}], ",", 
   RowBox[{"IMesoMax", "->", "0.`"}], ",", 
   RowBox[{"IMesoInitial", "->", "0.35`"}], ",", 
   RowBox[{"MaxStepRatio", "->", "20"}], ",", 
   RowBox[{"IMesoFactor", "->", "1.`"}], ",", 
   RowBox[{"NSamples", "->", "6"}], ",", 
   RowBox[{"LaserSpotMesoFWHM", "->", "0"}]}], "}"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Sce", "[", 
    RowBox[{"params_", ",", "PsiVector_", ",", 
     RowBox[{"opts", ":", 
      RowBox[{"OptionsPattern", "[", "]"}]}]}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "sceOut", ",", "FluxMesoInt", ",", "FWHM2mom", ",", "IMesoMax1", ",", 
       "airmass", ",", "FluxGroundInt", ",", "FWHMMeso1", ",", "params1", ",",
        "fitFunction", ",", "fitRules", ",", "errTotRel"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"{", 
        RowBox[{
        "FluxMesoInt", ",", "FWHMMeso1", ",", "FWHM2mom", ",", "IMesoMax1", 
         ",", "fitFunction", ",", "fitRules", ",", "errTotRel"}], "}"}], "=", 
       "\[IndentingNewLine]", 
       RowBox[{"InterBeamMeso", "[", 
        RowBox[{"params", ",", "PsiVector", ",", "opts"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"params1", "=", 
       RowBox[{"Join", "[", 
        RowBox[{"params", ",", 
         RowBox[{"OptionValue", "[", "DefaultParameters", "]"}]}], "]"}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"airmass", " ", "=", " ", 
       RowBox[{
        RowBox[{"Sec", "[", "Zenith", "]"}], "/.", "params1"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"FluxGroundInt", "=", 
       RowBox[{
        RowBox[{
         SuperscriptBox["Ta", "airmass"], "*", 
         RowBox[{
          RowBox[{"FluxMesoInt", "/", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"HNa", "-", "Htele"}], ")"}], "2"]}], "/", "airmass"}]}],
         "/.", "params1"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"sceOut", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"FluxMesoInt", "/", "PLaunch"}], "/", 
         SuperscriptBox["Ta", "airmass"]}], "/.", "params1"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "sceOut", ",", "FluxGroundInt", ",", "FluxMesoInt", ",", "FWHMMeso1", 
        ",", "FWHM2mom", ",", "IMesoMax1", ",", "fitFunction", ",", 
        "fitRules", ",", "errTotRel"}], "}"}]}]}], "\[IndentingNewLine]", 
    "]"}]}], " ", ";"}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["AbsCrossSection", "Subsection"],

Cell[TextData[{
 "Absorption cross section and cycle time\nInput values:\n\tAs for ",
 "PsiMeso",
 "[]\nOutput values:\n\tsig: absorption cross section [m^2]\n\tps: Result of \
PhiSec (computed for 200 velocity classes)  [ph/s/sr/(W/m^2)]\n\tW: number of \
spontaneous excitations per time [ph/s/atom], inverse of cycle time \
(neglecting stimulated emission)\nExample:\n\t",
 Cell[BoxData[
  RowBox[{"AbsCrossSection", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"IMeso", "->", "50.0"}], ",", 
     RowBox[{"\[CapitalDelta]fLaser", "->", 
      RowBox[{"0.0", "*", 
       SuperscriptBox["10", "9"]}]}], ",", 
     RowBox[{"RepumpFraction", "->", "0.12"}], ",", 
     RowBox[{"BG", "->", "0.0"}], ",", 
     RowBox[{"LightPolarization", "->", "1"}]}], "}"}], "]"}]],
  CellChangeTimes->{{3.453458642270107*^9, 3.4534586839266233`*^9}, {
    3.45345932097462*^9, 3.4534593735826483`*^9}, {3.4534604026281776`*^9, 
    3.4534604677992187`*^9}, {3.4534614038653617`*^9, 
    3.4534614052872186`*^9}, {3.4534614473804297`*^9, 3.453461449552277*^9}, {
    3.4534618595782785`*^9, 3.4534618900466385`*^9}, 3.45346197588929*^9, {
    3.4534620377947474`*^9, 3.4534620381072435`*^9}, {3.45346207038808*^9, 
    3.4534620767786236`*^9}}],
 "\nyields\n\t",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{
   "1.1424135229284697`*^-15", ",", "402.5300545224092`", ",", 
    "169414.22073090656`"}], "}"}]],
  CellChangeTimes->{3.453461899280895*^9, 3.453461982904825*^9, 
   3.4534620459665174`*^9, 3.4534620882472267`*^9}],
 "\nhence a cross-section of 1.14 times the low-irradiance cross section (1.0 \
x 10^(-11) cm^2), psi=402 ph/s/sr/(W/m^2), and the cycle time is\n\t\
1/169414.22073090656`*10^6 us = 5.90 us,\nhence very close to the Larmor time \
in Paranal (5.8us)."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Options", "[", "AbsCrossSection", "]"}], "=", 
  RowBox[{"FilterRules", "[", 
   RowBox[{
    RowBox[{"Options", "[", "PsiMeso", "]"}], ",", 
    RowBox[{"Except", "[", 
     RowBox[{"{", 
      RowBox[{"ReturnAllSteps", ",", "FluxReport"}], "}"}], "]"}]}], 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DefaultParameters", ":>", "$DefaultLGSParameters"}], ",", 
   RowBox[{"RefineResult", "->", 
    RowBox[{"{", "100", "}"}]}], ",", 
   RowBox[{"InitVGParams", "->", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "3"}], "}"}]}], "}"}]}], ",", 
   RowBox[{"PlotVelocityGroups", "->", "False"}], ",", 
   RowBox[{"InitVG", "->", "Automatic"}], ",", 
   RowBox[{"SkipLinearSolve", "->", "False"}]}], "}"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"AbsCrossSection", "[", 
    RowBox[{"syst_", ",", "params_", ",", 
     RowBox[{"opts", ":", 
      RowBox[{"OptionsPattern", "[", "]"}]}]}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "ps", ",", "exStates", ",", "fb2", ",", "W", ",", "constc", ",", 
       "consth", ",", "nu", ",", "sig", ",", "nfopts", ",", "params1"}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"params1", "=", 
       RowBox[{"Join", "[", 
        RowBox[{"params", ",", 
         RowBox[{"OptionValue", "[", "DefaultParameters", "]"}]}], "]"}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"nfopts", "=", 
       RowBox[{"FilterRules", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"FluxReport", "->", 
              RowBox[{"{", 
               RowBox[{"DMSolution", ",", "BlochMatrix"}], "}"}]}], ",", 
             RowBox[{"ReturnAllSteps", "->", "False"}]}], "}"}], ",", "opts", 
           ",", 
           RowBox[{"Options", "[", "AbsCrossSection", "]"}]}], "}"}], ",", 
         RowBox[{"Options", "[", "PsiMeso", "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"ps", "=", 
       RowBox[{"PsiMeso", "[", 
        RowBox[{"syst", ",", "params", ",", "nfopts"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"fb2", "=", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"BlochMatrix", "/.", 
          RowBox[{"ps", "[", 
           RowBox[{"[", "2", "]"}], "]"}]}], ")"}], "[", 
        RowBox[{"[", "3", "]"}], "]"}]}], ";", " ", "\[IndentingNewLine]", 
      RowBox[{"exStates", "=", 
       RowBox[{"Flatten", "[", 
        RowBox[{"Position", "[", 
         RowBox[{"fb2", ",", 
          RowBox[{"x_", "/;", 
           RowBox[{"x", ">", "0"}]}]}], "]"}], "]"}]}], ";", " ", 
      RowBox[{"(*", " ", 
       RowBox[{
       "Find", " ", "indices", " ", "of", " ", "radiating", " ", "states"}], 
       " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"fb2", "[", 
        RowBox[{"[", "exStates", "]"}], "]"}], "=", 
       RowBox[{"6.154149127341655`", "*", 
        RowBox[{"10", "^", "7"}]}]}], ";", " ", 
      RowBox[{"(*", " ", 
       RowBox[{"Neglect", " ", "antenna", " ", "factors", " ", 
        RowBox[{"(", 
         RowBox[{"set", " ", "to", " ", "1.0"}], ")"}]}], " ", "*)"}], 
      RowBox[{"W", "=", 
       RowBox[{"Re", "[", 
        RowBox[{"fb2", ".", 
         RowBox[{"(", 
          RowBox[{"DMSolution", "/.", 
           RowBox[{"ps", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], ")"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"constc", "=", 
       RowBox[{"2.99792", "*", 
        RowBox[{"10", "^", "8"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"consth", "=", 
       RowBox[{"6.626069", "*", 
        RowBox[{"10", "^", 
         RowBox[{"(", 
          RowBox[{"-", "34"}], ")"}]}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"nu", "=", 
       RowBox[{
        RowBox[{"consth", "*", 
         RowBox[{"constc", "/", "LightWavelength"}]}], "/.", "params1"}]}], 
      ";", " ", 
      RowBox[{"(*", "  ", 
       RowBox[{
       "Compute", " ", "frequency", " ", "of", " ", "D2b", " ", "line"}], " ",
        "*)"}], "\[IndentingNewLine]", 
      RowBox[{"sig", "=", 
       RowBox[{
        RowBox[{"nu", "*", 
         RowBox[{"W", "/", "IMeso"}]}], "/.", "params1"}]}], ";", 
      RowBox[{"(*", " ", 
       RowBox[{
        RowBox[{"absorption", " ", "cross"}], "-", 
        RowBox[{"section", " ", "in", " ", "m"}]}], " ", "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"sig", ",", 
        RowBox[{"ps", "[", 
         RowBox[{"[", "1", "]"}], "]"}], ",", "W"}], "}"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], " ", ";"}]], "Input",
 InitializationCell->True],

Cell[" ", "Text",
 Editable->False,
 Selectable->False,
 CellFrame->{{0, 0}, {0, 0.5}},
 ShowCellBracket->False,
 CellMargins->{{0, 0}, {1, 1}},
 CellElementSpacings->{"CellMinHeight"->1},
 CellFrameMargins->0,
 CellFrameColor->RGBColor[0, 0, 1],
 CellSize->{Inherited, 3}],

Cell[" ", "Text",
 Editable->False,
 Selectable->False,
 CellFrame->{{0, 0}, {0, 3}},
 ShowCellBracket->False,
 CellMargins->{{0, 0}, {1, 1}},
 CellElementSpacings->{"CellMinHeight"->1},
 CellFrameMargins->0,
 CellFrameColor->RGBColor[0, 0, 1],
 CellSize->{Inherited, 5}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
StepPulseFlux -- Time-dependent solution with piecewise constant coefficients\
\>", "Subsection"],

Cell[BoxData[
 RowBox[{
  RowBox[{"convertDM", "[", 
   RowBox[{"olddm_", ",", "oldvg_", ",", "newvg_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "oldbins", ",", "newbins", ",", "dm", ",", "convertlist", ",", "leftpos",
       ",", "leftfrac", ",", "middlepos", ",", "middlefrac", ",", "rightpos", 
      ",", "rightfrac", ",", "containedpos", ",", "containedfrac", ",", "inf",
       ",", "leftovers"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"oldbins", "=", 
      RowBox[{
       RowBox[{
        RowBox[{"oldvg", "[", 
         RowBox[{"[", 
          RowBox[{"All", ",", "3"}], "]"}], "]"}], "/.", 
        RowBox[{
         RowBox[{"-", "Infinity"}], "->", 
         RowBox[{"-", "inf"}]}]}], "/.", 
       RowBox[{"Infinity", "->", "inf"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"newbins", "=", 
      RowBox[{
       RowBox[{
        RowBox[{"newvg", "[", 
         RowBox[{"[", 
          RowBox[{"All", ",", "3"}], "]"}], "]"}], "/.", 
        RowBox[{
         RowBox[{"-", "Infinity"}], "->", 
         RowBox[{"-", "inf"}]}]}], "/.", 
       RowBox[{"Infinity", "->", "inf"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"dm", "=", 
      RowBox[{"Partition", "[", 
       RowBox[{"olddm", ",", 
        RowBox[{
         RowBox[{"Length", "[", "olddm", "]"}], "/", 
         RowBox[{"Length", "[", "oldbins", "]"}]}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"convertlist", "=", 
      RowBox[{"Table", "[", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"leftpos", "=", 
          RowBox[{"Position", "[", 
           RowBox[{"oldbins", ",", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"x1_", ",", "x2_"}], "}"}], "/;", 
             RowBox[{"x1", "<", 
              RowBox[{"bin", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "<", "x2", "<", 
              RowBox[{"bin", "[", 
               RowBox[{"[", "2", "]"}], "]"}]}]}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"leftfrac", "=", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"#", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "-", 
               RowBox[{"bin", "[", 
                RowBox[{"[", "1", "]"}], "]"}]}], ")"}], "/", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"#", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "-", 
               RowBox[{"#", "[", 
                RowBox[{"[", "1", "]"}], "]"}]}], ")"}]}], "&"}], "/@", 
           RowBox[{"Extract", "[", 
            RowBox[{"oldbins", ",", "leftpos"}], "]"}]}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"middlepos", "=", 
          RowBox[{"Position", "[", 
           RowBox[{"oldbins", ",", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"x1_", ",", "x2_"}], "}"}], "/;", 
             RowBox[{
              RowBox[{"bin", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "<=", " ", "x1", "<=", "x2", "<=", 
              RowBox[{"bin", "[", 
               RowBox[{"[", "2", "]"}], "]"}]}]}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"middlefrac", "=", 
          RowBox[{
           RowBox[{"1", "&"}], "/@", 
           RowBox[{"Extract", "[", 
            RowBox[{"oldbins", ",", "middlepos"}], "]"}]}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"rightpos", "=", 
          RowBox[{"Position", "[", 
           RowBox[{"oldbins", ",", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"x1_", ",", "x2_"}], "}"}], "/;", 
             RowBox[{
              RowBox[{"bin", "[", 
               RowBox[{"[", "1", "]"}], "]"}], "<", "x1", "<", 
              RowBox[{"bin", "[", 
               RowBox[{"[", "2", "]"}], "]"}], "<", "x2"}]}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"rightfrac", "=", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"bin", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "-", 
               RowBox[{"#", "[", 
                RowBox[{"[", "1", "]"}], "]"}]}], ")"}], "/", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"#", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "-", 
               RowBox[{"#", "[", 
                RowBox[{"[", "1", "]"}], "]"}]}], ")"}]}], "&"}], "/@", 
           RowBox[{"Extract", "[", 
            RowBox[{"oldbins", ",", "rightpos"}], "]"}]}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"containedpos", "=", 
          RowBox[{"Position", "[", 
           RowBox[{"oldbins", ",", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"x1_", ",", "x2_"}], "}"}], "/;", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"x1", "<=", 
                RowBox[{"bin", "[", 
                 RowBox[{"[", "1", "]"}], "]"}], "<=", 
                RowBox[{"bin", "[", 
                 RowBox[{"[", "2", "]"}], "]"}], "<", "x2"}], ")"}], "||", 
              RowBox[{"(", 
               RowBox[{"x1", "<", 
                RowBox[{"bin", "[", 
                 RowBox[{"[", "1", "]"}], "]"}], "<=", 
                RowBox[{"bin", "[", 
                 RowBox[{"[", "2", "]"}], "]"}], "<=", "x2"}], ")"}]}]}]}], 
           "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"containedfrac", "=", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"bin", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "-", 
               RowBox[{"bin", "[", 
                RowBox[{"[", "1", "]"}], "]"}]}], ")"}], "/", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"#", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "-", 
               RowBox[{"#", "[", 
                RowBox[{"[", "1", "]"}], "]"}]}], ")"}]}], "&"}], "/@", 
           RowBox[{"Extract", "[", 
            RowBox[{"oldbins", ",", "containedpos"}], "]"}]}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"Flatten", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Transpose", "@", 
              RowBox[{"{", 
               RowBox[{"leftfrac", ",", "leftpos"}], "}"}]}], ",", 
             RowBox[{"Transpose", "@", 
              RowBox[{"{", 
               RowBox[{"middlefrac", ",", "middlepos"}], "}"}]}], ",", 
             RowBox[{"Transpose", "@", 
              RowBox[{"{", 
               RowBox[{"rightfrac", ",", "rightpos"}], "}"}]}], ",", 
             RowBox[{"Transpose", "@", 
              RowBox[{"{", 
               RowBox[{"containedfrac", ",", "containedpos"}], "}"}]}]}], 
            "}"}], ",", "1"}], "]"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"bin", ",", "newbins"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"leftovers", "=", 
      RowBox[{"Chop", "[", 
       RowBox[{
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"1", "-", 
             RowBox[{"Total", "@", 
              RowBox[{"Cases", "[", 
               RowBox[{
                RowBox[{"Flatten", "[", 
                 RowBox[{"convertlist", ",", "1"}], "]"}], ",", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{"x_", ",", 
                   RowBox[{"{", "i", "}"}]}], "}"}], "->", "x"}]}], "]"}]}]}],
             ",", 
            RowBox[{"{", "i", "}"}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", 
            RowBox[{"Length", "[", "oldbins", "]"}]}], "}"}]}], "]"}], ",", 
        RowBox[{"2", "$MachineEpsilon"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"convertlist", "[", 
       RowBox[{"[", "1", "]"}], "]"}], "=", 
      RowBox[{"DeleteCases", "[", 
       RowBox[{
        RowBox[{"Take", "[", 
         RowBox[{"leftovers", ",", 
          RowBox[{"Floor", "[", 
           RowBox[{
            RowBox[{"Length", "[", "oldbins", "]"}], "/", "2"}], "]"}]}], 
         "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"convertlist", "[", 
       RowBox[{"[", 
        RowBox[{"-", "1"}], "]"}], "]"}], "=", 
      RowBox[{"DeleteCases", "[", 
       RowBox[{
        RowBox[{"Drop", "[", 
         RowBox[{"leftovers", ",", 
          RowBox[{"Floor", "[", 
           RowBox[{
            RowBox[{"Length", "[", "oldbins", "]"}], "/", "2"}], "]"}]}], 
         "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "_"}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Flatten", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Sum", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"conv", "[", 
            RowBox[{"[", "1", "]"}], "]"}], 
           RowBox[{"Extract", "[", 
            RowBox[{"dm", ",", 
             RowBox[{"conv", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], ",", 
          RowBox[{"{", 
           RowBox[{"conv", ",", "#"}], "}"}]}], "]"}], "&"}], "/@", 
       "convertlist"}], "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[" ", "Text",
 Editable->False,
 Selectable->False,
 CellFrame->{{0, 0}, {0, 0.5}},
 ShowCellBracket->False,
 CellMargins->{{0, 0}, {1, 1}},
 CellElementSpacings->{"CellMinHeight"->1},
 CellFrameMargins->0,
 CellFrameColor->RGBColor[0, 0, 1],
 CellSize->{Inherited, 3}],

Cell[BoxData[
 RowBox[{"(*", "  ", 
  RowBox[{
  "Convert", " ", "velocity", " ", "groups", " ", "for", " ", "use", " ", 
   "within", " ", 
   RowBox[{"StepPulseFlux", "[", "]"}]}], " ", "*)"}]], "Input"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Options", "[", "makeVg", "]"}], "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"DefaultParameters", ":>", "$DefaultLGSParameters"}], "}"}], 
     ",", 
     RowBox[{"Options", "[", "ReturnFlux", "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"SetOptions", "[", 
  RowBox[{"makeVg", ",", 
   RowBox[{"RefineResult", "->", 
    RowBox[{"{", "100", "}"}]}]}], "]"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DefaultParameters", ":>", "$DefaultLGSParameters"}], ",", 
   RowBox[{"RefineResult", "->", 
    RowBox[{"{", "100", "}"}]}], ",", 
   RowBox[{"InitVGParams", "->", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "3"}], "}"}]}], "}"}]}], ",", 
   RowBox[{"FluxReport", "->", 
    RowBox[{"{", "}"}]}], ",", 
   RowBox[{"ReturnAllSteps", "->", "False"}], ",", 
   RowBox[{"PlotVelocityGroups", "->", "False"}], ",", 
   RowBox[{"InitVG", "->", "Automatic"}], ",", 
   RowBox[{"SkipLinearSolve", "->", "False"}]}], "}"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"makeVg", "[", 
   RowBox[{"speclist_", ",", "atdata1_", ",", "params_", ",", 
    RowBox[{"opts", ":", 
     RowBox[{"OptionsPattern", "[", "]"}]}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "dw", ",", "nog", ",", "allfreqs", ",", "vg", ",", "freqs", ",", "spec", 
      ",", "bendfunc", ",", "wf", ",", "nb"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"nb", "=", 
      RowBox[{"NDither", "/.", "params"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"dw", "=", 
      RowBox[{"DopplerWidth", "/.", "params"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"nog", "=", 
      RowBox[{
       RowBox[{"OptionValue", "[", "RefineResult", "]"}], "/.", 
       RowBox[{
        RowBox[{"False", "|", "None"}], "->", 
        RowBox[{"{", "}"}]}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"freqs", "=", 
      RowBox[{"ResonanceFrequencies", "/.", "atdata1"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"allfreqs", "=", 
      RowBox[{"Union", "@", 
       RowBox[{"Flatten", "@", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"freqs", "-", 
              RowBox[{"DetuningA", "[", "i", "]"}]}], "/.", "params"}], ",", 
            RowBox[{
             RowBox[{"freqs", "-", 
              RowBox[{"DetuningB", "[", "i", "]"}]}], "/.", "params"}]}], 
           "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "nb"}], "}"}]}], "]"}]}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"spec", "=", 
      RowBox[{"Interpolation", "[", 
       RowBox[{"speclist", ",", 
        RowBox[{"InterpolationOrder", "->", "1"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"bendfunc", "=", 
      RowBox[{"Interpolation", "[", 
       RowBox[{
        RowBox[{"BendList", "[", "speclist", "]"}], ",", 
        RowBox[{"InterpolationOrder", "->", "1"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"wf", "=", 
      RowBox[{"NWeightFunction", "[", 
       RowBox[{"spec", ",", "bendfunc", ",", 
        RowBox[{"{", 
         RowBox[{"5", ",", "5", ",", "5"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"x", ",", 
          RowBox[{
           RowBox[{"-", "3"}], "dw"}], ",", 
          RowBox[{"3", "dw"}]}], "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"vg", "=", 
      RowBox[{"VelocityGroups2", "[", 
       RowBox[{"wf", ",", 
        RowBox[{"{", 
         RowBox[{"x", ",", 
          RowBox[{
           RowBox[{"-", "3"}], "dw"}], ",", 
          RowBox[{"3", "dw"}]}], "}"}], ",", 
        RowBox[{"nog", "[", 
         RowBox[{"[", 
          RowBox[{"-", "1"}], "]"}], "]"}], ",", "dw", ",", 
        RowBox[{
         RowBox[{"5", " ", "RecoilShift"}], "/.", "params"}], ",", "allfreqs",
         ",", 
        RowBox[{"ShowPlot", "->", 
         RowBox[{"OptionValue", "[", "PlotVelocityGroups", "]"}]}]}], 
       "]"}]}]}]}], "\[IndentingNewLine]", 
   RowBox[{"(*", "*)"}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[" ", "Text",
 Editable->False,
 Selectable->False,
 CellFrame->{{0, 0}, {0, 0.5}},
 ShowCellBracket->False,
 CellMargins->{{0, 0}, {1, 1}},
 CellElementSpacings->{"CellMinHeight"->1},
 CellFrameMargins->0,
 CellFrameColor->RGBColor[0, 0, 1],
 CellSize->{Inherited, 3}],

Cell["\<\
filterPops[] first extracts the populations specified by vec from the sol \
input. It then partitions that list into sublists that correspond to each \
velocity group, and scales the sublists by the width of each bin. That's \
necessary because the wider bins will have more population just because \
they're wider; we want to remove this effect. It then totals the populations \
for each velocity group and forms a two-column table with the positions of \
the bins as the first column and the scaled populations as the second column. \
If one removes the Total command you can get the scaled populations for each \
sublevel, rather than the sum of all sublevels. \
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"filterPops", "[", 
    RowBox[{"sol_", ",", "vec_", ",", "vg_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "pops", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"pops", "=", 
       RowBox[{"getPops", "[", 
        RowBox[{"sol", ",", "vec"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Mod", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Length", "[", "pops", "]"}], "/", 
             RowBox[{"Length", "[", "vg", "]"}]}], ",", "1"}], "]"}], "!=", 
          "0"}], "||", 
         RowBox[{
          RowBox[{
           RowBox[{"Length", "[", "pops", "]"}], "/", 
           RowBox[{"Length", "[", "vg", "]"}]}], "<", "1"}]}], ",", 
        RowBox[{
         RowBox[{
         "Print", "[", 
          "\"\<filterPops[]: Length[pops] is not a multiple of Length[vg], \
Aborting!\>\"", "]"}], ";", 
         RowBox[{"Abort", "[", "]"}], ";"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Transpose", "@", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"vg", "[", 
           RowBox[{"[", 
            RowBox[{"All", ",", "1"}], "]"}], "]"}], "/", 
          RowBox[{"(", 
           RowBox[{"2.0", "\[Pi]", " ", 
            RowBox[{"10", "^", "9"}]}], ")"}]}], ",", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{"2.0", "\[Pi]", " ", 
             RowBox[{"10", "^", "9"}]}], ")"}]}], 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"Total", "/@", 
             RowBox[{"Partition", "[", 
              RowBox[{"pops", ",", 
               RowBox[{
                RowBox[{"Length", "[", "pops", "]"}], "/", 
                RowBox[{"Length", "[", "vg", "]"}]}]}], "]"}]}], ")"}], "/", 
           RowBox[{"Subtract", "@@@", 
            RowBox[{"vg", "[", 
             RowBox[{"[", 
              RowBox[{"All", ",", "3"}], "]"}], "]"}]}]}]}]}], "}"}]}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]], "Input",
 InitializationCell->True],

Cell[" ", "Text",
 Editable->False,
 Selectable->False,
 CellFrame->{{0, 0}, {0, 0.5}},
 ShowCellBracket->False,
 CellMargins->{{0, 0}, {1, 1}},
 CellElementSpacings->{"CellMinHeight"->1},
 CellFrameMargins->0,
 CellFrameColor->RGBColor[0, 0, 1],
 CellSize->{Inherited, 3}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"getPops", "[", 
    RowBox[{"sol_", ",", "vec_"}], "]"}], ":=", 
   RowBox[{"sol", "[", 
    RowBox[{"[", 
     RowBox[{"Flatten", "[", 
      RowBox[{"Position", "[", 
       RowBox[{"vec", ",", 
        RowBox[{"x_", "/;", 
         RowBox[{"x", "!=", "0"}]}]}], "]"}], "]"}], "]"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True],

Cell[" ", "Text",
 Editable->False,
 Selectable->False,
 CellFrame->{{0, 0}, {0, 0.5}},
 ShowCellBracket->False,
 CellMargins->{{0, 0}, {1, 1}},
 CellElementSpacings->{"CellMinHeight"->1},
 CellFrameMargins->0,
 CellFrameColor->RGBColor[0, 0, 1],
 CellSize->{Inherited, 3}],

Cell[TextData[{
 "Solve the time-dependent problem for periodic irradiation pattern and \
integrate flux return over the last period. Uses phiv[] to\nsolve the \
inhomogeneous problem (d/dt)\[Rho] = A\[Rho]+",
 StyleBox["b",
  FontSlant->"Italic"],
 " for piecewise constant coefficients.\nInput:\n\tsyst: Bloch equation \
system\n\tparams: Physical simulation parameters for ReturnFlux[]\n\t\
nperiods: maximum number of periods to simulate\n\tcomplexE: Vector, \
subdividing the pulse into n subspans, scaling the \n\t\tirradiance in each \
subspan so that I[[j]] == IMeso |complexE[[j,2]]|^2, \n\t\tand \
Arg[complexE[[j,2]]] is the optical phase at step j. The entries \
complexE[[j,1]] contain the corresponding \ttime steps relative to the \
beginning of the simulated period. Hence, complexE[[-1,1]] equals the \
duration of the simulated period. The time steps do not have to be constant. \
Note that all time steps, including complexE[[1,1]], must be larger than 0 \
and strictly increasing.\n\tchirpRate: Chirp rate (positive values yield blue \
shifting of laser line over time) [Hz/s]\n\tfname: File name to save the \
results structure \"res\" in, using DumpSave[fname,res]\n\topts: option set \
for PsiMeso[]\nOutput:\nGlobal`Res={psiAvg, psiAvgTotal, psiSpanAll, \
psiAvgHist, Iavg, popgr8All, popex16All, {PhiVErrors -> errAll, PsiIavg -> \
psiIavg1, GroundPopulation -> popgr, ExcitedPopulation -> popex, \
FinalGroundPops -> groundPops, FinalExcitedPops -> excitedPops, ExcitedPops \
-> excitedPopsAll, LastDM -> {w, vg}, RFParamsOut -> {rfparams1, params1, \
opts}}}\n\n\tpsiAvg: Specific return flux averaged over the last period \
[ph/s/sr/atom/(W/m^2)]\n\tpsiAvgTotal: psiAvg summed over all periods \
[ph/s/sr/atom/(W/m^2)]\n\tpsiSpanAll: vector of pairs {tn, return flux(tn)}, \
return flux [ph/s/sr/atom/(W/m^2)]\n\tpsiAvgHist: vector of psiAvg values for \
each period\n\tIavg: Irradiance, averaged over a simulation period [W/m^2]\n\t\
popgr8All: Array of dimension Nx8, where N equals the length of psiSpanAll \
and corresponds to the time steps given by psiSpanAll[[All,1]]. The values of \
popgr8All yield the 8 ground state populations of the velocity class that is \
closest to +5 MHz, where in many systems a peak of the polarization is \
observed (\"busy velocity class\"), normalized by the total ground+excited \
state population in this velocity class. For darkness in equilibrium we thus \
have popgr8All[[All,All]]==1/8. The states are ordered {(F=2,m=2), \
(F=2,m=1)..., (F=2,m=-2), (F=1,m=1), (F=1,m=0), (F=2,m=-1)} (with \
LightPolarization->+1, the (F=2,m=2) is pumped most strongly)\n\tpopex16All: \
same as popgr8All, but for the 16 excited states. Sequence starts with \
(F=3,m=3).\n\tPhiVErrors: vector of numerical errors returned by phiv[], \
corresponding to psiSpanAll\n\tpsiIavg: value of psi for average irradiance\n\
\tpopgr: ground state population\n\tpopex: Excited state population\n\tw: \
final solution vector (b)\nThe routine computes the average return flux \
psiAvg defined as\n\n   psiAvg = 1/(trep * Iavg)  \\int_{t0}^{t0+trep} Psi(t) \
dt,\n\nwhere Psi(t) is the instantaneous non-normalized return flux in \
ph/s/sr/atom, t0 is the begin of the last period, and Iavg = Ipulse*duty, \
where Ipulse is the pulse irradiance.\n\nThe iteration starts from thermal \
equilibrium (InitialState -> LightOff) or from the steady-state solution at \
Ipulse (InitialState ",
 "->",
 " LightOn) and stops once the difference of psiAvg between two subsequent \
iterations drops below some threshold. In the verbose simulation output, \
\"peak cw psi\" is the steady-state result psi(I_peak), and \"flux from avg \
irradiance\" is similarly psi(I_avg).\nTo achieve convergence, allow for a \
total simulation time of several times 16ns. At low irradiance, you may have \
to simulate several collision time scales."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Options", "[", "StepPulseFlux", "]"}], "=", 
  RowBox[{"FilterRules", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Options", "[", "PsiMeso", "]"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"VGMethod", "->", "ConvertDM"}], ",", 
        RowBox[{"InitialState", "->", "LightOff"}], ",", 
        RowBox[{"KrylovDimension", "->", "Automatic"}], ",", 
        RowBox[{"VerbosityLevel", "->", "2"}], ",", 
        RowBox[{"Tolerance", "->", 
         RowBox[{"10.0", "^", 
          RowBox[{"(", 
           RowBox[{"-", "7"}], ")"}]}]}], ",", 
        RowBox[{"TerminationThresh", "->", 
         RowBox[{"0.25", "*", 
          RowBox[{"10", "^", 
           RowBox[{"(", 
            RowBox[{"-", "2"}], ")"}]}]}]}], ",", 
        RowBox[{"VGfreezePeriod", "->", "12"}]}], "}"}]}], "}"}], ",", 
    RowBox[{"Except", "[", 
     RowBox[{"{", 
      RowBox[{"FluxReport", ",", "ReturnAllSteps"}], "}"}], "]"}]}], 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DefaultParameters", ":>", "$DefaultLGSParameters"}], ",", 
   RowBox[{"RefineResult", "->", 
    RowBox[{"{", "100", "}"}]}], ",", 
   RowBox[{"InitVGParams", "->", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "2"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "3"}], "}"}]}], "}"}]}], ",", 
   RowBox[{"PlotVelocityGroups", "->", "False"}], ",", 
   RowBox[{"InitVG", "->", "Automatic"}], ",", 
   RowBox[{"SkipLinearSolve", "->", "False"}], ",", 
   RowBox[{"VGMethod", "->", "ConvertDM"}], ",", 
   RowBox[{"InitialState", "->", "LightOff"}], ",", 
   RowBox[{"KrylovDimension", "->", "Automatic"}], ",", 
   RowBox[{"VerbosityLevel", "->", "2"}], ",", 
   RowBox[{"Tolerance", "->", "1.`*^-7"}], ",", 
   RowBox[{"TerminationThresh", "->", "0.0025`"}], ",", 
   RowBox[{"VGfreezePeriod", "->", "12"}]}], "}"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"StepPulseFlux", "[", 
    RowBox[{
    "syst_", ",", "params_", ",", "nperiods_", ",", "complexE_", ",", 
     "chirpRate_", ",", "fname_", ",", 
     RowBox[{"opts", ":", 
      RowBox[{"OptionsPattern", "[", "]"}]}]}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "trep", ",", "tNow", ",", "i", ",", "fullAmat", ",", "w", ",", "err", 
       ",", "errAll", ",", "psiSpan", ",", "psiSpanAll", ",", "tend", ",", 
       "psiInit", ",", "fns", ",", "hh", ",", "psiAvg", ",", "psiAvgTotal", 
       ",", "tns", ",", "tspanMaxFactor", ",", "tol", ",", "m", ",", "Iavg", 
       ",", "psiIavg1", ",", "popgr", ",", "popex", ",", "refine1", ",", 
       "fullbvecOn", ",", "psiAvgLast", ",", "psiAvgHist", ",", "j", ",", 
       "errLoc", ",", "psiSpanLoc", ",", "IMesoNow", ",", "relativeI", ",", 
       "phase", ",", "tDiffs", ",", "psi", ",", "inst", ",", "lastvg", ",", 
       "vg", ",", "nfopts", ",", "params1", ",", "sol", ",", "fullfluorvecOn",
        ",", "fullbvecOff", ",", "fullfluorvecOff", ",", "bvec", ",", "fvec", 
       ",", "groundPops", ",", "excitedPops", ",", "excitedPopsAll", ",", 
       "excitedPopsHelp", ",", "speclist", ",", "NDitherLoc", ",", 
       "rfparams1", ",", "indVgMaxEx", ",", "popgr8", ",", "popex16", ",", 
       "popgr8All", ",", "popex16All", ",", "\[CapitalDelta]fLaserNow", ",", 
       "tu1", ",", "tu2", ",", "tuLast", ",", "terminationCriterion", ",", 
       "fwriteCriterion", ",", "help1", ",", "tu11", ",", "tPrep", ",", 
       "popgr8Weighted", ",", "popgr8WeightedAll", ",", "iVGfreezeLoc", ",", 
       "fullAmatLI", ",", "A110L", ",", "grStates", ",", "exStates"}], "}"}], 
     ",", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{"Argument", " ", "checking"}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"complexE", "[", 
          RowBox[{"[", 
           RowBox[{"1", ",", "1"}], "]"}], "]"}], "<=", "0"}], ",", 
        RowBox[{
         RowBox[{
         "Print", "[", 
          "\"\<First time step must be larger than zero. Aborting.\>\"", 
          "]"}], ";", 
         RowBox[{"Abort", "[", "]"}], ";"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", "Period", " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"trep", "=", 
       RowBox[{"complexE", "[", 
        RowBox[{"[", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "1"}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"trep", "<=", "0.0"}], "||", 
         RowBox[{"trep", ">", "0.1"}]}], ",", 
        RowBox[{
         RowBox[{"Print", "[", 
          RowBox[{
          "\"\<trep: \>\"", ",", " ", "trep", ",", 
           "\"\<s, which may not be what you want. Aborting.\>\""}], "]"}], 
         ";", 
         RowBox[{"Abort", "[", "]"}], ";"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"Massage", " ", "options"}], "*)"}], "\[IndentingNewLine]", 
      RowBox[{"inst", "=", 
       RowBox[{"OptionValue", "[", "InitialState", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"Tolerance", " ", "for", " ", 
        RowBox[{"phiv", "[", "]"}]}], "*)"}], "\[IndentingNewLine]", 
      RowBox[{"tol", "=", 
       RowBox[{"OptionValue", "[", "Tolerance", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"Krylov", " ", "subspace", " ", "dimension", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", "1"}], " ", "means", " ", 
          RowBox[{"default", ":", "30"}]}], ")"}]}], "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"m", "=", 
       RowBox[{
        RowBox[{"OptionValue", "[", "KrylovDimension", "]"}], "/.", 
        RowBox[{"Automatic", "->", 
         RowBox[{"-", "1"}]}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"Solver", " ", "parameters"}], " ", "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"nfopts", "=", 
       RowBox[{"FilterRules", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"ReturnAllSteps", "->", "False"}], ",", "opts", ",", 
           RowBox[{"Options", "[", "StepPulseFlux", "]"}]}], "}"}], ",", 
         RowBox[{"Options", "[", "PsiMeso", "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"LGS", " ", "Parameters", " ", "for", " ", 
        RowBox[{"PsiMeso", "[", "]"}]}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"params1", "=", 
       RowBox[{"Join", "[", 
        RowBox[{"params", ",", 
         RowBox[{"OptionValue", "[", "DefaultParameters", "]"}]}], "]"}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "Normalize", " ", "the", " ", "relative", " ", "irradiances", " ", 
        "by", " ", "the", " ", "peak", " ", "irradiance", " ", "in", " ", 
        "time", " ", "domain"}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"relativeI", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{"complexE", "[", 
           RowBox[{"[", 
            RowBox[{"All", ",", "2"}], "]"}], "]"}], "]"}], "^", "2"}], "/", 
        RowBox[{"Max", "[", 
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"complexE", "[", 
            RowBox[{"[", 
             RowBox[{"All", ",", "2"}], "]"}], "]"}], "]"}], "^", "2"}], 
         "]"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"phase", "=", 
       RowBox[{"Arg", "[", 
        RowBox[{"complexE", "[", 
         RowBox[{"[", 
          RowBox[{"All", ",", "2"}], "]"}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "Factor", " ", "from", " ", "seconds", " ", "to", " ", "nanoseconds"}],
        "*)"}], "\[IndentingNewLine]", 
      RowBox[{"fns", "=", 
       RowBox[{"1.0", "*", 
        RowBox[{"10", "^", "9"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "Minimum", " ", "number", " ", "of", " ", "time", " ", "steps", " ", 
        "per", " ", "constant", " ", "intensity", " ", "span"}], "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"tspanMaxFactor", "=", "11.0"}], ";", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"Speed", "-", 
        RowBox[{
        "up", " ", "computation", " ", "of", " ", "fullAmat", " ", "starting",
          " ", "with", " ", "this", " ", "period"}]}], " ", "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"iVGfreezeLoc", "=", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"OptionValue", "[", "VGMethod", "]"}], "===", "FixVG"}], 
         ",", "1", ",", 
         RowBox[{"OptionValue", "[", "VGfreezePeriod", "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"time", " ", "sample", " ", "steps"}], " ", "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"tDiffs", "=", 
       RowBox[{"Differences", "[", 
        RowBox[{"complexE", "[", 
         RowBox[{"[", 
          RowBox[{"All", ",", "1"}], "]"}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"PrependTo", "[", 
       RowBox[{"tDiffs", ",", 
        RowBox[{"complexE", "[", 
         RowBox[{"[", 
          RowBox[{"1", ",", "1"}], "]"}], "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{"Average", " ", "irradiance", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
          "use", " ", "piecewise", " ", "constant", " ", "irradiance"}], ",", 
          " ", 
          RowBox[{"exactly", " ", "like", " ", "input", " ", "irradiance"}]}],
          ")"}]}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"Iavg", "=", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"relativeI", ".", "tDiffs"}], ")"}], "/", "trep"}], "*", 
         "IMeso"}], "/.", "params1"}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "Run", " ", "steady", " ", "state", " ", "calculation", " ", "at", " ",
         "unperturbed", " ", "intensity", " ", "and", " ", "frequency"}], " ",
        "*)"}], "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Head", "[", "inst", "]"}], "===", "List"}], ",", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"w", ",", "vg"}], "}"}], "=", "inst"}], ",", 
        RowBox[{"vg", "=", "Automatic"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"psi", "=", 
       RowBox[{"PsiMeso", "[", 
        RowBox[{"syst", ",", "params", ",", 
         RowBox[{"FluxReport", "->", 
          RowBox[{"{", 
           RowBox[{
           "BlochMatrix", ",", "DMSolution", ",", "VelocityGroups", ",", 
            "RFParams"}], "}"}]}], ",", 
         RowBox[{"InitVG", "->", "vg"}], ",", "nfopts"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Head", "[", "inst", "]"}], "===", "List"}], ",", 
        RowBox[{"(*", " ", 
         RowBox[{
         "w", " ", "and", " ", "vg", " ", "have", " ", "been", " ", 
          "supplied"}], " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"fullAmat", ",", "fullbvecOn", ",", "fullfluorvecOn"}], 
             "}"}], ",", "rfparams1"}], "}"}], "=", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"BlochMatrix", ",", "RFParams"}], "}"}], "/.", 
           RowBox[{"psi", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"excitedPops", "=", 
          RowBox[{"filterPops", "[", 
           RowBox[{"w", ",", "fullbvecOn", ",", "vg"}], "]"}]}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"fullAmat", ",", "fullbvecOn", ",", "fullfluorvecOn"}], 
             "}"}], ",", "sol", ",", "vg", ",", "rfparams1"}], "}"}], "=", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
            "BlochMatrix", ",", "DMSolution", ",", "VelocityGroups", ",", 
             "RFParams"}], "}"}], "/.", 
           RowBox[{"psi", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"excitedPops", "=", 
          RowBox[{"filterPops", "[", 
           RowBox[{"sol", ",", "fullbvecOn", ",", "vg"}], "]"}]}]}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"psiIavg1", "=", 
       RowBox[{"PsiMeso", "[", 
        RowBox[{"syst", ",", 
         RowBox[{"Join", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"IMeso", "->", "Iavg"}], "}"}], ",", "params"}], "]"}], 
         ",", 
         RowBox[{"RefineResult", "->", 
          RowBox[{"{", "}"}]}], ",", 
         RowBox[{"FluxReport", "->", 
          RowBox[{"{", "}"}]}], ",", 
         RowBox[{"InitVG", "->", "vg"}], ",", "nfopts"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
        RowBox[{
         RowBox[{
         "indVgMaxEx", " ", "is", " ", "the", " ", "index", " ", "of", " ", 
          "the", " ", "velocity", " ", "group", " ", "nearest", " ", "to"}], 
         " ", "+", 
         RowBox[{"5", "MHz"}]}], ",", " ", 
        RowBox[{
        "where", " ", "we", " ", "usually", " ", "find", " ", "a", " ", 
         "peak", " ", "in", " ", "the", " ", 
         RowBox[{"spectrum", ".", " ", "We"}], " ", "pass", " ", "this", " ", 
         "index", " ", "to", " ", 
         RowBox[{"phiv", "[", "]"}], " ", "to", " ", "extract", " ", 
         "population", " ", "information", " ", "of", " ", "this", " ", 
         "\"\<busy\>\"", " ", "velocity", " ", 
         RowBox[{"group", "."}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"help1", "=", 
       RowBox[{"Abs", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"vg", "[", 
           RowBox[{"[", 
            RowBox[{"All", ",", "1"}], "]"}], "]"}], "-", 
          RowBox[{"0.5", "*", 
           RowBox[{"NaturalWidth", "[", "2", "]"}]}]}], "/.", 
         RowBox[{"Last", "[", "syst", "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"indVgMaxEx", "=", 
       RowBox[{
        RowBox[{"Position", "[", 
         RowBox[{"help1", ",", 
          RowBox[{"Min", "[", "help1", "]"}]}], "]"}], "[", 
        RowBox[{"[", 
         RowBox[{"1", ",", "1"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"OptionValue", "[", "VerbosityLevel", "]"}], ">", "3"}], ",", 
        RowBox[{
         RowBox[{"Print", "[", 
          RowBox[{
          "\"\<   indVgMaxEx: \>\"", ",", "indVgMaxEx", ",", " ", 
           "\"\<, memory size of fullAmat: \>\"", ",", 
           RowBox[{
            RowBox[{"ByteCount", "[", "fullAmat", "]"}], "/", 
            RowBox[{"1024.0", "^", "2"}]}], ",", "\"\< MB\>\""}], "]"}], 
         ";"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"refine1", "=", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"OptionValue", "[", "VGMethod", "]"}], "===", "FixVG"}], 
         ",", 
         RowBox[{"{", "}"}], ",", 
         RowBox[{"OptionValue", "[", "RefineResult", "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"popex", "=", 
       RowBox[{"Total", "[", 
        RowBox[{"getPops", "[", 
         RowBox[{
          RowBox[{"s1", "=", "sol"}], ",", 
          RowBox[{"f1", "=", "fullfluorvecOn"}]}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{"Initialize", " ", "with", " ", "steady"}], "-", 
        RowBox[{"state", " ", "vector", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"the", " ", "system", " ", "Aw"}], "==", 
           RowBox[{
            RowBox[{"-", "b"}], " ", "was", " ", "solved", " ", "and", " ", 
            "use", " ", "the", " ", "formula", " ", "fullbvec"}], "==", 
           RowBox[{"-", "\[Gamma]b"}]}], ")"}]}]}], "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Head", "[", "inst", "]"}], "===", "List"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"psiInit", "=", 
         RowBox[{"psi", "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"inst", "===", "LightOn"}], ",", 
           RowBox[{
            RowBox[{"w", "=", "sol"}], ";", 
            RowBox[{"bvec", "=", "fullbvecOn"}], ";", 
            RowBox[{"fvec", "=", "fullfluorvecOn"}], ";", 
            RowBox[{"psiInit", "=", 
             RowBox[{"psi", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}]}]}], "]"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"OptionValue", "[", "VerbosityLevel", "]"}], ">", "0"}], ",", 
        RowBox[{
         RowBox[{"Print", "[", 
          RowBox[{"\"\<I: \>\"", ",", 
           RowBox[{"IMeso", "/.", "params1"}], ",", "\"\< W/m^2, Iavg: \>\"", 
           ",", "Iavg", ",", "\"\<, frep: \>\"", ",", 
           RowBox[{
            RowBox[{
             RowBox[{"10", "^", 
              RowBox[{"-", "6"}]}], "/", "trep"}], " ", 
            "\"\< MHz, peak cw psi: \>\""}], ",", 
           RowBox[{"psi", "[", 
            RowBox[{"[", "1", "]"}], "]"}], ",", 
           "\"\< ph/s/sr/atom/(W/m^2), psi from Iavg (cw): \>\"", ",", 
           "psiIavg1", ",", "\"\<, excited state population: \>\"", ",", 
           "popex"}], "]"}], ";"}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"inst", "===", "LightOff"}], ",", "\[IndentingNewLine]", 
        RowBox[{"(*", 
         RowBox[{
          RowBox[{"Run", " ", "steady"}], "-", 
          RowBox[{
          "state", " ", "calculation", " ", "at", " ", "close", " ", "to", 
           " ", "zero", " ", "intensity"}]}], "*)"}], 
        RowBox[{
         RowBox[{"psi", "=", 
          RowBox[{"PsiMeso", "[", 
           RowBox[{"syst", ",", 
            RowBox[{"Join", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"IMeso", "->", "$MachineEpsilon"}], "}"}], ",", 
              "params"}], "]"}], ",", 
            RowBox[{"RefineResult", "->", 
             RowBox[{"{", "}"}]}], ",", 
            RowBox[{"FluxReport", "->", 
             RowBox[{"{", 
              RowBox[{"BlochMatrix", ",", "DMSolution"}], "}"}]}], ",", 
            RowBox[{"InitVG", "->", "vg"}], ",", "nfopts"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"fullAmat", ",", "fullbvecOff", ",", "fullfluorvecOff"}],
              "}"}], ",", "sol"}], "}"}], "=", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"BlochMatrix", ",", "DMSolution"}], "}"}], "/.", 
           RowBox[{"psi", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"inst", "===", "LightOff"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"w", "=", "sol"}], ";", 
            RowBox[{"bvec", "=", "fullbvecOff"}], ";", 
            RowBox[{"fvec", "=", "fullfluorvecOff"}], ";", 
            RowBox[{"psiInit", "=", "0.0"}], ";"}]}], "\[IndentingNewLine]", 
          "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Dimensions", "[", "fullbvecOn", "]"}], "!=", " ", 
            RowBox[{"Dimensions", "[", "fullbvecOff", "]"}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Print", "[", 
             RowBox[{
             "\"\<StepReturnFlux[]: Dimensions[fullbvecOn]: \>\"", ",", " ", 
              RowBox[{"Dimensions", "[", "fullbvecOn", "]"}], ",", " ", 
              "\"\< != Dimensions[fullbvecOff]: \>\"", ",", " ", 
              RowBox[{"Dimensions", "[", "fullbvecOff", "]"}], ",", 
              "\"\<, aborting\>\""}], "]"}], ";", 
            RowBox[{"Abort", "[", "]"}], ";"}]}], "]"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", 
      RowBox[{"(*", "  ", 
       RowBox[{"If", "[", 
        RowBox[{"inst", "===", "LightOff"}], "]"}], " ", "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"psiSpanAll", "=", 
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{"0.0", ",", "psiInit"}], "}"}], "}"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"errAll", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"tend", "=", "0.0"}], ";", 
      RowBox[{"psiAvgLast", "=", "0.0"}], ";", 
      RowBox[{"psiAvgTotal", "=", "0.0"}], ";", "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
       "Ground", " ", "and", " ", "excited", " ", "state", " ", 
        "populations"}], "*)"}], "\[IndentingNewLine]", 
      RowBox[{"popgr", "=", 
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{"0.0", ",", 
          RowBox[{"Total", "[", 
           RowBox[{"getPops", "[", 
            RowBox[{"w", ",", "bvec"}], "]"}], "]"}]}], "}"}], "}"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"popex", "=", 
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{"0.0", ",", 
          RowBox[{"Total", "[", 
           RowBox[{"getPops", "[", 
            RowBox[{"w", ",", "fvec"}], "]"}], "]"}]}], "}"}], "}"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "Here", " ", "we", " ", "assume", " ", "the", " ", "\"\<Off\>\"", " ", 
        "inital", " ", 
        RowBox[{"state", ".", " ", "Worth"}], " ", 
        RowBox[{"generalizing", "?"}]}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"popgr8All", "=", 
       RowBox[{"{", 
        RowBox[{"ConstantArray", "[", 
         RowBox[{
          RowBox[{"1.0", "/", "8"}], ",", "8"}], "]"}], "}"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"popex16All", "=", 
       RowBox[{"{", 
        RowBox[{"ConstantArray", "[", 
         RowBox[{"0.0", ",", "16"}], "]"}], "}"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"popgr8WeightedAll", "=", 
       RowBox[{"{", 
        RowBox[{"ConstantArray", "[", 
         RowBox[{
          RowBox[{"1.0", "/", "8"}], ",", "8"}], "]"}], "}"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"psiAvgHist", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"excitedPopsAll", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"tuLast", "=", 
       RowBox[{"TimeUsed", "[", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"lastvg", "=", "vg"}], ";", "\[IndentingNewLine]", 
      RowBox[{"tPrep", "=", "0.0"}], ";", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "Positions", " ", "of", " ", "the", " ", "ground", " ", "state", " ", 
        "populations"}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"grStates", "=", 
       RowBox[{"Flatten", "[", 
        RowBox[{"Position", "[", 
         RowBox[{"fullbvecOn", ",", 
          RowBox[{"x_", "/;", 
           RowBox[{"x", "!=", "0"}]}]}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "Positions", " ", "of", " ", "the", " ", "excited", " ", "state", " ", 
        "populations"}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{"exStates", "=", 
       RowBox[{"Flatten", "[", 
        RowBox[{"Position", "[", 
         RowBox[{"fullfluorvecOn", ",", 
          RowBox[{"x_", "/;", 
           RowBox[{"x", ">", "0"}]}]}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<Dimensions[grStates]: \>\"", ",", 
        RowBox[{"Dimensions", "[", "grStates", "]"}], ",", 
        "\"\<, Dimensions[exStates]: \>\"", ",", 
        RowBox[{"Dimensions", "[", "exStates", "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Dimensions", "[", "w", "]"}], "!=", " ", 
         RowBox[{"Dimensions", "[", "fullbvecOn", "]"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Print", "[", 
          RowBox[{"\"\<StepReturnFlux[]: Dimensions[w]: \>\"", ",", " ", 
           RowBox[{"Dimensions", "[", "w", "]"}], ",", " ", 
           "\"\< != Dimensions[fullbvecOn]: \>\"", ",", " ", 
           RowBox[{"Dimensions", "[", "fullbvecOn", "]"}], ",", 
           "\"\<, aborting\>\""}], "]"}], ";", 
         RowBox[{"Abort", "[", "]"}], ";"}]}], "]"}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "===", "===", "===", "===", "===", "===", "===", "===", "===", "===", 
        " ", 
        RowBox[{"MAIN", " ", "LOOP", " ", "BEGIN"}], " ", "===", "===", "===",
         "===", "===", "===", "===", "=="}], "*)"}], "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"i", "=", "1"}], ",", 
        RowBox[{"i", "<=", "nperiods"}], ",", 
        RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
         "CPU", " ", "time", " ", "spent", " ", "in", " ", "this", " ", 
          "session", " ", "so", " ", "far"}], " ", "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"tu1", "=", 
          RowBox[{"TimeUsed", "[", "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"OptionValue", "[", "VerbosityLevel", "]"}], ">", "2"}], 
           ",", "\[IndentingNewLine]", 
           RowBox[{"(*", " ", 
            RowBox[{
             RowBox[{
             "Current", " ", "central", " ", "frequency", " ", "offset"}], 
             ",", " ", 
             RowBox[{
             "possibly", " ", "changed", " ", "due", " ", "to", " ", 
              "chirping"}]}], " ", "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"\[CapitalDelta]fLaserNow", "=", 
             RowBox[{
              RowBox[{
               RowBox[{"tend", "*", "chirpRate"}], "+", 
               "\[CapitalDelta]fLaser"}], "/.", "params1"}]}], ";", 
            RowBox[{"Print", "[", 
             RowBox[{"\"\<  Laser period of duration \>\"", ",", 
              RowBox[{"fns", "*", "trep"}], ",", 
              "\"\< ns, DeltafLaserNow: \>\"", ",", 
              RowBox[{
               SuperscriptBox["10.0", 
                RowBox[{"-", "6"}]], "*", "\[CapitalDelta]fLaserNow"}], ",", 
              "\"\< MHz\>\""}], "]"}], ";"}]}], "]"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"err", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"psiSpan", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"i", "==", "iVGfreezeLoc"}], ",", "\[IndentingNewLine]", 
           RowBox[{"(*", 
            RowBox[{
             RowBox[{"Added", " ", "by", " ", 
              RowBox[{"S", ".", "R", "."}]}], ":", " ", 
             RowBox[{
              RowBox[{"create", " ", "light"}], "-", 
              RowBox[{
              "independent", " ", "part", " ", "of", " ", "Bloch", " ", 
               "matrix"}]}]}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"psi", "=", 
             RowBox[{"PsiMeso", "[", 
              RowBox[{
               RowBox[{"SparseReplace", "[", 
                RowBox[{"syst", ",", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"RabiFrequencyA", "[", "1", "]"}], "->", "0"}], 
                   ",", 
                   RowBox[{
                    RowBox[{"RabiFrequencyB", "[", "1", "]"}], "->", "0"}], 
                   ",", 
                   RowBox[{
                    RowBox[{"DetuningA", "[", "1", "]"}], "->", "0"}], ",", 
                   RowBox[{
                    RowBox[{"DetuningB", "[", "1", "]"}], "->", "0"}], ",", 
                   RowBox[{
                    RowBox[{"LightPhase", "[", "1", "]"}], "->", "0"}]}], 
                  "}"}]}], "]"}], ",", "params", ",", 
               RowBox[{"RefineResult", "->", 
                RowBox[{"{", "}"}]}], ",", 
               RowBox[{"FluxReport", "->", 
                RowBox[{"{", "BlochMatrix", "}"}]}], ",", 
               RowBox[{"InitVG", "->", "vg"}], ",", 
               RowBox[{"SkipLinearSolve", "->", "True"}], ",", "nfopts"}], 
              "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
              "fullAmatLI", ",", "fullbvecOn", ",", "fullfluorvecOn"}], "}"}],
              " ", "=", " ", 
             RowBox[{"BlochMatrix", "/.", 
              RowBox[{"psi", "[", 
               RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"(*", 
             RowBox[{
              RowBox[{"Added", " ", "by", " ", 
               RowBox[{"S", ".", "R", "."}]}], ":", " ", 
              RowBox[{
               RowBox[{"create", " ", "light"}], "-", 
               RowBox[{
               "only", " ", "part", " ", "of", " ", "one", " ", "block", " ", 
                "of", " ", "Bloch", " ", "matrix"}]}]}], "*)"}], 
            "\[IndentingNewLine]", 
            RowBox[{"A110L", "=", 
             RowBox[{"SparseArray", "@@", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"Expand", "[", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"ArrayRules", "[", 
                    RowBox[{
                    RowBox[{"syst", "[", 
                    RowBox[{"[", "2", "]"}], "]"}], "-", 
                    RowBox[{"SparseReplace", "[", 
                    RowBox[{
                    RowBox[{"syst", "[", 
                    RowBox[{"[", "2", "]"}], "]"}], ",", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"RabiFrequencyA", "[", "1", "]"}], "->", "0"}], 
                    ",", 
                    RowBox[{
                    RowBox[{"RabiFrequencyB", "[", "1", "]"}], "->", "0"}], 
                    ",", 
                    RowBox[{
                    RowBox[{"DetuningA", "[", "1", "]"}], "->", "0"}], ",", 
                    RowBox[{
                    RowBox[{"DetuningB", "[", "1", "]"}], "->", "0"}], ",", 
                    RowBox[{
                    RowBox[{"LightPhase", "[", "1", "]"}], "->", "0"}]}], 
                    "}"}]}], "]"}]}], "]"}], ",", 
                   RowBox[{"Dimensions", "[", 
                    RowBox[{"syst", "[", 
                    RowBox[{"[", "2", "]"}], "]"}], "]"}]}], "}"}], "]"}], "/.", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"{", 
                    RowBox[{"x_Integer", ",", "y_Integer"}], "}"}], "->", 
                   "0"}], ")"}], "->", 
                 RowBox[{"Sequence", "[", "]"}]}]}], ")"}]}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"OptionValue", "[", "VerbosityLevel", "]"}], ">", 
                "2"}], "&&", 
               RowBox[{"i", ">", "1"}]}], ",", 
              RowBox[{
               RowBox[{
               "Print", "[", 
                "\"\<  --> Freezing velocity groups (rebinning stopped)\>\"", 
                "]"}], ";"}]}], "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], 
         ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"(*", "  ", 
          RowBox[{"Bright", " ", 
           RowBox[{"period", ":", " ", 
            RowBox[{
            "Integrate", " ", "over", " ", "sub", " ", "time", " ", 
             "spans"}]}]}], " ", "*)"}], "\[IndentingNewLine]", 
         RowBox[{"For", "[", 
          RowBox[{
           RowBox[{"j", "=", "1"}], ",", 
           RowBox[{"j", "<=", 
            RowBox[{"Length", "[", "relativeI", "]"}]}], ",", 
           RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
           RowBox[{"(*", " ", 
            RowBox[{
            "time", " ", "since", " ", "beginning", " ", "of", " ", "current",
              " ", "period"}], " ", "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"tNow", "=", 
             RowBox[{"complexE", "[", 
              RowBox[{"[", 
               RowBox[{"j", ",", "1"}], "]"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"(*", " ", 
             RowBox[{"Current", " ", 
              RowBox[{"irradiance", ".", " ", "A"}], " ", "value", " ", "of", 
              " ", "zero", " ", "leads", " ", "to", " ", "a", " ", 
              RowBox[{"1", "/", "0"}], " ", "division"}], " ", "*)"}], 
            "\[IndentingNewLine]", 
            RowBox[{"IMesoNow", "=", 
             RowBox[{"Max", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"IMeso", "*", 
                 RowBox[{"relativeI", "[", 
                  RowBox[{"[", "j", "]"}], "]"}]}], "/.", "params1"}], ",", 
               "$MachineEpsilon"}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"(*", " ", 
             RowBox[{
              RowBox[{
              "Current", " ", "central", " ", "frequency", " ", "offset"}], 
              ",", " ", 
              RowBox[{
              "possibly", " ", "changed", " ", "due", " ", "to", " ", 
               "chirping"}]}], " ", "*)"}], "\[IndentingNewLine]", 
            RowBox[{"\[CapitalDelta]fLaserNow", "=", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"tend", "+", 
                  RowBox[{"0.5", "*", 
                   RowBox[{"tDiffs", "[", 
                    RowBox[{"[", "j", "]"}], "]"}]}]}], ")"}], "*", 
                "chirpRate"}], "+", "\[CapitalDelta]fLaser"}], "/.", 
              "params1"}]}], ";", "\[IndentingNewLine]", 
            "\[IndentingNewLine]", 
            RowBox[{"(*", " ", 
             RowBox[{
              RowBox[{
              "Just", " ", "assemble", " ", "the", " ", "Bloch", " ", 
               "matrices"}], ";", " ", 
              RowBox[{
               RowBox[{"don", "'"}], "t", " ", "actually", " ", "solve", " ", 
               "the", " ", "equations"}]}], " ", "*)"}], 
            "\[IndentingNewLine]", 
            RowBox[{"tu11", "=", 
             RowBox[{"TimeUsed", "[", "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"(*", 
             RowBox[{
              RowBox[{
               RowBox[{"Added", " ", "by", " ", 
                RowBox[{"S", ".", "R", "."}]}], ":", " ", 
               RowBox[{
               "for", " ", "fixed", " ", "velocity", " ", "groups"}]}], ",", 
              " ", 
              RowBox[{
               RowBox[{"just", " ", "add", " ", "light"}], "-", 
               RowBox[{
               "dependent", " ", "part", " ", "to", " ", "precalculated", " ",
                 "light"}], "-", 
               RowBox[{"independent", " ", 
                RowBox[{"part", ".", "\[IndentingNewLine]", "Commented"}], 
                " ", "out", " ", "by", " ", 
                RowBox[{"R", ".", "H", ".", " ", "on"}], " ", "17", "Nov11", 
                " ", "in", " ", "order", " ", "to", " ", "accommodate", " ", 
                RowBox[{"MatrixExp", "[", "]"}]}]}], ",", " ", 
              RowBox[{"see", " ", 
               RowBox[{"below", "."}]}]}], "\[IndentingNewLine]", "*)"}], 
            "\[IndentingNewLine]", 
            RowBox[{"(*", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{"i", ">=", "iVGfreezeLoc"}], ",", 
                "\[IndentingNewLine]", 
                RowBox[{"Global`fullAmatG", "=", 
                 RowBox[{"fullAmatLI", "+", 
                  RowBox[{"BuildLightDependentPart", "[", 
                   RowBox[{
                    RowBox[{"syst", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], ",", "A110L", ",", 
                    RowBox[{"syst", "[", 
                    RowBox[{"[", 
                    RowBox[{"-", "1"}], "]"}], "]"}], ",", 
                    RowBox[{"Join", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"IMeso", "->", "IMesoNow"}], ",", 
                    RowBox[{"Phase", "->", 
                    RowBox[{"phase", "[", 
                    RowBox[{"[", "j", "]"}], "]"}]}], ",", 
                    RowBox[{
                    "\[CapitalDelta]fLaser", "->", 
                    "\[CapitalDelta]fLaserNow"}]}], "}"}], ",", "params"}], 
                    "]"}], ",", "vg"}], "]"}]}]}], "\[IndentingNewLine]", ",",
                 " ", 
                RowBox[{"(*", " ", 
                 RowBox[{"Otherwise", ",", " ", 
                  RowBox[{"create", " ", "entire", " ", "matrix"}]}], " ", 
                 "*)"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"psi", "=", 
                  RowBox[{"PsiMeso", "[", 
                   RowBox[{"syst", ",", 
                    RowBox[{"Join", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"IMeso", "->", "IMesoNow"}], ",", 
                    RowBox[{"Phase", "->", 
                    RowBox[{"phase", "[", 
                    RowBox[{"[", "j", "]"}], "]"}]}], ",", 
                    RowBox[{
                    "\[CapitalDelta]fLaser", "->", 
                    "\[CapitalDelta]fLaserNow"}]}], "}"}], ",", "params"}], 
                    "]"}], ",", 
                    RowBox[{"RefineResult", "->", 
                    RowBox[{"{", "}"}]}], ",", 
                    RowBox[{"FluxReport", "->", 
                    RowBox[{"{", "BlochMatrix", "}"}]}], ",", 
                    RowBox[{"InitVG", "->", "vg"}], ",", 
                    RowBox[{"SkipLinearSolve", "->", "True"}], ",", 
                    "nfopts"}], "]"}]}], ";", "\[IndentingNewLine]", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{
                   "Global`fullAmatG", ",", "fullbvecOn", ",", 
                    "fullfluorvecOn"}], "}"}], " ", "=", " ", 
                  RowBox[{"BlochMatrix", "/.", 
                   RowBox[{"psi", "[", 
                    RowBox[{"[", "2", "]"}], "]"}]}]}], ";"}]}], 
               "\[IndentingNewLine]", "]"}], ";"}], "\[IndentingNewLine]", 
             "*)"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
            RowBox[{"(*", " ", 
             RowBox[{
              RowBox[{
              "We", " ", "do", " ", "solve", " ", "the", " ", "full", " ", 
               "system", " ", "here", " ", "to", " ", "update", " ", "the", 
               " ", "variable", " ", "\"\<sol\>\"", " ", 
               RowBox[{"(", 
                RowBox[{"option", " ", "DMSolution"}], ")"}]}], ",", " ", 
              RowBox[{"used", " ", "in", " ", 
               RowBox[{"MatrixExp", "[", "]"}], " ", "below"}]}], "  ", 
             "*)"}], "\[IndentingNewLine]", 
            RowBox[{"psi", "=", 
             RowBox[{"PsiMeso", "[", 
              RowBox[{"syst", ",", 
               RowBox[{"Join", "[", 
                RowBox[{
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"IMeso", "->", "IMesoNow"}], ",", 
                   RowBox[{"Phase", "->", 
                    RowBox[{"phase", "[", 
                    RowBox[{"[", "j", "]"}], "]"}]}], ",", 
                   RowBox[{
                   "\[CapitalDelta]fLaser", "->", 
                    "\[CapitalDelta]fLaserNow"}]}], "}"}], ",", "params"}], 
                "]"}], ",", 
               RowBox[{"RefineResult", "->", 
                RowBox[{"{", "}"}]}], ",", 
               RowBox[{"FluxReport", "->", 
                RowBox[{"{", 
                 RowBox[{"BlochMatrix", ",", "DMSolution"}], "}"}]}], ",", 
               RowBox[{"InitVG", "->", "vg"}], ",", 
               RowBox[{"SkipLinearSolve", "->", "False"}], ",", "nfopts"}], 
              "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{
                "fullAmat", ",", "fullbvecOn", ",", "fullfluorvecOn"}], "}"}],
                ",", "sol"}], "}"}], " ", "=", " ", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"BlochMatrix", ",", "DMSolution"}], "}"}], "/.", 
              RowBox[{"psi", "[", 
               RowBox[{"[", "2", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]",
             "\[IndentingNewLine]", 
            RowBox[{"tPrep", "+=", 
             RowBox[{
              RowBox[{"TimeUsed", "[", "]"}], "-", "tu11"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"OptionValue", "[", "VerbosityLevel", "]"}], ">", 
                "4"}], "||", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{
                  RowBox[{"OptionValue", "[", "VerbosityLevel", "]"}], ">", 
                  "3"}], "&&", 
                 RowBox[{"i", "==", "1"}]}], ")"}]}], ",", 
              RowBox[{
               RowBox[{"Print", "[", 
                RowBox[{
                "\"\<    Subspan \>\"", ",", "j", ",", "\"\</\>\"", ",", 
                 RowBox[{"Length", "[", "relativeI", "]"}], ",", 
                 "\"\< of \>\"", ",", 
                 RowBox[{"fns", "*", 
                  RowBox[{"tDiffs", "[", 
                   RowBox[{"[", "j", "]"}], "]"}]}], ",", "\"\< ns, I: \>\"", 
                 ",", "IMesoNow", ",", " ", "\"\< W/m^2, Phase: \>\"", ",", 
                 RowBox[{"phase", "[", 
                  RowBox[{"[", "j", "]"}], "]"}], ",", 
                 "\"\<, time for initial solution: \>\"", ",", 
                 RowBox[{
                  RowBox[{"TimeUsed", "[", "]"}], "-", "tu11"}], ",", 
                 "\"\< s\>\""}], "]"}], ";"}]}], "\[IndentingNewLine]", "]"}],
             ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
            RowBox[{"(*", 
             RowBox[{
              RowBox[{
               RowBox[{"[", 
                RowBox[{"w", ",", "err", ",", "..."}], "]"}], "=", 
               RowBox[{
                RowBox[{
                 RowBox[{"phivCompiled", "[", 
                  RowBox[{"t_", ",", "u_", ",", "v_", ",", 
                   RowBox[{"obsmat_:", 
                    RowBox[{"{", "}"}]}], ",", 
                   RowBox[{"tspanMax_:", "1.0*^200"}], ",", 
                   RowBox[{"tol_:", "1.0*^-7"}], ",", 
                   RowBox[{"mIn_:", "-", "1"}], ",", 
                   RowBox[{"indVgMaxEx_:", "-", "1"}], ",", 
                   RowBox[{"OptionsPattern", "[", "]"}]}], "]"}], " ", 
                 "solves", " ", "the", " ", "ODE", " ", 
                 RowBox[{"w", "'"}]}], " ", "=", " ", 
                RowBox[{"Aw", "+", "b"}]}]}], ",", " ", 
              RowBox[{
               RowBox[{"w", 
                RowBox[{"(", "0", ")"}]}], " ", "=", " ", "v"}]}], " ", 
             "*)"}], "\[IndentingNewLine]", 
            RowBox[{"(*", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"Global`fullAmatG", "=", "fullAmat"}], ";", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{
                "w", ",", "errLoc", ",", "psiSpanLoc", ",", "popgr8", ",", 
                 "popex16", ",", "popgr8Weighted"}], "}"}], "=", 
               RowBox[{"phivCompiled", "[", 
                RowBox[{
                 RowBox[{"tDiffs", "[", 
                  RowBox[{"[", "j", "]"}], "]"}], ",", "fullbvecOn", ",", "w",
                  ",", "fullfluorvecOn", ",", 
                 RowBox[{"trep", "/", "tspanMaxFactor"}], ",", "tol", ",", 
                 "m", ",", "indVgMaxEx", ",", 
                 RowBox[{"VerbosityLevel", "->", 
                  RowBox[{
                   RowBox[{"OptionValue", "[", "VerbosityLevel", "]"}], "-", 
                   "1"}]}]}], "]"}]}], ";"}], "\[IndentingNewLine]", "*)"}], 
            "\[IndentingNewLine]", "\[IndentingNewLine]", 
            RowBox[{"(*", " ", 
             RowBox[{
              RowBox[{
              "Solver", " ", "for", " ", "constant", " ", "light", " ", 
               "field", " ", "based", " ", "on", " ", 
               RowBox[{"Mathematica", "'"}], "s", " ", "routine", " ", 
               RowBox[{"MatrixExp", "[", 
                RowBox[{"m", ",", "v"}], "]"}]}], ",", " ", 
              RowBox[{"faster", " ", "than", " ", 
               RowBox[{"phiv", "[", "]"}], " ", "and", " ", 
               RowBox[{"phivCompiled", "[", "]"}], " ", "in", " ", "many", 
               " ", "cases"}]}], "  ", "*)"}], "\[IndentingNewLine]", 
            RowBox[{"w", "=", 
             RowBox[{"Re", "[", 
              RowBox[{
               RowBox[{"MatrixExp", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"tDiffs", "[", 
                   RowBox[{"[", "j", "]"}], "]"}], "*", "fullAmat"}], ",", 
                 RowBox[{"(", 
                  RowBox[{"w", "+", "sol"}], ")"}]}], "]"}], "-", "sol"}], 
              "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
              "errLoc", ",", "psiSpanLoc", ",", "popgr8", ",", "popex16", ",",
                "popgr8Weighted"}], "}"}], "=", 
             RowBox[{"phivMatrixExp", "[", 
              RowBox[{
               RowBox[{"tDiffs", "[", 
                RowBox[{"[", "j", "]"}], "]"}], ",", "w", ",", "grStates", 
               ",", "exStates", ",", "fullfluorvecOn", ",", "indVgMaxEx", ",", 
               RowBox[{"VerbosityLevel", "->", 
                RowBox[{
                 RowBox[{"OptionValue", "[", "VerbosityLevel", "]"}], "-", 
                 "1"}]}]}], "]"}]}], ";", "\[IndentingNewLine]", 
            "\[IndentingNewLine]", 
            RowBox[{"AppendTo", "[", 
             RowBox[{"err", ",", "errLoc"}], "]"}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"psiSpanLoc", "[", 
              RowBox[{"[", 
               RowBox[{"All", ",", "1"}], "]"}], "]"}], "+=", "tend"}], ";", 
            RowBox[{"(*", " ", 
             RowBox[{"Advance", " ", "time", " ", "axis", " ", 
              RowBox[{"(", 
               RowBox[{
               "tend", " ", "is", " ", "the", " ", "time", " ", "from", " ", 
                "beginning", " ", "of", " ", "first", " ", "pulse"}], ")"}]}],
              " ", "*)"}], "\[IndentingNewLine]", 
            RowBox[{"psiSpan", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"psiSpan", ",", "psiSpanLoc"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"tend", "+=", 
             RowBox[{"tDiffs", "[", 
              RowBox[{"[", "j", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"popgr", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"popgr", ",", 
               RowBox[{"{", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"fns", "*", "tend"}], ",", 
                  RowBox[{"Total", "[", 
                   RowBox[{"getPops", "[", 
                    RowBox[{"w", ",", "fullbvecOn"}], "]"}], "]"}]}], "}"}], 
                "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"popex", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"popex", ",", 
               RowBox[{"{", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"fns", "*", "tend"}], ",", 
                  RowBox[{"Total", "[", 
                   RowBox[{"getPops", "[", 
                    RowBox[{"w", ",", "fullfluorvecOn"}], "]"}], "]"}]}], 
                 "}"}], "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"popgr8All", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"popgr8All", ",", "popgr8"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"popex16All", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"popex16All", ",", "popex16"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"popgr8WeightedAll", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"popgr8WeightedAll", ",", "popgr8Weighted"}], "]"}]}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
         RowBox[{"(*", 
          RowBox[{"For", "[", "j", "]"}], "*)"}], "\[IndentingNewLine]", 
         "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{
          "Scale", " ", "time", " ", "axis", " ", "to", " ", "ns", " ", "to", 
           " ", "avoid", " ", "precision", " ", "loss", " ", "in", " ", 
           RowBox[{"Integrate", "[", "]"}]}], " ", "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"psiSpan", "[", 
           RowBox[{"[", 
            RowBox[{"All", ",", "1"}], "]"}], "]"}], "*=", "fns"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{
          "Divide", " ", "return", " ", "flux", " ", "samples", " ", "by", 
           " ", "average", " ", "intensity"}], " ", "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"psiSpan", "[", 
           RowBox[{"[", 
            RowBox[{"All", ",", "2"}], "]"}], "]"}], "/=", "Iavg"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{"Integrate", " ", "Flux", " ", "during", " ", "each", " ", 
           RowBox[{"period", ".", " ", "Prepend"}], " ", "last", " ", "value",
            " ", "of", " ", "previous", " ", "span", " ", "in", " ", "order", 
           " ", "to", " ", "properly", " ", "begin", " ", "the", " ", 
           RowBox[{"integral", "."}]}], "  ", "*)"}], "\[IndentingNewLine]", 
         RowBox[{"hh", "=", 
          RowBox[{"Interpolation", "[", 
           RowBox[{
            RowBox[{"Join", "[", 
             RowBox[{
              RowBox[{"{", 
               RowBox[{"psiSpanAll", "[", 
                RowBox[{"[", 
                 RowBox[{
                  RowBox[{"-", "1"}], ",", "All"}], "]"}], "]"}], "}"}], ",", 
              "psiSpan"}], "]"}], ",", 
            RowBox[{"InterpolationOrder", "->", "1"}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{
           RowBox[{
            RowBox[{"Must", " ", "use", " ", 
             RowBox[{"NIntegrate", "[", "]"}], " ", "with", " ", "Method"}], "->", 
            RowBox[{"\"\<Spline\>\"", " ", "in", " ", 
             RowBox[{"Interpolation", "[", "]"}]}]}], ",", " ", 
           RowBox[{"not", " ", 
            RowBox[{"Integrate", "[", "]"}]}]}], " ", "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"psiAvg", "=", 
          RowBox[{
           RowBox[{"NIntegrate", "[", 
            RowBox[{
             RowBox[{"hh", "[", "tns", "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"tns", ",", 
               RowBox[{"hh", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ",", "1", ",", "1"}], "]"}], "]"}], ",", 
               RowBox[{"hh", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ",", "1", ",", "2"}], "]"}], "]"}]}], "}"}]}], 
            "]"}], "/", 
           RowBox[{"(", 
            RowBox[{"fns", "*", "trep"}], ")"}]}]}], ";", 
         "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"groundPops", "=", 
          RowBox[{"filterPops", "[", 
           RowBox[{"w", ",", "fullbvecOn", ",", "vg"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"excitedPops", "=", 
          RowBox[{"filterPops", "[", 
           RowBox[{"w", ",", "fullfluorvecOn", ",", "vg"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"excitedPopsAll", ",", "excitedPops"}], "]"}], ";", 
         "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{
           RowBox[{"For", " ", "debugging", " ", "only"}], ";", " ", 
           RowBox[{"remove", " ", "later"}]}], " ", "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"OptionValue", "[", "VerbosityLevel", "]"}], ">", "3"}], 
           ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Print", "[", 
             RowBox[{"ListLinePlot", "[", 
              RowBox[{"excitedPops", ",", 
               RowBox[{"Mesh", "->", "All"}], ",", 
               RowBox[{"AxesOrigin", "->", 
                RowBox[{"{", 
                 RowBox[{"0.0", ",", "0.0"}], "}"}]}], ",", 
               RowBox[{"PlotRange", "->", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{"-", "1.0"}], ",", "1.0"}], "}"}], ",", "All"}], 
                 "}"}]}], ",", 
               RowBox[{"Frame", "->", "True"}], ",", 
               RowBox[{"FrameLabel", "->", 
                RowBox[{"{", 
                 RowBox[{
                 "\"\<Frequency (GHz)\>\"", ",", 
                  "\"\<Excited State Population\>\""}], "}"}]}]}], "]"}], 
             "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
         "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{
           RowBox[{"Convert", " ", "velocity", " ", "classes"}], ",", " ", 
           RowBox[{
            RowBox[{"based", " ", "on", " ", "current", " ", "time"}], "-", 
            RowBox[{"dependent", " ", 
             RowBox[{"sol", "'"}], "n"}]}]}], " ", "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"OptionValue", "[", "VGMethod", "]"}], "===", 
             "ConvertDM"}], " ", "&&", " ", 
            RowBox[{"i", "<", "iVGfreezeLoc"}]}], ",", "\[IndentingNewLine]", 
           RowBox[{"(*", " ", 
            RowBox[{
             RowBox[{"This", " ", "is", " ", "a", " ", "kludge"}], "..."}], 
            " ", "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"NDitherLoc", "=", 
             RowBox[{"1", "+", 
              RowBox[{"Sign", "[", 
               RowBox[{"FWHMbw", "/.", "params1"}], "]"}]}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"speclist", "=", 
             RowBox[{"MapThread", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"#3", ",", 
                  RowBox[{"#1", "/", 
                   RowBox[{"(", 
                    RowBox[{"-", 
                    RowBox[{"Subtract", "@@", "#2"}]}], ")"}]}]}], "}"}], 
                "&"}], ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"Total", "/@", 
                  RowBox[{"Partition", "[", 
                   RowBox[{
                    RowBox[{"Total", "/@", 
                    RowBox[{"Partition", "[", 
                    RowBox[{
                    RowBox[{"fullfluorvecOn", "  ", "w"}], ",", 
                    RowBox[{"First", "[", "syst", "]"}]}], "]"}]}], ",", 
                    "NDitherLoc"}], "]"}]}], ",", 
                 RowBox[{"vg", "[", 
                  RowBox[{"[", 
                   RowBox[{"All", ",", "3"}], "]"}], "]"}], ",", 
                 RowBox[{"vg", "[", 
                  RowBox[{"[", 
                   RowBox[{"All", ",", "1"}], "]"}], "]"}]}], "}"}]}], 
              "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"lastvg", "=", "vg"}], ";", "\[IndentingNewLine]", 
            RowBox[{"(*", " ", 
             RowBox[{"makeVg", "[", 
              RowBox[{"speclist_", ",", "atdata1_", ",", "params_", ",", 
               RowBox[{"opts", ":", 
                RowBox[{"OptionsPattern", "[", "]"}]}]}], "]"}], "  ", "*)"}],
             "\[IndentingNewLine]", 
            RowBox[{"vg", "=", 
             RowBox[{"makeVg", "[", 
              RowBox[{"speclist", ",", 
               RowBox[{"Last", "[", "syst", "]"}], ",", "rfparams1", ",", 
               "nfopts"}], "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"w", "=", 
             RowBox[{"convertDM", "[", 
              RowBox[{"w", ",", "lastvg", ",", "vg"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"fullfluorvecOn", "=", 
             RowBox[{"Flatten", "@", 
              RowBox[{"Table", "[", 
               RowBox[{
                RowBox[{"Evaluate", "@", 
                 RowBox[{"Take", "[", 
                  RowBox[{"fullfluorvecOn", ",", 
                   RowBox[{
                    RowBox[{"Length", "[", "fullfluorvecOn", "]"}], "/", 
                    RowBox[{"Length", "[", "lastvg", "]"}]}]}], "]"}]}], ",", 
                RowBox[{"{", 
                 RowBox[{"Length", "[", "vg", "]"}], "}"}]}], "]"}]}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"(*", " ", 
             RowBox[{
             "velocity", " ", "group", " ", "index", " ", "of", " ", 
              "maximum", " ", "excitation"}], " ", "*)"}], 
            "\[IndentingNewLine]", 
            RowBox[{"excitedPopsHelp", "=", 
             RowBox[{"filterPops", "[", 
              RowBox[{"w", ",", "fullfluorvecOn", ",", "vg"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"help1", "=", 
             RowBox[{"Abs", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"vg", "[", 
                 RowBox[{"[", 
                  RowBox[{"All", ",", "1"}], "]"}], "]"}], "-", 
                RowBox[{"0.5", "*", 
                 RowBox[{"NaturalWidth", "[", "2", "]"}]}]}], "/.", 
               RowBox[{"Last", "[", "syst", "]"}]}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"indVgMaxEx", "=", 
             RowBox[{
              RowBox[{"Position", "[", 
               RowBox[{"help1", ",", 
                RowBox[{"Min", "[", "help1", "]"}]}], "]"}], "[", 
              RowBox[{"[", 
               RowBox[{"1", ",", "1"}], "]"}], "]"}]}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"OptionValue", "[", "VerbosityLevel", "]"}], ">", 
               "3"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"Print", "[", 
                RowBox[{"\"\<    indVgMaxEx: \>\"", ",", "indVgMaxEx"}], 
                "]"}], ";"}]}], "]"}], ";"}], "\[IndentingNewLine]", ",", " ", 
           RowBox[{"(*", " ", "ELSE", " ", "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"excitedPopsHelp", "=", "excitedPops"}], ";"}]}], 
          "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         "\[IndentingNewLine]", 
         RowBox[{"psiSpanAll", "=", 
          RowBox[{"Join", "[", 
           RowBox[{"psiSpanAll", ",", "psiSpan"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"psiAvgTotal", "+=", "psiAvg"}], ";", "\[IndentingNewLine]", 
         RowBox[{"errAll", "=", 
          RowBox[{"Join", "[", 
           RowBox[{"errAll", ",", 
            RowBox[{"{", "err", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"psiAvgHist", "=", 
          RowBox[{"Join", "[", 
           RowBox[{"psiAvgHist", ",", 
            RowBox[{"{", "psiAvg", "}"}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{
          "Scale", " ", "w", " ", "to", " ", "compensate", " ", "for", " ", 
           "small", " ", "normalization", " ", "errors"}], " ", "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"w", "*=", 
          RowBox[{"1.0", "/", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"popgr", "[", 
              RowBox[{"[", 
               RowBox[{
                RowBox[{"-", "1"}], ",", "2"}], "]"}], "]"}], "+", 
             RowBox[{"popex", "[", 
              RowBox[{"[", 
               RowBox[{
                RowBox[{"-", "1"}], ",", "2"}], "]"}], "]"}]}], ")"}]}]}], 
         ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{
          "Terminate", " ", "iteration", " ", "if", " ", "two", " ", 
           "subsequent", " ", "psi", " ", "values", " ", "are", " ", "close", 
           " ", "to", " ", "each", " ", "other"}], " ", "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"terminationCriterion", "=", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"Abs", "[", 
              RowBox[{
               RowBox[{"psiAvgLast", "/", "psiAvg"}], "-", "1.0"}], "]"}], 
             "<", "TerminationThresh"}], "&&", 
            RowBox[{"i", ">", "4"}]}], ")"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{
           RowBox[{"Save", " ", "data", " ", 
            RowBox[{"file", ":", " ", 
             RowBox[{"first", " ", "4", " ", "iterations"}]}]}], ",", " ", 
           RowBox[{
           "then", " ", "once", " ", "per", " ", "CPU", " ", "hour", " ", 
            "at", " ", "the", " ", "most"}], ",", " ", 
           RowBox[{"then", " ", "last"}]}], " ", "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"fwriteCriterion", "=", 
          RowBox[{
           RowBox[{
            RowBox[{"StringLength", "[", "fname", "]"}], ">", "0"}], "&&", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"i", "<", "5"}], "||", "terminationCriterion", "||", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"TimeUsed", "[", "]"}], "-", "tuLast"}], ")"}], ">", 
              "3600.0"}], "||", 
             RowBox[{"i", "==", "nperiods"}]}], ")"}]}]}], ";", 
         "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"Global`res", "=", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"terminationCriterion", "||", "fwriteCriterion", "||", 
             RowBox[{"i", "==", "nperiods"}]}], ",", "\[IndentingNewLine]", 
            RowBox[{"{", 
             RowBox[{
             "psiAvg", ",", "psiAvgTotal", ",", "psiSpanAll", ",", 
              "psiAvgHist", ",", "Iavg", ",", "popgr8All", ",", "popex16All", 
              ",", "popgr8WeightedAll", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"PhiVErrors", "->", "errAll"}], ",", 
                RowBox[{"PsiIavg", "->", "psiIavg1"}], ",", 
                RowBox[{"GroundPopulation", "->", "popgr"}], ",", 
                RowBox[{"ExcitedPopulation", "->", "popex"}], ",", 
                RowBox[{"FinalGroundPops", "->", "groundPops"}], ",", 
                RowBox[{"FinalExcitedPops", "->", "excitedPops"}], ",", 
                RowBox[{"ExcitedPops", "->", "excitedPopsAll"}], ",", 
                RowBox[{"LastDM", "->", 
                 RowBox[{"{", 
                  RowBox[{"w", ",", "vg"}], "}"}]}], ",", 
                RowBox[{"RFParamsOut", "->", 
                 RowBox[{"{", 
                  RowBox[{"rfparams1", ",", "params1", ",", "opts"}], 
                  "}"}]}]}], "}"}]}], "}"}], "\[IndentingNewLine]", ",", 
            RowBox[{"{", "}"}]}], "\[IndentingNewLine]", "]"}]}], ";", 
         "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{"fwriteCriterion", ",", "\[IndentingNewLine]", 
           RowBox[{"(*", " ", 
            RowBox[{
             RowBox[{
             "Make", " ", "sure", " ", "\"\<res\>\"", " ", "does", " ", "not",
               " ", "appear", " ", "in", " ", "the", " ", "variable", " ", 
              "list", " ", "of", " ", "this", " ", "Module", " ", 
              RowBox[{"(", 
               RowBox[{
               "must", " ", "not", " ", "appear", " ", "green", " ", "in", 
                " ", "the", " ", "following", " ", "line"}], ")"}]}], ",", 
             " ", 
             RowBox[{
             "otherwise", " ", "it", " ", "cannot", " ", "be", " ", "loaded", 
              " ", "by", " ", 
              RowBox[{"Get", "[", "]"}], " ", "into", " ", "the", " ", 
              "global", " ", 
              RowBox[{"context", "!"}]}]}], " ", "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Export", "[", 
             RowBox[{"fname", ",", "Global`res"}], "]"}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"tuLast", "=", 
             RowBox[{"TimeUsed", "[", "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"OptionValue", "[", "VerbosityLevel", "]"}], ">", 
               "2"}], ",", 
              RowBox[{
               RowBox[{"Print", "[", 
                RowBox[{
                "\"\< ==> Saved (intermediate) results to \>\"", ",", 
                 "fname"}], "]"}], ";"}]}], "]"}], ";"}]}], 
          "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"tu2", "=", 
          RowBox[{"TimeUsed", "[", "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"OptionValue", "[", "VerbosityLevel", "]"}], ">", "1"}], 
           ",", 
           RowBox[{
            RowBox[{"Print", "[", 
             RowBox[{
             "\"\< Period \>\"", ",", "i", ",", "\"\< done, CPU time: \>\"", 
              ",", 
              RowBox[{"tu2", "-", "tu1"}], ",", 
              "\"\<s, matrix prep CPU time: \>\"", ",", "tPrep", ",", 
              "\"\<s, psi avg: \>\"", ",", "psiAvg", ",", "\"\<\\n\>\""}], 
             "]"}], ";"}]}], "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"(*", " ", 
          RowBox[{
          "Terminate", " ", "iteration", " ", "if", " ", "two", " ", 
           "subsequent", " ", "psi", " ", "values", " ", "are", " ", "close", 
           " ", "to", " ", "each", " ", "other"}], " ", "*)"}], 
         RowBox[{"If", "[", 
          RowBox[{"terminationCriterion", ",", 
           RowBox[{
            RowBox[{"Print", "[", 
             RowBox[{"\"\< Last two psi values differ by only \>\"", ",", 
              RowBox[{"100.0", "*", 
               RowBox[{"Abs", "[", 
                RowBox[{
                 RowBox[{"psiAvgLast", "/", "psiAvg"}], "-", "1.0"}], "]"}]}],
               ",", "\"\<%, terminating iteration now.\>\""}], "]"}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"Break", "[", "]"}], ";"}]}], "]"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"psiAvgLast", "=", "psiAvg"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{"For", "[", "i", "]"}], ",", 
        RowBox[{"main", " ", "loop", " ", "done"}]}], "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
        RowBox[{
        "===", "===", "===", "===", "===", "===", "===", "===", "==="}], "=", 
        " ", 
        RowBox[{
         RowBox[{"MAIN", " ", "LOOP", " ", "END"}], " ", "===", "===", "===", 
         RowBox[{"==", 
          RowBox[{
          "===", "===", "===", "===", "===", "===", "===", "==="}]}]}]}], " ",
        "*)"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"OptionValue", "[", "VerbosityLevel", "]"}], ">", "0"}], ",",
         "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Print", "[", 
          RowBox[{
          "\"\<Completed, psi averaged over last period: \>\"", ",", "psiAvg",
            ",", "\"\< ph/s/sr/atom/(W/m^2)\>\""}], "]"}], ";", 
         RowBox[{"Print", "[", 
          RowBox[{
          "\"\<accumulated psi over all periods: \>\"", ",", "psiAvgTotal", 
           " ", ",", "\"\<, psi from avg cw irradiance: \>\"", ",", 
           "psiIavg1"}], "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
      "\[IndentingNewLine]", "Global`res"}]}], "\[IndentingNewLine]", "]"}]}],
   ";"}]], "Input",
 InitializationCell->True],

Cell[" ", "Text",
 Editable->False,
 Selectable->False,
 CellFrame->{{0, 0}, {0, 2}},
 ShowCellBracket->False,
 CellMargins->{{0, 0}, {1, 1}},
 CellElementSpacings->{"CellMinHeight"->1},
 CellFrameMargins->0,
 CellFrameColor->RGBColor[0, 0, 1],
 CellSize->{Inherited, 4}]
}, Open  ]],

Cell[CellGroupData[{

Cell["End the private context", "Subsection"],

Cell[BoxData[
 RowBox[{"End", "[", "]"}]], "Input",
 InitializationCell->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Epilog", "Section"],

Cell["This section protects exported symbols and ends the package.", "Text"],

Cell[CellGroupData[{

Cell["Protect exported symbols", "Subsection"],

Cell[BoxData[
 RowBox[{"Protect", "[", 
  RowBox[{"Evaluate", "[", 
   RowBox[{"$Context", "<>", "\"\<*\>\""}], "]"}], "]"}]], "Input",
 InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["End the package context", "Subsection"],

Cell[BoxData[
 RowBox[{"EndPackage", "[", "]"}]], "Input",
 InitializationCell->True]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->Automatic,
WindowSize->{1440, 851},
WindowMargins->{{0, Automatic}, {Automatic, -1}},
PrivateNotebookOptions->{"VersionedStylesheet"->{"Default.nb"[8.] -> True}},
TrackCellChangeTimes->False,
FrontEndVersion->"10.4 for Mac OS X x86 (32-bit, 64-bit Kernel) (April 11, \
2016)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 26, 0, 77, "Title"],
Cell[CellGroupData[{
Cell[618, 26, 28, 0, 68, "Section"],
Cell[CellGroupData[{
Cell[671, 30, 27, 0, 35, "Subsection"],
Cell[701, 32, 26, 0, 26, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[764, 37, 29, 0, 35, "Subsection"],
Cell[796, 39, 129, 3, 26, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[962, 47, 29, 0, 35, "Subsection"],
Cell[994, 49, 117, 3, 26, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1148, 57, 31, 0, 35, "Subsection"],
Cell[1182, 59, 552, 11, 101, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1771, 75, 38, 0, 35, "Subsection"],
Cell[1812, 77, 34, 0, 26, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1883, 82, 94, 4, 35, "Subsection"],
Cell[1980, 88, 20, 0, 26, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2037, 93, 30, 0, 35, "Subsection"],
Cell[2070, 95, 98, 2, 26, "Text"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2217, 103, 28, 0, 68, "Section"],
Cell[2248, 105, 60, 0, 26, "Text"],
Cell[CellGroupData[{
Cell[2333, 109, 43, 0, 35, "Subsection"],
Cell[2379, 111, 182, 4, 29, "Input",
 InitializationCell->True],
Cell[2564, 117, 110, 2, 29, "Input",
 InitializationCell->True],
Cell[2677, 121, 545, 12, 45, "Input",
 InitializationCell->True],
Cell[3225, 135, 164, 4, 29, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[3426, 144, 36, 0, 35, "Subsection"],
Cell[CellGroupData[{
Cell[3487, 148, 30, 0, 24, "Subsubsection"],
Cell[CellGroupData[{
Cell[3542, 152, 274, 6, 29, "Input",
 InitializationCell->True],
Cell[3819, 160, 216, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4072, 167, 292, 6, 29, "Input",
 InitializationCell->True],
Cell[4367, 175, 222, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4626, 182, 1249, 21, 173, "Input",
 InitializationCell->True],
Cell[5878, 205, 1259, 16, 157, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[7174, 226, 276, 6, 29, "Input",
 InitializationCell->True],
Cell[7453, 234, 182, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[7672, 241, 347, 6, 61, "Input",
 InitializationCell->True],
Cell[8022, 249, 245, 3, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[8304, 257, 304, 6, 45, "Input",
 InitializationCell->True],
Cell[8611, 265, 201, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[8849, 272, 285, 6, 29, "Input",
 InitializationCell->True],
Cell[9137, 280, 195, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[9369, 287, 280, 7, 29, "Input",
 InitializationCell->True],
Cell[9652, 296, 123, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[9812, 302, 306, 7, 31, "Input",
 InitializationCell->True],
Cell[10121, 311, 152, 1, 31, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[10310, 317, 304, 7, 29, "Input",
 InitializationCell->True],
Cell[10617, 326, 131, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[10785, 332, 207, 5, 29, "Input",
 InitializationCell->True],
Cell[10995, 339, 119, 1, 29, "Output"]
}, Open  ]],
Cell[11129, 343, 122, 3, 29, "Input",
 InitializationCell->True],
Cell[11254, 348, 120, 3, 29, "Input",
 InitializationCell->True],
Cell[11377, 353, 128, 3, 29, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[11530, 360, 325, 7, 29, "Input",
 InitializationCell->True],
Cell[11858, 369, 152, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[12047, 375, 272, 7, 29, "Input",
 InitializationCell->True],
Cell[12322, 384, 111, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[12470, 390, 266, 7, 29, "Input",
 InitializationCell->True],
Cell[12739, 399, 103, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[12879, 405, 295, 7, 31, "Input",
 InitializationCell->True],
Cell[13177, 414, 147, 1, 31, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[13361, 420, 242, 6, 29, "Input",
 InitializationCell->True],
Cell[13606, 428, 73, 0, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[13716, 433, 272, 7, 29, "Input",
 InitializationCell->True],
Cell[13991, 442, 111, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[14139, 448, 266, 7, 29, "Input",
 InitializationCell->True],
Cell[14408, 457, 103, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[14548, 463, 295, 7, 31, "Input",
 InitializationCell->True],
Cell[14846, 472, 147, 1, 31, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[15030, 478, 242, 6, 29, "Input",
 InitializationCell->True],
Cell[15275, 486, 73, 0, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[15385, 491, 278, 7, 31, "Input",
 InitializationCell->True],
Cell[15666, 500, 133, 1, 31, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[15836, 506, 314, 7, 31, "Input",
 InitializationCell->True],
Cell[16153, 515, 163, 2, 31, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[16353, 522, 287, 7, 31, "Input",
 InitializationCell->True],
Cell[16643, 531, 136, 1, 31, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[16816, 537, 538, 10, 63, "Input",
 InitializationCell->True],
Cell[17357, 549, 386, 5, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[17780, 559, 214, 6, 29, "Input",
 InitializationCell->True],
Cell[17997, 567, 61, 0, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[18095, 572, 253, 7, 29, "Input",
 InitializationCell->True],
Cell[18351, 581, 102, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[18490, 587, 212, 5, 29, "Input",
 InitializationCell->True],
Cell[18705, 594, 114, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[18856, 600, 184, 5, 29, "Input",
 InitializationCell->True],
Cell[19043, 607, 94, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[19174, 613, 314, 7, 29, "Input",
 InitializationCell->True],
Cell[19491, 622, 145, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[19673, 628, 662, 12, 77, "Input",
 InitializationCell->True],
Cell[20338, 642, 124, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[20499, 648, 329, 7, 29, "Input",
 InitializationCell->True],
Cell[20831, 657, 164, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[21032, 664, 282, 7, 29, "Input",
 InitializationCell->True],
Cell[21317, 673, 131, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[21485, 679, 346, 9, 29, "Input",
 InitializationCell->True],
Cell[21834, 690, 216, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[22087, 697, 314, 8, 29, "Input",
 InitializationCell->True],
Cell[22404, 707, 199, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[22640, 714, 326, 8, 29, "Input",
 InitializationCell->True],
Cell[22969, 724, 208, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[23214, 731, 312, 6, 45, "Input",
 InitializationCell->True],
Cell[23529, 739, 214, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[23780, 746, 288, 6, 29, "Input",
 InitializationCell->True],
Cell[24071, 754, 190, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[24298, 761, 283, 6, 29, "Input",
 InitializationCell->True],
Cell[24584, 769, 192, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[24813, 776, 1543, 27, 189, "Input",
 InitializationCell->True],
Cell[26359, 805, 1596, 21, 173, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[27992, 831, 878, 14, 125, "Input",
 InitializationCell->True],
Cell[28873, 847, 796, 10, 125, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[29706, 862, 380, 7, 61, "Input",
 InitializationCell->True],
Cell[30089, 871, 276, 3, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[30402, 879, 973, 17, 95, "Input",
 InitializationCell->True],
Cell[31378, 898, 998, 14, 79, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[32413, 917, 1308, 21, 111, "Input",
 InitializationCell->True],
Cell[33724, 940, 1397, 19, 79, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[35158, 964, 300, 6, 45, "Input",
 InitializationCell->True],
Cell[35461, 972, 203, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[35701, 979, 296, 6, 45, "Input",
 InitializationCell->True],
Cell[36000, 987, 195, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[36232, 994, 306, 6, 45, "Input",
 InitializationCell->True],
Cell[36541, 1002, 207, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[36785, 1009, 284, 6, 29, "Input",
 InitializationCell->True],
Cell[37072, 1017, 187, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[37296, 1024, 322, 6, 61, "Input",
 InitializationCell->True],
Cell[37621, 1032, 229, 2, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[37887, 1039, 382, 7, 61, "Input",
 InitializationCell->True],
Cell[38272, 1048, 288, 3, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[38597, 1056, 229, 5, 45, "Input",
 InitializationCell->True],
Cell[38829, 1063, 120, 1, 29, "Output"]
}, Open  ]],
Cell[38964, 1067, 744, 12, 173, "Input",
 InitializationCell->True],
Cell[39711, 1081, 129, 3, 29, "Input",
 InitializationCell->True],
Cell[39843, 1086, 121, 3, 29, "Input",
 InitializationCell->True],
Cell[39967, 1091, 126, 3, 29, "Input",
 InitializationCell->True],
Cell[40096, 1096, 126, 3, 29, "Input",
 InitializationCell->True],
Cell[40225, 1101, 126, 3, 29, "Input",
 InitializationCell->True],
Cell[40354, 1106, 127, 3, 29, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[40518, 1114, 61, 0, 24, "Subsubsection"],
Cell[CellGroupData[{
Cell[40604, 1118, 236, 5, 29, "Input",
 InitializationCell->True],
Cell[40843, 1125, 144, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[41024, 1131, 224, 5, 29, "Input",
 InitializationCell->True],
Cell[41251, 1138, 136, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[41424, 1144, 250, 5, 29, "Input",
 InitializationCell->True],
Cell[41677, 1151, 154, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[41868, 1157, 242, 5, 29, "Input",
 InitializationCell->True],
Cell[42113, 1164, 149, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[42299, 1170, 239, 5, 29, "Input",
 InitializationCell->True],
Cell[42541, 1177, 133, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[42711, 1183, 335, 7, 61, "Input",
 InitializationCell->True],
Cell[43049, 1192, 241, 3, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[43327, 1200, 354, 7, 61, "Input",
 InitializationCell->True],
Cell[43684, 1209, 259, 3, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[43980, 1217, 364, 12, 33, "Input",
 InitializationCell->True],
Cell[44347, 1231, 356, 4, 33, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[44740, 1240, 237, 5, 29, "Input",
 InitializationCell->True],
Cell[44980, 1247, 140, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[45157, 1253, 228, 5, 29, "Input",
 InitializationCell->True],
Cell[45388, 1260, 128, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[45553, 1266, 195, 5, 29, "Input",
 InitializationCell->True],
Cell[45751, 1273, 96, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[45884, 1279, 166, 4, 29, "Input",
 InitializationCell->True],
Cell[46053, 1285, 78, 0, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[46168, 1290, 207, 5, 29, "Input",
 InitializationCell->True],
Cell[46378, 1297, 118, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[46533, 1303, 354, 7, 61, "Input",
 InitializationCell->True],
Cell[46890, 1312, 249, 3, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[47176, 1320, 305, 6, 29, "Input",
 InitializationCell->True],
Cell[47484, 1328, 195, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[47716, 1335, 275, 8, 29, "Input",
 InitializationCell->True],
Cell[47994, 1345, 228, 3, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[48259, 1353, 202, 5, 29, "Input",
 InitializationCell->True],
Cell[48464, 1360, 117, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[48618, 1366, 203, 5, 29, "Input",
 InitializationCell->True],
Cell[48824, 1373, 108, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[48969, 1379, 201, 5, 29, "Input",
 InitializationCell->True],
Cell[49173, 1386, 111, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[49321, 1392, 190, 5, 29, "Input",
 InitializationCell->True],
Cell[49514, 1399, 104, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[49655, 1405, 239, 5, 29, "Input",
 InitializationCell->True],
Cell[49897, 1412, 144, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[50078, 1418, 216, 5, 29, "Input",
 InitializationCell->True],
Cell[50297, 1425, 126, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[50460, 1431, 186, 5, 29, "Input",
 InitializationCell->True],
Cell[50649, 1438, 96, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[50782, 1444, 195, 5, 29, "Input",
 InitializationCell->True],
Cell[50980, 1451, 107, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[51124, 1457, 170, 4, 29, "Input",
 InitializationCell->True],
Cell[51297, 1463, 86, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[51420, 1469, 288, 6, 33, "Input",
 InitializationCell->True],
Cell[51711, 1477, 207, 2, 33, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[51955, 1484, 276, 6, 31, "Input",
 InitializationCell->True],
Cell[52234, 1492, 206, 3, 31, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[52477, 1500, 409, 10, 47, "Input",
 InitializationCell->True],
Cell[52889, 1512, 359, 4, 31, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[53285, 1521, 484, 15, 33, "Input",
 InitializationCell->True],
Cell[53772, 1538, 482, 6, 33, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[54291, 1549, 237, 5, 29, "Input",
 InitializationCell->True],
Cell[54531, 1556, 142, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[54710, 1562, 301, 6, 45, "Input",
 InitializationCell->True],
Cell[55014, 1570, 207, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[55258, 1577, 246, 5, 29, "Input",
 InitializationCell->True],
Cell[55507, 1584, 157, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[55701, 1591, 393, 7, 61, "Input",
 InitializationCell->True],
Cell[56097, 1600, 293, 3, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[56427, 1608, 274, 6, 29, "Input",
 InitializationCell->True],
Cell[56704, 1616, 169, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[56910, 1623, 221, 5, 29, "Input",
 InitializationCell->True],
Cell[57134, 1630, 130, 1, 29, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[57313, 1637, 32, 0, 24, "Subsubsection"],
Cell[57348, 1639, 122, 3, 29, "Input",
 InitializationCell->True],
Cell[57473, 1644, 135, 4, 29, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[57645, 1653, 39, 0, 24, "Subsubsection"],
Cell[CellGroupData[{
Cell[57709, 1657, 201, 5, 29, "Input",
 InitializationCell->True],
Cell[57913, 1664, 96, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[58046, 1670, 195, 5, 29, "Input",
 InitializationCell->True],
Cell[58244, 1677, 95, 1, 29, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[58388, 1684, 38, 0, 24, "Subsubsection"],
Cell[CellGroupData[{
Cell[58451, 1688, 231, 5, 70, "Input",
 InitializationCell->True],
Cell[58685, 1695, 135, 1, 70, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[58869, 1702, 28, 0, 18, "Subsubsection"],
Cell[58900, 1704, 1677, 26, 323, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[60614, 1735, 40, 0, 24, "Subsubsection"],
Cell[60657, 1737, 943, 15, 223, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[61637, 1757, 38, 0, 24, "Subsubsection"],
Cell[61678, 1759, 3984, 56, 733, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[65687, 1819, 280, 6, 29, "Input",
 InitializationCell->True],
Cell[65970, 1827, 187, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[66194, 1834, 211, 5, 29, "Input",
 InitializationCell->True],
Cell[66408, 1841, 124, 1, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[66569, 1847, 347, 7, 61, "Input",
 InitializationCell->True],
Cell[66919, 1856, 252, 3, 45, "Output"]
}, Open  ]],
Cell[67186, 1862, 656, 13, 93, "Input",
 InitializationCell->True],
Cell[67845, 1877, 631, 8, 77, "Input"],
Cell[CellGroupData[{
Cell[68501, 1889, 269, 6, 29, "Input",
 InitializationCell->True],
Cell[68773, 1897, 184, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[68994, 1904, 271, 6, 29, "Input",
 InitializationCell->True],
Cell[69268, 1912, 185, 2, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[69490, 1919, 650, 11, 93, "Input",
 InitializationCell->True],
Cell[70143, 1932, 567, 7, 93, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[70747, 1944, 350, 7, 61, "Input",
 InitializationCell->True],
Cell[71100, 1953, 267, 3, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[71404, 1961, 392, 7, 61, "Input",
 InitializationCell->True],
Cell[71799, 1970, 305, 3, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[72141, 1978, 396, 7, 61, "Input",
 InitializationCell->True],
Cell[72540, 1987, 281, 3, 45, "Output"]
}, Open  ]],
Cell[72836, 1993, 120, 3, 29, "Input",
 InitializationCell->True],
Cell[72959, 1998, 117, 3, 29, "Input",
 InitializationCell->True],
Cell[73079, 2003, 126, 3, 29, "Input",
 InitializationCell->True],
Cell[73208, 2008, 127, 3, 29, "Input",
 InitializationCell->True],
Cell[73338, 2013, 125, 3, 29, "Input",
 InitializationCell->True],
Cell[73466, 2018, 126, 3, 29, "Input",
 InitializationCell->True],
Cell[73595, 2023, 121, 3, 29, "Input",
 InitializationCell->True],
Cell[73719, 2028, 116, 3, 29, "Input",
 InitializationCell->True],
Cell[73838, 2033, 121, 3, 29, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[73996, 2041, 43, 0, 24, "Subsubsection"],
Cell[74042, 2043, 1906, 51, 317, "Input",
 InitializationCell->True]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[76009, 2101, 33, 0, 68, "Section"],
Cell[CellGroupData[{
Cell[76067, 2105, 43, 0, 35, "Subsection"],
Cell[76113, 2107, 101, 2, 29, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[76251, 2114, 27, 0, 35, "Subsection"],
Cell[76281, 2116, 75, 0, 26, "Text"],
Cell[76359, 2118, 255, 7, 29, "Input",
 InitializationCell->True],
Cell[76617, 2127, 93, 2, 26, "Text"],
Cell[76713, 2131, 1772, 47, 77, "Input",
 InitializationCell->True],
Cell[78488, 2180, 60, 0, 26, "Text"],
Cell[78551, 2182, 892, 25, 45, "Input",
 InitializationCell->True],
Cell[79446, 2209, 48, 0, 26, "Text"],
Cell[79497, 2211, 338, 11, 29, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[79872, 2227, 39, 0, 35, "Subsection"],
Cell[79914, 2229, 224, 7, 29, "Input",
 InitializationCell->True],
Cell[80141, 2238, 249, 8, 29, "Input",
 InitializationCell->True],
Cell[80393, 2248, 399, 10, 29, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[80829, 2263, 37, 0, 35, "Subsection"],
Cell[80869, 2265, 286, 5, 26, "Text"],
Cell[81158, 2272, 1023, 20, 161, "Text"],
Cell[82184, 2294, 174, 3, 26, "Text"],
Cell[82361, 2299, 122, 3, 28, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[82508, 2306, 767, 23, 39, "Input",
 InitializationCell->True],
Cell[83278, 2331, 282, 9, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[83597, 2345, 795, 24, 39, "Input",
 InitializationCell->True],
Cell[84395, 2371, 189, 6, 45, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[84621, 2382, 436, 14, 39, "Input",
 InitializationCell->True],
Cell[85060, 2398, 264, 8, 68, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[85361, 2411, 428, 13, 29, "Input",
 InitializationCell->True],
Cell[85792, 2426, 222, 6, 43, "Output"]
}, Open  ]],
Cell[86029, 2435, 505, 15, 29, "Input",
 InitializationCell->True],
Cell[86537, 2452, 203, 6, 29, "Input",
 InitializationCell->True],
Cell[86743, 2460, 555, 17, 29, "Input",
 InitializationCell->True],
Cell[87301, 2479, 7904, 210, 347, "Input",
 InitializationCell->True],
Cell[95208, 2691, 631, 22, 33, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[95876, 2718, 37, 0, 35, "Subsection"],
Cell[95916, 2720, 136, 3, 26, "Text"],
Cell[96055, 2725, 359, 13, 50, "Input",
 InitializationCell->True],
Cell[96417, 2740, 355, 13, 52, "Input",
 InitializationCell->True],
Cell[96775, 2755, 1571, 45, 88, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[98371, 2804, 767, 23, 39, "Input",
 InitializationCell->True],
Cell[99141, 2829, 282, 9, 45, "Output"]
}, Open  ]],
Cell[99438, 2841, 951, 30, 29, "Input",
 InitializationCell->True],
Cell[100392, 2873, 203, 6, 29, "Input",
 InitializationCell->True],
Cell[100598, 2881, 6668, 173, 331, "Input",
 InitializationCell->True],
Cell[107269, 3056, 725, 25, 49, "Input",
 InitializationCell->True],
Cell[107997, 3083, 1861, 58, 75, "Input",
 InitializationCell->True],
Cell[109861, 3143, 2771, 79, 93, "Input",
 InitializationCell->True],
Cell[112635, 3224, 1859, 53, 68, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[114531, 3282, 37, 0, 35, "Subsection"],
Cell[114571, 3284, 1004, 20, 146, "Text"],
Cell[115578, 3306, 172, 3, 26, "Text"],
Cell[115753, 3311, 449, 7, 41, "Text"],
Cell[116205, 3320, 743, 21, 45, "Input",
 InitializationCell->True],
Cell[116951, 3343, 3547, 85, 157, "Input",
 InitializationCell->True],
Cell[120501, 3430, 9098, 231, 539, "Input",
 InitializationCell->True],
Cell[129602, 3663, 2273, 62, 125, "Input",
 InitializationCell->True],
Cell[131878, 3727, 293, 9, 29, "Input",
 InitializationCell->True],
Cell[132174, 3738, 22400, 560, 1276, "Input",
 InitializationCell->True],
Cell[154577, 4300, 5725, 144, 517, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[160339, 4449, 43, 0, 35, "Subsection"],
Cell[160385, 4451, 102, 2, 26, "Text"],
Cell[160490, 4455, 9753, 261, 571, "Input",
 InitializationCell->True],
Cell[170246, 4718, 70, 0, 26, "Text"],
Cell[170319, 4720, 1090, 31, 61, "Input",
 InitializationCell->True],
Cell[171412, 4753, 472, 12, 92, "Text"],
Cell[171887, 4767, 6241, 151, 481, "Input",
 InitializationCell->True],
Cell[178131, 4920, 801, 19, 56, "Text"],
Cell[178935, 4941, 1396, 23, 131, "Text"],
Cell[180334, 4966, 141, 3, 26, "Text"],
Cell[180478, 4971, 3603, 87, 125, "Input",
 InitializationCell->True],
Cell[184084, 5060, 142, 4, 41, "Text"],
Cell[184229, 5066, 1760, 54, 66, "Input",
 InitializationCell->True],
Cell[185992, 5122, 849, 23, 45, "Input",
 InitializationCell->True],
Cell[186844, 5147, 124, 3, 26, "Text"],
Cell[186971, 5152, 2665, 65, 139, "Input",
 InitializationCell->True],
Cell[189639, 5219, 56, 0, 26, "Text"],
Cell[189698, 5221, 350, 7, 41, "Text"],
Cell[190051, 5230, 2556, 68, 93, "Input",
 InitializationCell->True],
Cell[192610, 5300, 76, 0, 26, "Text"],
Cell[192689, 5302, 3380, 82, 141, "Input",
 InitializationCell->True],
Cell[196072, 5386, 273, 5, 26, "Text"],
Cell[196348, 5393, 279, 8, 29, "Input",
 InitializationCell->True],
Cell[196630, 5403, 466, 12, 44, "Text"],
Cell[197099, 5417, 1704, 50, 141, "Input",
 InitializationCell->True],
Cell[198806, 5469, 270, 5, 26, "Text"],
Cell[199079, 5476, 135, 3, 26, "Text"],
Cell[199217, 5481, 3026, 85, 165, "Input",
 InitializationCell->True],
Cell[202246, 5568, 142, 3, 26, "Text"],
Cell[202391, 5573, 148, 3, 26, "Text"],
Cell[202542, 5578, 756, 22, 29, "Input",
 InitializationCell->True],
Cell[203301, 5602, 87, 2, 26, "Text"],
Cell[203391, 5606, 529, 17, 29, "Input",
 InitializationCell->True],
Cell[203923, 5625, 602, 18, 29, "Input",
 InitializationCell->True],
Cell[204528, 5645, 267, 5, 26, "Text"],
Cell[204798, 5652, 496, 16, 29, "Input",
 InitializationCell->True],
Cell[205297, 5670, 386, 11, 26, "Text"],
Cell[205686, 5683, 388, 11, 29, "Input",
 InitializationCell->True],
Cell[206077, 5696, 164, 3, 26, "Text"],
Cell[206244, 5701, 365, 11, 29, "Input",
 InitializationCell->True],
Cell[206612, 5714, 169, 3, 26, "Text"],
Cell[206784, 5719, 317, 9, 29, "Input",
 InitializationCell->True],
Cell[207104, 5730, 367, 10, 29, "Input",
 InitializationCell->True],
Cell[207474, 5742, 376, 11, 29, "Input",
 InitializationCell->True],
Cell[207853, 5755, 328, 9, 29, "Input",
 InitializationCell->True],
Cell[208184, 5766, 319, 5, 41, "Text"],
Cell[208506, 5773, 3046, 87, 200, "Input",
 InitializationCell->True],
Cell[211555, 5862, 418, 11, 41, "Text"],
Cell[211976, 5875, 4827, 136, 298, "Input",
 InitializationCell->True],
Cell[216806, 6013, 435, 10, 29, "Input",
 InitializationCell->True],
Cell[217244, 6025, 231, 5, 29, "Input"],
Cell[217478, 6032, 4214, 94, 506, "Input",
 InitializationCell->True],
Cell[221695, 6128, 271, 9, 8, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[222003, 6142, 26, 0, 35, "Subsection"],
Cell[222032, 6144, 2307, 79, 1166, "Text"],
Cell[224342, 6225, 195, 6, 29, "Input",
 InitializationCell->True],
Cell[224540, 6233, 46517, 1108, 2936, "Input",
 InitializationCell->True],
Cell[271060, 7343, 271, 9, 8, "Text"],
Cell[271334, 7354, 216, 4, 29, "Input"],
Cell[271553, 7360, 23014, 552, 1666, "Input",
 InitializationCell->True],
Cell[294570, 7914, 273, 9, 5, "Text"],
Cell[294846, 7925, 778, 16, 61, "Input"],
Cell[295627, 7943, 32862, 820, 2003, "Input",
 InitializationCell->True],
Cell[328492, 8765, 273, 9, 5, "Text"],
Cell[328768, 8776, 772, 19, 45, "Input",
 InitializationCell->True],
Cell[329543, 8797, 273, 9, 5, "Text"],
Cell[329819, 8808, 401, 9, 29, "Input"],
Cell[330223, 8819, 9527, 238, 808, "Input",
 InitializationCell->True],
Cell[339753, 9059, 271, 9, 8, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[340061, 9073, 58, 0, 35, "Subsection"],
Cell[340122, 9075, 3467, 97, 191, "Input",
 InitializationCell->True],
Cell[343592, 9174, 747, 17, 152, "Text"],
Cell[344342, 9193, 692, 17, 53, "Input",
 InitializationCell->True],
Cell[345037, 9212, 559, 13, 116, "Text"],
Cell[345599, 9227, 847, 23, 53, "Input",
 InitializationCell->True],
Cell[346449, 9252, 709, 18, 140, "Text"],
Cell[347161, 9272, 669, 18, 29, "Input",
 InitializationCell->True],
Cell[347833, 9292, 838, 25, 113, "Text"],
Cell[348674, 9319, 377, 10, 29, "Input",
 InitializationCell->True],
Cell[349054, 9331, 310, 7, 29, "Input",
 InitializationCell->True],
Cell[349367, 9340, 203, 6, 29, "Input",
 InitializationCell->True],
Cell[349573, 9348, 1330, 28, 77, "Input"],
Cell[350906, 9378, 108, 2, 28, "Input",
 InitializationCell->True],
Cell[351017, 9382, 1330, 28, 77, "Input"],
Cell[352350, 9412, 109, 2, 29, "Input",
 InitializationCell->True],
Cell[352462, 9416, 1129, 25, 61, "Input"],
Cell[353594, 9443, 2070, 52, 77, "Input",
 InitializationCell->True],
Cell[355667, 9497, 118, 3, 26, "Text"],
Cell[355788, 9502, 1857, 46, 77, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[357682, 9553, 29, 0, 35, "Subsection"],
Cell[357714, 9555, 495, 15, 45, "Input",
 InitializationCell->True],
Cell[358212, 9572, 676, 18, 45, "Input"],
Cell[358891, 9592, 2971, 79, 188, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[361899, 9676, 36, 0, 35, "Subsection"],
Cell[361938, 9678, 755, 21, 105, "Input",
 InitializationCell->True],
Cell[362696, 9701, 655, 17, 110, "Text"],
Cell[363354, 9720, 594, 18, 33, "Input",
 InitializationCell->True],
Cell[363951, 9740, 622, 19, 45, "Input",
 InitializationCell->True],
Cell[364576, 9761, 120, 3, 26, "Text"],
Cell[364699, 9766, 612, 16, 45, "Input",
 InitializationCell->True],
Cell[365314, 9784, 404, 9, 29, "Input"],
Cell[365721, 9795, 5829, 140, 446, "Input",
 InitializationCell->True],
Cell[371553, 9937, 173, 3, 26, "Text"],
Cell[371729, 9942, 720, 21, 29, "Input",
 InitializationCell->True],
Cell[372452, 9965, 3296, 86, 157, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[375785, 10056, 35, 0, 35, "Subsection"],
Cell[375823, 10058, 209, 4, 41, "Text"],
Cell[376035, 10064, 196, 5, 29, "Input",
 InitializationCell->True],
Cell[376234, 10071, 19188, 448, 1085, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[395459, 10524, 25, 0, 35, "Subsection"],
Cell[395487, 10526, 2970, 76, 395, "Text"],
Cell[398460, 10604, 158, 4, 29, "Input",
 InitializationCell->True],
Cell[398621, 10610, 404, 9, 29, "Input"],
Cell[399028, 10621, 2192, 56, 172, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[401257, 10682, 37, 0, 35, "Subsection"],
Cell[401297, 10684, 1777, 37, 233, "Text"],
Cell[403077, 10723, 342, 10, 29, "Input",
 InitializationCell->True],
Cell[403422, 10735, 560, 15, 29, "Input"],
Cell[403985, 10752, 3930, 104, 220, "Input",
 InitializationCell->True],
Cell[407918, 10858, 273, 9, 5, "Text"],
Cell[408194, 10869, 271, 9, 10, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[408502, 10883, 107, 2, 35, "Subsection"],
Cell[408612, 10887, 9474, 251, 332, "Input",
 InitializationCell->True],
Cell[418089, 11140, 273, 9, 5, "Text"],
Cell[418365, 11151, 205, 5, 29, "Input"],
Cell[418573, 11158, 493, 15, 45, "Input",
 InitializationCell->True],
Cell[419069, 11175, 676, 18, 45, "Input"],
Cell[419748, 11195, 3135, 83, 205, "Input",
 InitializationCell->True],
Cell[422886, 11280, 273, 9, 5, "Text"],
Cell[423162, 11291, 688, 10, 56, "Text"],
Cell[423853, 11303, 2156, 61, 93, "Input",
 InitializationCell->True],
Cell[426012, 11366, 273, 9, 5, "Text"],
Cell[426288, 11377, 388, 13, 29, "Input",
 InitializationCell->True],
Cell[426679, 11392, 273, 9, 5, "Text"],
Cell[426955, 11403, 3877, 56, 611, "Text"],
Cell[430835, 11461, 1030, 28, 61, "Input",
 InitializationCell->True],
Cell[431868, 11491, 923, 22, 45, "Input"],
Cell[432794, 11515, 67325, 1538, 4131, "Input",
 InitializationCell->True],
Cell[500122, 13055, 271, 9, 8, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[500430, 13069, 45, 0, 35, "Subsection"],
Cell[500478, 13071, 78, 2, 28, "Input",
 InitializationCell->True]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[500605, 13079, 25, 0, 68, "Section"],
Cell[500633, 13081, 76, 0, 26, "Text"],
Cell[CellGroupData[{
Cell[500734, 13085, 46, 0, 35, "Subsection"],
Cell[500783, 13087, 162, 4, 28, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[500982, 13096, 45, 0, 35, "Subsection"],
Cell[501030, 13098, 85, 2, 29, "Input",
 InitializationCell->True]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

